Package: fdaSP
Type: Package
Title: Sparse Functional Data Analysis Methods
Version: 1.1.3
Date: 2026-07-21
Author: Mauro Bernardi [aut, cre],
  Marco Stefanucci [aut],
  Antonio Canale [ctb]
Authors@R: c(person("Mauro", "Bernardi", role= c("aut","cre"), email="mauro.bernardi@unipd.it"),
            person("Marco", "Stefanucci", role= c("aut"), email="marco.stefanucci@uniroma2.it"),
	    person("Antonio", "Canale", role= "ctb", email="antonio.canale@unipd.it"))
Maintainer: Mauro Bernardi <mauro.bernardi@unipd.it>
Description: Provides algorithms to fit linear regression models under several popular penalization techniques and functional linear regression models based on Majorizing-Minimizing (MM) and Alternating Direction Method of Multipliers (ADMM) techniques.
    See Boyd et al (2010) <doi:10.1561/2200000016> for complete introduction to the method.
License: GPL (>= 3)
Suggests: RColorBrewer, gglasso, glmnet, latex2exp, utils
Imports: Rcpp, Rdpack, grDevices, graphics, stats, parallel,
        doParallel, foreach, splines, ks, pracma, CVXR
RdMacros: Rdpack
LinkingTo: Rcpp, RcppArmadillo
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-07-22 17:08:26 UTC; maurobernardi
Repository: CRAN
Date/Publication: 2026-07-22 17:50:21 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-07-22 18:42:55 UTC; unix
Archs: fdaSP.so.dSYM

useDynLib(fdaSP, .registration=TRUE)
importFrom("Rcpp", "evalCpp")
importFrom("Rdpack","reprompt")
importFrom("grDevices", "rgb")
importFrom("graphics", "polygon")
importFrom("stats", "acf", "ccf", "lag", "na.omit", "pacf", "pnorm",
           "qnorm", "sd", "uniroot", "var")
importFrom("parallel", "detectCores", "makeCluster")
importFrom("doParallel", "registerDoParallel")
importFrom("foreach", "%dopar%")
importFrom("splines", "bs")
importFrom("ks", "invvec", "vec")
importFrom("pracma", "strcmpi")
importFrom("CVXR", "Variable", "norm2", "Maximize", "Problem", "psolve")

## export all regularly named functions
## (but allow for private functions whose name starts with a dot).name <- function(...)
##exportPattern("^[[:alpha:]]+")
export(lmSP)
export(lmSP_cv)
export(f2sSP)
export(f2sSP_cv)
export(f2fSP)
export(f2fSP_cv)
export(softhresh)
export(confband)
export(forward_diff)

# Authors: Mauro Bernardi 
#          Department Statistical Sciences
#       	 University of Padova
#      	   Via Cesare Battisti, 241
#      	   35121 PADOVA, Italy
#          E-mail: mauro.bernardi@unipd.it  

# Last change: March 16, 2023

# List of functions
# 1. linreg_ADMM_LASSO
# 2. linreg_ADMM_LASSO_1lambda
# 3. linreg_ADMM_LASSO_cv

#' Least Absolute Shrinkage and Selection Operator
#'
#' LASSO, or L1-regularized regression, solves the following optimization problem
#' \deqn{\textrm{min}_\beta ~ \frac{1}{2}\|y-X\beta-Z\gamma\|_2^2 + \lambda \|T \beta\|_1}
#' to obtain a sparse coefficient vector \eqn{\beta\in\mathbb{R}^p} and a coefficient vector \eqn{\gamma\in\mathbb{R}^q}. The diagonal matrix \eqn{T\in\mathbb{R}^{p\times p}} 
#' contains the variable-specific weights. When \eqn{T} is equal to the identity matrix the standard LASSO method is performed, otherwise the adaptive 
#' LASSO method of Zou (2006) is performed. These values are provided by the argument \code{var_weights} (see below).
#' The regularization path is computed for the LASSO penalty at a grid of values for the regularization parameter \eqn{\lambda} using the 
#' alternating direction method of multipliers (ADMM). See Boyd et al. (2011) for details on the ADMM method.
#'  
#' @param X an \eqn{(n\times p)} matrix of penalized predictors.
#' @param Z an \eqn{(n\times q)} full column rank matrix of predictors that are not penalized. 
#' @param y a length-\eqn{n} response vector.
#' @param var_weights a vector of length \eqn{p} containing variable-specific weights. The default is NULL.
#' @param adaptive a flag for running adaptive Lasso. The default is FALSE.
#' @param standardize.data logical. Should data be standardized?
#' @param lambda either a regularization parameter or a vector of regularization parameters. 
#' In this latter case the routine computes the whole path. If it is NULL the path values for lambda are provided by the routine.
#' @param lambda.min.ratio smallest value for lambda, as a fraction of the maximum lambda value, the (data derived) entry value (i.e. the smallest value for which all coefficients are zero). 
#' The default depends on the sample size nobs relative to the number of variables nvars. If nobs > nvars, the default is 0.0001, 
#' close to zero. If nobs < nvars, the default is 0.01. A very small value of lambda.min.ratio will lead to a saturated fit 
#' in the nobs < nvars case.
#' @param nlambda the number of lambda values - default is 30.
#' @param intercept logical. If it is TRUE, a column of ones is added to the design matrix. 
#' @param control	 a list of control parameters. See ‘Details’.
#'
#' @return A named list containing \describe{
#' \item{sp.coefficients}{a length-\eqn{p} solution vector for the parameters \eqn{\beta}, which corresponds to the minimum in-sample MSE.}
#' \item{coefficients}{a length-\eqn{q} solution vector for the parameters \eqn{\gamma}, which corresponds to the minimum in-sample MSE. 
#' It is provided only when either the matrix \eqn{Z} in input is not NULL or the intercept is set to TRUE.}
#' \item{sp.coef.path}{an \eqn{(nlambda\times p)} matrix of estimated \eqn{\beta} coefficients for each lambda of the provided sequence.}
#' \item{coef.path}{an \eqn{(nlambda\times q)} matrix of estimated \eqn{\gamma} coefficients for each lambda of the provided sequence. 
#' It is provided only when either the matrix \eqn{Z} in input is not NULL or the intercept is set to TRUE.}
#' \item{lambda}{sequence of lambda.}
#' \item{lambda.min}{value of lambda that attains the minimum in-sample MSE.}
#' \item{mse}{in-sample mean squared error.}
#' \item{min.mse}{minimum value of the MSE for the sequence of lambdas.}
#' \item{convergence}{logical. 1 denotes achieved convergence.}
#' \item{elapsedTime}{elapsed time in seconds.}
#' \item{iternum}{number of iterations.}
#' }
#'
#' When you run the algorithm, output returns not only the solution, but also the iteration history recording
#' following fields over iterates:
#' \describe{
#' \item{objval}{objective function value}
#' \item{r_norm}{norm of primal residual}
#' \item{s_norm}{norm of dual residual}
#' \item{eps_pri}{feasibility tolerance for primal feasibility condition}
#' \item{eps_dual}{feasibility tolerance for dual feasibility condition.}
#' }
#' Iteration stops when both \code{r_norm} and \code{s_norm} values
#' become smaller than \code{eps_pri} and \code{eps_dual}, respectively.
#' @section Details: The control argument is a list that can supply any of the following components:\describe{
#' (To understand exactly what these do see the source code: higher levels give more detail.).
#' \item{adaptation}{logical. If it is TRUE, ADMM with adaptation is performed. The default value is TRUE.}
#' \item{rho}{an augmented Lagrangian parameter. The default value is 1.}
#' \item{tau.ada}{an adaptation parameter in [1,2]. Only needed if adaptation = TRUE. The default value is 2. See Boyd et al. (2011) for details.}
#' \item{mu.ada}{an adaptation parameter. Only needed if adaptation = TRUE. The default value is 10. See Boyd et al. (2011) for details.}
#' \item{abstol}{absolute tolerance stopping criterion. The default value is sqrt(sqrt(.Machine$double.eps)).}
#' \item{reltol}{relative tolerance stopping criterion. The default value is sqrt(.Machine$double.eps).}
#' \item{maxit}{maximum number of iterations. The default value is 100.}
#' \item{dof.toler_c}{numeric tolerance for identifying effective zeros in the degrees-of-freedom calculation. The default is 5.”}
#' \item{dof.toler_d}{numeric tolerance for identifying effective zeros in the degrees-of-freedom calculation. The default is 0.001.}
#' \item{print.out}{logical. If it is TRUE, a message about the procedure is printed. The default value is TRUE.}
#' }
#' 
#' @examples
#' \donttest{
#' 
#' ### generate sample data
#' set.seed(2023)
#' n    <- 50
#' p    <- 30 
#' X    <- matrix(rnorm(n * p), n, p)
#' beta <- apply(matrix(rnorm(p, sd = 1), p, 1), 1, fdaSP:::softhresh, 1.5)
#' y    <- X %*% beta + rnorm(n, sd = sqrt(crossprod(X %*% beta)) / 20)
#' 
#' ### set regularization parameter grid
#' lam   <- 10^seq(0, -2, length.out = 30)
#' 
#' ### set the hyper-parameters of the ADMM algorithm
#' maxit      <- 1000
#' adaptation <- TRUE
#' rho        <- 1
#' reltol     <- 1e-5
#' abstol     <- 1e-5
#' 
#' ### run example
#' mod <- linreg_ADMM_LASSO(X = X, y = y, standardize.data = FALSE, intercept = FALSE, lambda = lam,
#'                          control = list("adaptation" = adaptation, "rho" = rho, 
#'                                         "maxit" = maxit, "reltol" = reltol, "abstol" = abstol, 
#'                                         "print.out" = FALSE)) 
#'                                         
#' ### graphical presentation
#' matplot(log(lam), mod$sp.coef.path, type = "l", main = "Lasso solution path",
#'         bty = "n", xlab = TeX("$\\log(\\lambda)$"), ylab = "")
#' }
#'
#' @references
#' \insertRef{tibshirani_regression_1996a}{fdaSP}
#' 
#' \insertRef{boyd_etal.2011}{fdaSP}
#' 
#' \insertRef{hastie_etal.2015}{fdaSP}
#' 
#' \insertRef{lin_etal.2022}{fdaSP} 
#' 
#' \insertRef{zou.2006}{fdaSP} 
#'
#' @noRd
linreg_ADMM_LASSO <- function(X, Z = NULL, y, var_weights = NULL, 
                              adaptive = FALSE, standardize.data = FALSE, 
                              lambda = NULL, lambda.min.ratio = NULL, nlambda = 30, 
                              intercept = FALSE, control = list()) {
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # add constant
  if (standardize.data == TRUE) {
    if (intercept == TRUE){
      intercept <- FALSE
      warning("* linreg_ADMM_LASSO : intercept should be set to FALSE is data are standardized!\n")
    }
  }
  if (intercept == TRUE) {
    if (!is.null(Z)) {
      Z <- cbind(matrix(data = 1, nrow = length(y), ncol = 1), Z)
    } else {
      Z <- matrix(data = 1, nrow = length(y), ncol = 1)
    }
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Pre-processing: data validity
  if (!check_data_matrix(X)) {
    stop("* linreg_ADMM_LASSO : input 'X' is invalid data matrix.")  
  }
  if (!check_data_vector(y)) {
    stop("* linreg_ADMM_LASSO : input 'y' is invalid data vector.")  
  }
  y <- as.vector(y)
  # data size
  if (dim(X)[1] != length(y)) {
    stop("* linreg_ADMM_LASSO : two inputs 'X' and 'Y' have non-matching dimension.")
  }
  if (!is.null(Z)) {
    if (dim(Z)[1] != length(y)) {
      stop("* linreg_ADMM_OVGLASSO : two inputs 'Z' and 'Y' have non-matching dimension.")
    }
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # get data dimensions
  n <- dim(X)[1]
  p <- dim(X)[2]  
  if (!is.null(Z)) {
    q <- dim(Z)[2]
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Standardise response and design matrix
  if (standardize.data == TRUE) {
    res   <- standardizemat(X, y)
    X.std <- res$X.std
    y.std <- res$y.std
    mU    <- res$mU
    mV    <- res$mV
  } else {
    X.std <- X
    y.std <- y
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # check for adaptive-LASSO vs weighted LASSO vs LASSO
  # se l'utente da pesi var_weights NULL ma ha messo adaptive = TRUE allora
  # fa adaptive LASSO con pesi OLS calcolati di seguito. 
  if (!is.null(adaptive)) {
    if (adaptive == TRUE) {
      #adalasso <- TRUE
      warning("* lmSP : adaptive Lasso is running!\n")
      if (!is.null(var_weights)) {
        warning("* lmSP : var_weights has been over-written by the inverse of the absolute value of the LS estimates!\n")
      }
      if (is.null(Z)) {
        coeff_LS    <- as.numeric(lm_ols(X = X.std, y = y.std))
        var_weights <- 1.0 / abs(coeff_LS)
      } else {
        coeff_LS    <- as.numeric(lm_ols_FWL(X = X.std, Z = Z, y = y.std)$coeff_X)
        var_weights <- 1.0 / abs(coeff_LS)
      }
    }
  } else {
    warning("* lmSP : adaptive is set to FALSE if it is not provided by the user!\n")
    adaptive <- FALSE
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # check for NULL var_weights
  if (is.null(var_weights)) {
    #adalasso    <- FALSE
    var_weights <- rep(1L, p)
    if (adaptive == TRUE) {
      warning("* lmSP : var_weights are set all equal to one if var_weights are not provided by the user!\n")  
    }
  } else if (adaptive == FALSE) {
    diff_ <- var_weights - rep(1L, p)
    if (max(abs(diff_)) > .Machine$double.eps) {
      warning("* lmSP : weighted Lasso is running with weights provided by the user!\n")
    }
    #adalasso <- FALSE
  }

  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Pre-processing: lambda parameter (in case of a single lambda)
  if (!is.null(lambda)) {
    if (length(lambda) == 1) {
      meps     <- (.Machine$double.eps)
      negsmall <- -meps
      if (!check_param_constant(lambda, negsmall)) {
        stop("* linreg_ADMM_LASSO : reg. parameter 'lambda' is invalid.")
      }
      if (lambda < meps){
        message("* linreg_ADMM_LASSO : since 'lambda' is effectively zero, a least-squares solution is returned.")
        xsol     <- as.vector(aux_pinv(X) %*% matrix(y))
        output   <- list()
        output$x <- xsol
        return(output)
      }
    }
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # check for inputs
  npar   <- dim(X.std)[2]
  con    <- list(maxit                = 100L, 
                 abstol               = sqrt(.Machine$double.eps), 
                 reltol               = sqrt(sqrt(.Machine$double.eps)), 
                 adaptation           = TRUE, 
                 rho                  = 1, 
                 tau.ada              = 2,               
                 mu.ada               = 10,
                 dof.toler_c          = 5,
                 dof.toler_d          = 0.001,
                 print.out            = TRUE)
  
  nmsC                          <- names(con)
  con[(namc <- names(control))] <- control
  if (length(noNms <- namc[!namc %in% nmsC])) {
    warning("unknown names in control: ", paste(noNms, collapse = ", "))
  }
  # set default values (for inputs)
  maxit      <- con$maxit
  abstol     <- con$abstol
  reltol     <- con$reltol
  adaptation <- con$adaptation
  rho        <- con$rho
  tau.ada    <- con$tau.ada      
  mu.ada     <- con$mu.ada
  toler_c    <- con$dof.toler_c
  toler_d    <- con$dof.toler_d
  print.out  <- con$print.out
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Pre-processing: other parameters
  if (!check_param_constant_multiple(c(abstol, reltol))) {
    stop("* linreg_ADMM_LASSO : tolerance level is invalid.")
  }
  if (!check_param_integer(maxit, 0.0)) {
    stop("* linreg_ADMM_LASSO : 'maxiter' should be a positive integer.")
  }
  maxit = as.integer(maxit)
  if (!check_param_constant(rho, 0.0)) {
    stop("* linreg_ADMM_LASSO : 'rho' should be a positive real number.")
  }
  if (!check_param_constant(mu.ada, 0.0)) {
    stop("* linreg_ADMM_LASSO : 'mu.ada' should be a positive real number.")
  }
  if ((tau.ada < 1.0) || (tau.ada > 2.0)) {
    warning("* linreg_ADMM_LASSO : 'tau.ada' value is suggested to be in [1,2].")
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Pre-processing: variable weights
  check_weights(x = var_weights, n = dim(X)[2], algname = "linreg_ADMM_LASSO", funname = "var_weights")
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get the sequence of lambdas
  if (!is.null(nlambda) && is.null(lambda)) {
    # get the smallest value of such that the regression 
    # coefficients estimated by the lasso are all equal to zero
    lambda <- lm_lambdamax_ADALASSO(y = y.std, X = X.std, Z, var_weights = var_weights, lambda.min.ratio = lambda.min.ratio, maxl = nlambda)$lambda.seq
  }
  if (is.null(nlambda) && is.null(lambda)) {
    # get the smallest value of such that the regression 
    # coefficients estimated by the lasso are all equal to zero
    lambda <- lm_lambdamax_ADALASSO(y = y.std, X = X.std, Z, var_weights = var_weights, lambda.min.ratio = lambda.min.ratio, maxl = 30)$lambda.seq
  }
  if (!is.null(lambda)) {
    nlambda <- length(lambda)
  }

  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Run lasso
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Run overlap group Lasso
  if (is.null(Z)) {
    if (adaptive == TRUE) {
      ret <- .admm_adalasso_fast(A = X.std, b = y.std, var_weights = var_weights, lambda = lambda, rho_adaptation = adaptation, rho = rho,
                                 tau = tau.ada, mu = mu.ada, reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
    } else {
      ret <- .admm_lasso_fast(A = X.std, b = y.std, lambda = lambda, rho_adaptation = adaptation, rho = rho,
                              tau = tau.ada, mu = mu.ada, reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
    }
    # get estimated coefficients and path
    mSpRegP <- t(ret$coef.path)
    vSpRegP <- ret$coefficients
  } else {
    if (adaptive == TRUE) {
      ret <- .admm_adalasso_cov_fast(W = X.std, Z = Z, y = y.std, var_weights = var_weights, lambda = lambda, rho_adaptation = adaptation, rho = rho,
                                     tau = tau.ada, mu = mu.ada, reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
    } else {
      ret <- .admm_lasso_cov_fast(W = X.std, Z = Z, y = y.std, lambda = lambda, rho_adaptation = adaptation, rho = rho,
                                  tau = tau.ada, mu = mu.ada, reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
    }
    # get estimated coefficients and path
    mSpRegP <- t(ret$sp.coef.path)
    mRegP   <- t(ret$coef.path)
    vSpRegP <- ret$sp.coefficients
    vRegP   <- ret$coefficients
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get the path and retrieve the scaled estimates
  if (standardize.data == TRUE) {
    sp.path <- matrix(nlambda, p, data = t(apply(mSpRegP, 2, function(x) solve(mU) %*% x %*% mV)))
    vSpRegP <- solve(mU) %*% vSpRegP %*% mV
    if (!is.null(Z)) {
      path  <- matrix(nlambda, q, data = t(apply(mRegP, 2, function(x) x %*% mV)))
      vRegP <- vRegP %*% mV
    }
  } else {
    vSpRegP <- matrix(vSpRegP, length(vSpRegP), 1)
    sp.path <- t(mSpRegP)
    if (!is.null(Z)) {
      path <- t(mRegP)
    }
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Compute the degrees of freedom 
  if (adaptive == TRUE) {
    dof.out <- lm_dof_OVGLASSO(X                = X.std,
                               Z                = Z,
                               coeff            = sp.path,
                               lambda           = lambda,
                               GRmat            = diag(1, dim(X.std)[2]),
                               group_weights    = var_weights,
                               var_weights      = rep(1, dim(X.std)[2]),
                               Umat             = ret$U,
                               err_primal       = ret$err_pri,
                               err_dual         = ret$err_dual,
                               rho              = ret$rho,
                               toler_c          = toler_c,
                               toler_d          = toler_d)
  } else {
    dof.out <- lm_dof_LASSO(X                = X.std, 
                            coeff            = sp.path, 
                            lambda           = lambda, 
                            var_weights      = var_weights,
                            Umat             = ret$U,
                            err_primal       = ret$err_pri,
                            err_dual         = ret$err_dual,
                            rho              = ret$rho,
                            toler_c          = toler_c,
                            toler_d          = toler_d)
  }

  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # compute the BIC
  bic <- dim(X.std)[1] * log(ret$mse) + dof.out$dof * log(dim(X.std)[1]) 
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Print to screen
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  if (print.out == TRUE) {
    cat("\n\n\n")
    cat("Linear regression model with LASSO penalty\n")
    cat("Alternating direction method of multipliers\n")
    cat("running time (for ", reltol, " relative error):",
        sum(ret$elapsedTime), "seconds \n\n\n")
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get output
  res.names <- c("sp.coefficients", 
                 "sp.coef.path", 
                 "coefficients",
                 "coef.path",
                 "dof",
                 "dof.coeff_active",
                 "bic",
                 "lambda.min",
                 "lambda",
                 "mse",
                 "min.mse",
                 "convergence",
                 "elapsedTime",
                 "iternum",
                 "objfun",
                 "r_norm",
                 "s_norm",
                 "err_pri",
                 "err_dual",
                 "rho") 
  res        <- vector(mode = "list", length = length(res.names))
  names(res) <- res.names
  if (!is.null(Z)) {
    res$coefficients <- vRegP
    res$coef.path    <- path
  } 
  res$sp.coefficients  <- vSpRegP
  res$sp.coef.path     <- sp.path
  res$dof              <- dof.out$dof
  res$dof.coeff_active <- dof.out$coeff_active
  res$bic              <- bic
  res$lambda.min       <- ret$lambda.min
  res$lambda           <- ret$lambda
  res$mse              <- ret$mse
  res$min.mse          <- ret$min.mse
  res$convergence      <- ret$convergence
  res$elapsedTime      <- ret$elapsedTime
  res$iternum          <- ret$iternum
  res$objfun           <- ret$objfun
  res$r_norm           <- ret$r_norm
  res$s_norm           <- ret$s_norm
  res$err_pri          <- ret$err_pri
  res$err_dual         <- ret$err_dual
  res$rho              <- ret$rho
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Return output
  return(res)
}

# Authors: Mauro Bernardi 
#          Department Statistical Sciences
#       	 University of Padova
#      	   Via Cesare Battisti, 241
#      	   35121 PADOVA, Italy
#          E-mail: mauro.bernardi@unipd.it  

# Last change: July 20, 2021


# ================================
# Check functions
check_data_matrix <- function(A) {
  cond1 = (is.matrix(A))  # matrix
  cond2 = (!(any(is.infinite(A))||any(is.na(A))))
  if (cond1 && cond2) {
    return(TRUE)
  } else {
    return(FALSE)
  }
}

check_data_vector <- function(b) {
  cond1 = ((is.vector(b))||((is.matrix(b))&&
                              (length(b)==nrow(b))||(length(b)==ncol(b))))
  cond2 = (!(any(is.infinite(b))||any(is.na(b))))
  if (cond1&&cond2){
    return(TRUE)
  } else {
    return(FALSE)
  }
}

check_data_positive_vector <- function(b) {
  cond1 = ((is.vector(b))||((is.matrix(b))&&
                              (length(b)==nrow(b))||(length(b)==ncol(b))))
  cond2 = (!(any(is.infinite(b))||any(is.na(b))))
  cond3 = (!(any(b<=0)))
  if (cond1&&cond2&&cond3){
    return(TRUE)
  } else {
    return(FALSE)
  }
}

check_param_constant <- function(num, lowerbound=0){
  cond1 = (length(num)==1)
  cond2 = ((!is.infinite(num))&&(!is.na(num)))
  cond3 = (num > lowerbound)
  
  if (cond1&&cond2&&cond3){
    return(TRUE)
  } else {
    return(FALSE)
  }
}

check_param_constant_multiple <- function(numvec, lowerbound=0){
  for (i in 1:length(numvec)){
    if (!check_param_constant(numvec[i], lowerbound)){
      return(FALSE)
    }
  }
  return(TRUE)
}

check_param_integer <- function(num, lowerbound=0){
  cond1 = (length(num)==1)
  cond2 = ((!is.infinite(num))&&(!is.na(num)))
  cond3 = (num > lowerbound)
  cond4 = (abs(num-round(num)) < sqrt(.Machine$double.eps))
  
  if (cond1&&cond2&&cond3&&cond4){
    return(TRUE)
  } else {
    return(FALSE)
  }
}

check_weights <- function(x, n, algname, funname) {
  if (!is.null(x)) {
    if (any(is.na(x))) {
      stop(paste0("*", algname, " : the vector ", funname, " has missing values."))
    }
    if (!is.numeric(x)) {
      stop(paste0("*", algname, " : the vector ", funname, " is not a numeric vector."))
    }
    if (n != length(x)) {
      stop(paste0("*", algname, " : the length of the vector ", funname, " does not match the number of penalized groups."))
    }
    if (min(x) < 0.0) {
      stop(paste0("*", algname, " : the values in ", funname, " must be positive."))
    }
  }
}

check_group_weights <- function(x, n, algname, funname) {
  if (!is.null(x)) {
    if (any(is.na(x))) {
      stop(paste0("*", algname, " : the vector ", funname, " has missing values."))
    }
    if (!is.numeric(x)) {
      stop(paste0("*", algname, " : the vector ", funname, " is not a numeric vector."))
    }
    if ((n != length(x)) && (n != (length(x)+1))) {
      stop(paste0("*", algname, " : the length of the vector ", funname, " does not match the number of penalized predictors."))
    }
    if (min(x) < 0.0) {
      stop(paste0("*", algname, " : the values in ", funname, " must be positive."))
    }
  }
}

check_group_overlaps <- function(GRmat) {
  
  csum <- colSums(GRmat)
  if (any(csum != 1)) {
    res <- "ovglasso"
  } else if (any(rowSums(GRmat) != 1)) {
    res <- "glasso"
  } else {
    res <- "lasso"
  }
  return(res)
}

# ================================
# AUXILIARY COMPUTATIONS 

# 2. PseudoInverse using SVD and NumPy Scheme
aux_pinv <- function(A){
  svdA      <- svd(A)
  tolerance <- (.Machine$double.eps)*max(c(nrow(A),ncol(A)))*as.double(max(svdA$d))
  
  idxcut          <- which(svdA$d <= tolerance)
  invDvec         <- (1.0 / svdA$d)
  invDvec[idxcut] <- 0
  output          <- (svdA$v %*% diag(invDvec) %*% t(svdA$u))
  return(output)
}

logseq <- function(from = 1, to = 1000, length.out = 6, reverse = FALSE) {
  
  # ================================
  # logarithmic spaced sequence
  out <- exp(seq(log(from), log(to), length.out = length.out))
  
  # ================================
  # reverse the sequence
  if (reverse == TRUE) {
    res <- out[length(out):1]
  } else {
    res <- out
  }
  
  # ================================
  # Get output
  return(res)
}

linreg <- function(vY, mX) {
  
  # ================================
  # Get the relevant quantities
  mXX <- crossprod(mX)
  vXy <- crossprod(mX, vY)
  
  # ================================
  # Perform the QR decomposition of mXX
  mXX_QR <- qr(mXX)
  mQ     <- qr.Q(mXX_QR)
  mR     <- qr.R(mXX_QR)
  
  # ================================
  # Compute the estimate
  vXy_  <- crossprod(mQ, vXy)
  vRegP <- backsolve(mR, vXy_, k = ncol(mR), upper.tri = TRUE, transpose = FALSE)
  
  # ================================
  # Return output
  return(vRegP)
}

scaledata <- function(x) {
  if (is.vector(x) == TRUE) {
    x.scaled <- (x - mean(x)) / sd(x)
  } else {
    x.scaled <- apply(x, 2, function(x) (x-mean(x))/sd(x))
  }
  return(x.scaled)
}

# ================================
# Standardise response and design matrix
standardizemat <- function(X, y) {
  
  # ================================
  # Get dimensions
  n <- dim(X)[1]
  
  # ================================
  # Get std
  y  <- matrix(y, nrow = n, ncol = 1)
  mU <- diag(sqrt(diag(var(X))))
  if (ncol(y) > 1) {
    mV <- diag(sqrt(diag(var(y))))
  } else {
    mV <- sqrt(var(y))
  }
  y.ctr <- apply(y, 2, function(x) x - mean(x))
  y.std <- y.ctr %*% solve(mV)
  X.ctr <- apply(X, 2, function(x) x - mean(x))
  X.std <- X.ctr %*% solve(mU)
  
  # ================================
  # Get output
  out       <- NULL
  out$X.std <- X.std
  out$y.std <- y.std
  out$mU    <- mU
  out$mV    <- mV
  
  # ================================
  # Return output
  return(out)
}

# return the regression parameters in the original scale
# after standardization
invstdparms <- function(parms, mU, mV) {

  # get dimensions
  n <- dim(parms)[1]
  p <- dim(parms)[2]
  
  # standardize data
  if (p == 1) {
    parms.std <- solve(mU) %*% parms[1,] %*% mV
  } else {
    #parms.std <- t(apply(t(parms), 2, function(x) solve(mU) %*% x %*% mV))
    parms.std <- t(apply(parms, 1, function(x) solve(mU) %*% x %*% mV))
  }
  
  # convert to a mtrix object
  parms.std <- matrix(data = parms.std, nrow = n, ncol = p)
  
  # return output
  return(parms.std)
}

groups.cv <- function(n, k = 10) {
  if (k == 0) {
    k <- n
  }
  if (!is.numeric(k) || k < 0 || k > n) {
    stop("Invalid values of 'k'. Must be between 0 (for leave-one-out CV) and 'n'.")
  }
  dummyorder <- sample(1:n, size = n)
  f          <- ceiling(seq_along(dummyorder)/(n/k))
  
  groups     <- vector("list", length = max(f))
  groups.all <- NULL
  for (it in 1:max(f)) {
    indi         <- which(f == it)
    groups[[it]] <- dummyorder[indi]
    groups.all   <- c(groups.all, dummyorder[indi])
  }
  
  # groups cv
  groups.cv <- vector("list", length = max(f))
  for (it in 1:max(f)) {
    groups.cv[[it]] <- setdiff(groups.all, groups[[it]])
  }
  
  # define foldid
  foldid <- rep(0, n)
  for (i in 1:k) {
    idx         <- groups[[i]]
    foldid[idx] <- i
  }
  
  # get output
  ret             <- NULL
  ret$groups.pred <- groups
  ret$groups.cv   <- groups.cv
  ret$shuffle     <- groups.all
  ret$foldid      <- foldid
  
  # return output
  return(ret)
}

#' Function to solve the soft thresholding problem 
#'  
#' @param x the data value.
#' @param lambda the lambda value. 
#'
#' @return the solution to the soft thresholding operator. 
#' 
#' @references
#'  
#' \insertRef{hastie_etal.2015}{fdaSP}
#' 
#' @export softhresh
softhresh <- function(x, lambda) {
  return(sign(x) * pmax(rep(0, length(x)), abs(x) - lambda))
}

softhresh_group <- function(x, lam){
  return(max(0L, 1L - lam / normvec(x)) * x)
}

normvec <- function(va) {
  return(as.numeric(sqrt(crossprod(va))))
}

getsparsevec <- function(x, toler) {
  
  # ================================
  # Get dimensions
  n <- length(x)
  
  y <- x
  for (it in 1:n) {
    if (abs(x[it]) < toler) {
      y[it] <- 0.0
    }
  }
  
  # ================================
  # Return output
  return(y)
}

softthreshold <- function(xx, lambda) {

  # ================================
  # LASSO soft threshold operator
  zz <- abs(xx) - lambda
  if (zz > 0.0) {
    yy <- zz * sign(xx)
  } else {
    yy <- 0.0
  }
  # ================================
  # Return output
  return(yy)
}

getglassopenalty <- function(vRegP, mSelMat) {
  
  # ================================
  # Get indicators
  vPen <- sqrt(t(mSelMat) %*% (vRegP^2))
  
  # ================================
  # Return output
  return(vPen)
}

scaledata <- function(x) {
  if (is.vector(x) == TRUE) {
    x.scaled <- (x - mean(x)) / sd(x)
  } else {
    x.scaled <- apply(x, 2, function(x) (x-mean(x))/sd(x))
  }
  return(x.scaled)
}

# 6. fround
fround <- function(x, n) {
  
  y <- x
  for (it in 1:n) {
    if (abs(x[it]) < .Machine$double.eps) {
      y[it] <- 0.0
    }
  }
  return(y)
}

# 7. fthreshold
fthreshold <- function(x, n, toler) {
  
  y <- x
  for (it in 1:n) {
    if (abs(x[it]) < toler) {
      y[it] <- 0.0
    }
  }
  return(y)
}


# 8. set to a given tolerance
fSet2GivenTolerance <- function(x, n, toler) {
  
  y <- x
  for (it in 1:n) {
    if (abs(x[it]) < toler) {
      y[it] <- toler
    }
  }
  return(y)
}

f2s_reg_RIDGE <- function(y, Z = NULL, X, diff_order = 1, lambda, lambda2 = 0.0) {
  
  # get dimensions
  n <- dim(X)[1]
  p <- dim(X)[2]
  if (!is.null(Z)) {
    q <- dim(Z)[2]
  }

  # output returned in case of no valid options
  output <- NULL
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # both lambdas are NOT NULL
  if (!is.null(lambda) && !is.null(lambda2)) {
    # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    # both lambdas are single values
    if ((length(lambda) == 1) && (length(lambda2) == 1)) {
      meps     <- (.Machine$double.eps)
      negsmall <- -meps
      
      # check lambda
      if (!check_param_constant(lambda, negsmall)) {
        stop("* f2sSP : parameter 'lambda' is invalid.")
      }
      # check lambda2
      if (!check_param_constant(lambda2, negsmall)) {
        stop("* f2sSP : parameter 'lambda2' is invalid.")
      }
      
      # case 1: lambdas parameters are single values, both equal to zero, OLS solution
      if ((lambda < meps) && (lambda2 < meps)) {
        message("* f2sSP : since 'lambda' and 'lambda2' are effectively zero, a least-squares solution is returned.")
        if (is.null(Z)) {
          xsol <- as.vector(aux_pinv(X) %*% matrix(y))
          dof  <- p
        } else {
          xsol <- as.vector(aux_pinv(cbind(X, Z)) %*% matrix(y))
          dof  <- p + q
        }
        # get output
        sp.coef.path <- xsol[1:p]
        if (!is.null(Z)) {
          coef.path        <- xsol[(p+1):(p+q)]
          output$coef.path <- coef.path
        }
        output$sp.coef.path <- sp.coef.path
      }
      # case 2: lambdas parameters are single values
      #         lambda is zero and lambda2 in NOT zero: RIDGE solution
      if ((lambda < meps) && (lambda2 >= meps)) {
        message("* f2sSP : since 'lambda' is effectively zero but 'lambda2' is not, a RIDGE solution is returned.")
        if (!is.null(Z)) {
          R_          <- .forward_diff_penalty_matrix(n = p, h = 1, d = diff_order)
          R           <- matrix(data = 0, nrow = p + q, p + q)
          R[1:p, 1:p] <- R_
          xsol        <- as.vector(solve(crossprod(cbind(X, Z)) + lambda2 * R) %*% crossprod(cbind(X, Z), y))

          # compute DoF
          C   <- cbind(X, Z)
          S   <- solve(crossprod(C) + lambda2 * R) %*% crossprod(C)
          dof <- sum(diag(S)) 
        } else {
          R    <- .forward_diff_penalty_matrix(n = p, h = 1, d = diff_order)
          xsol <- as.vector(solve(crossprod(X) + lambda2 * R) %*% crossprod(X, y))
  
          # compute DoF
          S   <- solve(crossprod(X) + lambda2 * R) %*% crossprod(X)
          dof <- sum(diag(S)) 
        }
        # get output
        sp.coef.path <- xsol[1:p]
        if (!is.null(Z)) {
          coef.path        <- xsol[(p+1):(p+q)]
          output$coef.path <- coef.path
        }
        output$sp.coef.path <- sp.coef.path
      }
    } else if ((length(lambda) == 1) && (length(lambda2) > 1)) {
      # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
      # both lambda is a single value while lambda2 is not
      message("* f2sSP : since 'lambda' is effectively zero, a RIDGE solution is returned.")
      meps     <- (.Machine$double.eps)
      negsmall <- -meps
      
      # check lambda
      if (!check_param_constant(lambda, negsmall)) {
        stop("* f2sSP : parameter 'lambda' is invalid.")
      }
      # case 1: lambda is zero
      if (lambda < meps) {
        message("* f2sSP : since 'lambda' is effectively zero, a least-squares solution is returned.")
        # path over lambda2, OLS solution
        if (is.null(Z)) {
          R <- .forward_diff_penalty_matrix(n = p,
                                            h = 1, d = diff_order)
          Xsol <- matrix(data = 0, nrow = p, ncol = length(lambda2))
          dof  <- rep(0, length(lambda2))
          for (j in 1:length(lambda2)) {
            Xsol[, j] <- as.vector(solve(crossprod(X) +
                                           lambda2[j] * R) %*% crossprod(X, y))
            
            # compute DoF (ols path lambda2)
            S      <- solve(crossprod(X) + lambda2[j] * R) %*% crossprod(X)
            dof[j] <- sum(diag(S)) 
          }
          # get output
          sp.coef.path        <- Xsol[1:p,]
          output$sp.coef.path <- sp.coef.path
        } else {
          # path over lambda2, RIDGE solution
          R_ <- .forward_diff_penalty_matrix(n = p,
                                             h = 1,
                                             d = diff_order)
          R           <- matrix(data = 0, nrow = p + q, p + q)
          R[1:p, 1:p] <- R_
          Xsol        <- matrix(data = 0, nrow = p + q, ncol = length(lambda2))
          dof         <- rep(0, length(lambda2))
          for (j in 1:length(lambda2)) {
            Xsol[, j] <- as.vector(solve(crossprod(cbind(X, Z)) +
                                           lambda2[j] * R) %*% crossprod(cbind(X, Z), y))
            
            # compute DoF
            C      <- cbind(X, Z)
            S      <- solve(crossprod(C) + lambda2[j] * R) %*% crossprod(C)
            dof[j] <- sum(diag(S)) 
          }
          # get output
          sp.coef.path        <- Xsol[1:p,]
          coef.path           <- Xsol[(p+1):(q+p),]
          output$coef.path    <- coef.path
          output$sp.coef.path <- sp.coef.path
        }
      }
    }
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # retrieve MSE
  if (!is.null(output)) {
    if (!is.null( Z)) { 
      # get estimated coefficients
      mSpRegP <- sp.coef.path
      mRegP   <- coef.path
      
      # get MSE
      fitted    <- X %*% mSpRegP + Z %*% mRegP
      residuals <- apply(X = fitted, MARGIN = 2, FUN = function(x){x-y})
      mse       <- diag(crossprod(residuals)) / n
      mse.min   <- which.min(mse)
      
      # get estimated parameters at max MSE
      if (length(lambda2) > 1) {
        vSpRegP <- mSpRegP[,mse.min]
        vRegP   <- mRegP[mse.min]
      } else {
        vSpRegP <- mSpRegP
        vRegP   <- mRegP
      }
    
      # get output
      output$vSpRegP      <- vSpRegP
      output$vRegP        <- vRegP
      output$mse          <- mse
      output$min.mse      <- mse.min
      output$dof          <- dof
    } else {
      # get estimated coefficients
      mSpRegP <- sp.coef.path
      
      # get MSE
      fitted    <- X %*% mSpRegP
      residuals <- apply(X = fitted, MARGIN = 2, FUN = function(x){x-y})
      mse       <- diag(crossprod(residuals)) / n
      mse.min   <- which.min(mse)
    
      # get estimated parameters at max MSE
      if (length(lambda2) > 1) {
        vSpRegP <- mSpRegP[,mse.min]
      } else {
        vSpRegP <- mSpRegP
      }
      
      # get output
      output$vSpRegP      <- vSpRegP
      output$mse          <- mse
      output$min.mse      <- mse.min
      output$dof          <- dof
    }
    # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    # compute the BIC
    output$bic <- n * log(mse) + dof * log(n) 
  }
  
  # return output
  return(output)
}

# Authors: Mauro Bernardi 
#          Department Statistical Sciences
#       	 University of Padova
#      	   Via Cesare Battisti, 241
#      	   35121 PADOVA, Italy
#          E-mail: mauro.bernardi@unipd.it  

# Last change: March 16, 2023

#' Overlap Group Least Absolute Shrinkage and Selection Operator for function-on-function regression model
#'
#' Overlap Group-LASSO for function-on-function regression model solves the following optimization problem
#' \deqn{\textrm{min}_{\psi} ~ \frac{1}{2} \sum_{i=1}^n \int \left( y_i(s) - \int x_i(t) \psi(t,s) dt \right)^2 ds + \lambda \sum_{g=1}^{G} \Vert S_{g}T\psi \Vert_2}
#' to obtain a sparse coefficient vector \eqn{\psi=\mathsf{vec}(\Psi)\in\mathbb{R}^{ML}} for the functional penalized predictor \eqn{x(t)}, where the coefficient matrix \eqn{\Psi\in\mathbb{R}^{M\times L}}, 
#' the regression function \eqn{\psi(t,s)=\varphi(t)^\intercal\Psi\theta(s)}, 
#' \eqn{\varphi(t)} and \eqn{\theta(s)} are two B-splines bases of order \eqn{d} and dimension \eqn{M} and \eqn{L}, respectively. For each group \eqn{g}, each row of 
#' the matrix \eqn{S_g\in\mathbb{R}^{d\times ML}} has non-zero entries only for those bases belonging 
#' to that group. These values are provided by the arguments \code{groups} and \code{group_weights} (see below). 
#' Each basis function belongs to more than one group. The diagonal matrix \eqn{T\in\mathbb{R}^{ML\times ML}} contains 
#' the basis-specific weights. These values are provided by the argument \code{var_weights} (see below).
#' The regularization path is computed for the overlap group-LASSO penalty at a grid of values for the regularization 
#' parameter \eqn{\lambda} using the alternating direction method of multipliers (ADMM). See Boyd et al. (2011) and Lin et al. (2022) 
#' for details on the ADMM method.
#'  
#' @param mX an \eqn{(n\times r_x)} matrix of observations of the functional covariate.
#' @param mY an \eqn{(n\times r_y)} matrix of observations of the functional response variable.
#' @param L number of elements of the B-spline basis vector \eqn{\theta(s)}.
#' @param M number of elements of the B-spline basis vector \eqn{\varphi(t)}.
#' @param group_weights a vector of length \eqn{G} containing group-specific weights. The default is square root of the group cardinality, see Bernardi et al. (2022).
#' @param var_weights a vector of length \eqn{ML} containing basis-specific weights. The default is a vector where 
#' each entry is the reciprocal of the number of groups including that basis. See Bernardi et al. (2022) for details.
#' @param standardize.data logical. Should data be standardized?
#' @param splOrd the order \eqn{d} of the spline basis.
#' @param lambda either a regularization parameter or a vector of regularization parameters. 
#' In this latter case the routine computes the whole path. If it is NULL values for lambda are provided by the routine.
#' @param lambda.min.ratio smallest value for lambda, as a fraction of the maximum lambda value. If \eqn{nr_y>LM}, the default is 0.0001, and if \eqn{nr_y<LM}, the default is 0.01. 
#' @param nlambda the number of lambda values - default is 30.
#' @param overall.group logical. If it is TRUE, an overall group including all penalized covariates is added.
#' @param control	 a list of control parameters for the ADMM algorithm. See ‘Details’.
#'
#' @return A named list containing \describe{
#' \item{sp.coefficients}{an \eqn{(M\times L)} solution matrix for the parameters \eqn{\Psi}, which corresponds to the minimum in-sample MSE.}
#' \item{sp.coef.path}{an \eqn{(n_\lambda\times M \times L)} array of estimated \eqn{\Psi} coefficients for each lambda.}
#' \item{sp.fun}{an \eqn{(r_x\times r_y)} matrix providing the estimated functional coefficient for \eqn{\psi(t,s)}.}
#' \item{sp.fun.path}{an \eqn{(n_\lambda\times r_x\times r_y)} array providing the estimated functional coefficients for \eqn{\psi(t,s)} for each lambda.}
#' \item{lambda}{sequence of lambda.}
#' \item{lambda.min}{value of lambda that attains the minimum in-sample MSE.}
#' \item{mse}{in-sample mean squared error.}
#' \item{min.mse}{minimum value of the in-sample MSE for the sequence of lambda.}
#' \item{convergence}{logical. 1 denotes achieved convergence.}
#' \item{elapsedTime}{elapsed time in seconds.}
#' \item{iternum}{number of iterations.}
#' }
#'
#' When you run the algorithm, output returns not only the solution, but also the iteration history recording
#' following fields over iterates,
#' \describe{
#' \item{objval}{objective function value.}
#' \item{r_norm}{norm of primal residual.}
#' \item{s_norm}{norm of dual residual.}
#' \item{eps_pri}{feasibility tolerance for primal feasibility condition.}
#' \item{eps_dual}{feasibility tolerance for dual feasibility condition.}
#' }
#' Iteration stops when both \code{r_norm} and \code{s_norm} values
#' become smaller than \code{eps_pri} and \code{eps_dual}, respectively.
#' @section Details: The control argument is a list that can supply any of the following components:\describe{
#' \item{adaptation}{logical. If it is TRUE, ADMM with adaptation is performed. The default value is TRUE. See Boyd et al. (2011) for details.}
#' \item{rho}{an augmented Lagrangian parameter. The default value is 1.}
#' \item{tau.ada}{an adaptation parameter greater than one. Only needed if adaptation = TRUE. The default value is 2. See Boyd et al. (2011) and Lin et al. (2022) for details.}
#' \item{mu.ada}{an adaptation parameter greater than one. Only needed if adaptation = TRUE. The default value is 10. See Boyd et al. (2011) and Lin et al. (2022) for details.}
#' \item{abstol}{absolute tolerance stopping criterion. The default value is sqrt(sqrt(.Machine$double.eps)).}
#' \item{reltol}{relative tolerance stopping criterion. The default value is sqrt(.Machine$double.eps).}
#' \item{maxit}{maximum number of iterations. The default value is 100.}
#' \item{print.out}{logical. If it is TRUE, a message about the procedure is printed. The default value is TRUE.}
#' }
#' 
#' @examples 
#' 
#' ## generate sample data
#' set.seed(4321)
#' s  <- seq(0, 1, length.out = 100)
#' t  <- seq(0, 1, length.out = 100)
#' p1 <- 5
#' p2 <- 6
#' r  <- 10
#' n  <- 50
#' 
#' beta_basis1 <- splines::bs(s, df = p1, intercept = TRUE)    # first basis for beta
#' beta_basis2 <- splines::bs(s, df = p2, intercept = TRUE)    # second basis for beta
#' 
#' data_basis <- splines::bs(s, df = r, intercept = TRUE)    # basis for X
#' 
#' x_0   <- apply(matrix(rnorm(p1 * p2, sd = 1), p1, p2), 1, 
#'                fdaSP::softhresh, 1.5)  # regression coefficients 
#' x_fun <- beta_basis2 %*% x_0 %*%  t(beta_basis1)  
#' 
#' fun_data <- matrix(rnorm(n*r), n, r) %*% t(data_basis)
#' b        <- fun_data %*% x_fun + rnorm(n * 100, sd = sd(fun_data %*% x_fun )/3)
#' 
#' ## set the hyper-parameters
#' maxit          <- 1000
#' rho_adaptation <- FALSE
#' rho            <- 1
#' reltol         <- 1e-5
#' abstol         <- 1e-5
#' 
#' ## fit functional regression model
#' mod <- f2fSP(mY = b, mX = fun_data, L = p1, M = p2,
#'              group_weights = NULL, var_weights = NULL, standardize.data = FALSE, splOrd = 4,
#'              lambda = NULL, nlambda = 30, lambda.min.ratio = NULL, 
#'              control = list("abstol" = abstol, 
#'                             "reltol" = reltol, 
#'                             "maxit" = maxit, 
#'                             "adaptation" = rho_adaptation, 
#'                             rho = rho, 
#'              "print.out" = FALSE))
#'  
#' mycol <- function (n) {
#' palette <- colorRampPalette(RColorBrewer::brewer.pal(11, "Spectral"))
#' palette(n)
#' }
#' cols <- mycol(1000)
#' 
#' oldpar <- par(mfrow = c(1, 2))
#' image(x_0, col = cols)
#' image(mod$sp.coefficients, col = cols)
#' par(oldpar)
#' 
#' oldpar <- par(mfrow = c(1, 2))
#' image(x_fun, col = cols)
#' contour(x_fun, add = TRUE)
#' image(beta_basis2 %*% mod$sp.coefficients %*% t(beta_basis1), col = cols)
#' contour(beta_basis2 %*% mod$sp.coefficients %*% t(beta_basis1), add = TRUE)
#' par(oldpar)
#' 
#' @references
#' \insertRef{bernardi_etal.2022}{fdaSP}
#' 
#' \insertRef{boyd_etal.2011}{fdaSP}
#' 
#' \insertRef{jenatton_etal.2011}{fdaSP}
#' 
#' \insertRef{lin_etal.2022}{fdaSP}
#'
#' @export
f2fSP <- function(mY, mX, L, M, group_weights = NULL, var_weights = NULL, standardize.data = TRUE,
                  splOrd = 4, lambda = NULL, lambda.min.ratio = NULL, nlambda = 30, overall.group = FALSE,
                  control = list()) {
    
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get dimensions
  n <- dim(mX)[1]             # number of observations
  p <- dim(mX)[2]
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Create the B-spline basis functions
  s      <- seq(0, 1, length.out = p)
  mH     <- t(splines::bs(s, df = M, intercept = TRUE))
  mTheta <- t(splines::bs(s, df = L, intercept = TRUE))
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Define response and design matrix
  mY_B <- mY
  mX_B <- tcrossprod(mX, mH)
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Vectorisation form
  X_ <- kronecker(t(mTheta), mX_B)
  y_ <- matrix(ks::vec(mY_B, byrow = FALSE), ncol = 1)
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Standardise response and design matrix
  if (standardize.data == TRUE) {
    res   <- standardizemat(X_, y_)
    X.std <- res$X.std
    y.std <- res$y.std
    mU    <- res$mU
    mV    <- res$mV
  } else {
    X.std <- X_
    y.std <- y_
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Pre-processing: lambda parameter (in case of a single lambda)
  if (!is.null(lambda)) {
    if (length(lambda) == 1) {
      meps     <- (.Machine$double.eps)
      negsmall <- -meps
      if (!check_param_constant(lambda, negsmall)) {
        stop("* f2fSP : reg. parameter 'lambda' is invalid.")
      }
      if (lambda < meps){
        message("* f2fSP : since 'lambda' is effectively zero, a least-squares solution is returned.")
        xsol     <- as.vector(aux_pinv(X_) %*% matrix(y_))
        output   <- list()
        output$x <- xsol
        return(output)
      }
    }
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # check for inputs
  npar   <- dim(X.std)[2]
  con    <- list(maxit                = 100L, 
                 abstol               = sqrt(.Machine$double.eps), 
                 reltol               = sqrt(sqrt(.Machine$double.eps)), 
                 adaptation           = TRUE, 
                 rho                  = 1, 
                 tau.ada              = 2,               
                 mu.ada               = 10,
                 par.init             = NULL,
                 init.rnd             = FALSE,
                 print.out            = TRUE)
  
  nmsC                          <- names(con)
  con[(namc <- names(control))] <- control
  if (length(noNms <- namc[!namc %in% nmsC])) {
    warning("unknown names in control: ", paste(noNms, collapse = ", "))
  }
  # set default values (for inputs)
  maxit                <- con$maxit
  abstol               <- con$abstol
  reltol               <- con$reltol
  adaptation           <- con$adaptation
  rho                  <- con$rho
  tau.ada              <- con$tau.ada      
  mu.ada               <- con$mu.ada
  par.init             <- con$par.init
  init.rnd             <- con$init.rnd
  print.out            <- con$print.out
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Pre-processing: variable weights
  check_weights(x = var_weights, n = dim(X_)[2], algname = "f2fSP", funname = "var_weights")
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get the groups of the overlap group LASSO penalty
  if (overall.group == TRUE) {
    out <- penfun(method = "ogl&1", p = M, q = L, splOrd = splOrd, regpars = NULL)
  } else {
    out <- penfun(method = "ogl", p = M, q = L, splOrd = splOrd, regpars = NULL)
  }
  GRmat <- out$grMat
  G     <- dim(GRmat)[1]
  nG    <- rowSums(GRmat)
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Manage group & overall group weights
  if (is.null(group_weights)) {
    # Define the weights as in Yuan and Lin 2006, JRSSB
    #group_weights <- sqrt(diag(tcrossprod(GRmat)))
    group_weights <- rep(1, G)
  } else {
    if (overall.group == TRUE) {
      check_group_weights(x = group_weights, n = G, algname = "f2fSP", funname = "group_weights")
      if (length(group_weights) == (G-1)) {
        group_weights <- c(group_weights, sqrt(dim(GRmat)[2]))
      }
    } else {
      check_weights(x = group_weights, n = G, algname = "f2fSP", funname = "group_weights")
    }
  } 
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Manage variable weights
  if (is.null(var_weights)) {
    #var_weights <- 1.0 / colSums(GRmat)
    var_weights <- 1.0 / sqrt(colSums(GRmat))
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get the sequence of lambda
  if (!is.null(nlambda) && is.null(lambda)) {
    # get the smallest value of such that the regression 
    # coefficients estimated by the lasso are all equal to zero
    lambda <- lm_lambdamax_OVGLASSO(X = X.std, y = y.std, 
                                    groups = NULL, GRmat = out$grMat, group_weights = group_weights, var_weights = var_weights, 
                                    lambda.min.ratio = lambda.min.ratio, maxl = nlambda)$lambda.seq
  }
  if (is.null(nlambda) && is.null(lambda)) {
    # get the smallest value of such that the regression 
    # coefficients estimated by the lasso are all equal to zero
    lambda <- lm_lambdamax_OVGLASSO(X = X.std, y = y.std, 
                                    groups = NULL, GRmat = out$grMat, group_weights = group_weights, var_weights = var_weights, 
                                    lambda.min.ratio = lambda.min.ratio, maxl = 30)$lambda.seq
  }
  if (!is.null(lambda)) {
    nlambda <- length(lambda)
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Store output
  mRegP    <- matrix(0, nrow = M * L, ncol = nlambda)
  vMse     <- rep(0, nlambda)
  vConv    <- rep(0, nlambda)
  vRunTime <- rep(0, nlambda)
  vIterN   <- rep(0, nlambda)
  objfun   <- vector(mode = "list", length = nlambda)
  r_norm   <- vector(mode = "list", length = nlambda)
  s_norm   <- vector(mode = "list", length = nlambda)
  err_pri  <- vector(mode = "list", length = nlambda)
  err_dual <- vector(mode = "list", length = nlambda)
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Run the ADMM algorithm
  ret <- .admm_ovglasso_fast(A = X.std, b = y.std, groups = out$grMat, group_weights = group_weights, var_weights = var_weights,
                             lambda = lambda, rho_adaptation = adaptation, rho = rho, tau = tau.ada, mu = mu.ada,
                             reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)

  # get estimated coefficients and path
  mSpRegP <- t(ret$coef.path)
  vSpRegP <- ret$coefficients
  
  if (standardize.data == TRUE) {
    # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    # Get the path and retrieve the scaled estimates
    sp.path <- array(dim = c(nlambda, M, L), data = t(apply(mSpRegP, 2, function(x) solve(mU) %*% x %*% mV)))
    
    # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    # Retrieve estimated parameters
    mSpRegP_ <- matrix(solve(mU) %*% vSpRegP %*% mV, nrow = M, ncol = L)
  } else {
    sp.path  <- array(dim = c(nlambda, M, L), data = t(mSpRegP))
    mSpRegP_ <- invvec(vSpRegP, L, M, byrow = FALSE)
  }
  # get the estimated functions
  # sp.fun.path <- t(mTheta) %*% sp.path %*% mH
  # sp.fun.path <- array(data = apply(sp.path, 1, function(x){t(mTheta) %*% x %*% mH}), 
  #                      dim = c(nlambda, p, p))
  sp.fun.path <- array(data = apply(sp.path, 1, function(x){t(mH) %*% x %*% mTheta}), 
                       dim = c(nlambda, p, p))
  # sp.fun      <- t(mTheta) %*% mSpRegP_ %*% mH
  sp.fun      <- t(mH) %*% mSpRegP_ %*% mTheta
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Print to screen
  if (print.out == TRUE) {
    cat("\n\n\n")
    cat("Function to function regression model with overlap group-LASSO penalty\n")
    cat("running time (for ", reltol, " relative error):",
        sum(ret$elapsedTime), "seconds \n\n\n")
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get output
  res.names <- c("sp.coefficients", 
                 "sp.coef.path", 
                 "sp.fun.path",
                 "sp.fun",
                 "lambda.min",
                 "lambda",
                 "mse",
                 "min.mse",
                 "convergence",
                 "elapsedTime",
                 "iternum",
                 "objfun",
                 "r_norm",
                 "s_norm",
                 "err_pri",
                 "err_dual",
                 "rho") 
  res                 <- vector(mode = "list", length = length(res.names))
  names(res)          <- res.names
  res$sp.coefficients <- mSpRegP_
  res$sp.coef.path    <- sp.path
  res$sp.fun.path     <- sp.fun.path
  res$sp.fun          <- sp.fun
  res$lambda.min      <- ret$lambda.min
  res$lambda          <- ret$lambda
  res$mse             <- ret$mse
  res$min.mse         <- ret$min.mse
  res$convergence     <- ret$convergence
  res$elapsedTime     <- ret$elapsedTime
  res$iternum         <- ret$iternum
  res$objfun          <- ret$objfun
  res$r_norm          <- ret$r_norm
  res$s_norm          <- ret$s_norm
  res$err_pri         <- ret$err_pri
  res$err_dual        <- ret$err_dual
  res$rho             <- ret$rho
 
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Return output
  return(res)
}

#' Cross-validation for Overlap Group Least Absolute Shrinkage and Selection Operator for function-on-function regression model
#'
#' Overlap Group-LASSO for function-on-function regression model solves the following optimization problem
#' \deqn{\textrm{min}_{\psi} ~ \frac{1}{2} \sum_{i=1}^n \int \left( y_i(s) - \int x_i(t) \psi(t,s) dt \right)^2 ds + \lambda \sum_{g=1}^{G} \Vert S_{g}T\psi \Vert_2}
#' to obtain a sparse coefficient vector \eqn{\psi=\mathsf{vec}(\Psi)\in\mathbb{R}^{ML}} for the functional penalized predictor \eqn{x(t)}, where the coefficient matrix \eqn{\Psi\in\mathbb{R}^{M\times L}}, 
#' the regression function \eqn{\psi(t,s)=\varphi(t)^\intercal\Psi\theta(s)}, 
#' \eqn{\varphi(t)} and \eqn{\theta(s)} are two B-splines bases of order \eqn{d} and dimension \eqn{M} and \eqn{L}, respectively. For each group \eqn{g}, each row of 
#' the matrix \eqn{S_g\in\mathbb{R}^{d\times ML}} has non-zero entries only for those bases belonging 
#' to that group. These values are provided by the arguments \code{groups} and \code{group_weights} (see below). 
#' Each basis function belongs to more than one group. The diagonal matrix \eqn{T\in\mathbb{R}^{ML\times ML}} contains 
#' the basis-specific weights. These values are provided by the argument \code{var_weights} (see below).
#' The regularization path is computed for the overlap group-LASSO penalty at a grid of values for the regularization 
#' parameter \eqn{\lambda} using the alternating direction method of multipliers (ADMM). See Boyd et al. (2011) and Lin et al. (2022) 
#' for details on the ADMM method.
#'  
#' @param mX an \eqn{(n\times r_x)} matrix of observations of the functional covariate.
#' @param mY an \eqn{(n\times r_y)} matrix of observations of the functional response variable.
#' @param L number of elements of the B-spline basis vector \eqn{\theta(s)}.
#' @param M number of elements of the B-spline basis vector \eqn{\varphi(t)}.
#' @param group_weights a vector of length \eqn{G} containing group-specific weights. The default is square root of the group cardinality, see Bernardi et al. (2022).
#' @param var_weights a vector of length \eqn{ML} containing basis-specific weights. The default is a vector where 
#' each entry is the reciprocal of the number of groups including that basis. See Bernardi et al. (2022) for details.
#' @param standardize.data logical. Should data be standardized?
#' @param splOrd the order \eqn{d} of the spline basis.
#' @param lambda either a regularization parameter or a vector of regularization parameters. 
#' In this latter case the routine computes the whole path. If it is NULL values for lambda are provided by the routine.
#' @param lambda.min.ratio smallest value for lambda, as a fraction of the maximum lambda value. If \eqn{nr_y>LM}, the default is 0.0001, and if \eqn{nr_y<LM}, the default is 0.01. 
#' @param nlambda the number of lambda values - default is 30.
#' @param overall.group logical. If it is TRUE, an overall group including all penalized covariates is added.
#' @param cv.fold the number of folds - default is 5.
#' @param control	 a list of control parameters for the ADMM algorithm. See ‘Details’.
#'
#' @return A named list containing \describe{
#' \item{sp.coefficients}{an \eqn{(M\times L)} solution matrix for the parameters \eqn{\Psi}, which corresponds to the minimum cross-validated MSE.}
#' \item{sp.fun}{an \eqn{(r_x\times r_y)} matrix providing the estimated functional coefficient for \eqn{\psi(t,s)} corresponding to the minimum cross-validated MSE.} 
#' \item{lambda}{sequence of lambda.}
#' \item{lambda.min}{value of lambda that attains the cross-validated minimum mean squared error.}
#' \item{indi.min.mse}{index of the lambda sequence corresponding to lambda.min.}
#' \item{mse}{cross-validated mean squared error.}
#' \item{min.mse}{minimum value of the cross-validated MSE for the sequence of lambda.}
#' \item{mse.sd}{standard deviation of the cross-validated mean squared error.}
#' \item{convergence}{logical. 1 denotes achieved convergence.}
#' \item{elapsedTime}{elapsed time in seconds.}
#' \item{iternum}{number of iterations.}
#' }
#'
#' Iteration stops when both \code{r_norm} and \code{s_norm} values
#' become smaller than \code{eps_pri} and \code{eps_dual}, respectively.
#' @section Details: The control argument is a list that can supply any of the following components:\describe{
#' \item{adaptation}{logical. If it is TRUE, ADMM with adaptation is performed. The default value is TRUE. See Boyd et al. (2011) for details.}
#' \item{rho}{an augmented Lagrangian parameter. The default value is 1.}
#' \item{tau.ada}{an adaptation parameter greater than one. Only needed if adaptation = TRUE. The default value is 2. See Boyd et al. (2011) and Lin et al. (2022) for details.}
#' \item{mu.ada}{an adaptation parameter greater than one. Only needed if adaptation = TRUE. The default value is 10. See Boyd et al. (2011) and Lin et al. (2022) for details.}
#' \item{abstol}{absolute tolerance stopping criterion. The default value is sqrt(sqrt(.Machine$double.eps)).}
#' \item{reltol}{relative tolerance stopping criterion. The default value is sqrt(.Machine$double.eps).}
#' \item{maxit}{maximum number of iterations. The default value is 100.}
#' \item{print.out}{logical. If it is TRUE, a message about the procedure is printed. The default value is TRUE.}
#' }
#' 
#' @examples 
#' 
#' ## generate sample data
#' set.seed(4321)
#' s  <- seq(0, 1, length.out = 100)
#' t  <- seq(0, 1, length.out = 100)
#' p1 <- 5
#' p2 <- 6
#' r  <- 10
#' n  <- 50
#' 
#' beta_basis1 <- splines::bs(s, df = p1, intercept = TRUE)    # first basis for beta
#' beta_basis2 <- splines::bs(s, df = p2, intercept = TRUE)    # second basis for beta
#' 
#' data_basis <- splines::bs(s, df = r, intercept = TRUE)    # basis for X
#' 
#' x_0   <- apply(matrix(rnorm(p1 * p2, sd = 1), p1, p2), 1, 
#'                fdaSP::softhresh, 1.5)  # regression coefficients 
#' x_fun <- beta_basis2 %*% x_0 %*%  t(beta_basis1)  
#' 
#' fun_data <- matrix(rnorm(n*r), n, r) %*% t(data_basis)
#' b        <- fun_data %*% x_fun + rnorm(n * 100, sd = sd(fun_data %*% x_fun )/3)
#' 
#' ## set the hyper-parameters
#' maxit          <- 1000
#' rho_adaptation <- FALSE
#' rho            <- 0.01
#' reltol         <- 1e-5
#' abstol         <- 1e-5
#' 
#' ## fit functional regression model
#' mod_cv <- f2fSP_cv(mY = b, mX = fun_data, L = p1, M = p2,
#'                    group_weights = NULL, var_weights = NULL, 
#'                    standardize.data = FALSE, splOrd = 4,
#'                    lambda = NULL, nlambda = 30, cv.fold = 5, 
#'                    lambda.min.ratio = NULL,
#'                    control = list("abstol" = abstol, 
#'                                   "reltol" = reltol, 
#'                                   "maxit" = maxit, 
#'                                   "adaptation" = rho_adaptation, 
#'                                   "rho" = rho, 
#'                                   "print.out" = FALSE))
#'
#' ### graphical presentation
#' plot(log(mod_cv$lambda), mod_cv$mse, type = "l", col = "blue", lwd = 2, bty = "n", 
#'      xlab = latex2exp::TeX("$\\log(\\lambda)$"), ylab = "Prediction Error", 
#'      ylim = range(mod_cv$mse - mod_cv$mse.sd, mod_cv$mse + mod_cv$mse.sd),
#'      main = "Cross-validated Prediction Error")
#' fdaSP::confband(xV = log(mod_cv$lambda), yVmin = mod_cv$mse - mod_cv$mse.sd, 
#'                 yVmax = mod_cv$mse + mod_cv$mse.sd)       
#' abline(v = log(mod_cv$lambda[which(mod_cv$lambda == mod_cv$lambda.min)]), col = "red", lwd = 1.0)
#' 
#' ### comparison with oracle error
#' mod <- f2fSP(mY = b, mX = fun_data, L = p1, M = p2,
#'              group_weights = NULL, var_weights = NULL, 
#'              standardize.data = FALSE, splOrd = 4,
#'              lambda = NULL, nlambda = 30, lambda.min.ratio = NULL, 
#'              control = list("abstol" = abstol, 
#'                             "reltol" = reltol, 
#'                             "maxit" = maxit,
#'                             "adaptation" = rho_adaptation, 
#'                             "rho" = rho, 
#'                             "print.out" = FALSE))
#' err_mod <- apply(mod$sp.coef.path, 1, function(x) sum((x - x_0)^2))
#' plot(log(mod$lambda), err_mod, type = "l", col = "blue", lwd = 2, 
#'      xlab = latex2exp::TeX("$\\log(\\lambda)$"), 
#'      ylab = "Estimation Error", main = "True Estimation Error", bty = "n")
#' abline(v = log(mod$lambda[which(err_mod == min(err_mod))]), col = "red", lwd = 1.0)
#' abline(v = log(mod_cv$lambda[which(mod_cv$lambda == mod_cv$lambda.min)]), 
#'        col = "red", lwd = 1.0, lty = 2)
#' 
#' @references
#' \insertRef{bernardi_etal.2022}{fdaSP}
#' 
#' \insertRef{boyd_etal.2011}{fdaSP}
#' 
#' \insertRef{jenatton_etal.2011}{fdaSP}
#' 
#' \insertRef{lin_etal.2022}{fdaSP}
#'
#' @export
f2fSP_cv <- function(mY, mX, L, M, group_weights = NULL, var_weights = NULL, standardize.data = FALSE, splOrd = 4,
                     lambda = NULL, lambda.min.ratio = NULL, nlambda = NULL, cv.fold = 5, overall.group = FALSE, control = list()) {
    
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get dimensions
  n <- dim(mX)[1]             # number of observations
  p <- dim(mX)[2]
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Create the B-spline basis functions
  s      <- seq(0, 1, length.out = p)
  mH     <- t(bs(s, df = M, intercept = TRUE))
  mTheta <- t(bs(s, df = L, intercept = TRUE))
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Pre-processing
  # data validity
  if (!check_data_matrix(mX)) {
    stop("* f2fSP_cv : input 'mX' is invalid data matrix.")
  }
  if (!check_data_matrix(mY)) {
    stop("* f2fSP_cv : input 'mY' is invalid data matrix.")
  }
  mY <- as.matrix(mY)
  # data size
  if (n != dim(mY)[1]) {
    stop("* f2fSP_cv : two inputs 'mX' and 'mY' have non-matching dimension.")
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Define response and design matrix
  mY_B <- mY
  mX_B <- tcrossprod(mX, mH)

  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Vectorisation form
  X_ <- kronecker(t(mTheta), mX_B)
  y_ <- matrix(ks::vec(mY_B, byrow = FALSE), ncol = 1)

  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Standardise response and design matrix
  if (standardize.data == TRUE) {
    res   <- standardizemat(X_, y_)
    X.std <- res$X.std
    y.std <- res$y.std
    mU    <- res$mU
    mV    <- res$mV
  } else {
    X.std <- X_
    y.std <- y_
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Pre-processing: lambda parameter (in case of a single lambda)
  if (!is.null(lambda)) {
    if (length(lambda) == 1) {
      meps     <- (.Machine$double.eps)
      negsmall <- -meps
      if (!check_param_constant(lambda, negsmall)) {
        stop("* f2fSP_cv : reg. parameter 'lambda' is invalid.")
      }
      if (lambda < meps){
        message("* f2fSP_cv : since 'lambda' is effectively zero, a least-squares solution is returned.")
        xsol     <- as.vector(aux_pinv(X_) %*% matrix(y_))
        output   <- list()
        output$x <- xsol
        return(output)
      }
    }
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # check for inputs
  npar   <- dim(X.std)[2]
  con    <- list(maxit                = 100L, 
                 abstol               = sqrt(.Machine$double.eps), 
                 reltol               = sqrt(sqrt(.Machine$double.eps)), 
                 adaptation           = TRUE, 
                 rho                  = 1, 
                 tau.ada              = 2,               
                 mu.ada               = 10,
                 par.init             = NULL,
                 init.rnd             = FALSE,
                 print.out            = TRUE)
  
  nmsC                          <- names(con)
  con[(namc <- names(control))] <- control
  if (length(noNms <- namc[!namc %in% nmsC])) {
    warning("unknown names in control: ", paste(noNms, collapse = ", "))
  }
  # set default values (for inputs)
  maxit                <- con$maxit
  abstol               <- con$abstol
  reltol               <- con$reltol
  adaptation           <- con$adaptation
  rho                  <- con$rho
  tau.ada              <- con$tau.ada      
  mu.ada               <- con$mu.ada
  par.init             <- con$par.init
  init.rnd             <- con$init.rnd
  print.out            <- con$print.out
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Pre-processing
  # data validity
  if (!check_param_constant_multiple(c(abstol, reltol))) {
    stop("* f2fSP_cv : tolerance level is invalid.")
  }
  if (!check_param_integer(maxit, 0.0)) {
    stop("* f2fSP_cv : 'maxit' should be a positive integer.")
  }
  maxit = as.integer(maxit)
  if (!check_param_constant(rho, 0.0)) {
    stop("* f2fSP_cv : 'rho' should be a positive real number.")
  }
  if (!check_param_constant(mu.ada, 0.0)) {
    stop("* f2fSP_cv : 'mu' should be a positive real number.")
  }
  if ((tau.ada < 1.0) || (tau.ada > 2.0)) {
    warning("* f2fSP_cv : 'tau' value is suggested to be in [1,2].")
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Pre-processing: variable weights
  check_weights(x = var_weights, n = dim(X_)[2], algname = "f2fSP", funname = "var_weights")
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get the groups of the overlap group LASSO penalty
  if (overall.group == TRUE) {
    out <- penfun(method = "ogl&1", p = M, q = L, splOrd = splOrd, regpars = NULL)
  } else {
    out <- penfun(method = "ogl", p = M, q = L, splOrd = splOrd, regpars = NULL)
  }
  GRmat <- out$grMat
  G     <- dim(GRmat)[1]
  nG    <- rowSums(GRmat)
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Manage group & overall group weights
  if (is.null(group_weights)) {
    # Define the weights as in Yuan and Lin 2006, JRSSB
    #group_weights <- sqrt(diag(tcrossprod(GRmat)))
    group_weights <- rep(1, G)
  } else {
    if (overall.group == TRUE) {
      check_group_weights(x = group_weights, n = G, algname = "f2fSP", funname = "group_weights")
      if (length(group_weights) == (G-1)) {
        group_weights <- c(group_weights, sqrt(dim(GRmat)[2]))
      }
    } else {
      check_weights(x = group_weights, n = G, algname = "f2fSP", funname = "group_weights")
    }
  } 
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Manage variable weights
  if (is.null(var_weights)) {
    var_weights <- 1.0 / colSums(out$grMat)
    var_weights <- 1.0 / sqrt(colSums(out$grMat))
  }

  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get the sequence of lambda
  if (!is.null(nlambda) && is.null(lambda)) {
    # get the smallest value of such that the regression 
    # coefficients estimated by the lasso are all equal to zero
    lambda <- lm_lambdamax_OVGLASSO(y = y.std, X = X.std,
                                    groups = NULL, GRmat = out$grMat, group_weights = group_weights, var_weights = var_weights, 
                                    lambda.min.ratio = lambda.min.ratio, maxl = nlambda)$lambda.seq
  }
  if (is.null(nlambda) && is.null(lambda)) {
    # get the smallest value of such that the regression 
    # coefficients estimated by the lasso are all equal to zero
    lambda <- lm_lambdamax_OVGLASSO(y = y.std, X = X.std, 
                                    groups = NULL, GRmat = out$grMat, group_weights = group_weights, var_weights = var_weights, 
                                    lambda.min.ratio = lambda.min.ratio, maxl = 30)$lambda.seq
  }
  if (!is.null(lambda)) {
    nlambda <- length(lambda)
  }
  
  # other parameters
  meps     = (.Machine$double.eps)
  negsmall = -meps
  if (!check_param_constant(min(lambda), negsmall)) {
    stop("* f2fSP_cv : reg. parameter 'lambda' is invalid.")
  }
  if (min(lambda) < meps) {
    message("* f2fSP_cv : since 'lambda' is effectively zero, a least-squares solution is returned.")
    xsol     = as.vector(aux_pinv(mX) %*% matrix(mY))
    output   = list()
    output$x = xsol
    return(output)
  }
  if (!is.null(lambda)) {
    nlambda <- length(lambda)
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get groups
  ret.gr <- groups.cv(n = n, k = cv.fold)
  
  # store Lasso regression for each fold
  vMse_cv <- matrix(0.0, cv.fold, nlambda)
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Initial time 
  startTime <- Sys.time()
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Run lasso
  for (kt in 1:cv.fold) {
    # run ADMM
    # mX_ <- X.std[ret.gr$groups.cv[[kt]],]
    # vY_ <- y.std[ret.gr$groups.cv[[kt]]]
    data.gr <- f2freg_cv_manage_data(mY = mY, mX = mX, mTheta = mTheta, mH = mH, 
                                     ix.ins = ret.gr$groups.cv[[kt]], 
                                     ix.oos = ret.gr$groups.pred[[kt]], standardize.data = standardize.data)
    mX_ <- data.gr$X_ins.std
    vY_ <- data.gr$y_ins.std
    ret <- .admm_ovglasso_fast(A = mX_, b = vY_, groups = out$grMat, group_weights = group_weights, var_weights = var_weights, 
                               lambda = lambda, rho_adaptation = adaptation, rho = rho, tau = tau.ada, mu = mu.ada, 
                               reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
    
    # Evaluate MSE
    mSpRegP_     <- ret$coef.path
    fit          <- data.gr$X_oos.std %*% t(mSpRegP_)
    n_           <- length(data.gr$y_oos.std)
    mse          <- apply(fit, 2, function(x) {sum((data.gr$y_oos.std - x)^2)/n_})
    vMse_cv[kt,] <- mse
  }
  
  # get mean squared error
  vMse <- apply(vMse_cv, 2, mean) 
  vSd  <- apply(vMse_cv, 2, sd) 
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # system time
  endTime <- Sys.time()
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # compute run time
  eltime <- difftime(time1 = endTime, time2 = startTime, units = "secs")
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get lasso output
  min.mse      <- min(vMse)
  indi.min.mse <- which(vMse == min.mse) 
  lambda.min   <- lambda[indi.min.mse]
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # get estimate over the full sample
  rho_ <- ret$rho[[indi.min.mse]]
  ret  <- .admm_ovglasso(A = X.std, b = y.std,
                         groups = out$grMat, group_weights = group_weights, var_weights = var_weights,
                         u = t(ret$U[indi.min.mse,]), z = t(ret$Z[indi.min.mse,]), lambda = lambda.min,
                         rho_adaptation = adaptation, rho = rho_[length(rho_)], tau = tau.ada, mu = mu.ada,
                         reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)

  # Store output: full sample estimate
  niter <- ret$niter
  if (niter < maxit) {
    converged <- TRUE
  } else {
    converged <- FALSE
  }
  vSpRegP <- ret$x
  dConv   <- ifelse(converged == TRUE, 1, 0) 
  nIterN  <- ret$niter

  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Retrieve estimated parameters
  if (standardize.data == TRUE) {
    vSpRegP_ <- matrix(solve(mU) %*% vSpRegP %*% mV, nrow = npar, ncol = 1)
  } else {
    vSpRegP_ <- matrix(vSpRegP, nrow = npar, ncol = 1)
  }
  mSpRegP_ <- matrix(vSpRegP_, nrow = M, ncol = L, byrow = FALSE)
  
  # get the estimated functions
  # sp.fun <- t(mTheta) %*% mSpRegP_ %*% mH
  sp.fun <- t(mH) %*% mSpRegP_ %*% mTheta

  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Print to screen
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  if (print.out == TRUE) {
    cat("\n\n\n")
    cat("function to function regression model with overlap group-LASSO penalty, Cross Validation\n")
    cat("Alternating direction method of multipliers\n")
    cat("running time (for ", reltol, " relative error):",
        eltime, "seconds \n\n\n")
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get output
  res.names <- c("sp.coefficients",
                 "sp.fun",
                 "mse",
                 "mse.sd",
                 "min.mse",
                 "lambda",
                 "lambda.min",
                 "convergence",
                 "elapsedTime",
                 "iternum",
                 "indi.min.mse") 
  res        <- vector(mode = "list", length = length(res.names))
  names(res) <- res.names
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get output
  ret                 <- NULL
  ret$sp.coefficients <- mSpRegP_
  ret$sp.fun          <- sp.fun
  ret$mse             <- vMse
  ret$mse.sd          <- vSd
  ret$min.mse         <- min.mse
  ret$lambda          <- lambda
  ret$lambda.min      <- lambda.min
  ret$convergence     <- dConv
  ret$elapsedTime     <- eltime
  ret$iternum         <- nIterN
  ret$indi.min.mse    <- indi.min.mse
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Return output
  return(ret)
}

f2freg_cv_manage_data <- function(mY, mX, mTheta, mH, ix.ins = NULL, ix.oos = NULL, standardize.data = TRUE) {
    
  if (!is.null(ix.ins)) {
    # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    # Define the in-sample and out-of-sample data
    mY_ins <- mY[ix.ins,]
    mY_oos <- mY[ix.oos,]
    mX_ins <- mX[ix.ins,]
    mX_oos <- mX[ix.oos,]
    
    # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    # Define response and design matrix
    mY_B_ins <- mY_ins
    mY_B_oos <- mY_oos
    mX_B_ins <- tcrossprod(mX_ins, mH)
    mX_B_oos <- tcrossprod(mX_oos, mH)
    
    # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    # Vectorisation form
    X_ins_ <- kronecker(t(mTheta), mX_B_ins)
    X_oos_ <- kronecker(t(mTheta), mX_B_oos)
    y_ins_ <- matrix(ks::vec(mY_B_ins, byrow = FALSE), ncol = 1)
    y_oos_ <- matrix(ks::vec(mY_B_oos, byrow = FALSE), ncol = 1)
    X_     <- rbind(X_ins_, X_oos_)
    y_     <- as.matrix(c(y_ins_, y_oos_))
    
    # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    # Standardise response and design matrix
    if (standardize.data == TRUE) {
      res       <- standardizemat(X_, y_)
      mU        <- res$mU
      mV        <- res$mV
      mx        <- apply(X_, 2, mean)
      my        <- mean(y_)
      y_ins.std <- (y_ins_ - my) %*% solve(mV)
      y_oos.std <- (y_oos_ - my) %*% solve(mV)
      X_ins.std <- t(apply(X_ins_, 1, function(x) {(x-mx)})) %*% solve(mU)
      X_oos.std <- t(apply(X_oos_, 1, function(x) {(x-mx)})) %*% solve(mU)
    } else {
      X_ins.std <- X_ins_
      y_ins.std <- y_ins_
      X_oos.std <- X_oos_
      y_oos.std <- y_oos_
    }
  } else {
    # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    # Define response and design matrix
    mY_B <- mY
    mX_B <- tcrossprod(mX, mH)
    
    # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    # Vectorisation form
    X_ <- kronecker(t(mTheta), mX_B)
    y_ <- matrix(ks::vec(mY_B, byrow = FALSE), ncol = 1)
    
    # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    # Standardise response and design matrix
    if (standardize.data == TRUE) {
      res   <- standardizemat(X_, y_)
      X.std <- res$X.std
      y.std <- res$y.std
    } else {
      X.std <- X_
      y.std <- y_
    }
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get output
  res <- NULL
  if (!is.null(ix.ins)) {
    res$X_ins.std <- X_ins.std
    res$y_ins.std <- y_ins.std
    res$X_oos.std <- X_oos.std
    res$y_oos.std <- y_oos.std
  } else {
    res$X.std <- X.std
    res$y.std <- y.std
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Return output
  return(res)
}

# Authors: Mauro Bernardi 
#          Department Statistical Sciences
#       	 University of Padova
#      	   Via Cesare Battisti, 241
#      	   35121 PADOVA, Italy
#          E-mail: mauro.bernardi@unipd.it  

# Last change: July 20, 2020

# :::::::::::::::::::::::::::::::::::::::::::::::::
# Functions
# 0. lambdamax (wrap)
# 1. getlambda_LASSO
# 2. getlambda_ELNET
# 3. getlambda_OVGLASSO
# 4. penfun
# 5. getGroupMat_OVGLASSO

# 4. penfun
penfun <- function(method, p, q, splOrd, regpars) {
  
  # get the number of groups
  if (q == 1) {
    B <- p - splOrd + 1
  } else {
    B <- (p - splOrd + 1) * (q - splOrd + 1)
  }
  
  # lasso
  if (method == "lasso") {
    Cmat         <- diag(1L, p * q)
    Pmat         <- Cmat
    Cvec         <- rep(1L, p * q)
    grMat        <- NULL
    grMatWeights <- diag(1L, p * q)
    norme        <- NULL
  }
  
  # ridge
  if (method == "ridge") {
    Cmat         <- diag(1L, p * q)
    Pmat         <- Cmat
    Cvec         <- rep(1L, p * q)
    grMat        <- NULL
    grMatWeights <- diag(1L, p * q)
    norme        <- NULL
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # overlap group lasso (function-to-function)
  # no overall group
  if ((method == "ogl") & (q > 1)) {
    
    # get the group matrices
    grMat        <- getGroupMat_OVGLASSO(p = p, q = q, splOrd = splOrd)
    grMatWeights <- 1.0 / apply(grMat, 2, sum)
    
    if (is.null(regpars)) {
      Cpen <- NULL
      Cmat <- NULL
      Cvec <- NULL
      norme <- NULL
    } else {
      
      # get the weights
      norme_     <- grMat %*% (regpars^2)
      
      # Set the norm to a given threshold
      norme <- fSet2GivenTolerance(x = norme_, n = B, toler = .Machine$double.eps)
      
      Cvec       <- as.vector(t(grMat) %*% (1.0 / (2.0 * sqrt(norme))))
      Cpen       <- diag(1, p * q)
      diag(Cpen) <- Cvec
      Cmat       <- invvec(Cvec, p, q)
    }
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # overlap group lasso (function-to-function)
  # overall group
  if ((method == "ogl&1") & (q > 1)) {
    # get the group matrices
    grMat        <- getGroupMat_OVGLASSO(p = p, q = q, splOrd = splOrd)
    grMat        <- rbind(grMat, rep(1L, p * q))
    grMatWeights <- 1.0 / apply(grMat, 2, sum)
    
    # get the weights
    if (is.null(regpars)) {
      Cpen  <- NULL
      Cmat  <- NULL
      Cvec  <- NULL
      norme <- NULL
    } else {
      # get the weights
      # Cvec   <- 1.0 / (2.0 * sqrt(crossprod(regpars)))
      norme_ <- grMat %*% (regpars^2)
      
      # Set the norm to a given threshold
      norme <- fSet2GivenTolerance(x = norme_, n = B, toler = .Machine$double.eps)
      
      # add overall group
      #Cvec       <- rep(Cvec, p*q) + as.vector(t(grMat) %*% (1.0 / (2.0 * sqrt(norme))))
      Cvec       <- as.vector(t(grMat) %*% (1.0 / (2.0 * sqrt(norme))))
      Cpen       <- diag(1, p * q)
      diag(Cpen) <- Cvec
      Cmat       <- invvec(Cvec, p, q)
    }
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # overlap group lasso (function-to-scalar)
  # no overall group
  if ((method == "ogl") & (q == 1)) {
    
    # get the group matrices
    grMat        <- getGroupMat_OVGLASSO(p = p, q = 1, splOrd = splOrd)
    grMatWeights <- 1.0 / apply(grMat, 2, sum)
    
    if (is.null(regpars)) {
      Cpen <- NULL
      Cmat <- NULL
      Cvec <- NULL
      norme <- NULL
    } else {
      
      # get the weights
      norme_    <- grMat %*% (regpars^2)
      
      # Set the norm to a given threshold
      norme <- fSet2GivenTolerance(x = norme_, n = B, toler = .Machine$double.eps)
      
      Cvec       <- as.vector(t(grMat) %*% (1.0 / (2.0 * sqrt(norme))))
      Cpen       <- diag(1, p)
      diag(Cpen) <- Cvec
      Cmat       <- Cpen
    }
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # overlap group lasso (function-to-scalar)
  # no overall group
  if ((method == "ogl&1") & (q == 1)) {
    
    # get the group matrices
    grMat        <- getGroupMat_OVGLASSO(p = p, q = 1, splOrd = splOrd)
    grMat        <- rbind(grMat, rep(1L, p))
    grMatWeights <- 1.0 / apply(grMat, 2, sum)
    
    if (is.null(regpars)) {
      Cpen  <- NULL
      Cmat  <- NULL
      Cvec  <- NULL
      norme <- NULL
    } else {
      
      # get the weights
      norme_    <- grMat %*% (regpars^2)
      
      # Set the norm to a given threshold
      norme <- fSet2GivenTolerance(x = norme_, n = B, toler = .Machine$double.eps)
      
      Cvec       <- as.vector(t(grMat) %*% (1.0 / (2.0 * sqrt(norme))))
      Cpen       <- diag(1, p)
      diag(Cpen) <- Cvec
      Cmat       <- Cpen
    }
  }
  
  # get the output
  res              <- NULL
  res$Cpen         <- Cpen
  res$Cmat         <- Cmat
  res$Cvec         <- Cvec
  res$grMat        <- grMat
  res$grMatWeights <- grMatWeights
  res$regp.grnorm  <- norme
  
  # Return output
  return(res)
}

# 5. getGroupMat_OVGLASSO
# get groups of overlap group LASSO
getGroupMat_OVGLASSO <- function(p, q, splOrd = c(3, 4)) {
  
  if (q > 1) {
    # get the number of groups
    B     <- (p - splOrd + 1) * (q - splOrd + 1)
    grMat <- matrix(0, B, p * q)
    
    # get the groups matrix
    if ((splOrd == 3) || (splOrd == 4)) {
      # spline of order 3
      if (splOrd == 3) {
        cnt <- 1
        for (i in 0:(p-splOrd)) {
          for (j in 1:(q-splOrd+1)){
            ind             <- c(q*i+j, q*i+j+1, q*i+j+2, p+q*i+j, p+q*i+j+1, p+q*i+j+2, 2*p+q*i+j, 2*p+q*i+j+1, 2*p+q*i+j+2)
            grMat[cnt, ind] <- 1
            cnt             <- cnt + 1
          }
        }
      }
      
      # spline of order 4
      if (splOrd == 4) {
        # cnt <- 1
        # for (i in 0:(p-splOrd)) {
        #   for (j in 1:(q-splOrd+1)) {
        #     ind             <- c(q*i+j, q*i+j+1, q*i+j+2, q*i+j+3, p+q*i+j, p+q*i+j+1, p+q*i+j+2, p+q*i+j+3, 2*p+q*i+j, 2*p+q*i+j+1, 2*p+q*i+j+2, 2*p+q*i+j+3, 3*p+q*i+j, 3*p+q*i+j+1, 3*p+q*i+j+2, 3*p+q*i+j+3)
        #     grMat[cnt, ind] <- 1
        #     cnt             <- cnt + 1
        #   }
        # }
        # cnt <- 1
        # for (i in 0:(p - splOrd)) {
        #   init_row <- i * q
        #   for (j in 1:(q - splOrd + 1)) {
        #     init_col <- j
        #     v1       <- init_row + init_col + 0:(splOrd-1)
        #     v2       <- q + init_row + init_col + 0:(splOrd-1)
        #     v3       <- 2*q + init_row + init_col + 0:(splOrd-1)
        #     v4       <- 3*q + init_row + init_col + 0:(splOrd-1)
        #     ind      <- c(v1, v2, v3, v4)
        # 
        #     grMat[cnt, ind] <- 1
        #     cnt             <- cnt + 1
        #   }
        # }
        B <- (p - splOrd + 1) * (q - splOrd + 1)
        grMat <- matrix(0, B, p * q)
        cnt <- 1
        for (j in 1:(q - splOrd + 1)) {
          init_col <- j
          for (i in 0:(p - splOrd)) {
            init_row <- i*q
            
            v1  <- init_row + init_col + 0:3
            v2  <- q + init_row + init_col + 0:3
            v3  <- 2*q + init_row + init_col + 0:3
            v4  <- 3*q + init_row + init_col + 0:3
            ind <- c(v1, v2, v3, v4)
            
            grMat[cnt, ind] <- 1
            cnt <- cnt + 1
          }
        }
      }
    } else {
      cat("Error in getGroupMat_OVGLASSO, the spline order in not equat to 3 or 4!\n")
    }
  } else if (q == 1) {
    # get the number of groups
    B     <- (p - splOrd + 1)
    grMat <- matrix(0, B, p)
    
    # get the groups matrix
    if ((splOrd == 3) || (splOrd == 4)) {
      # spline of order 3
      if (splOrd == 3) {
        cnt <- 1
        for (i in 0:(p-splOrd)) {
          ind             <- c(i+1, i+2, i+3)
          grMat[cnt, ind] <- 1
          cnt             <- cnt + 1
        }
      }
      
      # spline of order 4
      if (splOrd == 4) {
        cnt <- 1
        for (i in 0:(p-splOrd)) {
          ind             <- c(i+1, i+2, i+3, i+4)
          grMat[cnt, ind] <- 1
          cnt             <- cnt + 1
        }
      }
    } else {
      cat("Error in getGroupMat_OVGLASSO, the spline order in not equat to 3 or 4!\n")
    }
  } else {
    cat("q is negative or equal to zero in getGroupMat_OVGLASSO!\n")
  }
  
  # return output
  return(grMat)
}

### codice per bande ombreggiate
# xV:    vettore asse x (sequenza lambda)
# yVmax: vettore asse y (MSE + sd(MSE)))
# yVmin: vettore asse y (MSE - sd(MSE)))

#' Function to plot the confidence bands 
#'  
#' @param xV the values for the x-axis.
#' @param yVmin the minimum values for the y-axis. 
#' @param yVmax the maximum values for the y-axis.
#'
#' @return a polygon. 
#' 
#' @export confband
confband <- function(xV, yVmin, yVmax) {
  polygon(x = c(xV, rev(xV)), y = c(yVmin, rev(yVmax)),  col = rgb(0,0,1,.3), border = NA)
}

#
## last update: July 22, 2026
#

#
## Description: compute the degrees of freedom for linear regression 
##              model with several Lasso-type penalties
#

# Functions: 
#
#           1.  lm_dof,                                  wrap of the dof for the linear regression
#           1.1 lm_dof_LASSO,                            wrapped in lm_dof
#           1.2 lm_dof_OVGLASSO,                         wrapped in  lm_dof
#           2.  f2s_dof                                  wrap of the dof for the scalar-on-function regression
#           3.  f2s_cov_dof                              wrap of the dof for the scalar-on-function regression with non-functional covariates
#           4.  ovglasso_group_selmat

####
#     Linear regression models
####

#' @keywords internal
#' @export
lm_dof <- function(y, 
                   X, 
                   Z = NULL,  
                   coeff,
                   lambda,
                   GRmat,
                   group_weights,
                   var_weights,
                   var_weights_L1 = NULL,
                   Umat,
                   err_primal,
                   err_dual,
                   rho,
                   toler_c,
                   toler_d,
                   method = c("lasso", 
                              "glasso", 
                              "ovglasso")) {
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # check maxl
  
  # compute DoF
  if (strcmpi(method, "glasso") || strcmpi(method, "ovglasso")) {
    res <- lm_dof_OVGLASSO(X = X,
                           Z = Z,
                           coeff = coeff,
                           lambda = lambda,
                           GRmat = GRmat,
                           group_weights = group_weights,
                           var_weights = var_weights,
                           Umat = Umat,
                           err_primal = err_primal,
                           err_dual = err_dual,
                           rho = rho,
                           toler_c = toler_c,
                           toler_d = toler_d)
  } else if (strcmpi(method, "lasso")) {
    # da fare
    res <- lm_dof_LASSO(X = X, 
                        Z = Z, 
                        coeff = coeff, 
                        lambda = lambda,
                        var_weights = var_weights,
                        Umat = Umat,
                        err_primal = err_primal,
                        err_dual = err_dual,
                        rho = rho,
                        toler_c = toler_c,
                        toler_d = toler_d)
  }
  
  # return output
  return(res)
}

#' @keywords internal
lm_dof_LASSO <- function(X,
                         Z = NULL,
                         coeff,
                         lambda,
                         var_weights = NULL,
                         Umat,
                         err_primal,
                         err_dual,
                         rho,
                         toler_c,
                         toler_d) {
  
  # get dimensions
  nlambda <- length(lambda)
  p       <- dim(X)[2]
  
  # store output
  dof_STORE           <- rep(0.0, nlambda)
  coeff_active_STORE  <- vector("list", nlambda)
  
  # compute the degrees of freedom
  for (k in 1:nlambda) {
    if (is.null(Z)) {
      if (is.null(var_weights)) {
        # attenzione che con var_weights NULL o altrimenti è uguale
        res <- .lm_dof_LASSO_1lambda(X = X,
                                     coeff = coeff[k,],
                                     lambda = lambda[k],
                                     Uvec = Umat[k,],
                                     err_primal = err_primal[[k]][length(err_primal[[k]])],
                                     err_dual = err_dual[[k]][length(err_dual[[k]])],
                                     rho = rho[[k]][length(rho[[k]])],
                                     toler_c = toler_c,
                                     toler_d = toler_d)
      } else {
        res <- .lm_dof_LASSO_1lambda(X = X,
                                     coeff = coeff[k,],
                                     lambda = lambda[k],
                                     Uvec = Umat[k,],
                                     err_primal = err_primal[[k]][length(err_primal[[k]])],
                                     err_dual = err_dual[[k]][length(err_dual[[k]])],
                                     rho = rho[[k]][length(rho[[k]])],
                                     toler_c = toler_c,
                                     toler_d = toler_d)
      }
    } else {
      
      # define the relevant quantities to specify the LASSO penalty using the OVGLASSO one.
      p             <- dim(X)[2]
      GRmat         <- diag(1, p)
      group_weights <- rep(1, p)
      var_weights   <- rep(1, p)
      
      # run dop
      res <- .lm_cov_dof_OVGLASSO_1lambda(X = X, 
                                          Z = Z,
                                          coeff_X = coeff[k,],
                                          lambda = lambda[k],
                                          GRmat = GRmat,
                                          group_weights = group_weights,
                                          var_weights = var_weights,
                                          Uvec = Umat[k,],
                                          err_primal = err_primal[[k]][length(err_primal[[k]])],
                                          err_dual = err_dual[[k]][length(err_dual[[k]])],
                                          rho = rho[[k]][length(rho[[k]])],
                                          toler_c = toler_c,
                                          toler_d = toler_d)
    }
    
    # store output
    dof_STORE[k]            <- res$dof
    coeff_active_STORE[[k]] <- res$coeff_active
  }
  
  # get output
  output               <- NULL
  output$dof           <- dof_STORE
  output$coeff_active  <- coeff_active_STORE
  
  # return output
  return(output)
}

#
## Overlap Group-Lasso DoF
#

#' @keywords internal
ovglasso_group_selmat <- function(y, x) {
  indi <- outer(y, x, function(xi, yj) {as.integer(yj == xi)})
  return(indi)
}

#' @keywords internal
lm_dof_OVGLASSO <- function(y,
                            X,
                            Z = NULL,
                            coeff,
                            lambda,
                            adaptive_weights = TRUE,
                            GRmat,
                            group_weights,
                            var_weights,
                            Umat,
                            err_primal,
                            err_dual,
                            rho,
                            toler_c,
                            toler_d) {
  
  # get dimensions
  nlambda <- length(lambda)
  p       <- dim(X)[2]
  G       <- length(group_weights)
  
  # store output
  dof_STORE           <- rep(0.0, nlambda)
  coeff_active_STORE  <- vector("list", nlambda)
  groups_active_STORE <- matrix(0, nlambda, G)
  
  # compute LS estimate for adaptive weights
  if (adaptive_weights == TRUE) {
    coeff_LS <- lm_ols(X = X, y = y)
  }
  
  # compute the degrees of freedom
  for (k in 1:nlambda) {
    if (is.null(Z)) {
      if (adaptive_weights == TRUE) {
        res <- .lm_adaptive_dof_OVGLASSO_1lambda(X = X,
                                                 coeff = coeff[k,],
                                                 coeff_LS = coeff_LS,
                                                 lambda = lambda[k],
                                                 GRmat = GRmat,
                                                 group_weights = group_weights,
                                                 var_weights = var_weights,
                                                 Uvec = Umat[k,],
                                                 err_primal = err_primal[[k]][length(err_primal[[k]])],
                                                 err_dual = err_dual[[k]][length(err_dual[[k]])],
                                                 rho = rho[[k]][length(rho[[k]])],
                                                 toler_c = toler_c,
                                                 toler_d = toler_d)
      } else {
        res <- .lm_dof_OVGLASSO_1lambda(X = X,
                                        coeff = coeff[k,], 
                                        lambda = lambda[k],
                                        GRmat = GRmat,
                                        group_weights = group_weights,
                                        var_weights = var_weights,
                                        Uvec = Umat[k,],
                                        err_primal = err_primal[[k]][length(err_primal[[k]])],
                                        err_dual = err_dual[[k]][length(err_dual[[k]])],
                                        rho = rho[[k]][length(rho[[k]])],
                                        toler_c = toler_c,
                                        toler_d = toler_d)
      }
    } else {
      if (adaptive_weights == TRUE) {
        res <- .lm_cov_adaptive_dof_OVGLASSO_1lamba(X = X, 
                                                    Z = Z,
                                                    coeff_X = coeff[k,],
                                                    coeff_X_LS = coeff_LS,
                                                    lambda = lambda[k],
                                                    GRmat = GRmat,
                                                    group_weights = group_weights,
                                                    var_weights = var_weights,
                                                    Uvec = Umat[k,],
                                                    err_primal = err_primal[[k]][length(err_primal[[k]])],
                                                    err_dual = err_dual[[k]][length(err_dual[[k]])],
                                                    rho = rho[[k]][length(rho[[k]])],
                                                    toler_c = toler_c,
                                                    toler_d = toler_d)
      } else {
        res <- .lm_cov_dof_OVGLASSO_1lambda(X = X, 
                                            Z = Z,
                                            coeff_X = coeff[k,],
                                            lambda = lambda[k],
                                            GRmat = GRmat,
                                            group_weights = group_weights,
                                            var_weights = var_weights,
                                            Uvec = Umat[k,],
                                            err_primal = err_primal[[k]][length(err_primal[[k]])],
                                            err_dual = err_dual[[k]][length(err_dual[[k]])],
                                            rho = rho[[k]][length(rho[[k]])],
                                            toler_c = toler_c,
                                            toler_d = toler_d)
      }
    }
    
    # store output
    dof_STORE[k]            <- res$dof
    coeff_active_STORE[[k]] <- res$coeff_active
    groups_active_STORE[k,] <- t(res$groups_active)
  }
  
  # get output
  output               <- NULL
  output$dof           <- dof_STORE
  output$coeff_active  <- coeff_active_STORE
  output$groups_active <- groups_active_STORE
  
  # return output
  return(output)
}


####
#     Scalar on function regression
####

#' @keywords internal
f2s_dof <- function(W,
                    y,
                    coeff,
                    lambda1,
                    lambda2 = NULL,
                    diff_order = 1,
                    adaptive_weights = TRUE,
                    GRmat,
                    group_weights,
                    var_weights,
                    Umat,
                    err_primal,
                    err_dual,
                    rho,
                    toler_c,
                    toler_d) {
  
  # get dimensions
  nlambda1  <- length(lambda1)
  nlambda2 <- length(lambda2)
  p        <- dim(W)[2]
  G        <- length(group_weights)
  
  # store output
  if (is.null(lambda2)) {
    # se lambda2 è NULL lo considero zero
    dof_STORE                  <- rep(0.0, nlambda1)
    groups_active_STORE        <- matrix(data = 0, nrow = nlambda1, ncol = G)
    vars_active_STORE          <- matrix(data = 0, nrow = nlambda1, ncol = p)
    coeff_active_STORE         <- vector("list", nlambda1)
    length(coeff_active_STORE) <- nlambda1
    lambda2                    <- 0                                    # 27 april 2026: set to zero in case NULL
  } else {
    if (nlambda2 == 1) {
      if (lambda2 == 0) {
        # qui ho un solo lambda1 che è zero
        dof_STORE                  <- rep(0.0, nlambda1)
        groups_active_STORE        <- matrix(data = 0, nrow = nlambda1, ncol = G)
        vars_active_STORE          <- matrix(data = 0, nrow = nlambda1, ncol = p)
        coeff_active_STORE         <- vector("list", nlambda1)
        length(coeff_active_STORE) <- nlambda1
      } else {
        # qui ho un solo lambda1 che non è zero
        dof_STORE               <- matrix(data = 0.0, nrow = nlambda1, ncol = nlambda2)
        groups_active_STORE     <- array(data = 0, dim = c(nlambda1, nlambda2, G))
        vars_active_STORE       <- array(data = 0, dim = c(nlambda1, nlambda2, p))
        coeff_active_STORE      <- vector("list", nlambda1 * nlambda2)
        dim(coeff_active_STORE) <- c(nlambda1, nlambda2)
      }
    } else {
      dof_STORE               <- matrix(data = 0.0, nrow = nlambda1, ncol = nlambda2)
      groups_active_STORE     <- array(data = 0, dim = c(nlambda1, nlambda2, G))
      vars_active_STORE       <- array(data = 0, dim = c(nlambda1, nlambda2, p))
      coeff_active_STORE      <- vector("list", nlambda1 * nlambda2)
      dim(coeff_active_STORE) <- c(nlambda1, nlambda2)
    }
  }
  
  # compute LS estimate for adaptive weights
  if (adaptive_weights == TRUE) {
    coeff_LS <- lm_ols(X = W, y = y)
  }
  
  # compute the degrees of freedom
  if ((!is.null(lambda2)) || (!is.null(diff_order))) {
    if (length(lambda2) == 1) {
      if (lambda2 == 0.0) {
        for (j in 1:nlambda1) {
          if (adaptive_weights == TRUE) {
            # compute degrees of freedom: adaptive weights
            res <- .f2s_adaptive_dof_1lambda(W = W,
                                             coeff = coeff[j,],
                                             coeff_LS = coeff_LS,
                                             lambda = lambda1[j],
                                             GRmat = GRmat,
                                             group_weights = group_weights,
                                             var_weights = var_weights,
                                             Uvec = Umat[j,],
                                             err_primal = err_primal[[j]][length(err_primal[[j]])],
                                             err_dual = err_dual[[j]][length(err_dual[[j]])],
                                             rho = rho[[j]][length(rho[[j]])],
                                             toler_c = toler_c,
                                             toler_d = toler_d)
          } else {
            # compute degrees of freedom: weights are not adaptive
            res <- .f2s_dof_1lambda(W = W,
                                    coeff = coeff[j,],
                                    lambda = lambda1[j],
                                    GRmat = GRmat,
                                    group_weights = group_weights,
                                    var_weights = var_weights,
                                    Uvec = Umat[j,],
                                    err_primal = err_primal[[j]][length(err_primal[[j]])],
                                    err_dual = err_dual[[j]][length(err_dual[[j]])],
                                    rho = rho[[j]][length(rho[[j]])],
                                    toler_c = toler_c,
                                    toler_d = toler_d)
          }
          
          # store output
          dof_STORE[j]            <- res$dof
          coeff_active_STORE[[j]] <- res$coeff_active
          groups_active_STORE[j,] <- as.numeric(res$groups_active)
          vars_active_STORE[j,]   <- as.numeric(res$var_active)
        }
      } else {
        for (j in 1:nlambda1) {
          for (k in 1:nlambda2) {
            if (adaptive_weights == TRUE) {
              # compute degrees of freedom: adaptive weights
              res <- .f2s_smo_adaptive_dof_1lambda(W = W, 
                                                   coeff = coeff[j,k,],
                                                   coeff_LS = coeff_LS,
                                                   lambda = lambda1[j], 
                                                   lambda2 = lambda2[k], 
                                                   diff_order = diff_order,
                                                   GRmat = GRmat,
                                                   group_weights = group_weights,
                                                   var_weights = var_weights,
                                                   Uvec = Umat[j,k,],
                                                   err_primal = err_primal[[j, k]][length(err_primal[[j, k]])], 
                                                   err_dual = err_dual[[j, k]][length(err_dual[[j, k]])], 
                                                   rho = rho[[j, k]][length(rho[[j, k]])],
                                                   toler_c = toler_c,
                                                   toler_d = toler_d)
            } else {
              res <- .f2s_smo_dof_1lambda(W = W, 
                                          coeff = coeff[j,k,],
                                          lambda = lambda1[j], 
                                          lambda2 = lambda2[k], 
                                          diff_order = diff_order,
                                          GRmat = GRmat,
                                          group_weights = group_weights,
                                          var_weights = var_weights,
                                          Uvec = Umat[j,k,],
                                          err_primal = err_primal[[j, k]][length(err_primal[[j, k]])], 
                                          err_dual = err_dual[[j, k]][length(err_dual[[j, k]])], 
                                          rho = rho[[j, k]][length(rho[[j, k]])],
                                          toler_c = toler_c,
                                          toler_d = toler_d)
            }
            
            # store output
            dof_STORE[j, k]            <- res$dof
            coeff_active_STORE[[j, k]] <- res$coeff_active
            groups_active_STORE[j, k,] <- as.numeric(res$groups_active)
            vars_active_STORE[j, k,]   <- as.numeric(res$var_active)
          }
        }
      }
    } else {
      for (j in 1:nlambda1) {
        for (k in 1:nlambda2) {
          if (adaptive_weights == TRUE) {
            # compute degrees of freedom: adaptive weights
            res <- .f2s_smo_adaptive_dof_1lambda(W = W, 
                                                 coeff = coeff[j,k,],
                                                 coeff_LS = coeff_LS,
                                                 lambda = lambda1[j], 
                                                 lambda2 = lambda2[k], 
                                                 diff_order = diff_order,
                                                 GRmat = GRmat,
                                                 group_weights = group_weights,
                                                 var_weights = var_weights,
                                                 Uvec = Umat[j,k,],
                                                 err_primal = err_primal[[j, k]][length(err_primal[[j, k]])], 
                                                 err_dual = err_dual[[j, k]][length(err_dual[[j, k]])], 
                                                 rho = rho[[j, k]][length(rho[[j, k]])],
                                                 toler_c = toler_c,
                                                 toler_d = toler_d)
          } else {
            res <- .f2s_smo_dof_1lambda(W = W, 
                                        coeff = coeff[j,k,],
                                        lambda = lambda1[j], 
                                        lambda2 = lambda2[k], 
                                        diff_order = diff_order,
                                        GRmat = GRmat,
                                        group_weights = group_weights,
                                        var_weights = var_weights,
                                        Uvec = Umat[j,k,],
                                        err_primal = err_primal[[j, k]][length(err_primal[[j, k]])], 
                                        err_dual = err_dual[[j, k]][length(err_dual[[j, k]])], 
                                        rho = rho[[j, k]][length(rho[[j, k]])],
                                        toler_c = toler_c,
                                        toler_d = toler_d)
          }
          
          # store output
          dof_STORE[j, k]            <- res$dof
          coeff_active_STORE[[j, k]] <- res$coeff_active
          groups_active_STORE[j, k,] <- as.numeric(res$groups_active)
          vars_active_STORE[j, k,]   <- as.numeric(res$var_active)
        }
      }
    }
  } else {
    cat("lambda2 is set to NULL!\n")
  }
  
  # get output
  output               <- NULL
  output$dof           <- dof_STORE
  output$coeff_active  <- coeff_active_STORE
  output$groups_active <- groups_active_STORE
  output$vars_active   <- vars_active_STORE
  
  # return output
  return(output)
}

#' @keywords internal
f2s_cov_dof <- function(W, 
                        Z,
                        y,
                        coeff_W,
                        coeff_Z,
                        coeff_W_LS,
                        lambda1,
                        lambda2 = NULL,
                        diff_order = 1,
                        adaptive_weights = TRUE,
                        GRmat,
                        group_weights,
                        var_weights,
                        Umat,
                        err_primal,
                        err_dual,
                        rho,
                        toler_c,
                        toler_d) {
  
  # get dimensions
  nlambda1 <- length(lambda1)
  nlambda2 <- length(lambda2)
  p        <- dim(W)[2]
  G        <- length(group_weights)
  
  # store output
  if (is.null(lambda2)) {
    # se lambda2 è NULL lo considero zero
    dof_STORE               <- rep(0.0, nlambda1)
    groups_active_STORE     <- matrix(data = 0, nrow = nlambda1, ncol = G)
    vars_active_STORE       <- matrix(data = 0, nrow = nlambda1, ncol = p)
    coeff_active_STORE      <- vector("list", nlambda1)
    dim(coeff_active_STORE) <- nlambda1
    lambda2                 <- 0
  } else {
    if (nlambda2 == 1) {
      if (lambda2 == 0) {
        # qui ho un solo lambda1 che è zero
        dof_STORE                  <- rep(0.0, nlambda1)
        groups_active_STORE        <- matrix(data = 0, nrow = nlambda1, ncol = G)
        vars_active_STORE          <- matrix(data = 0, nrow = nlambda1, ncol = p)
        coeff_active_STORE         <- vector("list", nlambda1)
        length(coeff_active_STORE) <- nlambda1
      } else {
        # qui ho un solo lambda1 che non è zero
        dof_STORE               <- matrix(data = 0.0, nrow = nlambda1, ncol = nlambda2)
        groups_active_STORE     <- array(data = 0, dim = c(nlambda1, nlambda2, G))
        vars_active_STORE       <- array(data = 0, dim = c(nlambda1, nlambda2, p))
        coeff_active_STORE      <- vector("list", nlambda1 * nlambda2)
        dim(coeff_active_STORE) <- c(nlambda1, nlambda2)
      }
    } else {
      dof_STORE               <- matrix(data = 0.0, nrow = nlambda1, ncol = nlambda2)
      groups_active_STORE     <- array(data = 0, dim = c(nlambda1, nlambda2, G))
      vars_active_STORE       <- array(data = 0, dim = c(nlambda1, nlambda2, p))
      coeff_active_STORE      <- vector("list", nlambda1 * nlambda2)
      dim(coeff_active_STORE) <- c(nlambda1, nlambda2)
    }
  }
  
  # compute LS estimate for adaptive weights
  if (adaptive_weights == TRUE) {
    out        <- lm_ols_FWL(X = W, Z = Z, y = y)
    coeff_W_LS <- out$coeff_X 
  }
  
  # compute the degrees of freedom
  if ((!is.null(lambda2)) || (!is.null(diff_order))) {
    if (length(lambda2) == 1) {
      if (lambda2 == 0.0) {
        for (j in 1:nlambda1) {
          if (adaptive_weights == TRUE) {
            res <- .f2s_cov_adaptive_dof_1lambda(W = W, 
                                                 Z = Z, 
                                                 coeff_W = coeff_W[j,],
                                                 coeff_W_LS = coeff_W_LS,
                                                 lambda = lambda1[j], 
                                                 GRmat = GRmat,
                                                 group_weights = group_weights,
                                                 var_weights = var_weights,
                                                 Uvec = Umat[j,],
                                                 err_primal = err_primal[[j]][length(err_primal[[j]])],
                                                 err_dual = err_dual[[j]][length(err_dual[[j]])],
                                                 rho = rho[[j]][length(rho[[j]])],
                                                 toler_c = toler_c,
                                                 toler_d = toler_d)
          } else {
            res <- .f2s_cov_dof_1lambda(W = W, 
                                        Z = Z, 
                                        coeff_W = coeff_W[j,],
                                        lambda = lambda1[j], 
                                        GRmat = GRmat,
                                        group_weights = group_weights,
                                        var_weights = var_weights,
                                        Uvec = Umat[j,],
                                        err_primal = err_primal[[j]][length(err_primal[[j]])],
                                        err_dual = err_dual[[j]][length(err_dual[[j]])],
                                        rho = rho[[j]][length(rho[[j]])],
                                        toler_c = toler_c,
                                        toler_d = toler_d)
          }
          
          # store output
          dof_STORE[j]            <- res$dof
          coeff_active_STORE[[j]] <- res$coeff_active
          groups_active_STORE[j,] <- as.numeric(res$groups_active)
          vars_active_STORE[j,]   <- as.numeric(res$var_active)
        }
      } else {
        for (j in 1:nlambda1) {
          for (k in 1:nlambda2) {
            if (adaptive_weights == TRUE) {
              res <- .f2s_adaptive_cov_smo_dof_1lambda(W = W, 
                                                       Z = Z, 
                                                       coeff_W = coeff_W[j,k,],
                                                       coeff_W_LS = coeff_W_LS,
                                                       lambda = lambda1[j], 
                                                       lambda2 = lambda2[k], 
                                                       diff_order = diff_order,
                                                       GRmat = GRmat,
                                                       group_weights = group_weights,
                                                       var_weights = var_weights,
                                                       Uvec = Umat[j,k,],
                                                       err_primal = err_primal[[j, k]][length(err_primal[[j, k]])], 
                                                       err_dual = err_dual[[j, k]][length(err_dual[[j, k]])], 
                                                       rho = rho[[j, k]][length(rho[[j, k]])],
                                                       toler_c = toler_c,
                                                       toler_d = toler_d)
            } else {
              res <- .f2s_cov_smo_dof_1lambda(W = W, 
                                              Z = Z, 
                                              coeff_W = coeff_W[j,k,],
                                              lambda = lambda1[j], 
                                              lambda2 = lambda2[k], 
                                              diff_order = diff_order,
                                              GRmat = GRmat,
                                              group_weights = group_weights,
                                              var_weights = var_weights,
                                              Uvec = Umat[j,k,],
                                              err_primal = err_primal[[j, k]][length(err_primal[[j, k]])], 
                                              err_dual = err_dual[[j, k]][length(err_dual[[j, k]])], 
                                              rho = rho[[j, k]][length(rho[[j, k]])],
                                              toler_c = toler_c,
                                              toler_d = toler_d)
            }
            
            # store output
            dof_STORE[j, k]            <- res$dof
            coeff_active_STORE[[j, k]] <- res$coeff_active
            groups_active_STORE[j, k,] <- as.numeric(res$groups_active)
            vars_active_STORE[j, k,]   <- as.numeric(res$var_active)
          }
        }
      }
    } else {
      for (j in 1:nlambda1) {
        for (k in 1:nlambda2) {
          if (adaptive_weights == TRUE) {
            res <- .f2s_adaptive_cov_smo_dof_1lambda(W = W, 
                                                     Z = Z, 
                                                     coeff_W = coeff_W[j,k,],
                                                     coeff_W_LS = coeff_W_LS,
                                                     lambda = lambda1[j], 
                                                     lambda2 = lambda2[k], 
                                                     diff_order = diff_order,
                                                     GRmat = GRmat,
                                                     group_weights = group_weights,
                                                     var_weights = var_weights,
                                                     Uvec = Umat[j,k,],
                                                     err_primal = err_primal[[j, k]][length(err_primal[[j, k]])], 
                                                     err_dual = err_dual[[j, k]][length(err_dual[[j, k]])], 
                                                     rho = rho[[j, k]][length(rho[[j, k]])],
                                                     toler_c = toler_c,
                                                     toler_d = toler_d)
          } else {
            res <- .f2s_cov_smo_dof_1lambda(W = W, 
                                            Z = Z, 
                                            coeff_W = coeff_W[j,k,],
                                            lambda = lambda1[j], 
                                            lambda2 = lambda2[k], 
                                            diff_order = diff_order,
                                            GRmat = GRmat,
                                            group_weights = group_weights,
                                            var_weights = var_weights,
                                            Uvec = Umat[j,k,],
                                            err_primal = err_primal[[j, k]][length(err_primal[[j, k]])], 
                                            err_dual = err_dual[[j, k]][length(err_dual[[j, k]])], 
                                            rho = rho[[j, k]][length(rho[[j, k]])],
                                            toler_c = toler_c,
                                            toler_d = toler_d)
          }
          
          # store output
          dof_STORE[j, k]            <- res$dof
          coeff_active_STORE[[j, k]] <- res$coeff_active
          groups_active_STORE[j, k,] <- as.numeric(res$groups_active)
          vars_active_STORE[j, k,]   <- as.numeric(res$var_active)
        }
      }
    }
  } else {
    cat("lambda2 is set to NULL!\n")
  }
  
  # get output
  output               <- NULL
  output$dof           <- dof_STORE
  output$coeff_active  <- coeff_active_STORE
  output$groups_active <- groups_active_STORE
  output$vars_active   <- vars_active_STORE

  # return output
  return(output)
}

# Authors: Mauro Bernardi 
#          Department Statistical Sciences
#       	 University of Padova
#      	   Via Cesare Battisti, 241
#      	   35121 PADOVA, Italy
#          E-mail: mauro.bernardi@unipd.it  

# Last change: October 21, 2025


#' Overlap Group Least Absolute Shrinkage and Selection Operator
#'
#' Overlap group-LASSO, or overlap group L2-regularized regression, solves the following optimization problem
#' \deqn{\textrm{min}_\beta ~ \frac{1}{2}\|y-X\beta-Z\gamma\|_2^2 + \lambda\sum_{g=1}^G \|S_g T\beta\|_2}
#' to obtain a sparse coefficient vector \eqn{\beta\in\mathbb{R}^p} and a coefficient vector \eqn{\gamma\in\mathbb{R}^q}. For each group, each row of 
#' the matrix \eqn{S_g\in\mathbb{R}^{n_g\times p}} has non-zero entries only for those variables belonging 
#' to that group. These values are provided by the arguments \code{groups} and \code{group_weights} (see below). 
#' Each variable can belong to more than one group. The diagonal matrix \eqn{T\in\mathbb{R}^{p\times p}} contains 
#' the variable-specific weights. These values are provided by the argument \code{var_weights} (see below).
#' The regularization path is computed for the overlap group-LASSO penalty at a grid of values for the regularization 
#' parameter \eqn{\lambda} using the alternating direction method of multipliers (ADMM). See Boyd et al. (2011) 
#' for details on the ADMM method.
#'  
#' @param X an \eqn{(n\times p)} matrix of penalized predictors.
#' @param Z an \eqn{(n\times q)} full column rank matrix of predictors that are not penalized. 
#' @param y a length-\eqn{n} response vector.
#' @param groups a list with two elements: the first element is a vector containing the variables belonging to each group, 
#' while the second element is a vector containing the group lengths (see example below).
#' @param group_weights a vector of length G containing group-specific weights. The default is square root of the group cardinality, see Bernardi et al. (2022).
#' @param var_weights a vector of length p containing variable-specific weights. The default is a vector where 
#' each entry is the reciprocal of the number of groups including that variable.
#' @param standardize.data logical. Should data be standardized?
#' @param lambda either a regularization parameter or a vector of regularization parameters. 
#' In this latter case the routine computes the whole path. If it is NULL the path values for lambda are provided by the routine.
#' @param lambda.min.ratio smallest value for lambda, as a fraction of the maximum lambda value, the (data derived) entry value (i.e. the smallest value for which all coefficients are zero). 
#' The default depends on the sample size nobs relative to the number of variables nvars. If nobs > nvars, the default is 0.0001, 
#' close to zero. If nobs < nvars, the default is 0.01. A very small value of lambda.min.ratio will lead to a saturated fit 
#' in the nobs < nvars case.
#' @param nlambda the number of lambda values - default is 30.
#' @param intercept logical. If it is TRUE, a column of ones is added to the design matrix. 
#' @param overall.group logical. If it is TRUE, an overall group including all penalized covariates is added.
#' @param control	 a list of control parameters. See ‘Details’.
#'
#' @return A named list containing \describe{
#' \item{sp.coefficients}{a length-\eqn{p} solution vector for the parameters \eqn{\beta}, which corresponds to the minimum in-sample MSE.}
#' \item{coefficients}{a length-\eqn{q} solution vector for the parameters \eqn{\gamma}, which corresponds to the minimum in-sample MSE. 
#' It is provided only when either the matrix \eqn{Z} in input is not NULL or the intercept is set to TRUE.}
#' \item{sp.coef.path}{an \eqn{(nlambda\times p)} matrix of estimated \eqn{\beta} coefficients for each lambda of the provided sequence.}
#' \item{coef.path}{an \eqn{(nlambda\times q)} matrix of estimated \eqn{\gamma} coefficients for each lambda of the provided sequence. 
#' It is provided only when either the matrix \eqn{Z} in input is not NULL or the intercept is set to TRUE.}
#' \item{lambda}{sequence of lambda.}
#' \item{lambda.min}{value of lambda that attains the in-sample minimum in-sample MSE.}
#' \item{mse}{in-sample mean squared error.}
#' \item{min.mse}{minimum value of the in-sample MSE for the sequence of lambdas.}
#' \item{convergence}{logical. 1 denotes achieved convergence.}
#' \item{elapsedTime}{elapsed time in seconds.}
#' \item{iternum}{number of iterations.}
#' }
#'
#' When you run the algorithm, output returns not only the solution, but also the iteration history recording
#' following fields over iterates:
#' \describe{
#' \item{objval}{objective function value}
#' \item{r_norm}{norm of primal residual}
#' \item{s_norm}{norm of dual residual}
#' \item{eps_pri}{feasibility tolerance for primal feasibility condition}
#' \item{eps_dual}{feasibility tolerance for dual feasibility condition.}
#' }
#' Iteration stops when both \code{r_norm} and \code{s_norm} values
#' become smaller than \code{eps_pri} and \code{eps_dual}, respectively.
#' @section Details: The control argument is a list that can supply any of the following components:\describe{
#' (To understand exactly what these do see the source code: higher levels give more detail.).
#' \item{adaptation}{logical. If it is TRUE, ADMM with adaptation is performed. The default value is TRUE.}
#' \item{rho}{an augmented Lagrangian parameter. The default value is 1.}
#' \item{tau.ada}{an adaptation parameter in [1,2]. Only needed if adaptation = TRUE. The default value is 2. See Boyd et al. (2011) for details.}
#' \item{mu.ada}{an adaptation parameter. Only needed if adaptation = TRUE. The default value is 10. See Boyd et al. (2011) for details.}
#' \item{abstol}{absolute tolerance stopping criterion. The default value is sqrt(sqrt(.Machine$double.eps)).}
#' \item{reltol}{relative tolerance stopping criterion. The default value is sqrt(.Machine$double.eps).}
#' \item{maxit}{maximum number of iterations. The default value is 100.}
#' \item{print.out}{logical. If it is TRUE, a message about the procedure is printed. The default value is TRUE.}
#' }
#' 
#' @examples 
#' \donttest{
#' 
#' ### generate sample data
#' set.seed(2023)
#' n    <- 50
#' p    <- 30 
#' X    <- matrix(rnorm(n*p), n, p)
#' beta <- rep(c(rep(4, 4), rep(0, 3), rep(-4, 3)), 3)
#' y    <- X %*% beta + rnorm(n, sd = sqrt(crossprod(X %*% beta )) / 20)
#' 
#' ## define overlapping groups of dimension 3 each 
#' group1 <- NULL
#' for(k in 0:(p - 3)) group1 <- c(group1, 1:3 + k)
#' Glen <- rep(3, p - 2)
#' 
#' ### set regularization parameter grid
#' lam   <- 10^seq(1, -2, length.out = 30)
#' 
#' ### set the hyper-parameters of the ADMM algorithm
#' maxit         <- 1000
#' adaptation    <- TRUE
#' rho           <- 1
#' reltol        <- 1e-5
#' abstol        <- 1e-5
#' 
#' ### run example
#' mod <- fdaSP:::linreg_ADMM_OVGLASSO(X = X, y = y, groups = list("groups" = group1, "Glen" = Glen),
#'                                     standardize.data = FALSE,  intercept = FALSE, lambda = lam,
#'                                     control = list("adaptation" = adaptation, "rho" = rho, 
#'                                                    "maxit" = maxit, "abstol" = abstol, "reltol" = reltol, 
#'                                                    "print.out" = FALSE))
#' 
#' ### graphical presentation
#' matplot(log(lam), mod$sp.coef.path, type = "l", main = "Overlap Group Lasso solution path", 
#'         bty = "n", xlab = TeX("$\\log(\\lambda)$"), ylab = "")
#' }
#' 
#' @references
#' \insertRef{bernardi_etal.2022}{fdaSP}
#' 
#' \insertRef{jenatton_etal.2011}{fdaSP}
#' 
#' \insertRef{boyd_etal.2011}{fdaSP}
#' 
#' \insertRef{lin_etal.2022}{fdaSP}
#'
#' @noRd
linreg_ADMM_OVGLASSO <- function(X, Z = NULL, y, groups, group_weights = NULL, var_weights = NULL, standardize.data = TRUE,
                                 lambda = NULL, lambda.min.ratio = NULL, nlambda = 30, 
                                 intercept = FALSE, overall.group = FALSE, control = list()) {
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # add constant
  if (standardize.data == TRUE) {
    if (intercept == TRUE){
      intercept = FALSE
      warning("* linreg_ADMM_OVGLASSO : intercept should be set to FALSE is data are standardized!\n")
    }
  }
  if (intercept == TRUE) {
    if (!is.null(Z)) {
      Z <- cbind(matrix(data = 1, nrow = length(y), ncol = 1), Z)
    } else {
      Z <- matrix(data = 1, nrow = length(y), ncol = 1)
    }
  }

  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Pre-processing: data validity
  if (!check_data_matrix(X)) {
    stop("* linreg_ADMM_OVGLASSO : input 'X' is invalid data matrix.")  
  }
  if (!check_data_vector(y)) {
    stop("* linreg_ADMM_OVGLASSO : input 'y' is invalid data vector.")  
  }
  y <- as.vector(y)
  # data size
  if (dim(X)[1] != length(y)) {
    stop("* linreg_ADMM_OVGLASSO : two inputs 'X' and 'Y' have non-matching dimension.")
  }
  if (!is.null(Z)) {
    if (dim(Z)[1] != length(y)) {
      stop("* linreg_ADMM_OVGLASSO : two inputs 'Z' and 'Y' have non-matching dimension.")
    }
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # get data dimensions
  n <- dim(X)[1]
  p <- dim(X)[2]  
  if (!is.null(Z)) {
    q <- dim(Z)[2]
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Standardise response and design matrix
  if (standardize.data == TRUE) {
    res   <- standardizemat(X, y)
    X.std <- res$X.std
    y.std <- res$y.std
    mU    <- res$mU
    mV    <- res$mV
  } else {
    X.std <- X
    y.std <- y
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Pre-processing: lambda parameter (in case of a single lambda)
  if (!is.null(lambda)) {
    if (length(lambda) == 1) {
      meps     <- (.Machine$double.eps)
      negsmall <- -meps
      if (!check_param_constant(lambda, negsmall)) {
        stop("* linreg_ADMM_OVGLASSO : reg. parameter 'lambda' is invalid.")
      }
      if (lambda < meps){
        message("* linreg_ADMM_OVGLASSO : since 'lambda' is effectively zero, a least-squares solution is returned.")
        xsol     <- as.vector(aux_pinv(X) %*% matrix(y))
        output   <- list()
        output$x <- xsol
        return(output)
      }
    }
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # check for inputs
  npar   <- dim(X.std)[2]
  con    <- list(maxit                = 100L, 
                 abstol               = sqrt(.Machine$double.eps), 
                 reltol               = sqrt(sqrt(.Machine$double.eps)), 
                 adaptation           = TRUE, 
                 rho                  = 1, 
                 tau.ada              = 2,               
                 mu.ada               = 10,
                 dof.toler_c          = 5,
                 dof.toler_d          = 0.001,
                 print.out            = TRUE)
  
  nmsC                          <- names(con)
  con[(namc <- names(control))] <- control
  if (length(noNms <- namc[!namc %in% nmsC])) {
    warning("unknown names in control: ", paste(noNms, collapse = ", "))
  }
  # set default values (for inputs)
  maxit      <- con$maxit
  abstol     <- con$abstol
  reltol     <- con$reltol
  adaptation <- con$adaptation
  rho        <- con$rho
  tau.ada    <- con$tau.ada      
  mu.ada     <- con$mu.ada
  toler_c    <- con$dof.toler_c
  toler_d    <- con$dof.toler_d
  print.out  <- con$print.out
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Pre-processing: other parameters
  if (!check_param_constant_multiple(c(abstol, reltol))) {
    stop("* linreg_ADMM_OVGLASSO : tolerance level is invalid.")
  }
  if (!check_param_integer(maxit, 0.0)) {
    stop("* linreg_ADMM_OVGLASSO : 'maxiter' should be a positive integer.")
  }
  maxit = as.integer(maxit)
  if (!check_param_constant(rho, 0.0)) {
    stop("* linreg_ADMM_OVGLASSO : 'rho' should be a positive real number.")
  }
  if (!check_param_constant(mu.ada, 0.0)) {
    stop("* linreg_ADMM_OVGLASSO : 'mu.ada' should be a positive real number.")
  }
  if ((tau.ada < 1.0) || (tau.ada > 2.0)) {
    warning("* linreg_ADMM_OVGLASSO : 'tau.ada' value is suggested to be in [1,2].")
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Pre-processing: variable weights
  check_weights(x = var_weights, n = dim(X)[2], algname = "linreg_ADMM_OVGLASSO", funname = "var_weights")
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Manage groups
  if (is.list(groups) && !is.null(groups$Glen)) {
    nG      <- groups$Glen
    groups_ <- groups$groups
    GRmat   <- groups2mat_OVGLASSO(groups = groups_, Glen = nG)
    G       <- dim(GRmat)[1]
  } else {
    groups_ <- groups
    GRmat   <- groups2mat_OVGLASSO(groups = groups_, Glen = NULL)
    G       <- dim(GRmat)[1]
    nG      <- rowSums(GRmat)
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Check overlapping groups
  res.method <- check_group_overlaps(GRmat)
  if (res.method != "ovglasso") {
    warning("* linreg_ADMM_OVGLASSO : this routine is for fitting the linear regression model with overlap group-LASSO penalty. 
            Use (faster) routine 'linreg_ADMM_GLASSO' instead or check the provided groups!")
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Manage overall group
  if ((overall.group == TRUE) && (!any(nG == dim(X)[2]))) {
    GRmat <- rbind(GRmat, rep(1, dim(GRmat)[2]))
    G     <- G + 1
    nG    <- c(nG, dim(GRmat)[2])
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Manage group & overall group weights
  if (is.null(group_weights)) {
    # Define the weights as in Yuan and Lin 2006, JRSSB
    group_weights <- sqrt(diag(tcrossprod(GRmat)))
  } else {
    if (overall.group == TRUE) {
      check_group_weights(x = group_weights, n = G, algname = "linreg_ADMM_OVGLASSO", funname = "group_weights")
      if (length(group_weights) == (G-1)) {
        group_weights <- c(group_weights, sqrt(dim(GRmat)[2]))
      }
    } else {
      check_weights(x = group_weights, n = G, algname = "linreg_ADMM_OVGLASSO", funname = "group_weights")
    }
  } 

  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Manage variable weights
  if (is.null(var_weights)) {
    var_weights <- 1.0 / colSums(GRmat)
  } 
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get the sequence of lambdas
  if (!is.null(nlambda) && is.null(lambda)) {
    # get the smallest value of such that the regression 
    # coefficients estimated by the lasso are all equal to zero
    lambda <- lm_lambdamax_OVGLASSO(y = y.std, X = X.std, Z = Z, 
                                    GRmat = GRmat, group_weights = group_weights, var_weights = var_weights, 
                                    lambda.min.ratio = lambda.min.ratio, maxl = nlambda)$lambda.seq
  }
  if (is.null(nlambda) && is.null(lambda)) {
    # get the smallest value of such that the regression 
    # coefficients estimated by the lasso are all equal to zero
    lambda <- lm_lambdamax_OVGLASSO(y = y.std, X = X.std, Z = Z, 
                                    GRmat = GRmat, group_weights = group_weights, var_weights = var_weights, 
                                    lambda.min.ratio = lambda.min.ratio, maxl = 30)$lambda.seq
  }
  if (!is.null(lambda)) {
    nlambda <- length(lambda)
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Run overlap group Lasso
  if (is.null(Z)) {
    ret <- .admm_ovglasso_fast(A = X.std, b = y.std, groups = GRmat, group_weights = group_weights, var_weights = var_weights,
                               lambda = lambda, rho_adaptation = adaptation, rho = rho, tau = tau.ada, mu = mu.ada,
                               reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
                               
    # get estimated coefficients and path
    mSpRegP <- t(ret$coef.path)
    vSpRegP <- ret$coefficients
  } else {
    ret <- .admm_ovglasso_cov_fast(W = X.std, Z = Z, y = y.std,
                                   groups = GRmat, group_weights = group_weights, var_weights = var_weights, 
                                   lambda = lambda, rho_adaptation = adaptation, rho = rho, tau = tau.ada, mu.ada,
                                   reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
                                   
    # get estimated coefficients and path
    mSpRegP <- t(ret$sp.coef.path)
    mRegP   <- t(ret$coef.path)
    vSpRegP <- ret$sp.coefficients
    vRegP   <- ret$coefficients
  }

  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get the path and retrieve the scaled estimates
  if (standardize.data == TRUE) {
    sp.path <- matrix(nlambda, p, data = t(apply(mSpRegP, 2, function(x) solve(mU) %*% x %*% mV)))
    vSpRegP <- solve(mU) %*% vSpRegP %*% mV
    if (!is.null(Z)) {
      path  <- matrix(nlambda, q, data = t(apply(mRegP, 2, function(x) x %*% mV)))
      vRegP <- vRegP %*% mV
    }
  } else {
    vSpRegP <- matrix(vSpRegP, length(vSpRegP), 1)
    sp.path <- t(mSpRegP)
    if (!is.null(Z)) {
      path <- t(mRegP)
    }
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Print to screen
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  if (print.out == TRUE) {
    cat("\n\n\n")
    cat("Linear regression model with overlap group-LASSO penalty\n")
    cat("Alternating direction method of multipliers\n")
    cat("running time (for ", reltol, " relative error):",
        sum(ret$elapsedTime), "seconds \n\n\n")
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get output
  res.names <- c("sp.coefficients", 
                 "sp.coef.path", 
                 "coefficients",
                 "coef.path",
                 "lambda.min",
                 "lambda",
                 "mse",
                 "min.mse",
                 "convergence",
                 "elapsedTime",
                 "iternum",
                 "objfun",
                 "r_norm",
                 "s_norm",
                 "err_pri",
                 "err_dual",
                 "rho")
  res        <- vector(mode = "list", length = length(res.names))
  names(res) <- res.names
  if (!is.null(Z)) {
    res$coefficients <- vRegP
    res$coef.path    <- path
  } 
  res$sp.coefficients   <- vSpRegP
  res$sp.coef.path      <- sp.path
  res$lambda.min        <- ret$lambda.min
  res$lambda            <- ret$lambda
  res$mse               <- ret$mse
  res$min.mse           <- ret$min.mse
  res$convergence       <- ret$convergence
  res$elapsedTime       <- ret$elapsedTime
  res$iternum           <- ret$iternum
  res$objfun            <- ret$objfun
  res$r_norm            <- ret$r_norm
  res$s_norm            <- ret$s_norm
  res$err_pri           <- ret$err_pri
  res$err_dual          <- ret$err_dual
  res$rho               <- ret$rho
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Return output
  return(res)
}

# Authors: Mauro Bernardi 
#          Department Statistical Sciences
#       	 University of Padova
#      	   Via Cesare Battisti, 241
#      	   35121 PADOVA, Italy
#          E-mail: mauro.bernardi@unipd.it  

# Last change: March 16, 2023

#' Cross-validation for Least Absolute Shrinkage and Selection Operator
#'
#' LASSO, or L1-regularized regression, solves the following optimization problem
#' \deqn{\textrm{min}_\beta ~ \frac{1}{2}\|y-X\beta-Z\gamma\|_2^2 + \lambda \|\beta\|_1}
#' to obtain a sparse coefficient vector \eqn{\beta\in\mathbb{R}^p} and a coefficient vector \eqn{\gamma\in\mathbb{R}^q}. The regularization path 
#' is computed for the LASSO penalty at a grid of values for the regularization parameter \eqn{\lambda} using the 
#' alternating direction method of multipliers (ADMM). See Boyd et al. (2011) for details on the ADMM method.
#'
#' @param X an \eqn{(n\times p)} matrix of penalized predictors.
#' @param Z an \eqn{(n\times q)} full column rank matrix of predictors that are not penalized. 
#' @param y a length-\eqn{n} response vector.
#' @param var_weights a vector of length \eqn{p} containing variable-specific weights. The default is a vector of ones.
#' @param standardize.data logical. Should data be standardized?
#' @param lambda either a regularization parameter or a vector of regularization parameters. 
#' In this latter case the routine computes the whole path. If it is NULL the path values for lambda are provided by the routine.
#' @param lambda.min.ratio smallest value for lambda, as a fraction of the maximum lambda value, the (data derived) entry value 
#' (i.e. the smallest value for which all coefficients are zero). 
#' The default depends on the sample size nobs relative to the number of variables nvars. If nobs > nvars, the default is 0.0001, 
#' close to zero. If nobs < nvars, the default is 0.01. A very small value of lambda.min.ratio will lead to a saturated fit 
#' in the nobs < nvars case.
#' @param nlambda the number of lambda values - default is 30.
#' @param cv.fold the number of folds - default is 5.
#' @param intercept logical. If it is TRUE, a column of ones is added to the design matrix. 
#' @param control	 a list of control parameters. See ‘Details’.
#'
#' @return A named list containing \describe{
#' \item{sp.coefficients}{a length-\eqn{p} solution vector for the parameters \eqn{\beta}, which corresponds to the model fitted on the full sample with \eqn{\lambda} corresponding to minimum cross-validated MSE.}
#' \item{coefficients}{a length-\eqn{q} solution vector for the parameters \eqn{\gamma}, which corresponds to the model fitted on the full sample with \eqn{\lambda} corresponding to minimum cross-validated MSE. 
#' It is provided only when either the matrix \eqn{Z} in input is not NULL or the intercept is set to TRUE.}
#' \item{convergence}{logical. 1 denotes achieved convergence for the model fitted on the full sample.}
#' \item{iternum}{number of iterations for the model fitted on the full sample.}
#' \item{lambda}{sequence of lambda.}
#' \item{lambda.min}{value of lambda that attains the minimum cross-validated MSE.}
#' \item{indi.min.mse}{index of the lambda sequence corresponding to lambda.min.}
#' \item{mse}{cross-validated mean squared error.}
#' \item{min.mse}{minimum value of the cross-validated MSE for the sequence of lambdas.}
#' \item{mse.sd}{standard deviation of the cross-validated mean squared error.}
#' \item{elapsedTime}{elapsed time in seconds for the whole procedure.}
#' \item{foldid}{a vector of values between 1 and cv.fold identifying what fold each observation is in.}
#' }
#'
#' Iteration stops when both \code{r_norm} and \code{s_norm} values
#' become smaller than \code{eps_pri} and \code{eps_dual}, respectively.
#' @section Details: The control argument is a list that can supply any of the following components:\describe{
#' (To understand exactly what these do see the source code: higher levels give more detail.).
#' \item{adaptation}{logical. If it is TRUE, ADMM with adaptation is performed. The default value is TRUE.}
#' \item{rho}{an augmented Lagrangian parameter. The default value is 1.}
#' \item{tau.ada}{an adaptation parameter in [1,2]. Only needed if adaptation = TRUE. The default value is 2. See Boyd et al. (2011) for details.}
#' \item{mu.ada}{an adaptation parameter. Only needed if adaptation = TRUE. The default value is 10. See Boyd et al. (2011) for details.}
#' \item{abstol}{absolute tolerance stopping criterion. The default value is sqrt(sqrt(.Machine$double.eps)).}
#' \item{reltol}{relative tolerance stopping criterion. The default value is sqrt(.Machine$double.eps).}
#' \item{maxit}{maximum number of iterations. The default value is 100.}
#' \item{print.out}{logical. If it is TRUE, a message about the procedure is printed. The default value is TRUE.}
#' }
#' @examples
#' 
#' ### generate sample data
#' set.seed(2023)
#' n    <- 50
#' p    <- 30 
#' X    <- matrix(rnorm(n * p), n, p)
#' beta <- apply(matrix(rnorm(p, sd = 1), p, 1), 1, fdaSP:::softhresh, 1.5)
#' y    <- X %*% beta + rnorm(n, sd = sqrt(crossprod(X %*% beta)) / 20)
#' 
#' ### set the hyper-parameters of the ADMM algorithm
#' maxit      <- 1000
#' adaptation <- TRUE
#' rho        <- 1
#' reltol     <- 1e-5
#' abstol     <- 1e-5
#' 
#' ### run cross-validation
#' mod_cv <- fdaSP:::linreg_ADMM_LASSO_cv(X = X, y = y, standardize.data = FALSE, intercept = FALSE,
#'                                        cv.fold = 5, nlambda = 30, 
#'                                        control = list("adaptation" = adaptation, "rho" = rho, 
#'                                                       "maxit" = maxit, "reltol" = reltol, "abstol" = abstol, 
#'                                                       "print.out" = FALSE)) 
#' 
#' ### graphical presentation
#' plot(log(mod_cv$lambda), mod_cv$mse, type = "l", col = "blue", lwd = 2, bty = "n", 
#'      xlab = TeX("$\\log(\\lambda)$"), ylab = "Prediction Error", 
#'      ylim = range(mod_cv$mse - mod_cv$mse.sd, mod_cv$mse + mod_cv$mse.sd),
#'      main = "Cross-validated Prediction Error")
#' fdaSP:::confband(xV = log(mod_cv$lambda), yVmin = mod_cv$mse - mod_cv$mse.sd, yVmax = mod_cv$mse + mod_cv$mse.sd)       
#' abline(v = log(mod_cv$lambda[which(mod_cv$lambda == mod_cv$lambda.min)]), col = "red", lwd = 1.0)
#' 
#' ### comparison with oracle error
#' mod <- fdaSP:::linreg_ADMM_LASSO(X = X, y = y, standardize.data = FALSE, intercept = FALSE,
#'                                  nlambda = 30, control = list("adaptation" = adaptation, "rho" = rho, 
#'                                                               "maxit" = maxit, "reltol" = reltol, "abstol" = abstol, 
#'                                                               "print.out" = FALSE)) 
#' err_mod <- apply(mod$sp.coef.path, 1, function(x) sum((x - beta)^2))
#' plot(log(mod$lambda), err_mod, type = "l", col = "blue", lwd = 2, xlab = TeX("$\\log(\\lambda)$"), 
#'      ylab = "Estimation Error", main = "True Estimation Error", bty = "n")
#' abline(v = log(mod$lambda[which(err_mod == min(err_mod))]), col = "red", lwd = 1.0)
#' abline(v = log(mod_cv$lambda[which(mod_cv$lambda == mod_cv$lambda.min)]), col = "red", lwd = 1.0, lty = 2)
#'
#' @references
#' \insertRef{tibshirani_regression_1996a}{fdaSP}
#' 
#' \insertRef{boyd_etal.2011}{fdaSP}
#' 
#' \insertRef{hastie_etal.2015}{fdaSP}
#' 
#' \insertRef{lin_etal.2022}{fdaSP} 
#' 
#' \insertRef{zou.2006}{fdaSP} 
#'
#' @noRd
linreg_ADMM_LASSO_cv <- function(X, Z = NULL, y, var_weights = NULL, standardize.data = TRUE, lambda = NULL, nlambda = NULL, lambda.min.ratio = NULL, 
                                 cv.fold = 5, intercept = TRUE, control = list()) {
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get dimensions
  n <- dim(X)[1]
  p <- dim(X)[2]  
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Pre-processing
  # data validity
  if (!check_data_matrix(X)) {
    stop("* linreg_ADMM_LASSO_cv : input 'X' is invalid data matrix.")  
  }
  if (!check_data_vector(y)) {
    stop("* linreg_ADMM_LASSO_cv : input 'y' is invalid data vector.")  
  }
  y = as.vector(y)
  # data size
  if (n != length(y)) {
    stop("* linreg_ADMM_LASSO_cv : two inputs 'X' and 'y' have non-matching dimension.")
  }
  
  if (!is.null(var_weights)) {
    if (!check_data_positive_vector(var_weights)) {
      stop("* linreg_ADMM_LASSO : input 'var_weights' is invalid vector of weights.")  
    } else {
      adalasso <- TRUE
    }
  } else {
    adalasso    <- FALSE
    var_weights <- rep(1L, p)
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get dimensions
  if (standardize.data == TRUE) {
    if (intercept == TRUE){
      intercept = FALSE
      warning("* linreg_ADMM_LASSO_cv : intercept should be set to FALSE is data are standardized!\n")
    }
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # add constant
  if (intercept == TRUE) {
    if (!is.null(Z)) {
      Z <- cbind(matrix(data = 1, nrow = length(y), ncol = 1), Z)
    } else {
      Z <- matrix(data = 1, nrow = length(y), ncol = 1)
    }
  }

  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Standardise response and design matrix
  if (standardize.data == TRUE) {
    res   <- standardizemat(X, y)
    X.std <- res$X.std
    y.std <- res$y.std
    mU    <- res$mU
    mV    <- res$mV
  } else {
    X.std <- X
    y.std <- y
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # check for inputs
  npar   <- dim(X.std)[2]
  con    <- list(maxit                = 100L, 
                 abstol               = sqrt(.Machine$double.eps), 
                 reltol               = sqrt(sqrt(.Machine$double.eps)), 
                 adaptation           = TRUE, 
                 rho                  = 1, 
                 tau.ada              = 2,               
                 mu.ada               = 10,
                 print.out            = TRUE)
  
  nmsC                          <- names(con)
  con[(namc <- names(control))] <- control
  if (length(noNms <- namc[!namc %in% nmsC])) {
    warning("unknown names in control: ", paste(noNms, collapse = ", "))
  }
  # set default values (for inputs)
  maxit                <- con$maxit
  abstol               <- con$abstol
  reltol               <- con$reltol
  adaptation           <- con$adaptation
  rho                  <- con$rho
  tau.ada              <- con$tau.ada      
  mu.ada               <- con$mu.ada
  print.out            <- con$print.out
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # data validity
  if (!check_param_constant_multiple(c(abstol, reltol))) {
    stop("* linreg_ADMM_LASSO_cv : tolerance level is invalid.")
  }
  if (!check_param_integer(maxit, 0.0)) {
    stop("* linreg_ADMM_LASSO_cv : 'maxiter' should be a positive integer.")
  }
  maxiter = as.integer(maxit)
  if (!check_param_constant(rho, 0.0)) {
    stop("* linreg_ADMM_LASSO_cv : 'rho' should be a positive real number.")
  }
  if (!check_param_constant(mu.ada, 0.0)) {
    stop("* linreg_ADMM_LASSO_cv : 'alpha' should be a positive real number.")
  }
  if ((tau.ada < 1.0) || (tau.ada > 2.0)) {
    warning("* linreg_ADMM_LASSO_cv : 'tau' value is suggested to be in [1,2].")
  }

  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get the sequence of lambdas
  if (!is.null(nlambda) && is.null(lambda)) {
    # get the smallest value of such that the regression 
    # coefficients estimated by the lasso are all equal to zero
    lambda <- lm_lambdamax_ADALASSO(y = y.std, X = X.std, Z = Z, var_weights = var_weights, lambda.min.ratio = lambda.min.ratio, maxl = nlambda)$lambda.seq
  }
  if (is.null(nlambda) && is.null(lambda)) {
    # get the smallest value of such that the regression 
    # coefficients estimated by the lasso are all equal to zero
    lambda <- lm_lambdamax_ADALASSO(y = y.std, X = X.std, Z = Z, var_weights = var_weights, lambda.min.ratio = lambda.min.ratio, maxl = 30)$lambda.seq
  }
  if (!is.null(lambda)) {
    nlambda <- length(lambda)
  }
  
  # other parameters
  meps     = (.Machine$double.eps)
  negsmall = -meps
  if (!check_param_constant(min(lambda), negsmall)) {
    stop("* linreg_ADMM_LASSO_cv : reg. parameter 'lambda' is invalid.")
  }
  if (min(lambda) < meps) {
    message("* linreg_ADMM_LASSO_cv : since 'lambda' is effectively zero, a least-squares solution is returned.")
    xsol     = as.vector(aux_pinv(X) %*% matrix(y))
    output   = list()
    output$x = xsol
    return(output)
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Output
  vMse     <- rep(0, nlambda)
  vSd      <- rep(0, nlambda)
  vRunTime <- rep(0, nlambda)

  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Store
  u_store   <- matrix(0L, nrow = p, ncol = nlambda)
  z_store   <- matrix(0L, nrow = p, ncol = nlambda)
  rho_store <- rep(0L, nlambda)
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get groups
  ret.gr <- groups.cv(n = n, k = cv.fold)
  
  # store Lasso regression for each fold
  vMse_cv <- matrix(0.0, cv.fold, nlambda)
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Initial time 
  startTime <- Sys.time()
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Run lasso
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Run overlap Group Lasso
  for (kt in 1:cv.fold) {
    # run ADMM
    mX_ <- X.std[ret.gr$groups.cv[[kt]],]
    vY_ <- y.std[ret.gr$groups.cv[[kt]]]
    if (is.null(Z)) {
      if (adalasso == FALSE) {
        ret <- .admm_lasso_fast(A = mX_, b = vY_, lambda = lambda, rho_adaptation = adaptation, rho = rho,
                                tau = tau.ada, mu = mu.ada, reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
        
        # ret <- .Call("admm_lasso_fast", 
        #              A = mX_, b = vY_, lambda = lambda, rho_adaptation = adaptation, rho = rho, 
        #              tau = tau.ada, mu = mu.ada, reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
      } else {
        ret <- .admm_adalasso_fast(A = mX_, b = vY_, var_weights = var_weights, lambda = lambda, rho_adaptation = adaptation, rho = rho,
                                   tau = tau.ada, mu = mu.ada, reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
        
        # ret <- .Call("admm_adalasso_fast", 
        #              A = mX_, b = vY_, var_weights = var_weights, lambda = lambda, rho_adaptation = adaptation, rho = rho, 
        #              tau = tau.ada, mu = mu.ada, reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
      }
      
      # fit
      mSpRegP_     <- ret$coef.path
      fit          <- X.std[ret.gr$groups.pred[[kt]],] %*% t(mSpRegP_)
    } else {
      if (dim(Z)[2] > 1) {
        mZ_ <- Z[ret.gr$groups.cv[[kt]],]
      } else {
        mZ_ <- matrix(Z[ret.gr$groups.cv[[kt]],], nrow = length(ret.gr$groups.cv[[kt]]), ncol = 1)
      }
      if (adalasso == FALSE) {
        ret <- .admm_lasso_cov_fast(W = mX_, Z = mZ_, y = vY_, lambda = lambda, rho_adaptation = adaptation, rho = rho,
                                    tau = tau.ada, mu = mu.ada, reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
        
        # ret <- .Call("admm_lasso_cov_fast",
        #              W = mX_, Z = mZ_, y = vY_, lambda = lambda, rho_adaptation = adaptation, rho = rho, 
        #              tau = tau.ada, mu = mu.ada, reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
      } else {
        ret <- .admm_adalasso_cov_fast(W = mX_, Z = mZ_, y = vY_, var_weights = var_weights, lambda = lambda, rho_adaptation = adaptation, rho = rho,
                                       tau = tau.ada, mu = mu.ada, reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
        
        # ret <- .Call("admm_adalasso_cov_fast",
        #              W = mX_, Z = mZ_, y = vY_, var_weights = var_weights, lambda = lambda, rho_adaptation = adaptation, rho = rho, 
        #              tau = tau.ada, mu = mu.ada, reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
      }
      
      # fit
      mSpRegP_     <- ret$sp.coef.path
      mRegP_       <- ret$coef.path
      fit          <- X.std[ret.gr$groups.pred[[kt]],] %*% t(mSpRegP_) + Z[ret.gr$groups.pred[[kt]],] %*% t(mRegP_)
    }
    
    # Evaluate MSE
    n_           <- length(y.std[ret.gr$groups.pred[[kt]]])
    mse          <- apply(fit, 2, function(x) {sum((y.std[ret.gr$groups.pred[[kt]]] - x)^2)/n_})
    vMse_cv[kt,] <- mse
  }
  
  # get mean squared error
  vMse <- apply(vMse_cv, 2, mean) 
  vSd  <- apply(vMse_cv, 2, sd) 

  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # system time
  endTime <- Sys.time()
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # compute run time
  eltime <- difftime(time1 = endTime, time2 = startTime, units = "secs")
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get lasso output
  min.mse      <- min(vMse)
  indi.min.mse <- which(vMse == min.mse) 
  lambda.min   <- lambda[indi.min.mse]
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # get estimate over the full sample
  #rho_ <- ret$rho[indi.min.mse]
  rho_ <- ret$rho[[indi.min.mse]]
  if (is.null(Z)) {
    if (adalasso == FALSE) {
      ret <- .admm_lasso(A = X.std, b = y.std, u = t(ret$U[indi.min.mse,]), z = t(ret$Z[indi.min.mse,]), lambda = lambda.min,
                         rho_adaptation = adaptation, rho = rho_[length(rho_)], tau = tau.ada, mu = mu.ada,
                         reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
      
      # ret <- .Call("admm_lasso", 
      #              A = X.std, b = y.std, u = t(ret$U[indi.min.mse,]), z = t(ret$Z[indi.min.mse,]), lambda = lambda.min, 
      #              rho_adaptation = adaptation, rho = rho_[length(rho_)], tau = tau.ada, mu = mu.ada, 
      #              reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
    } else {
      ret <- .admm_adalasso(A = X.std, b = y.std, var_weights = var_weights, u = t(ret$U[indi.min.mse,]), z = t(ret$Z[indi.min.mse,]), lambda = lambda.min,
                            rho_adaptation = adaptation, rho = rho_[length(rho_)], tau = tau.ada, mu = mu.ada,
                            reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
      
      # ret <- .Call("_admm_adalasso", 
      #              A = X.std, b = y.std, var_weights = var_weights, u = t(ret$U[indi.min.mse,]), z = t(ret$Z[indi.min.mse,]), lambda = lambda.min, 
      #              rho_adaptation = adaptation, rho = rho_[length(rho_)], tau = tau.ada, mu = mu.ada, 
      #              reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
    }
    # get estimated parameters
    vSpRegP <- ret$x
  } else {
    if (adalasso == FALSE) {
      ret <- .admm_lasso_cov(W = X.std, Z = Z, y = y.std, u = t(ret$U[indi.min.mse,]), z = t(ret$Z[indi.min.mse,]), lambda = lambda.min,
                             rho_adaptation = adaptation, rho = rho_[length(rho_)], tau = tau.ada, mu = mu.ada,
                             reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
      
      # ret <- .Call("admm_lasso_cov", 
      #              W = X.std, Z = Z, y = y.std, u = t(ret$U[indi.min.mse,]), z = t(ret$Z[indi.min.mse,]), lambda = lambda.min, 
      #              rho_adaptation = adaptation, rho = rho_[length(rho_)], tau = tau.ada, mu = mu.ada, 
      #              reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
      
    } else {
      ret <- .admm_adalasso_cov(W = X.std, Z = Z, y = y.std, var_weights = var_weights, u = t(ret$U[indi.min.mse,]), z = t(ret$Z[indi.min.mse,]), lambda = lambda.min,
                                rho_adaptation = adaptation, rho = rho_[length(rho_)], tau = tau.ada, mu = mu.ada,
                                reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
      
      # ret <- .Call("admm_adalasso_cov", 
      #              W = X.std, Z = Z, y = y.std, var_weights = var_weights, u = t(ret$U[indi.min.mse,]), z = t(ret$Z[indi.min.mse,]), lambda = lambda.min, 
      #              rho_adaptation = adaptation, rho = rho_[length(rho_)], tau = tau.ada, mu = mu.ada, 
      #              reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
    }
    
    # get estimated parameters
    vSpRegP <- ret$x
    vRegP   <- ret$v
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Store output: full sample estimate
  niter <- ret$niter
  if (niter < maxiter) {
    converged <- TRUE
  } else {
    converged <- FALSE
  }
  dConv  <- ifelse(converged == TRUE, 1, 0) 
  nIterN <- ret$niter
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Retrieve estimated parameters
  if (standardize.data == TRUE) {
    vSpRegP_ <- matrix(solve(mU) %*% vSpRegP %*% mV, nrow = p, ncol = 1)
    if (!is.null(Z)) {
      vRegP <- vRegP %*% mV
    }
  } else {
    vSpRegP_ <- matrix(vSpRegP, nrow = p, ncol = 1)
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Print to screen
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  if (print.out == TRUE) {
    cat("\n\n\n")
    cat("Linear regression model with LASSO penalty, Cross Validation\n")
    cat("Alternating direction method of multipliers\n")
    cat("running time (for ", reltol, " relative error):",
        endTime - startTime, "seconds \n\n\n")
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get output
  res.names <- c("sp.coefficients", 
                 "coefficients",
                 "mse",
                 "mse.sd",
                 "min.mse",
                 "lambda",
                 "lambda.min",
                 "convergence",
                 "elapsedTime",
                 "iternum",
                 "indi.min.mse",
                 "foldid") 
  res        <- vector(mode = "list", length = length(res.names))
  names(res) <- res.names
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get output
  ret                 <- NULL
  ret$sp.coefficients <- vSpRegP_
  if (!is.null(Z)) {
    ret$coefficients <- vRegP
  }
  ret$mse          <- vMse
  ret$mse.sd       <- vSd
  ret$min.mse      <- min.mse
  ret$lambda       <- lambda
  ret$lambda.min   <- lambda.min
  ret$convergence  <- dConv
  ret$elapsedTime  <- eltime
  ret$iternum      <- nIterN
  ret$indi.min.mse <- indi.min.mse
  ret$foldid       <- ret.gr$foldid
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Return output
  return(ret)
}

# Authors: Mauro Bernardi 
#          Department Statistical Sciences
#       	 University of Padova
#      	   Via Cesare Battisti, 241
#      	   35121 PADOVA, Italy
#          E-mail: mauro.bernardi@unipd.it  

# Last change: July 21, 2026


#' Overlap Group Least Absolute Shrinkage and Selection Operator for scalar-on-function regression model
#'
#' Overlap Group-LASSO for scalar-on-function regression model solves the following optimization problem
#' \deqn{\textrm{min}_{\psi,\gamma} ~ \frac{1}{2} \sum_{i=1}^n \left( y_i - \int x_i(t) \psi(t) dt-z_i^\intercal\gamma \right)^2 + \lambda_1 \sum_{g=1}^{G} \Vert S_{g}T\psi \Vert_2+\lambda_2\Vert D\psi\Vert_2^2}
#' to obtain a sparse coefficient vector \eqn{\psi\in\mathbb{R}^{M}} for the functional penalized predictor \eqn{x(t)} and a coefficient vector \eqn{\gamma\in\mathbb{R}^q} for the unpenalized scalar predictors \eqn{z_1,\dots,z_q}. The regression function is \eqn{\psi(t)=\varphi(t)^\intercal\psi}
#' where \eqn{\varphi(t)} is a B-spline basis of order \eqn{d} and dimension \eqn{M}. For each group \eqn{g}, each row of 
#' the matrix \eqn{S_g\in\mathbb{R}^{d\times M}} has non-zero entries only for those bases belonging 
#' to that group. These values are provided by the arguments \code{groups} and \code{group_weights} (see below). 
#' Each basis function belongs to more than one group. The diagonal matrix \eqn{T\in\mathbb{R}^{M\times M}} contains 
#' the basis-specific weights. These values are provided by the argument \code{var_weights} (see below).
#' The matrix \eqn{D} is a discrete difference matrix whose order is specified
#' by the argument \code{diff_order}, which must be greater than or equal to one.
#' The penalty \eqn{\lambda_2 \lVert D\psi\rVert_2^2} controls the smoothness
#' of the estimated coefficient function.
#' The regularization path is computed for the overlap group-LASSO penalty at a grid of values for the regularization
#' parameter \eqn{\lambda_1} using the alternating direction method of multipliers (ADMM). See Boyd et al. (2011) and Lin et al. (2022) 
#' for details on the ADMM method.
#'  
#' @param vY a length-\eqn{n} vector of observations of the scalar response variable.
#' @param mX a \eqn{(n\times r)} matrix of observations of the functional covariate.
#' @param mZ an \eqn{(n\times q)} full column rank matrix of scalar predictors that are not penalized. 
#' @param M number of elements of the B-spline basis vector \eqn{\varphi(t)}.
#' @param group_weights a vector of length \eqn{G} containing group-specific weights. The default is square root of the group cardinality, see Bernardi et al. (2022).
#' @param var_weights a vector of length \eqn{M} containing basis-specific weights. The default is a vector where 
#' each entry is the reciprocal of the number of groups including that basis. See Bernardi et al. (2022) for details.
#' @param weights_adaptive logical. If \code{TRUE}, adaptive weights are computed from an initial 
#' least squares estimate. This option is ignored (and set to \code{FALSE}) when 
#' \code{group_weights} are provided by the user, since external weights override the adaptive scheme.
#' @param standardize.data logical. Should data be standardized?
#' @param splOrd the order \eqn{d} of the spline basis.
#' @param diff_order a positive integer specifying the order of the discrete
#' difference operator used in the smoothness penalty. The default is 1.
#' @param lambda1 either a regularization parameter or a vector of regularization parameters.
#' In this latter case the routine computes the whole path. If it is NULL values for \code{lambda1} are provided by the routine.
#' @param lambda2 either a non-negative smoothing regularization parameter or a vector of
#' smoothing regularization parameters. If \code{NULL}, no smoothness penalty is used.
#' @param lambda1.min.ratio smallest value for \code{lambda1}, as a fraction of the maximum \code{lambda1} value. If \eqn{n>M}, the default is 0.0001, and if \eqn{n<M}, the default is 0.01.
#' @param nlambda1 the number of \code{lambda1} values - default is 30.
#' @param intercept logical. If it is TRUE, a column of ones is added to the design matrix. 
#' @param overall.group logical. If it is TRUE, an overall group including all penalized covariates is added.
#' @param control	 a list of control parameters for the ADMM algorithm. See ‘Details’.
#'
#' @return A named list containing \describe{
#' \item{sp.coefficients}{a length-\eqn{M} solution vector for the parameters \eqn{\psi}, which corresponds to the minimum in-sample MSE.}
#' \item{sp.coef.path}{an \eqn{(n_{\lambda_1}\times M)} matrix of estimated \eqn{\psi} coefficients for each \code{lambda1}.}
#' \item{sp.fun}{a length-\eqn{r} vector providing the estimated functional coefficient for \eqn{\psi(t)}.}
#' \item{sp.fun.path}{an \eqn{(n_{\lambda_1}\times r)} matrix providing the estimated functional coefficients for \eqn{\psi(t)} for each \code{lambda1}.}
#' \item{coefficients}{a length-\eqn{q} solution vector for the parameters \eqn{\gamma}, which corresponds to the minimum in-sample MSE. 
#' It is provided only when either the matrix \eqn{Z} in input is not NULL or the intercept is set to TRUE.}
#' \item{coef.path}{an \eqn{(n_{\lambda_1}\times q)} matrix of estimated \eqn{\gamma} coefficients for each \code{lambda1}. 
#' It is provided only when either the matrix \eqn{Z} in input is not NULL or the intercept is set to TRUE.}
#' \item{dof}{estimated degrees of freedom for each value of the regularization parameter.}
#' \item{dof.groups_active}{active groups used in the degrees-of-freedom computation for each value of the regularization parameter.}
#' \item{dof.coeff_active}{active coefficient indices used in the degrees-of-freedom computation for each value of the regularization parameter.}
#' \item{dof.vars_active}{active basis coefficients used in the degrees-of-freedom computation for each value of the regularization parameter.}
#' \item{bic}{Bayesian information criterion computed along the regularization path.}
#' \item{lambda1.min.bic}{optimal value of the \code{lambda1} regularization parameter selected by minimizing the Bayesian Information Criterion (BIC).}
#' \item{lambda1.min.bic}{optimal value of the \code{lambda2} regularization parameter selected by minimizing the Bayesian Information Criterion (BIC).}
#' \item{lambda1}{sequence of regularization parameters \code{lambda1}.}
#' \item{lambda2}{sequence of smoothing regularization parameters \code{lambda2}. It is \code{NULL} when no smoothness penalty is used.}
#' \item{mse}{in-sample mean squared error.}
#' \item{lambda1.min.mse}{value of \code{lambda1} that attains the minimum in-sample MSE.}
#' \item{lambda2.min.mse}{value of \code{lambda2} that attains the minimum in-sample MSE.}
#' \item{min.mse}{minimum value of the in-sample MSE for the sequence of \code{lambda1}.}
#' \item{convergence}{logical. 1 denotes achieved convergence.}
#' \item{elapsedTime}{elapsed time in seconds.}
#' \item{iternum}{number of iterations.}
#' }
#'
#' When you run the algorithm, output returns not only the solution, but also the iteration history recording
#' following fields over iterates,
#' \describe{
#' \item{objval}{objective function value.}
#' \item{r_norm}{norm of primal residual.}
#' \item{s_norm}{norm of dual residual.}
#' \item{eps_pri}{feasibility tolerance for primal feasibility condition.}
#' \item{eps_dual}{feasibility tolerance for dual feasibility condition.}
#' }
#' Iteration stops when both \code{r_norm} and \code{s_norm} values
#' become smaller than \code{eps_pri} and \code{eps_dual}, respectively.
#' @section Details: The control argument is a list that can supply any of the following components:\describe{
#' \item{adaptation}{logical. If it is TRUE, ADMM with adaptation is performed. The default value is TRUE. See Boyd et al. (2011) for details.}
#' \item{rho}{an augmented Lagrangian parameter. The default value is 1.}
#' \item{tau.ada}{an adaptation parameter greater than one. Only needed if adaptation = TRUE. The default value is 2. See Boyd et al. (2011) and Lin et al. (2022) for details.}
#' \item{mu.ada}{an adaptation parameter greater than one. Only needed if adaptation = TRUE. The default value is 10. See Boyd et al. (2011) and Lin et al. (2022) for details.}
#' \item{dof.toler_c}{tolerance parameter used in the degrees-of-freedom computation to determine active constraints.}
#' \item{dof.toler_d}{tolerance parameter used in the degrees-of-freedom computation to determine active coefficients or groups.}
#' \item{abstol}{absolute tolerance stopping criterion. The default value is sqrt(sqrt(.Machine$double.eps)).}
#' \item{reltol}{relative tolerance stopping criterion. The default value is sqrt(.Machine$double.eps).}
#' \item{maxit}{maximum number of iterations. The default value is 100.}
#' \item{print.out}{logical. If it is TRUE, a message about the procedure is printed. The default value is TRUE.}
#' }
#' 
#' @examples 
#' 
#' ## generate sample data
#' set.seed(1)
#' n     <- 40
#' p     <- 18                                  # number of basis to GENERATE beta
#' r     <- 100
#' s     <- seq(0, 1, length.out = r)
#' 
#' beta_basis <- splines::bs(s, df = p, intercept = TRUE)    # basis
#' coef_data  <- matrix(rnorm(n*floor(p/2)), n, floor(p/2))        
#' fun_data   <- coef_data %*% t(splines::bs(s, df = floor(p/2), intercept = TRUE))     
#' 
#' x_0   <- apply(matrix(rnorm(p, sd=1),p,1), 1, fdaSP::softhresh, 1)  # regression coefficients 
#' x_fun <- beta_basis %*% x_0                
#' 
#' b     <- fun_data %*% x_fun + rnorm(n, sd = sqrt(crossprod(fun_data %*% x_fun ))/10)
#' l     <- 10^seq(2, -4, length.out = 30)
#' 
#' ## set the hyper-parameters
#' maxit          <- 1000
#' rho_adaptation <- TRUE
#' rho            <- 1
#' reltol         <- 1e-5
#' abstol         <- 1e-5
#' 
#' # model fit
#' mod1 <- f2sSP(vY = b, mX = fun_data, 
#'               mZ = NULL, M = 5,
#'               group_weights = NULL, 
#'               var_weights = NULL, 
#'               standardize.data = FALSE, 
#'               intercept = FALSE, splOrd = 4,
#'               lambda1 = NULL, nlambda1 = 30, 
#'               lambda1.min.ratio = NULL, 
#'               weights_adaptive = TRUE, 
#'               overall.group = FALSE, 
#'               control = list("maxit"      = maxit,
#'                              "abstol"     = abstol, 
#'                              "reltol"     = reltol, 
#'                              "adaptation" = rho_adaptation, 
#'                              "rho"        = rho, 
#'                              "print.out"  = FALSE)) 
#' 
#' # plot coefficiente path
#' matplot(log(mod1$lambda1), mod1$sp.coef.path, type = "l", 
#'         xlab = latex2exp::TeX("$\\log(\\lambda_1)$"), ylab = "", bty = "n", lwd = 1.2)
#'         
#'  # model fit with smoothing      
#'  lam2 <- 10^seq(5, -3, length.out = 20)  # lambda2 sequence 
#'  mod2 <- f2sSP(vY = b, mX = fun_data, 
#'                mZ = NULL, M = 5,
#'                group_weights = NULL, 
#'                var_weights   = NULL, 
#'                standardize.data = FALSE, 
#'                intercept = FALSE, splOrd = 4,
#'                lambda1 = NULL, nlambda1 = 30, 
#'                lambda1.min.ratio = NULL,
#'                lambda2 = lam2, 
#'                diff_order = 2,
#'                weights_adaptive = TRUE, 
#'                overall.group = FALSE, 
#'                control = list("maxit"      = maxit,
#'                               "abstol"     = abstol, 
#'                               "reltol"     = reltol, 
#'                               "adaptation" = rho_adaptation, 
#'                               "rho"        = rho, 
#'                               "print.out"  = FALSE))       
#'
#' idx <- which(mod2$lambda2 == mod2$lambda2.min.bic)
#' matplot(log(mod2$lambda1), mod2$sp.coef.path[, idx,, drop = TRUE], type = "l",
#'         xlab = latex2exp::TeX("$\\log(\\lambda_1)$"), ylab = "",
#'         bty = "n", lwd = 1.2)                              
#'                                          
#' @references
#' \insertRef{bernardi_etal.2022}{fdaSP}
#' 
#' \insertRef{boyd_etal.2011}{fdaSP}
#' 
#' \insertRef{jenatton_etal.2011}{fdaSP}
#' 
#' \insertRef{lin_etal.2022}{fdaSP}
#'
#' @export
f2sSP <- function(vY, mX, mZ = NULL, M, 
                  group_weights = NULL, var_weights = NULL, weights_adaptive = FALSE,
                  standardize.data = TRUE, splOrd = 4, diff_order = 1,
                  lambda1 = NULL, lambda2 = NULL, nlambda1 = 30, lambda1.min.ratio = NULL, 
                  intercept = FALSE, overall.group = FALSE, control = list()) {
    
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Pre-processing: data validity
  if (!check_data_matrix(mX)) {
    stop("* f2sSP : input 'mX' is invalid data matrix.")
  }
  if (!check_data_vector(vY)) {
    stop("* f2sSP : input 'vY' is invalid data vector.")
  }
  vY <- as.vector(vY)
  # data size
  if (dim(mX)[1] != length(vY)) {
    stop("* f2sSP : two inputs 'mX' and 'vY' have non-matching dimension.")
  }
  if (!is.null(mZ)) {
    if (dim(mZ)[1] != length(vY)) {
      stop("* f2sSP : two inputs 'mZ' and 'vY' have non-matching dimension.")
    }
  }
  if (M <= 0) {
    stop("* f2sSP : the number of basis must be a positive integer!")
  }
  if (!is.null(diff_order)) {
    if (diff_order < 1) {
      stop("* f2sSP : the parameter diff_order is less than one!")
    }
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get dimensions
  n <- dim(mX)[1]             # number of observations
  p <- dim(mX)[2]
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Create the B-spline basis functions
  s  <- seq(0, 1, length.out = p)
  mH <- t(splines::bs(s, df = M, intercept = TRUE))
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Define response and design matrix
  mX_B <- tcrossprod(mX, mH)
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Vectorisation form
  X_ <- mX_B
  y_ <- matrix(vY, nrow = n, ncol = 1)
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # add constant
  if (standardize.data == TRUE) {
    if (intercept == TRUE){
      intercept = FALSE
      warning("* f2sSP : intercept should be set to FALSE is data are standardized!\n")
    }
  }
  if (intercept == TRUE) {
    if (!is.null(mZ)) {
      mZ <- cbind(matrix(data = 1, nrow = n, ncol = 1), mZ)
    } else {
      mZ <- matrix(data = 1, nrow = n, ncol = 1)
    }
  }
  q <- dim(mZ)[2]
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Standardise response and design matrix
  if (standardize.data == TRUE) {
    res   <- standardizemat(X_, y_)
    X.std <- res$X.std
    y.std <- res$y.std
    mU    <- res$mU
    mV    <- res$mV
  } else {
    X.std <- X_
    y.std <- y_
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Pre-processing: lambda1 parameter (in case of a single lambda1)
  output <- f2s_reg_RIDGE(y = y.std, Z = mZ, X = X.std, 
                          diff_order = diff_order, 
                          lambda = lambda1, lambda2 = lambda2)

  # retrieve estimate coefficients in case standardize is TRUE
  if (!is.null(output)) {
    mSpRegP <- output$sp.coef.path
    vSpRegP <- output$vSpRegP
    if (!is.null(mZ)) { 
      mRegP <- output$coef.path
      vRegP <- output$vRegP
    }
    # Get the scaled path
    if (standardize.data == TRUE) {
      if (length(lambda2) > 1) {
        sp.path <- matrix(data = t(apply(mSpRegP, 2, function(x) solve(mU) %*% x %*% mV)), nrow = length(lambda2), ncol = M)
      } else {
        sp.path <- matrix(data = t(solve(mU) %*% mSpRegP %*% mV), nrow = 1, ncol = M)
      }
      vSpRegP <- matrix(solve(mU) %*% vSpRegP %*% mV, nrow = 1, ncol = M)
      if (!is.null(mZ)) {
        if (length(lambda2) > 1) {
          path <- matrix(length(lambda2), q, data = t(apply(mRegP, 2, function(x) x %*% mV)))
        } else {
          path <- matrix(1, q, data = t(mRegP %*% mV))
        }
        vRegP <- vRegP %*% mV
      }
    } else {
      sp.path <- matrix(data = t(mSpRegP), length(lambda2), M)
      if (!is.null(mZ)) {
        path  <- t(mRegP)
      }
    }
  }
  
  # retrieve estimated functins and path
  if (!is.null(output)) {
    sp.fun.path <- sp.path %*% mH
    sp.fun      <- vSpRegP %*% mH
  }
  
  # Get output
  if (!is.null(output)) {
    res.names <- c("sp.coefficients", 
                   "sp.coef.path", 
                   "sp.fun.path",
                   "sp.fun",
                   "coefficients",
                   "coef.path",
                   "dof",
                   "lambda1",
                   "lambda2",
                   "mse",
                   "min.mse",
                   "bic") 
    res        <- vector(mode = "list", length = length(res.names))
    names(res) <- res.names
    if (!is.null(mZ)) {
      res$coefficients <- as.numeric(vRegP)
      res$coef.path    <- path
    } 
    res$sp.coefficients   <- t(mSpRegP)
    res$sp.coef.path      <- sp.path
    res$sp.fun.path       <- sp.fun.path
    res$sp.fun            <- t(sp.fun)
    res$dof               <- output$dof
    res$lambda1           <- lambda1
    res$lambda2           <- lambda2
    res$mse               <- output$mse
    res$min.mse           <- output$min.mse
    res$bic               <- output$bic
    
    # Return output
    return(res)
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # check for inputs
  npar   <- dim(X.std)[2]
  con    <- list(maxit                = 100L, 
                 abstol               = sqrt(.Machine$double.eps), 
                 reltol               = sqrt(sqrt(.Machine$double.eps)), 
                 adaptation           = TRUE, 
                 rho                  = 1, 
                 tau.ada              = 2,               
                 mu.ada               = 10,
                 dof.toler_c          = 5,
                 dof.toler_d          = 0.001,
                 par.init             = NULL,
                 init.rnd             = FALSE,
                 print.out            = TRUE)
  
  nmsC                          <- names(con)
  con[(namc <- names(control))] <- control
  if (length(noNms <- namc[!namc %in% nmsC])) {
    warning("unknown names in control: ", paste(noNms, collapse = ", "))
  }
  # set default values (for inputs)
  maxit                <- con$maxit
  abstol               <- con$abstol
  reltol               <- con$reltol
  adaptation           <- con$adaptation
  rho                  <- con$rho
  tau.ada              <- con$tau.ada      
  mu.ada               <- con$mu.ada
  toler_c              <- con$dof.toler_c
  toler_d              <- con$dof.toler_d
  par.init             <- con$par.init
  init.rnd             <- con$init.rnd
  print.out            <- con$print.out
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Pre-processing: other parameters
  if (!check_param_constant_multiple(c(abstol, reltol))) {
    stop("* f2sSP : tolerance level is invalid.")
  }
  if (!check_param_integer(maxit, 0.0)) {
    stop("* f2sSP : 'maxiter' should be a positive integer.")
  }
  maxit = as.integer(maxit)
  if (!check_param_constant(rho, 0.0)) {
    stop("* f2sSP : 'rho' should be a positive real number.")
  }
  if (!check_param_constant(mu.ada, 0.0)) {
    stop("* f2sSP : 'mu.ada' should be a positive real number.")
  }
  if ((tau.ada < 1.0) || (tau.ada > 2.0)) {
    warning("* f2sSP : 'tau.ada' value is suggested to be in [1,2].")
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Pre-processing: variable weights
  check_weights(x = var_weights, n = dim(X_)[2], algname = "f2sSP", funname = "var_weights")
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get the groups of the overlap group LASSO penalty
  if (overall.group == TRUE) {
    out <- penfun(method = "ogl&1", p = M, q = 1, splOrd = splOrd, regpars = NULL)
  } else {
    out <- penfun(method = "ogl", p = M, q = 1, splOrd = splOrd, regpars = NULL)
  }
  GRmat <- out$grMat
  G     <- dim(GRmat)[1]
  nG    <- rowSums(GRmat)
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Manage group & overall group weights
  # set weights_adaptive to FALSE if group_weights are provided by the user!
  if (!is.null(group_weights)) {
    if (weights_adaptive == TRUE) {
      warning("* f2sSP : weights_adaptive is set to FALSE if group_weights are provided by the user!\n")
      weights_adaptive <- FALSE
    }
  }
  # compute group_weights if they are not provided by the user
  if (is.null(group_weights)) {
    # Define the weights as in Yuan and Lin 2006, JRSSB
    #group_weights <- sqrt(diag(tcrossprod(GRmat)))
    #group_weights <- rep(1, G)
    if (!is.null(weights_adaptive)) {
      if (weights_adaptive == TRUE) {
        if (is.null(mZ)) {
          coeff_LS <- lm_ols(X = X.std, y = y.std)
          for (g in 1:G) {
            idx              <- which(GRmat[g,] == 1)
            group_weights[g] <- 1.0 / norm(coeff_LS[idx], type = "2")
          }
        } else {
          coeff_LS <- lm_ols_FWL(X = X.std, Z = mZ, y = y.std)$coeff_X
          for (g in 1:G) {
            idx              <- which(GRmat[g,] == 1)
            group_weights[g] <- 1.0 / norm(coeff_LS[idx], type = "2")
          }
        }
      } else {
        group_weights <- rep(1, G)
      }
    } else {
      group_weights <- rep(1, G)
    }
  } else {
    if (overall.group == TRUE) {
      check_group_weights(x = group_weights, n = G, algname = "f2sSP", funname = "group_weights")
      if (length(group_weights) == (G-1)) {
        group_weights <- c(group_weights, sqrt(dim(GRmat)[2]))
      }
    } else {
      check_weights(x = group_weights, n = G, algname = "f2sSP", funname = "group_weights")
    }
  } 
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Manage variable weights
  # le righe che seguono sono state cambiate il 27 ottobre 2025 da Mauro e Marco
  if (is.null(var_weights)) {
   var_weights <- 1.0 / colSums(GRmat)
  }
  # if (is.null(var_weights)) {
  #   var_weights <- 1.0 / sqrt(colSums(GRmat))
  # }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get the sequence of lambda1
  if (!is.null(nlambda1) && is.null(lambda1)) {
    # get the smallest value of such that the regression 
    # coefficients estimated by the lasso are all equal to zero
    lambda1 <- lm_lambdamax_OVGLASSO(y = y.std, X = X.std, mZ,
                                     GRmat = GRmat, group_weights = group_weights, var_weights = var_weights, 
                                     lambda.min.ratio = lambda1.min.ratio, maxl = nlambda1)$lambda.seq
  }
  if (is.null(nlambda1) && is.null(lambda1)) {
    # get the smallest value of such that the regression 
    # coefficients estimated by the lasso are all equal to zero
    lambda1 <- lm_lambdamax_OVGLASSO(y = y.std, X = X.std, mZ,
                                     GRmat = GRmat, group_weights = group_weights, var_weights = var_weights, 
                                     lambda.min.ratio = lambda1.min.ratio, maxl = 30)$lambda.seq
  }
  if (!is.null(lambda1)) {
    nlambda1 <- length(lambda1)
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Run the ADMM algorithm
  if (is.null(mZ)) {
  
    # check for that diff_order and lambda2 are not NULL
    if ((is.null(diff_order)) || (is.null(lambda2))) {
      ret <- .admm_ovglasso_fast(A = X.std, b = y.std, groups = GRmat,
                                 group_weights = group_weights, var_weights = var_weights,
                                 lambda = lambda1, rho_adaptation = adaptation,
                                 rho = rho, tau = tau.ada, mu = mu.ada,
                                 reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
    } else {
      if (length(lambda2) == 1) {
        if (lambda2 == 0) {
          ret <- .admm_ovglasso_fast(A = X.std, b = y.std, groups = GRmat,
                                     group_weights = group_weights, var_weights = var_weights,
                                     lambda = lambda1, rho_adaptation = adaptation,
                                     rho = rho, tau = tau.ada, mu = mu.ada,
                                     reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
        } else {
          ret <- .admm_ovglasso_smo_fast(A = X.std, b = y.std, groups = GRmat,
                                         group_weights = group_weights, var_weights = var_weights,
                                         lambda = lambda1, lambda2 = lambda2, diff_order = diff_order, 
                                         rho_adaptation = adaptation,
                                         rho = rho, tau = tau.ada, mu = mu.ada,
                                         reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
        }
      } else {
        ret <- .admm_ovglasso_smo_fast(A = X.std, b = y.std, groups = GRmat,
                                       group_weights = group_weights, var_weights = var_weights,
                                       lambda = lambda1, lambda2 = lambda2, diff_order = diff_order, 
                                       rho_adaptation = adaptation,
                                       rho = rho, tau = tau.ada, mu = mu.ada,
                                       reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
      }
    }
                               
    # get estimated coefficients and path
    if (length(dim(ret$coef.path)) == 2) {
      mSpRegP <- t(ret$coef.path)
      
    } else {
      mSpRegP <- ret$coef.path
    }
    vSpRegP <- ret$coefficients
  } else {
    # check for that diff_order and lambda2 are not NULL
    if ((is.null(diff_order)) || (is.null(lambda2))) {
      ret <- .admm_ovglasso_cov_fast(W = X.std, Z = mZ, y = y.std,
                                     groups = GRmat, group_weights = group_weights, var_weights = var_weights,
                                     lambda = lambda1, rho_adaptation = adaptation, rho = rho, tau = tau.ada, mu.ada,
                                     reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
    } else {
      if (length(lambda2) == 1) {
        if (lambda2 == 0) {
          ret <- .admm_ovglasso_cov_fast(W = X.std, Z = mZ, y = y.std,
                                         groups = GRmat, group_weights = group_weights, var_weights = var_weights,
                                         lambda = lambda1, rho_adaptation = adaptation, rho = rho, tau = tau.ada, mu.ada,
                                         reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
        } else {
          ret <- .admm_ovglasso_cov_smo_fast(W = X.std, Z = mZ, y = y.std,
                                             groups = GRmat, group_weights = group_weights, var_weights = var_weights,
                                             lambda = lambda1, lambda2 = lambda2, diff_order = diff_order,
                                             rho_adaptation = adaptation, rho = rho, tau = tau.ada, mu.ada,
                                             reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
        }
      } else {
        ret <- .admm_ovglasso_cov_smo_fast(W = X.std, Z = mZ, y = y.std,
                                           groups = GRmat, group_weights = group_weights, var_weights = var_weights,
                                           lambda = lambda1, lambda2 = lambda2, diff_order = diff_order,
                                           rho_adaptation = adaptation, rho = rho, tau = tau.ada, mu.ada,
                                           reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
      }
    }
        
    # get estimated coefficients and path
    if (length(dim(ret$sp.coef.path)) == 2) {
      mSpRegP <- t(ret$sp.coef.path)
      mRegP   <- t(ret$coef.path)
    } else {
      mSpRegP <- ret$sp.coef.path
      mRegP   <- ret$coef.path
    }
    vSpRegP <- ret$sp.coefficients
    vRegP   <- ret$coefficients
  }
  
  if (standardize.data == TRUE) {
    # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    # Get the path and retrieve the scaled estimates
    #path <- array(dim = c(nlambda1, L, M), data = t(apply(mRegP, 2, function(x) solve(mU) %*% x %*% mV)))
    #sp.path <- matrix(data = t(apply(mSpRegP, 2, function(x) solve(mU) %*% x %*% mV)), nlambda1, M)
    if (length(dim(mSpRegP)) == 2) {
      sp.path <- matrix(data = t(apply(mSpRegP, 2, function(x) solve(mU) %*% x %*% mV)), nlambda1, M)
    } else {
      sp.path <- array(
        data = 0,
        dim  = dim(mSpRegP)
      )
      for (k in 1:length(lambda2)) {
        sp.path[,k,] <- matrix(data = t(apply(t(mSpRegP[,k,]), 2, function(x) solve(mU) %*% x %*% mV)), nlambda1, M)
      }
    }
    
    # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    # Retrieve estimated parameters
    mSpRegP_ <- matrix(solve(mU) %*% vSpRegP %*% mV, nrow = 1, ncol = M)
    
    # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    # Retrieve estimated parameters (non penalized covariates)
    if (!is.null(mZ)) {
      if (length(dim(mSpRegP)) == 2) {
        path  <- matrix(nlambda1, q, data = t(apply(mRegP, 2, function(x) x %*% mV)))
      } else {
        path <- array(
          data = 0,
          dim  = dim(mRegP)
        )
        for (k in 1:length(lambda2)) {
          path[,k,] <- matrix(nlambda1, q, data = apply(mRegP[,k,], 2, function(x) x %*% mV))
        }
      }
      vRegP <- vRegP %*% mV
    }
  } else {
    #path   <- array(dim = c(nlambda1, L, M), data = t(mRegP))
    #sp.path  <- matrix(data = t(mSpRegP), nlambda1, M)
    #mSpRegP_ <- matrix(vSpRegP, nrow = 1, ncol = M)
    #if (!is.null(mZ)) {
    #  path <- t(mRegP)
    #}
    if (length(dim(mSpRegP)) == 2) {
      sp.path  <- matrix(data = t(mSpRegP), nlambda1, M)
      if (!is.null(mZ)) {
        path <- t(mRegP)
      }
    } else {
      sp.path <- array(
        data = 0,
        dim  = c(dim(mSpRegP)[1], dim(mSpRegP)[2], dim(mSpRegP)[3])
      )
      if (!is.null(mZ)) {
        path <- array(
          data = 0,
          dim = c(dim(mRegP)[1], dim(mRegP)[2], dim(mRegP)[3])
        )
      }
      for (k in 1:length(lambda2)) {
        sp.path[,k,] <- matrix(data = mSpRegP[,k,], nlambda1, M)
        if (!is.null(mZ)) {
          path[,k,] <- mRegP[,k,]
        }
      }
    }
    mSpRegP_ <- matrix(vSpRegP, nrow = 1, ncol = M)
  }
  # get the estimated functions
  # sp.fun.path <- sp.path %*% mH
  # sp.fun      <- mSpRegP_ %*% mH
  if (length(dim(sp.path)) == 2) {
    sp.fun.path <- sp.path %*% mH
  } else {
    sp.fun.path <- array(
      data = 0,
      dim  = c(dim(mSpRegP)[1], dim(mSpRegP)[2], dim(mH)[2])
    )
    for (k in 1:length(lambda2)) {
      sp.fun.path[,k,] <- sp.path[,k,] %*% mH
    }
  }
  sp.fun <- mSpRegP_ %*% mH
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Compute the degrees of freedom 
  if (is.null(mZ)) {
    # compute the DoF
    dof.out <- f2s_dof(W                = X.std, 
                       y                = y.std,
                       coeff            = ret$coef.path, 
                       lambda1          = lambda1, 
                       lambda2          = lambda2, 
                       diff_order       = diff_order, 
                       adaptive_weights = weights_adaptive,
                       GRmat            = GRmat, 
                       group_weights    = group_weights, 
                       var_weights      = var_weights, 
                       Umat             = ret$U,
                       err_primal       = ret$err_pri,
                       err_dual         = ret$err_dual,
                       rho              = ret$rho,
                       toler_c          = toler_c,
                       toler_d          = toler_d)
  } else {
    dof.out <- f2s_cov_dof(W                = X.std, 
                           Z                = mZ, 
                           y                = y.std,
                           coeff_W          = ret$sp.coef.path, 
                           coeff_Z          = ret$coef.path, 
                           lambda1          = lambda1, 
                           lambda2          = lambda2, 
                           diff_order       = diff_order,
                           adaptive_weights = weights_adaptive,
                           GRmat            = GRmat, 
                           group_weights    = group_weights, 
                           var_weights      = var_weights, 
                           Umat             = ret$U,
                           err_primal       = ret$err_pri,
                           err_dual         = ret$err_dual,
                           rho              = ret$rho,
                           toler_c          = toler_c,
                           toler_d          = toler_d)
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # compute the BIC
  bic <- dim(X.std)[1] * log(ret$mse) + dof.out$dof * log(dim(X.std)[1]) 
  if (is.null(lambda2)) {
    bic.min.pos     <- which.min(bic)
    bic.min         <- min(bic,  na.rm = TRUE)
    lambda1.min.bic <- lambda1[bic.min.pos]
  } else {
    bic.min     <- min(bic, na.rm = TRUE)
    bic.min.pos <- as.numeric(arrayInd(
      which.min(replace(bic, is.na(bic), Inf)), dim(bic)
      )
    )
    lambda1.min.bic <- lambda1[bic.min.pos[1]]
    lambda2.min.bic <- lambda2[bic.min.pos[2]]
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Print to screen
  if (print.out == TRUE) {
    cat("\n\n\n")
    cat("Function to scalar regression model with overlap group-LASSO penalty\n")
    cat("running time (for ", reltol, " relative error):",
        sum(ret$elapsedTime), "seconds \n\n\n")
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get output
  res.names <- c("sp.coefficients", 
                 "sp.coef.path", 
                 "sp.fun.path",
                 "sp.fun",
                 "coefficients",
                 "coef.path",
                 "dof",
                 "dof.groups_active",
                 "dof.coef_active",
                 "dof.vars_active",
                 "bic",
                 "lambda1.min.bic",
                 "lambda2.min.bic",
                 "lambda1",
                 "lambda2",
                 "GRmat",
                 "group_weights",
                 "var_weights",
                 "mse",
                 "min.mse",
                 "lambda1.min.mse",
                 "lambda2.min.mse",
                 "convergence",
                 "elapsedTime",
                 "iternum",
                 "objfun",
                 "r_norm",
                 "s_norm",
                 "err_pri",
                 "err_dual",
                 "rho") 
  res        <- vector(mode = "list", length = length(res.names))
  names(res) <- res.names
  if (!is.null(mZ)) {
    res$coefficients <- as.numeric(vRegP)
    res$coef.path    <- path
  } 
  res$sp.coefficients   <- t(mSpRegP_)
  res$sp.coef.path      <- sp.path
  res$sp.fun.path       <- sp.fun.path
  res$sp.fun            <- t(sp.fun)
  res$dof               <- dof.out$dof
  res$dof.groups_active <- dof.out$groups_active
  res$dof.coef_active   <- dof.out$coeff_active
  res$dof.vars_active   <- dof.out$vars_active
  res$bic               <- bic
  res$lambda1.min.bic   <- lambda1.min.bic
  if (!is.null(lambda2)) {
    res$lambda2.min.bic <- lambda2.min.bic
  } else {
    res$lambda2.min.bic <- NULL
  }
  res$lambda1.min.mse   <- ret$lambda.min
  if (!is.null(lambda2)) {
    res$lambda2.min.mse <- ret$lambda2.min
  } else {
    res$lambda2.min.mse <- NULL
  }
  res$lambda1           <- lambda1
  res$lambda2           <- lambda2
  res$GRmat             <- GRmat
  res$group_weights     <- group_weights
  res$var_weights       <- var_weights
  res$mse               <- ret$mse
  res$min.mse           <- ret$min.mse
  res$convergence       <- ret$convergence
  res$elapsedTime       <- ret$elapsedTime
  res$iternum           <- ret$iternum
  res$objfun            <- ret$objfun
  res$r_norm            <- ret$r_norm
  res$s_norm            <- ret$s_norm
  res$err_pri           <- ret$err_pri
  res$err_dual          <- ret$err_dual
  res$rho               <- ret$rho
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Return output
  return(res)
}

#' Cross-validation for Overlap Group Least Absolute Shrinkage and Selection Operator on scalar-on-function regression model
#'
#' Overlap Group-LASSO for scalar-on-function regression model solves the following optimization problem
#' \deqn{\textrm{min}_{\psi,\gamma} ~ \frac{1}{2} \sum_{i=1}^n \left( y_i - \int x_i(t) \psi(t) dt-z_i^\intercal\gamma \right)^2 + \lambda_1 \sum_{g=1}^{G} \Vert S_{g}T\psi \Vert_2+\lambda_2\Vert D\psi\Vert_2^2}
#' to obtain a sparse coefficient vector \eqn{\psi\in\mathbb{R}^{M}} for the functional penalized predictor \eqn{x(t)} and a coefficient vector \eqn{\gamma\in\mathbb{R}^q} for the unpenalized scalar predictors \eqn{z_1,\dots,z_q}. The regression function is \eqn{\psi(t)=\varphi(t)^\intercal\psi}
#' where \eqn{\varphi(t)} is a B-spline basis of order \eqn{d} and dimension \eqn{M}. 
#' For each group \eqn{g}, each row of the matrix \eqn{S_g\in\mathbb{R}^{d\times M}} has non-zero entries only for those bases belonging 
#' to that group. These values are provided by the arguments \code{groups} and \code{group_weights} (see below). 
#' Each basis function belongs to more than one group. The diagonal matrix \eqn{T\in\mathbb{R}^{M\times M}} contains 
#' the basis specific weights. These values are provided by the argument \code{var_weights} (see below).
#' The matrix \eqn{D} is a discrete difference matrix whose order is specified
#' by the argument \code{diff_order}, which must be greater than or equal to one.
#' The penalty \eqn{\lambda_2 \lVert D\psi\rVert_2^2} controls the smoothness
#' of the estimated coefficient function.
#' The regularization path is computed for the overlap group-LASSO penalty at a grid of values for the regularization
#' parameter \eqn{\lambda1} using the alternating direction method of multipliers (ADMM). See Boyd et al. (2011) and Lin et al. (2022) 
#' for details on the ADMM method.
#'  
#' @param vY a length-\eqn{n} vector of observations of the scalar response variable.
#' @param mX a \eqn{(n\times r)} matrix of observations of the functional covariate.
#' @param mZ an \eqn{(n\times q)} full column rank matrix of scalar predictors that are not penalized.  
#' @param M number of elements of the B-spline basis vector \eqn{\varphi(t)}.
#' @param group_weights a vector of length \eqn{G} containing group-specific weights. The default is square root of the group cardinality, see Bernardi et al. (2022).
#' @param var_weights a vector of length \eqn{M} containing basis-specific weights. The default is a vector where 
#' each entry is the reciprocal of the number of groups including that basis. See Bernardi et al. (2022) for details.
#' @param standardize.data logical. Should data be standardized?
#' @param splOrd the order \eqn{d} of the spline basis.
#' @param diff_order a positive integer specifying the order of the discrete
#' difference operator used in the smoothness penalty. The default is 1.
#' @param lambda1 either a regularization parameter or a vector of regularization parameters.
#' @param lambda2 either a non-negative smoothing regularization parameter or a vector of
#' smoothing regularization parameters. If \code{NULL}, no smoothness penalty is used.
#' In this latter case the routine computes the whole path. If it is NULL values for \code{lambda1} are provided by the routine.
#' @param lambda1.min.ratio smallest value for \code{lambda1}, as a fraction of the maximum \code{lambda1} value. If \eqn{n>M}, the default is 0.0001, and if \eqn{n<M}, the default is 0.01. 
#' @param nlambda1 the number of \code{lambda1} values - default is 30.
#' @param intercept logical. If it is TRUE, a column of ones is added to the design matrix. 
#' @param overall.group logical. If it is TRUE, an overall group including all penalized covariates is added.
#' @param cv.fold the number of folds - default is 5.
#' @param control	 a list of control parameters for the ADMM algorithm. See ‘Details’.
#'
#' @return A named list containing \describe{
#' \item{sp.coefficients}{a length-\eqn{M} solution vector for the parameters \eqn{\psi}, which corresponds to the minimum cross-validated MSE.}
#' \item{sp.fun}{a length-\eqn{r} vector providing the estimated functional coefficient for \eqn{\psi(t)} corresponding to the minimum cross-validated MSE.}
#' \item{coefficients}{a length-\eqn{q} solution vector for the parameters \eqn{\gamma}, which corresponds to the minimum cross-validated MSE.
#' It is provided only when either the matrix \eqn{Z} in input is not NULL or the intercept is set to TRUE.} 
#' \item{lambda1}{sequence of regularization parameters \code{lambda1}.}
#' \item{lambda2}{sequence of smoothing regularization parameters \code{lambda2}. It is \code{NULL} when no smoothness penalty is used.}
#' \item{mse}{cross-validated mean squared error.}
#' \item{min.mse}{minimum value of the cross-validated MSE for the sequence of lambda.}
#' \item{lambda1.min.mse}{value of \code{lambda1} that attains the minimum in-sample MSE.}
#' \item{lambda2.min.mse}{value of \code{lambda2} that attains the minimum in-sample MSE.}
#' \item{convergence}{logical. 1 denotes achieved convergence.}
#' \item{elapsedTime}{elapsed time in seconds.}
#' \item{iternum}{number of iterations.}
#' }
#'
#' Iteration stops when both \code{r_norm} and \code{s_norm} values
#' become smaller than \code{eps_pri} and \code{eps_dual}, respectively.
#' @section Details: The control argument is a list that can supply any of the following components:\describe{
#' \item{adaptation}{logical. If it is TRUE, ADMM with adaptation is performed. The default value is TRUE. See Boyd et al. (2011) for details.}
#' \item{rho}{an augmented Lagrangian parameter. The default value is 1.}
#' \item{tau.ada}{an adaptation parameter greater than one. Only needed if adaptation = TRUE. The default value is 2. See Boyd et al. (2011) and Lin et al. (2022) for details.}
#' \item{mu.ada}{an adaptation parameter greater than one. Only needed if adaptation = TRUE. The default value is 10. See Boyd et al. (2011) and Lin et al. (2022) for details.}
#' \item{abstol}{absolute tolerance stopping criterion. The default value is sqrt(sqrt(.Machine$double.eps)).}
#' \item{reltol}{relative tolerance stopping criterion. The default value is sqrt(.Machine$double.eps).}
#' \item{maxit}{maximum number of iterations. The default value is 100.}
#' \item{print.out}{logical. If it is TRUE, a message about the procedure is printed. The default value is TRUE.}
#' }
#' 
#' @examples 
#' 
#' ## generate sample data and functional coefficients
#' set.seed(1)
#' n     <- 40
#' p     <- 18                                 
#' r     <- 100
#' s     <- seq(0, 1, length.out = r)
#' 
#' beta_basis <- splines::bs(s, df = p, intercept = TRUE)    # basis
#' coef_data  <- matrix(rnorm(n*floor(p/2)), n, floor(p/2))        
#' fun_data   <- coef_data %*% t(splines::bs(s, 
#'                   df = floor(p/2), intercept = TRUE))     
#' 
#' x_0   <- apply(matrix(rnorm(p, sd=1),p,1), 1, fdaSP::softhresh, 1)  
#' x_fun <- beta_basis %*% x_0                
#' 
#' b     <- fun_data %*% x_fun + rnorm(n, sd = sqrt(crossprod(fun_data %*% x_fun ))/10)
#' l     <- 10^seq(2, -4, length.out = 30)
#' 
#' ## set the hyper-parameters
#' maxit          <- 1000
#' rho_adaptation <- TRUE
#' rho            <- 1
#' reltol         <- 1e-5
#' abstol         <- 1e-5
#' 
#' ## run cross-validation
#' mod_cv <- f2sSP_cv(vY = b, mX = fun_data, M = p,
#'                    group_weights = NULL, var_weights = NULL, 
#'                    standardize.data = FALSE, splOrd = 4,
#'                    lambda1 = NULL, lambda1.min.ratio = 1e-5, 
#'                    nlambda1 = 30, cv.fold = 5, intercept = FALSE, 
#'                    control = list("abstol" = abstol, 
#'                                   "reltol" = reltol, 
#'                                   "adaptation" = rho_adaptation,
#'                                   "rho" = rho, 
#'                                   "print.out" = FALSE))
#'                                           
#' ### graphical presentation
#' plot(log(mod_cv$lambda1), mod_cv$mse, type = "l", 
#'      col = "blue", lwd = 2, bty = "n", 
#'      xlab = latex2exp::TeX("$\\log(\\lambda_1)$"), 
#'      ylab = "Prediction Error", 
#'      ylim = range(mod_cv$mse - mod_cv$mse.sd, 
#'      mod_cv$mse + mod_cv$mse.sd),
#'      main = "Cross-validated Prediction Error")
#' fdaSP::confband(xV = log(mod_cv$lambda1), 
#'                 yVmin = mod_cv$mse - mod_cv$mse.sd, 
#'                 yVmax = mod_cv$mse + mod_cv$mse.sd)       
#' idx <- which(mod_cv$lambda1 == mod_cv$lambda1.min.mse)
#' abline(v = log(mod_cv$lambda1[idx]), 
#'        col = "red", lwd = 1.0)
#' 
#' ### comparison with oracle error
#' mod <- f2sSP(vY = b, mX = fun_data, M = p, 
#'              group_weights = NULL, var_weights = NULL, 
#'              standardize.data = FALSE, splOrd = 4,
#'              lambda1 = NULL, nlambda1 = 30, 
#'              lambda1.min.ratio = 1e-5, intercept = FALSE,
#'              control = list("abstol" = abstol, 
#'                             "reltol" = reltol, 
#'                             "adaptation" = rho_adaptation, 
#'                             "rho" = rho, 
#'                             "print.out" = FALSE))
#'                                     
#' err_mod <- apply(mod$sp.coef.path, 1, 
#'                  function(x) sum((x - x_0)^2))
#' plot(log(mod$lambda1), 
#' err_mod, type = "l", col = "blue", 
#'      lwd = 2, 
#'      xlab = latex2exp::TeX("$\\log(\\lambda_1)$"), 
#'      ylab = "Estimation Error", 
#'      main = "True Estimation Error", bty = "n")
#' abline(v = log(mod$lambda1[which(err_mod == min(err_mod))]), 
#'        col = "red", lwd = 1.0)
#' idx <- which(mod_cv$lambda1 == mod_cv$lambda1.min.mse)
#' abline(v = log(mod_cv$lambda1[idx]), 
#'        col = "red", lwd = 1.0, lty = 2)                                      
#' 
#' @references
#' \insertRef{bernardi_etal.2022}{fdaSP}
#' 
#' \insertRef{boyd_etal.2011}{fdaSP}
#' 
#' \insertRef{jenatton_etal.2011}{fdaSP}
#' 
#' \insertRef{lin_etal.2022}{fdaSP}
#'
#' @export 
f2sSP_cv <- function(vY, mX, mZ = NULL, M,
                     group_weights = NULL, var_weights = NULL, standardize.data = FALSE, 
                     splOrd = 4, diff_order = 1,
                     lambda1 = NULL, lambda1.min.ratio = NULL, nlambda1 = NULL, lambda2 = NULL,
                     cv.fold = 5, intercept = FALSE, overall.group = FALSE, control = list()) {
    
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Pre-processing
  # data validity
  if (!check_data_matrix(mX)) {
    stop("* f2sSP_cv : input 'mX' is invalid data matrix.")
  }
  if (!check_data_vector(vY)) {
    stop("* f2sSP_cv : input 'vY' is invalid data vector.")
  }
  # data size
  if (dim(mX)[1] != length(vY)) {
    stop("* f2sSP_cv : two inputs 'mX' and 'vY' have non-matching dimension.")
  }
  if (!is.null(mZ)) {
    if (dim(mZ)[1] != length(vY)) {
      stop("* f2sSP_cv : two inputs 'mZ' and 'vY' have non-matching dimension.")
    }
  }
  if (M <= 0) {
    stop("* f2sSP_cv : the number of basis must be a positive integer!")
  }
  if (is.null(diff_order) || is.null(lambda2)) {
    diff_order <- NULL
    lambda2    <- NULL
  }
  if (!is.null(diff_order)) {
    if (diff_order < 1) {
      stop("* f2sSP : the parameter diff_order is less than one!")
    }
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get dimensions
  n <- dim(mX)[1]             # number of observations
  p <- dim(mX)[2]
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Create the B-spline basis functions
  s  <- seq(0, 1, length.out = p)
  mH <- t(splines::bs(s, df = M, intercept = TRUE))
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Define response and design matrix
  mX_B <- tcrossprod(mX, mH)
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Vectorisation form
  X_ <- mX_B
  y_ <- matrix(vY, nrow = n, ncol = 1)
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # add constant
  if (standardize.data == TRUE) {
    if (intercept == TRUE){
      intercept = FALSE
      warning("* f2sSP_cv : intercept should be set to FALSE is data are standardized!\n")
    }
  }
  if (intercept == TRUE) {
    if (!is.null(mZ)) {
      mZ <- cbind(matrix(data = 1, nrow = n, ncol = 1), mZ)
    } else {
      mZ <- matrix(data = 1, nrow = n, ncol = 1)
    }
  }
  #q <- dim(mZ)[2]
  q <- if (is.null(mZ)) 0L else ncol(mZ)
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Standardise response and design matrix
  if (standardize.data == TRUE) {
    res   <- standardizemat(X_, y_)
    X.std <- res$X.std
    y.std <- res$y.std
    mU    <- res$mU
    mV    <- res$mV
  } else {
    X.std <- X_
    y.std <- y_
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Pre-processing: lambda parameter (in case of a single lambda)
  output <- f2s_reg_RIDGE(y = y.std, Z = mZ, X = X.std, 
                          diff_order = diff_order, 
                          lambda = lambda1, lambda2 = lambda2)
  
  # retrieve estimate coefficients in case standardize is TRUE
  if (!is.null(output)) {
    mSpRegP <- output$sp.coef.path
    vSpRegP <- output$vSpRegP
    if (!is.null(mZ)) { 
      mRegP <- output$coef.path
      vRegP <- output$vRegP
    }
    # Get the scaled path
    if (standardize.data == TRUE) {
      if (length(lambda2) > 1) {
        sp.path <- matrix(data = t(apply(mSpRegP, 2, function(x) solve(mU) %*% x %*% mV)), nrow = length(lambda2), ncol = M)
      } else {
        sp.path <- matrix(data = t(solve(mU) %*% mSpRegP %*% mV), nrow = 1, ncol = M)
      }
      vSpRegP <- matrix(solve(mU) %*% vSpRegP %*% mV, nrow = 1, ncol = M)
      if (!is.null(mZ)) {
        if (length(lambda2) > 1) {
          path <- matrix(length(lambda2), q, data = t(apply(mRegP, 2, function(x) x %*% mV)))
        } else {
          path <- matrix(1, q, data = t(mRegP %*% mV))
        }
        vRegP <- vRegP %*% mV
      }
    } else {
      sp.path <- matrix(data = t(mSpRegP), length(lambda2), M)
      if (!is.null(mZ)) {
        path  <- t(mRegP)
      }
    }
  }
  
  # retrieve estimated functins and path
  if (!is.null(output)) {
    sp.fun.path <- sp.path %*% mH
    sp.fun      <- vSpRegP %*% mH
  }
  
  # Get output
  if (!is.null(output)) {
    res.names <- c("sp.coefficients", 
                   "coefficients",
                   "sp.fun",
                   "mse",
                   "min.mse",
                   "lambda1",
                   "lambda2") 
    res        <- vector(mode = "list", length = length(res.names))
    names(res) <- res.names
    if (!is.null(mZ)) {
      res$coefficients <- as.numeric(vRegP)
    } 
    res$sp.coefficients   <- t(mSpRegP)
    res$sp.fun            <- t(sp.fun)
    res$lambda1           <- lambda1
    res$lambda2           <- lambda2
    res$mse               <- output$mse
    res$min.mse           <- output$min.mse
    
    # Return output
    return(res)
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # check for inputs
  npar   <- dim(X.std)[2]
  con    <- list(maxit                = 100L, 
                 abstol               = sqrt(.Machine$double.eps), 
                 reltol               = sqrt(sqrt(.Machine$double.eps)), 
                 adaptation           = TRUE, 
                 rho                  = 1, 
                 tau.ada              = 2,               
                 mu.ada               = 10,
                 par.init             = NULL,
                 init.rnd             = FALSE,
                 print.out            = TRUE)
  
  nmsC                          <- names(con)
  con[(namc <- names(control))] <- control
  if (length(noNms <- namc[!namc %in% nmsC])) {
    warning("unknown names in control: ", paste(noNms, collapse = ", "))
  }
  # set default values (for inputs)
  maxit      <- con$maxit
  abstol     <- con$abstol
  reltol     <- con$reltol
  adaptation <- con$adaptation
  rho        <- con$rho
  tau.ada    <- con$tau.ada      
  mu.ada     <- con$mu.ada
  par.init   <- con$par.init
  init.rnd   <- con$init.rnd
  print.out  <- con$print.out
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Pre-processing
  # data validity
  if (!check_param_constant_multiple(c(abstol, reltol))) {
    stop("* f2sSP_cv : tolerance level is invalid.")
  }
  if (!check_param_integer(maxit, 0.0)) {
    stop("* f2sSP_cv : 'maxit' should be a positive integer.")
  }
  maxit = as.integer(maxit)
  if (!check_param_constant(rho, 0.0)) {
    stop("* f2sSP_cv : 'rho' should be a positive real number.")
  }
  if (!check_param_constant(mu.ada, 0.0)) {
    stop("* f2sSP_cv : 'mu' should be a positive real number.")
  }
  if ((tau.ada < 1.0) || (tau.ada > 2.0)) {
    warning("* f2sSP_cv : 'tau' value is suggested to be in [1,2].")
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Pre-processing: variable weights
  check_weights(x = var_weights, n = dim(X_)[2], algname = "f2sSP", funname = "var_weights")
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get the groups of the overlap group LASSO penalty
  if (overall.group == TRUE) {
    out <- penfun(method = "ogl&1", p = M, q = 1, splOrd = splOrd, regpars = NULL)
  } else {
    out <- penfun(method = "ogl", p = M, q = 1, splOrd = splOrd, regpars = NULL)
  }
  GRmat <- out$grMat
  G     <- dim(GRmat)[1]
  nG    <- rowSums(GRmat)
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Manage group & overall group weights
  if (is.null(group_weights)) {
    # Define the weights as in Yuan and Lin 2006, JRSSB
    #group_weights <- sqrt(diag(tcrossprod(GRmat)))
    group_weights <- rep(1, G)
  } else {
    if (overall.group == TRUE) {
      check_group_weights(x = group_weights, n = G, algname = "f2sSP_cv", funname = "group_weights")
      if (length(group_weights) == (G-1)) {
        group_weights <- c(group_weights, sqrt(dim(GRmat)[2]))
      }
    } else {
      check_weights(x = group_weights, n = G, algname = "f2sSP_cv", funname = "group_weights")
    }
  } 
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Manage variable weights
  #if (is.null(var_weights)) {
  #  var_weights <- 1.0 / colSums(GRmat)
  #}
  if (is.null(var_weights)) {
    var_weights <- 1.0 / sqrt(colSums(GRmat))
  }

  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get the sequence of lambda
  if (!is.null(nlambda1) && is.null(lambda1)) {
    # get the smallest value of such that the regression 
    # coefficients estimated by the lasso are all equal to zero
    lambda1 <- lm_lambdamax_OVGLASSO(y = y.std, X = X.std, mZ,
                                     GRmat = GRmat, group_weights = group_weights, var_weights = var_weights, 
                                     lambda.min.ratio = lambda1.min.ratio, maxl = nlambda1)$lambda.seq
  }
  if (is.null(nlambda1) && is.null(lambda1)) {
    # get the smallest value of such that the regression 
    # coefficients estimated by the lasso are all equal to zero
    lambda1 <- lm_lambdamax_OVGLASSO(y = y.std, X = X.std, mZ,
                                     GRmat = GRmat, group_weights = group_weights, var_weights = var_weights, 
                                     lambda.min.ratio = lambda1.min.ratio, maxl = 30)$lambda.seq
  }
  if (!is.null(lambda1)) {
    nlambda1 <- length(lambda1)
  }

  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get groups
  ret.gr <- groups.cv(n = n, k = cv.fold)
  
  # store Lasso regression for each fold
  if (!is.null(lambda2)) {
    aMse_cv <- array(data = 0.0, dim = c(cv.fold, nlambda1, length(lambda2)))
  } else {
    #aMse_cv <- matrix(data = 0.0, dim = c(cv.fold, nlambda1))
    aMse_cv <- matrix(data = 0.0, nrow = cv.fold, ncol = nlambda1)
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Initial time 
  startTime <- Sys.time()

  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Run lasso
  for (kt in 1:cv.fold) {
    # run ADMM
    mX_ <- X.std[ret.gr$groups.cv[[kt]],]
    vY_ <- y.std[ret.gr$groups.cv[[kt]]]
    if (is.null(mZ)) {

      # check for that diff_order and lambda2 are not NULL
      if ((is.null(diff_order)) || (is.null(lambda2))) {
        ret <- .admm_ovglasso_fast(A = mX_, b = vY_, groups = GRmat, group_weights = group_weights, var_weights = var_weights,
                                   lambda = lambda1, rho_adaptation = adaptation, rho = rho, tau = tau.ada, mu = mu.ada,
                                   reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
      } else {
        ret <- .admm_ovglasso_smo_fast(A = mX_, b = vY_, groups = GRmat, group_weights = group_weights, var_weights = var_weights,
                                       lambda = lambda1, lambda2 = lambda2, diff_order = diff_order, 
                                       rho_adaptation = adaptation, rho = rho, tau = tau.ada, mu = mu.ada,
                                       reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
      }

      # fit 
      mSpRegP_ <- ret$coef.path
      if (length(dim(mSpRegP_)) == 3) {
        fit      <- array(
          data = 0,
          dim  = c(n-dim(mX_)[1], dim(mSpRegP_)[1], dim(mSpRegP_)[2])
        )
        for (k in 1:length(lambda2)) {
          fit[,,k] <- X.std[ret.gr$groups.pred[[kt]],] %*% t(mSpRegP_[,k,])
        }
      } else {
        fit <- X.std[ret.gr$groups.pred[[kt]],] %*% t(mSpRegP_)
      }
    } else {
      mZ_ <- as.matrix(mZ[ret.gr$groups.cv[[kt]],], ncol = 1)
    
      # check for that diff_order and lambda2 are not NULL
      if ((is.null(diff_order)) || (is.null(lambda2))) {
        ret <- .admm_ovglasso_cov_fast(W = mX_, Z = mZ_, y = vY_, groups = GRmat, 
                                       group_weights = group_weights, var_weights = var_weights,
                                       lambda = lambda1, rho_adaptation = adaptation, rho = rho, tau = tau.ada, mu = mu.ada,
                                       reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
      } else {
        ret <- .admm_ovglasso_cov_smo_fast(W = mX_, Z = mZ_, y = vY_, groups = GRmat, 
                                           group_weights = group_weights, var_weights = var_weights,
                                           lambda = lambda1, lambda2 = lambda2, diff_order = diff_order, 
                                           rho_adaptation = adaptation, rho = rho, tau = tau.ada, mu = mu.ada,
                                           reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
      }
      
      # fit
      mSpRegP_ <- ret$sp.coef.path
      mRegP_   <- ret$coef.path
      if (length(dim(mSpRegP_)) == 3) {
        fit      <- array(
          data = 0,
          dim  = c(n-dim(mX_)[1], dim(mSpRegP_)[1], dim(mSpRegP_)[2])
        )
        for (k in 1:length(lambda2)) {
          fit[,,k] <- X.std[ret.gr$groups.pred[[kt]],] %*% t(mSpRegP_[,k,]) + mZ[ret.gr$groups.pred[[kt]],] %*% t(mRegP_[,k,])
        }
      } else {
        fit <- X.std[ret.gr$groups.pred[[kt]],] %*% t(mSpRegP_) + mZ[ret.gr$groups.pred[[kt]],] %*% t(mRegP_)
      }
    }
    # Evaluate MSE
    if (length(dim(mSpRegP_)) == 3) {
      n_            <- length(y.std[ret.gr$groups.pred[[kt]]])
      mse           <- apply(fit, c(2, 3), function(x) {sum((y.std[ret.gr$groups.pred[[kt]]] - x)^2)/n_})
      aMse_cv[kt,,] <- mse
    } else {
      n_           <- length(y.std[ret.gr$groups.pred[[kt]]])
      mse          <- apply(fit, 2, function(x) {sum((y.std[ret.gr$groups.pred[[kt]]] - x)^2)/n_})
      aMse_cv[kt,] <- mse
    }
  }
  
  # get mean squared error
  if (length(dim(mSpRegP_)) == 2) {
    mMse <- apply(aMse_cv, 2, mean) 
    mSd  <- apply(aMse_cv, 2, sd) 
  } else {
    mMse <- apply(aMse_cv, c(2, 3), mean) 
    mSd  <- apply(aMse_cv, c(2, 3), sd) 
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # system time
  endTime <- Sys.time()
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # compute run time
  eltime <- difftime(time1 = endTime, time2 = startTime, units = "secs")
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get lasso output
  # min.mse      <- min(vMse)
  # indi.min.mse <- which(vMse == min.mse) 
  # lambda.min   <- lambda[indi.min.mse]
  if (length(dim(mSpRegP_)) == 2) {
    min.mse      <- min(mMse)
    indi.min.mse <- which(mMse == min.mse) 
    lambda.min   <- lambda1[indi.min.mse]
  } else {
    min.mse      <- min(mMse)
    indi.min.mse <- which(mMse == min.mse, arr.ind = TRUE)[1,, drop = FALSE]
    lambda.grid  <- expand.grid(lambda1 = lambda1, lambda2 = lambda2)
    lambda.min   <- c(lambda1[indi.min.mse[1]], lambda2[indi.min.mse[2]])
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # get estimate over the full sample
  if (length(dim(mSpRegP_)) == 2) {
    rho_ <- ret$rho[[indi.min.mse]]
    u_   <- t(ret$U[indi.min.mse,])
    z_   <- t(ret$Z[indi.min.mse,])
  } else {
    rho_ <- ret$rho[[indi.min.mse[1], indi.min.mse[2]]]
    u_   <- t(ret$U[indi.min.mse[1],indi.min.mse[2],])
    z_   <- t(ret$Z[indi.min.mse[1],indi.min.mse[2],])
  }
  if (is.null(mZ)) {
    
    # check for that diff_order and lambda2 are not NULL
    if ((is.null(diff_order)) || (is.null(lambda2))) {
      ret  <- .admm_ovglasso(A = X.std, b = y.std,
                             groups = GRmat, group_weights = group_weights, var_weights = var_weights,
                             u = u_, z = z_, lambda = lambda.min,
                             rho_adaptation = adaptation, rho = rho_[length(rho_)], tau = tau.ada, mu = mu.ada,
                             reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
    } else {
      ret  <- .admm_ovglasso_smo(A = X.std, b = y.std,
                                 groups = GRmat, group_weights = group_weights, var_weights = var_weights,
                                 u = u_, z = z_, lambda = lambda.min[1], lambda2 = lambda.min[2], diff_order = diff_order,
                                 rho_adaptation = adaptation, rho = rho_[length(rho_)], tau = tau.ada, mu = mu.ada,
                                 reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
    }
    
    # get estimated parameters
    vSpRegP <- ret$x
  } else {
    # check for that diff_order and lambda2 are not NULL
    if ((is.null(diff_order)) || (is.null(lambda2))) {
      ret  <- .admm_ovglasso_cov(W = X.std, Z = mZ, y = y.std,
                                 groups = GRmat, group_weights = group_weights, var_weights = var_weights,
                                 u = u_, z = z_, lambda = lambda.min,
                                 rho_adaptation = adaptation, rho = rho_[length(rho_)], tau = tau.ada, mu = mu.ada,
                                 reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
    } else {
      ret  <- .admm_ovglasso_cov_smo(W = X.std, Z = mZ, y = y.std,
                                     groups = GRmat, group_weights = group_weights, var_weights = var_weights,
                                     u = u_, z = z_, lambda = lambda.min[1], lambda2 = lambda.min[2], diff_order = diff_order,
                                     rho_adaptation = adaptation, rho = rho_[length(rho_)], tau = tau.ada, mu = mu.ada,
                                     reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
    }
  
    # get estimated parameters
    vSpRegP <- ret$x
    vRegP   <- ret$v
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Store output: full sample estimate
  niter <- ret$niter
  if (niter < maxit) {
    converged <- TRUE
  } else {
    converged <- FALSE
  }
  dConv  <- ifelse(converged == TRUE, 1, 0) 
  nIterN <- ret$niter
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Retrieve estimated parameters
  if (standardize.data == TRUE) {
    # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    # Retrieve estimated parameters
    mSpRegP <- matrix(solve(mU) %*% vSpRegP %*% mV, nrow = 1, ncol = M)
    
    # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    # Retrieve estimated parameters (non penalized covariates)
    if (!is.null(mZ)) {
      vRegP <- vRegP %*% mV
    }
  } else {
    mSpRegP <- matrix(vSpRegP, nrow = 1, ncol = M)
  }
  # get the estimated functions
  sp.fun <- mSpRegP %*% mH

  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Print to screen
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  if (print.out == TRUE) {
    cat("\n\n\n")
    cat("Function to scalar regression model with overlap group-LASSO penalty, Cross Validation\n")
    cat("Alternating direction method of multipliers\n")
    cat("running time (for ", reltol, " relative error):",
        eltime, "seconds \n\n\n")
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get output
  res.names <- c("sp.coefficients", 
                 "coefficients",
                 "sp.fun",
                 "mse",
                 "mse.sd",
                 "min.mse",
                 "lambda1.min.mse",
                 "lambda2.min.mse",
                 "lambda1",
                 "lambda2",
                 "convergence",
                 "elapsedTime",
                 "iternum",
                 "indi.min.mse") 
  res        <- vector(mode = "list", length = length(res.names))
  names(res) <- res.names
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get output
  ret                 <- NULL
  ret$sp.coefficients <- t(mSpRegP)
  if (!is.null(mZ)) {
    ret$coefficients <- t(vRegP)
  }
  ret$sp.fun          <- t(sp.fun)
  ret$mse             <- mMse
  ret$mse.sd          <- mSd
  ret$min.mse         <- min.mse
  ret$lambda1.min.mse <- lambda.min[1]
  ret$lambda2.min.mse <- if (!is.null(lambda2)) lambda.min[2] else NULL
  ret$lambda1         <- lambda1
  ret$lambda2         <- lambda2
  ret$convergence     <- dConv
  ret$elapsedTime     <- eltime
  ret$iternum         <- nIterN
  ret$indi.min.mse    <- indi.min.mse

  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Return output
  return(ret)
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

glasso_Gmat2Fmat_sparse <- function(Gmat, var_weights) {
    .Call(`_fdaSP_glasso_Gmat2Fmat_sparse`, Gmat, var_weights)
}

ovglasso_Gmat2Fmat_sparse <- function(Gmat, var_weights) {
    .Call(`_fdaSP_ovglasso_Gmat2Fmat_sparse`, Gmat, var_weights)
}

#' @name  fdaSP.internal
#' @keywords internal
.lm_dof_LASSO_1lambda <- function(X, coeff, lambda, Uvec, err_primal, err_dual, rho, toler_c, toler_d) {
    .Call(`_fdaSP_lm_dof_LASSO_1lambda`, X, coeff, lambda, Uvec, err_primal, err_dual, rho, toler_c, toler_d)
}

#' @name  fdaSP.internal
#' @keywords internal
.lm_dof_OVGLASSO_1lambda <- function(X, coeff, lambda, GRmat, group_weights, var_weights, Uvec, err_primal, err_dual, rho, toler_c, toler_d) {
    .Call(`_fdaSP_lm_dof_OVGLASSO_1lambda`, X, coeff, lambda, GRmat, group_weights, var_weights, Uvec, err_primal, err_dual, rho, toler_c, toler_d)
}

#' @name  fdaSP.internal
#' @keywords internal
.lm_cov_dof_OVGLASSO_1lambda <- function(X, Z, coeff_X, lambda, GRmat, group_weights, var_weights, Uvec, err_primal, err_dual, rho, toler_c, toler_d) {
    .Call(`_fdaSP_lm_cov_dof_OVGLASSO_1lambda`, X, Z, coeff_X, lambda, GRmat, group_weights, var_weights, Uvec, err_primal, err_dual, rho, toler_c, toler_d)
}

#' @name  fdaSP.internal
#' @keywords internal
.lm_adaptive_dof_OVGLASSO_1lambda <- function(X, coeff, coeff_LS, lambda, GRmat, group_weights, var_weights, Uvec, err_primal, err_dual, rho, toler_c, toler_d) {
    .Call(`_fdaSP_lm_adaptive_dof_OVGLASSO_1lambda`, X, coeff, coeff_LS, lambda, GRmat, group_weights, var_weights, Uvec, err_primal, err_dual, rho, toler_c, toler_d)
}

#' @name  fdaSP.internal
#' @keywords internal
.lm_cov_adaptive_dof_OVGLASSO_1lamba <- function(X, Z, coeff_X, coeff_X_LS, lambda, GRmat, group_weights, var_weights, Uvec, err_primal, err_dual, rho, toler_c, toler_d) {
    .Call(`_fdaSP_lm_cov_adaptive_dof_OVGLASSO_1lamba`, X, Z, coeff_X, coeff_X_LS, lambda, GRmat, group_weights, var_weights, Uvec, err_primal, err_dual, rho, toler_c, toler_d)
}

#' @name  fdaSP.internal
#' @keywords internal
.f2s_dof_1lambda <- function(W, coeff, lambda, GRmat, group_weights, var_weights, Uvec, err_primal, err_dual, rho, toler_c, toler_d) {
    .Call(`_fdaSP_f2s_dof_1lambda`, W, coeff, lambda, GRmat, group_weights, var_weights, Uvec, err_primal, err_dual, rho, toler_c, toler_d)
}

#' @name  fdaSP.internal
#' @keywords internal
.f2s_cov_dof_1lambda <- function(W, Z, coeff_W, lambda, GRmat, group_weights, var_weights, Uvec, err_primal, err_dual, rho, toler_c, toler_d) {
    .Call(`_fdaSP_f2s_cov_dof_1lambda`, W, Z, coeff_W, lambda, GRmat, group_weights, var_weights, Uvec, err_primal, err_dual, rho, toler_c, toler_d)
}

#' @name  fdaSP.internal
#' @keywords internal
.f2s_smo_dof_1lambda <- function(W, coeff, lambda, lambda2, diff_order, GRmat, group_weights, var_weights, Uvec, err_primal, err_dual, rho, toler_c, toler_d) {
    .Call(`_fdaSP_f2s_smo_dof_1lambda`, W, coeff, lambda, lambda2, diff_order, GRmat, group_weights, var_weights, Uvec, err_primal, err_dual, rho, toler_c, toler_d)
}

#' @name  fdaSP.internal
#' @keywords internal
.f2s_cov_smo_dof_1lambda <- function(W, Z, coeff_W, lambda, lambda2, diff_order, GRmat, group_weights, var_weights, Uvec, err_primal, err_dual, rho, toler_c, toler_d) {
    .Call(`_fdaSP_f2s_cov_smo_dof_1lambda`, W, Z, coeff_W, lambda, lambda2, diff_order, GRmat, group_weights, var_weights, Uvec, err_primal, err_dual, rho, toler_c, toler_d)
}

#' @name  fdaSP.internal
#' @keywords internal
.f2s_adaptive_dof_1lambda <- function(W, coeff, coeff_LS, lambda, GRmat, group_weights, var_weights, Uvec, err_primal, err_dual, rho, toler_c, toler_d) {
    .Call(`_fdaSP_f2s_adaptive_dof_1lambda`, W, coeff, coeff_LS, lambda, GRmat, group_weights, var_weights, Uvec, err_primal, err_dual, rho, toler_c, toler_d)
}

#' @name  fdaSP.internal
#' @keywords internal
.f2s_cov_adaptive_dof_1lambda <- function(W, Z, coeff_W, coeff_W_LS, lambda, GRmat, group_weights, var_weights, Uvec, err_primal, err_dual, rho, toler_c, toler_d) {
    .Call(`_fdaSP_f2s_cov_adaptive_dof_1lambda`, W, Z, coeff_W, coeff_W_LS, lambda, GRmat, group_weights, var_weights, Uvec, err_primal, err_dual, rho, toler_c, toler_d)
}

#' @name  fdaSP.internal
#' @keywords internal
.f2s_smo_adaptive_dof_1lambda <- function(W, coeff, coeff_LS, lambda, lambda2, diff_order, GRmat, group_weights, var_weights, Uvec, err_primal, err_dual, rho, toler_c, toler_d) {
    .Call(`_fdaSP_f2s_smo_adaptive_dof_1lambda`, W, coeff, coeff_LS, lambda, lambda2, diff_order, GRmat, group_weights, var_weights, Uvec, err_primal, err_dual, rho, toler_c, toler_d)
}

#' @name  fdaSP.internal
#' @keywords internal
.f2s_adaptive_cov_smo_dof_1lambda <- function(W, Z, coeff_W, coeff_W_LS, lambda, lambda2, diff_order, GRmat, group_weights, var_weights, Uvec, err_primal, err_dual, rho, toler_c, toler_d) {
    .Call(`_fdaSP_f2s_adaptive_cov_smo_dof_1lambda`, W, Z, coeff_W, coeff_W_LS, lambda, lambda2, diff_order, GRmat, group_weights, var_weights, Uvec, err_primal, err_dual, rho, toler_c, toler_d)
}

#' @name  fdaSP.internal
#' @keywords internal
.f2s_bic <- function(y, W, coeff_path, df) {
    .Call(`_fdaSP_f2s_bic`, y, W, coeff_path, df)
}

#' @name  fdaSP.internal
#' @keywords internal
.f2s_cov_bic <- function(y, W, Z, W_coeff_path, Z_coeff_path, df) {
    .Call(`_fdaSP_f2s_cov_bic`, y, W, Z, W_coeff_path, Z_coeff_path, df)
}

#' @name  fdaSP.internal
#' @keywords internal
.f2s_ebic <- function(y, X, coeff_path, df, kappa) {
    .Call(`_fdaSP_f2s_ebic`, y, X, coeff_path, df, kappa)
}

#' @name  fdaSP.internal
#' @keywords internal
.f2s_cov_ebic <- function(y, W, Z, W_coeff_path, Z_coeff_path, df, kappa) {
    .Call(`_fdaSP_f2s_cov_ebic`, y, W, Z, W_coeff_path, Z_coeff_path, df, kappa)
}

#' @name  fdaSP.internal
#' @keywords internal
.admm_glasso <- function(A, b, groups, group_weights, var_weights, u, z, lambda, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping) {
    .Call(`_fdaSP_admm_glasso`, A, b, groups, group_weights, var_weights, u, z, lambda, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping)
}

#' @name  fdaSP.internal
#' @keywords internal
.admm_glasso_cov <- function(W, Z, y, u, z, groups, group_weights, var_weights, lambda, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping) {
    .Call(`_fdaSP_admm_glasso_cov`, W, Z, y, u, z, groups, group_weights, var_weights, lambda, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping)
}

#' @name  fdaSP.internal
#' @keywords internal
.admm_glasso_fast <- function(A, b, groups, group_weights, var_weights, lambda, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping) {
    .Call(`_fdaSP_admm_glasso_fast`, A, b, groups, group_weights, var_weights, lambda, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping)
}

#' @name  fdaSP.internal
#' @keywords internal
.admm_glasso_cov_fast <- function(W, Z, y, groups, group_weights, var_weights, lambda, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping) {
    .Call(`_fdaSP_admm_glasso_cov_fast`, W, Z, y, groups, group_weights, var_weights, lambda, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping)
}

#' @name  fdaSP.internal
#' @keywords internal
.admm_spglasso <- function(A, b, groups, group_weights, var_weights, var_weights_L1, u, z, lambda1, lambda2, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping) {
    .Call(`_fdaSP_admm_spglasso`, A, b, groups, group_weights, var_weights, var_weights_L1, u, z, lambda1, lambda2, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping)
}

#' @name  fdaSP.internal
#' @keywords internal
.admm_spglasso_cov <- function(W, Z, y, groups, group_weights, var_weights, var_weights_L1, u, z, lambda1, lambda2, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping) {
    .Call(`_fdaSP_admm_spglasso_cov`, W, Z, y, groups, group_weights, var_weights, var_weights_L1, u, z, lambda1, lambda2, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping)
}

#' @name  fdaSP.internal
#' @keywords internal
.admm_spglasso_fast <- function(A, b, groups, group_weights, var_weights, var_weights_L1, lambda, alpha, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping) {
    .Call(`_fdaSP_admm_spglasso_fast`, A, b, groups, group_weights, var_weights, var_weights_L1, lambda, alpha, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping)
}

#' @name  fdaSP.internal
#' @keywords internal
.admm_spglasso_cov_fast <- function(W, Z, y, groups, group_weights, var_weights, var_weights_L1, lambda, alpha, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping) {
    .Call(`_fdaSP_admm_spglasso_cov_fast`, W, Z, y, groups, group_weights, var_weights, var_weights_L1, lambda, alpha, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping)
}

#' @name  fdaSP.internal
#' @keywords internal
.admm_lasso <- function(A, b, u, z, lambda, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping) {
    .Call(`_fdaSP_admm_lasso`, A, b, u, z, lambda, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping)
}

#' @name  fdaSP.internal
#' @keywords internal
.admm_lasso_cov <- function(W, Z, y, u, z, lambda, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping) {
    .Call(`_fdaSP_admm_lasso_cov`, W, Z, y, u, z, lambda, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping)
}

#' @name  fdaSP.internal
#' @keywords internal
.admm_adalasso <- function(A, b, var_weights, u, z, lambda, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping) {
    .Call(`_fdaSP_admm_adalasso`, A, b, var_weights, u, z, lambda, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping)
}

#' @name  fdaSP.internal
#' @keywords internal
.admm_adalasso_cov <- function(W, Z, y, var_weights, u, z, lambda, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping) {
    .Call(`_fdaSP_admm_adalasso_cov`, W, Z, y, var_weights, u, z, lambda, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping)
}

#' @name  fdaSP.internal
#' @keywords internal
.admm_lasso_fast <- function(A, b, lambda, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping) {
    .Call(`_fdaSP_admm_lasso_fast`, A, b, lambda, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping)
}

#' @name  fdaSP.internal
#' @keywords internal
.admm_lasso_cov_fast <- function(W, Z, y, lambda, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping) {
    .Call(`_fdaSP_admm_lasso_cov_fast`, W, Z, y, lambda, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping)
}

#' @name  fdaSP.internal
#' @keywords internal
.admm_adalasso_fast <- function(A, b, var_weights, lambda, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping) {
    .Call(`_fdaSP_admm_adalasso_fast`, A, b, var_weights, lambda, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping)
}

#' @name  fdaSP.internal
#' @keywords internal
.admm_adalasso_cov_fast <- function(W, Z, y, var_weights, lambda, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping) {
    .Call(`_fdaSP_admm_adalasso_cov_fast`, W, Z, y, var_weights, lambda, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping)
}

#' @name  fdaSP.internal
#' @keywords internal
.admm_ovglasso <- function(A, b, groups, group_weights, var_weights, u, z, lambda, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping) {
    .Call(`_fdaSP_admm_ovglasso`, A, b, groups, group_weights, var_weights, u, z, lambda, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping)
}

#' @name  fdaSP.internal
#' @keywords internal
.admm_ovglasso_smo <- function(A, b, groups, group_weights, var_weights, u, z, lambda, lambda2, diff_order, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping) {
    .Call(`_fdaSP_admm_ovglasso_smo`, A, b, groups, group_weights, var_weights, u, z, lambda, lambda2, diff_order, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping)
}

#' @name  fdaSP.internal
#' @keywords internal
.admm_ovglasso_cov <- function(W, Z, y, u, z, groups, group_weights, var_weights, lambda, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping) {
    .Call(`_fdaSP_admm_ovglasso_cov`, W, Z, y, u, z, groups, group_weights, var_weights, lambda, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping)
}

#' @name  fdaSP.internal
#' @keywords internal
.admm_ovglasso_cov_smo <- function(W, Z, y, u, z, groups, group_weights, var_weights, lambda, lambda2, diff_order, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping) {
    .Call(`_fdaSP_admm_ovglasso_cov_smo`, W, Z, y, u, z, groups, group_weights, var_weights, lambda, lambda2, diff_order, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping)
}

#' @name  fdaSP.internal
#' @keywords internal
.admm_ovglasso_fast <- function(A, b, groups, group_weights, var_weights, lambda, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping) {
    .Call(`_fdaSP_admm_ovglasso_fast`, A, b, groups, group_weights, var_weights, lambda, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping)
}

#' @name  fdaSP.internal
#' @keywords internal
.admm_ovglasso_smo_fast <- function(A, b, groups, group_weights, var_weights, lambda, lambda2, diff_order, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping) {
    .Call(`_fdaSP_admm_ovglasso_smo_fast`, A, b, groups, group_weights, var_weights, lambda, lambda2, diff_order, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping)
}

#' @name  fdaSP.internal
#' @keywords internal
.admm_ovglasso_cov_fast <- function(W, Z, y, groups, group_weights, var_weights, lambda, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping) {
    .Call(`_fdaSP_admm_ovglasso_cov_fast`, W, Z, y, groups, group_weights, var_weights, lambda, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping)
}

#' @name  fdaSP.internal
#' @keywords internal
.admm_ovglasso_cov_smo_fast <- function(W, Z, y, groups, group_weights, var_weights, lambda, lambda2, diff_order, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping) {
    .Call(`_fdaSP_admm_ovglasso_cov_smo_fast`, W, Z, y, groups, group_weights, var_weights, lambda, lambda2, diff_order, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping)
}

#' @name  fdaSP.internal
#' @keywords internal
.admm_spovglasso <- function(A, b, groups, group_weights, var_weights, var_weights_L1, u, z, lambda, alpha, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping) {
    .Call(`_fdaSP_admm_spovglasso`, A, b, groups, group_weights, var_weights, var_weights_L1, u, z, lambda, alpha, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping)
}

#' @name  fdaSP.internal
#' @keywords internal
.admm_spovglasso_cov <- function(W, Z, y, u, z, groups, group_weights, var_weights, var_weights_L1, lambda, alpha, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping) {
    .Call(`_fdaSP_admm_spovglasso_cov`, W, Z, y, u, z, groups, group_weights, var_weights, var_weights_L1, lambda, alpha, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping)
}

#' @name  fdaSP.internal
#' @keywords internal
.admm_spovglasso_fast <- function(A, b, groups, group_weights, var_weights, var_weights_L1, lambda, alpha, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping) {
    .Call(`_fdaSP_admm_spovglasso_fast`, A, b, groups, group_weights, var_weights, var_weights_L1, lambda, alpha, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping)
}

#' @name  fdaSP.internal
#' @keywords internal
.admm_spovglasso_cov_fast <- function(W, Z, y, groups, group_weights, var_weights, var_weights_L1, lambda, alpha, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping) {
    .Call(`_fdaSP_admm_spovglasso_cov_fast`, W, Z, y, groups, group_weights, var_weights, var_weights_L1, lambda, alpha, rho_adaptation, rho, tau, mu, reltol, abstol, maxiter, ping)
}

#' @keywords internal
#' @noRd
NULL

#' @keywords internal
#' @noRd
NULL

#' @keywords internal
#' @noRd
NULL

#' @keywords internal
#' @noRd
NULL

#' @keywords internal
#' @noRd
NULL

#' @keywords internal
#' @noRd
NULL

#' @keywords internal
#' @noRd
NULL

#' @keywords internal
#' @noRd
NULL

#' @keywords internal
#' @noRd
NULL

#' @keywords internal
#' @noRd
NULL

#' @keywords internal
#' @noRd
NULL

#' @keywords internal
#' @noRd
NULL

#' Forward finite difference approximation of order d
#'
#' @param f Numeric vector containing function values.
#' @param h Step size (default = 1.0).
#' @param d Order of the derivative (integer >= 1).
#' @return Numeric vector of forward differences of order d (length n, last d elements = NA).
#' #@examples
#' #f <- c(0, 1, 4, 9, 16)
#' #forward_diff(f = f, h = 1, d = 2)
forward_diff <- function(f, h, d) {
    .Call(`_fdaSP_forward_diff`, f, h, d)
}

#' Compute the discrete difference penalty matrix Q = D^T D
#'
#' Constructs the d-th order forward-difference operator D (scaled by 1/h^d)
#' and returns Q = D^T D.
#'
#' @param n Length of the vector the operator acts on.
#' @param h Step size (default = 1.0).
#' @param d Derivative order (integer >= 1).
#' @return Symmetric n x n matrix Q = D^T D.
#' @examples
#' #Q1 <- forward_diff_penalty_matrix(5, h = 1, d = 1)
#' #Q2 <- forward_diff_penalty_matrix(5, h = 1, d = 2)
#' #Q1
#' @name  fdaSP.internal
#' @keywords internal
.forward_diff_penalty_matrix <- function(n, h, d) {
    .Call(`_fdaSP_forward_diff_penalty_matrix`, n, h, d)
}

#' @name  fdaSP.internal
#' @keywords internal
.forward_diff_difference_matrix <- function(n, h, d) {
    .Call(`_fdaSP_forward_diff_difference_matrix`, n, h, d)
}

svd_check <- function(X, U, V, s) {
    .Call(`_fdaSP_svd_check`, X, U, V, s)
}

pseudoinv <- function(X) {
    .Call(`_fdaSP_pseudoinv`, X)
}

lm_ols <- function(X, y) {
    .Call(`_fdaSP_lm_ols`, X, y)
}

lm_ols_fast <- function(XTX_CHOL, XTy) {
    .Call(`_fdaSP_lm_ols_fast`, XTX_CHOL, XTy)
}

lm_ols_FWL <- function(X, Z, y) {
    .Call(`_fdaSP_lm_ols_FWL`, X, Z, y)
}

lm_ols_FWL_fast <- function(L_ZZ, L_XMX, P_ZX, XMy, Zy) {
    .Call(`_fdaSP_lm_ols_FWL_fast`, L_ZZ, L_XMX, P_ZX, XMy, Zy)
}

gennorm <- function(x, type) {
    .Call(`_fdaSP_gennorm`, x, type)
}

# Authors: Mauro Bernardi
#          Department Statistical Sciences
#       	 University of Padova
#      	   Via Cesare Battisti, 241
#      	   35121 PADOVA, Italy
#          E-mail: mauro.bernardi@unipd.it  

# Last change: March 16, 2023

#' Cross-validation for group Least Absolute Shrinkage and Selection Operator
#'
#' Group-LASSO, or Group L2-regularized regression, solves the following optimization problem
#' \deqn{\textrm{min}_\beta ~ \frac{1}{2}\|y-X\beta-Z\gamma\|_2^2 + \lambda \sum_{g=1}^G \|S_g T\beta\|_2}
#' to obtain a sparse coefficient vector \eqn{\beta\in\mathbb{R}^p} and a coefficient vector \eqn{\gamma\in\mathbb{R}^q}. For each group \eqn{g}, each row of 
#' the matrix \eqn{S_g\in\mathbb{R}^{n_g\times p}} has non-zero entries only for those variables belonging 
#' to that group. These values are provided by the arguments \code{groups} and \code{group_weights} (see below). 
#' Each variable must belong to only one group. The diagonal matrix \eqn{T\in\mathbb{R}^{p\times p}} contains 
#' the variable-specific weights. These values are provided by the argument \code{var_weights} (see below).
#' The regularization path is computed for the group-LASSO penalty at a grid of values for the regularization 
#' parameter \eqn{\lambda} using the alternating direction method of multipliers (ADMM). See Boyd et al. (2011) 
#' for details on the ADMM method.
#'
#' @param X an \eqn{(n\times p)} matrix of penalized predictors.
#' @param Z an \eqn{(n\times q)} full column rank matrix of predictors that are not penalized. 
#' @param y a length-\eqn{n} response vector.
#' @param groups either a vector of consecutive integers describing the grouping of the coefficients, 
#' or a list with two elements: the first element is a vector containing the variables belonging to each group, 
#' while the second element is a vector containing the group lenghts (see example below).
#' @param group_weights a vector of length G containing group-specific weights. The default is square root of the group cardinality, see Yuan and Lin (2006).
#' @param var_weights a vector of length p containing variable-specific weights. The default is a vector of ones.
#' @param standardize.data logical. Should data be standardized?
#' @param lambda either a regularization parameter or a vector of regularization parameters. 
#' In this latter case the routine computes the whole path. If it is NULL the path values for lambda are provided by the routine.
#' @param lambda.min.ratio smallest value for lambda, as a fraction of the maximum lambda value, the (data derived) entry value (i.e. the smallest value for which all coefficients are zero). 
#' The default depends on the sample size nobs relative to the number of variables nvars. If nobs > nvars, the default is 0.0001, 
#' close to zero. If nobs < nvars, the default is 0.01. A very small value of lambda.min.ratio will lead to a saturated fit 
#' in the nobs < nvars case.
#' @param nlambda the number of lambda values - default is 30.
#' @param cv.fold the number of folds - default is 5.
#' @param intercept logical. If it is TRUE, a column of ones is added to the design matrix. 
#' @param control	 a list of control parameters. See ‘Details’.
#'
#' @return A named list containing \describe{
#' \item{sp.coefficients}{a length-\eqn{p} solution vector for the parameters \eqn{\beta}, which corresponds to the model fitted on the full sample with \eqn{\lambda} corresponding to minimum cross-validated MSE.}
#' \item{coefficients}{a length-\eqn{q} solution vector for the parameters \eqn{\gamma}, which corresponds to the model fitted on the full sample with \eqn{\lambda} corresponding to minimum cross-validated MSE. 
#' It is provided only when either the matrix \eqn{Z} in input is not NULL or the intercept is set to TRUE.}
#' \item{convergence}{logical. 1 denotes achieved convergence for the model fitted on the full sample.}
#' \item{iternum}{number of iterations for the model fitted on the full sample.}
#' \item{lambda}{sequence of lambda.}
#' \item{lambda.min}{value of lambda that attains the minimum cross-validated MSE.}
#' \item{indi.min.mse}{index of the lambda sequence corresponding to lambda.min.}
#' \item{mse}{cross-validated mean squared error.}
#' \item{min.mse}{minimum value of the cross-validated MSE for the sequence of lambdas.}
#' \item{mse.sd}{standard deviation of the cross-validated mean squared error.}
#' \item{elapsedTime}{elapsed time in seconds for the whole procedure.}
#' \item{foldid}{a vector of values between 1 and cv.fold identifying what fold each observation is in.}
#' }
#'
#' Iteration stops when both \code{r_norm} and \code{s_norm} values
#' become smaller than \code{eps_pri} and \code{eps_dual}, respectively.
#' @section Details: The control argument is a list that can supply any of the following components:\describe{
#' (To understand exactly what these do see the source code: higher levels give more detail.).
#' \item{adaptation}{logical. If it is TRUE, ADMM with adaptation is performed. The default value is TRUE.}
#' \item{rho}{an augmented Lagrangian parameter. The default value is 1.}
#' \item{tau.ada}{an adaptation parameter in [1,2]. Only needed if adaptation = TRUE. The default value is 2. See Boyd et al. (2011) for details.}
#' \item{mu.ada}{an adaptation parameter. Only needed if adaptation = TRUE. The default value is 10. See Boyd et al. (2011) for details.}
#' \item{abstol}{absolute tolerance stopping criterion. The default value is sqrt(sqrt(.Machine$double.eps)).}
#' \item{reltol}{relative tolerance stopping criterion. The default value is sqrt(.Machine$double.eps).}
#' \item{maxit}{maximum number of iterations. The default value is 100.}
#' \item{print.out}{logical. If it is TRUE, a message about the procedure is printed. The default value is TRUE.}
#' }
#' 
#' @examples
#'\donttest{
#' 
#' ### generate sample data
#' set.seed(2023)
#' n    <- 50
#' p    <- 30 
#' X    <- matrix(rnorm(n * p), n, p)
#' beta <- c(rep(4, 12), rep(0, p - 12))
#' y    <- X %*% beta + rnorm(n, sd = sqrt(crossprod(X %*% beta)) / 20)
#' 
#' ### define groups of dimension 3 each
#' group1 <- rep(1:10, each = 3)
#' 
#' ### set the hyper-parameters of the ADMM algorithm
#' maxit      <- 1000
#' adaptation <- TRUE
#' rho        <- 1
#' reltol     <- 1e-5
#' abstol     <- 1e-5
#' 
#' ### run cross-validation
#' mod_cv <- linreg_ADMM_GLASSO_cv(X = X, y = y, groups = group1, standardize.data = FALSE, 
#'                                 intercept = FALSE, cv.fold = 5, nlambda = 30, 
#'                                 control = list("adaptation" = adaptation, "rho" = rho, 
#'                                                "maxit" = maxit, "reltol" = reltol, "abstol" = abstol, 
#'                                                "print.out" = FALSE)) 
#' 
#' 
#' ### graphical presentation
#' plot(log(mod_cv$lambda), mod_cv$mse, type = "l", col = "blue", lwd = 2, bty = "n", 
#'      xlab = TeX("$\\log(\\lambda)$"), ylab = "Prediction Error", 
#'      ylim = range(mod_cv$mse - mod_cv$mse.sd, mod_cv$mse + mod_cv$mse.sd),
#'      main = "Cross-validated Prediction Error")
#' fdaSP:::confband(xV = log(mod_cv$lambda), yVmin = mod_cv$mse - mod_cv$mse.sd, yVmax = mod_cv$mse + mod_cv$mse.sd)       
#' abline(v = log(mod_cv$lambda[which(mod_cv$lambda == mod_cv$lambda.min)]), col = "red", lwd = 1.0)
#' 
#' ### comparison with oracle error
#' mod <- fdaSP:::linreg_ADMM_GLASSO(X = X, y = y, groups = group1, standardize.data = FALSE, intercept = FALSE,
#'                                   nlambda = 30, control = list("adaptation" = adaptation, "rho" = rho, 
#'                                                                "maxit" = maxit, "reltol" = reltol, "abstol" = abstol, 
#'                                                                "print.out" = FALSE)) 
#' err_mod <- apply(mod$sp.coef.path, 1, function(x) sum((x - beta)^2))
#' plot(log(mod$lambda), err_mod, type = "l", col = "blue", lwd = 2, xlab = TeX("$\\log(\\lambda)$"), 
#'      ylab = "Estimation Error", main = "True Estimation Error", bty = "n")
#' abline(v = log(mod$lambda[which(err_mod == min(err_mod))]), col = "red", lwd = 1.0)
#' abline(v = log(mod_cv$lambda[which(mod_cv$lambda == mod_cv$lambda.min)]), col = "red", lwd = 1.0, lty = 2)                                   
#' }
#' 
#' @references
#' \insertRef{yuan_lin.2006}{fdaSP}
#' 
#' \insertRef{boyd_etal.2011}{fdaSP}
#' 
#' \insertRef{hastie_etal.2015}{fdaSP}
#' 
#' \insertRef{lin_etal.2022}{fdaSP} 
#'
#' @noRd
linreg_ADMM_GLASSO_cv <- function(X, Z = NULL, y, groups, group_weights = NULL, var_weights = NULL, standardize.data = TRUE, 
                                  lambda = NULL, lambda.min.ratio = NULL, nlambda = NULL, cv.fold = 5, intercept = FALSE,
                                  control = list()) {
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get dimensions
  n <- dim(X)[1]
  p <- dim(X)[2]  

  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Pre-processing
  # data validity
  if (!check_data_matrix(X)) {
    stop("* linreg_ADMM_GLASSO_cv : input 'X' is invalid data matrix.")  
  }
  if (!check_data_vector(y)) {
    stop("* linreg_ADMM_GLASSO_cv : input 'y' is invalid data vector.")  
  }
  y = as.vector(y)
  
  # data size
  if (n != length(y)) {
    stop("* linreg_ADMM_GLASSO_cv : two inputs 'X' and 'y' have non-matching dimension.")
  }
  if (!is.null(Z)) {
    if (dim(Z)[1] != length(y)) {
      stop("* linreg_ADMM_GLASSO : two inputs 'Z' and 'Y' have non-matching dimension.")
    }
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get dimensions
  if (standardize.data == TRUE) {
    if (intercept == TRUE){
      intercept = FALSE
      warning("* linreg_ADMM_OVGLASSO_cv : intercept should be set to FALSE is data are standardized!\n")
    }
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # add constant
  if (intercept == TRUE) {
    if (!is.null(Z)) {
      Z <- cbind(matrix(data = 1, nrow = length(y), ncol = 1), Z)
    } else {
      Z <- matrix(data = 1, nrow = length(y), ncol = 1)
    }
  }

  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Standardise response and design matrix
  if (standardize.data == TRUE) {
    res   <- standardizemat(X, y)
    X.std <- res$X.std
    y.std <- res$y.std
    mU    <- res$mU
    mV    <- res$mV
  } else {
    X.std <- X
    y.std <- y
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # check for inputs
  npar   <- dim(X.std)[2]
  con    <- list(maxit                = 100L, 
                 abstol               = sqrt(.Machine$double.eps), 
                 reltol               = sqrt(sqrt(.Machine$double.eps)), 
                 adaptation           = TRUE, 
                 rho                  = 1, 
                 tau.ada              = 2,               
                 mu.ada               = 10,
                 print.out            = TRUE)
  
  nmsC                          <- names(con)
  con[(namc <- names(control))] <- control
  if (length(noNms <- namc[!namc %in% nmsC])) {
    warning("unknown names in control: ", paste(noNms, collapse = ", "))
  }
  # set default values (for inputs)
  maxit                <- con$maxit
  abstol               <- con$abstol
  reltol               <- con$reltol
  adaptation           <- con$adaptation
  rho                  <- con$rho
  tau.ada              <- con$tau.ada      
  mu.ada               <- con$mu.ada
  print.out            <- con$print.out
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Pre-processing
  # data validity
  if (!check_param_constant_multiple(c(abstol, reltol))) {
    stop("* linreg_ADMM_GLASSO_cv : tolerance level is invalid.")
  }
  if (!check_param_integer(maxit, 0.0)) {
    stop("* linreg_ADMM_GLASSO_cv : 'maxiter' should be a positive integer.")
  }
  maxiter = as.integer(maxit)
  if (!check_param_constant(rho, 0.0)) {
    stop("* linreg_ADMM_GLASSO_cv : 'rho' should be a positive real number.")
  }
  if (!check_param_constant(mu.ada, 0.0)) {
    stop("* linreg_ADMM_GLASSO_cv : 'alpha' should be a positive real number.")
  }
  if ((tau.ada < 1.0) || (tau.ada > 2.0)) {
    warning("* linreg_ADMM_GLASSO_cv : 'tau' value is suggested to be in [1,2].")
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Pre-processing: variable weights
  if (!is.null(var_weights)) {
    if (!is.numeric(var_weights)) {
      stop("* linreg_ADMM_GLASSO_cv : the vector 'var_weights' is not a numeric vector.")
    }
    if (dim(X)[2] != length(var_weights)) {
      stop("* linreg_ADMM_GLASSO_cv : the length of the vector 'var_weights' does not match the number of penalized predictors.")
    }
    if (min(var_weights) < 0.0) {
      stop("* linreg_ADMM_GLASSO_cv : the values in 'var_weights' must be positive.")
    }
  }

  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Manage groups
  if (is.list(groups) && !is.null(groups$Glen)) {
    nG      <- groups$Glen
    groups_ <- groups$groups
    GRmat   <- groups2mat_GLASSO(groups = groups_, Glen = nG)
    G       <- dim(GRmat)[1]
  } else {
    groups_ <- groups
    GRmat   <- groups2mat_GLASSO(groups = groups_, Glen = NULL)
    G       <- dim(GRmat)[1]
    nG      <- rowSums(GRmat)
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Check overlapping groups
  res.method <- check_group_overlaps(GRmat)
  if (res.method == "ovglasso") {
    stop("* linreg_ADMM_GLASSO : this routine is for fitting the linear regression model with group-LASSO penalty. 
            Use routine 'linreg_ADMM_OVGLASSO' for fitting the linear regression model with overlap group-LASSO penalty!")
  }
  if (res.method == "lasso") {
    warning("* linreg_ADMM_GLASSO : this routine is for fitting the linear regression model with group-LASSO penalty. 
            Use (faster) routine 'linreg_ADMM_LASSO' instead or check the provided groups!")
  }

  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Manage group & overall group weights
  if (is.null(group_weights)) {
    # Define the weights as in Yuan and Lin 2006, JRSSB
    group_weights <- sqrt(diag(tcrossprod(GRmat)))
  } else {
    check_weights(x = group_weights, n = G, algname = "linreg_ADMM_GLASSO", funname = "group_weights")
  } 
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Manage variable weights
  if (is.null(var_weights)) {
    var_weights <- 1.0 / colSums(GRmat)
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get the sequence of lambdas
  if (!is.null(nlambda) && is.null(lambda)) {
    # get the smallest value of such that the regression 
    # coefficients estimated by the lasso are all equal to zero
    lambda <- lm_lambdamax_GLASSO(y = y.std, X = X.std, Z = Z,
                                  GRmat = GRmat, group_weights = group_weights, var_weights = var_weights, 
                                  lambda.min.ratio = lambda.min.ratio, maxl = nlambda)$lambda.seq
  }
  if (is.null(nlambda) && is.null(lambda)) {
    # get the smallest value of such that the regression 
    # coefficients estimated by the lasso are all equal to zero
    lambda <- lm_lambdamax_GLASSO(y = y.std, X = X.std, Z = Z,
                                  GRmat = GRmat, group_weights = group_weights, var_weights = var_weights, 
                                  lambda.min.ratio = lambda.min.ratio, maxl = 30)$lambda.seq
  }
  if (!is.null(lambda)) {
    nlambda <- length(lambda)
  }
  
  # other parameters
  meps     = (.Machine$double.eps)
  negsmall = -meps
  if (!check_param_constant(min(lambda), negsmall)) {
    stop("* linreg_ADMM_GLASSO_cv : reg. parameter 'lambda' is invalid.")
  }
  if (min(lambda) < meps) {
    message("* linreg_ADMM_GLASSO_cv : since 'lambda' is effectively zero, a least-squares solution is returned.")
    xsol     = as.vector(aux_pinv(X) %*% matrix(y))
    output   = list()
    output$x = xsol
    return(output)
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get groups
  ret.gr <- groups.cv(n = n, k = cv.fold)
  
  # store Lasso regression for each fold
  vMse_cv <- matrix(0.0, cv.fold, nlambda)
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Initial time 
  startTime <- Sys.time()
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Run lasso
  for (kt in 1:cv.fold) {
    # run ADMM
    mX_ <- X.std[ret.gr$groups.cv[[kt]],]
    vY_ <- y.std[ret.gr$groups.cv[[kt]]]
    if (is.null(Z)) {
      ret <- .admm_glasso_fast(A =  mX_, b = vY_, groups = GRmat, group_weights = group_weights, var_weights = var_weights,
                               lambda = lambda, rho_adaptation = adaptation, rho = rho, tau = tau.ada, mu = mu.ada,
                               reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
      
      # ret <- .Call("admm_glasso_fast", 
      #              A =  mX_, b = vY_, groups = GRmat, group_weights = group_weights, var_weights = var_weights,
      #              lambda = lambda, rho_adaptation = adaptation, rho = rho, tau = tau.ada, mu = mu.ada,
      #              reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
      
      # fit
      mSpRegP_     <- ret$coef.path
      fit          <- X.std[ret.gr$groups.pred[[kt]],] %*% t(mSpRegP_)
    } else {
      if (dim(Z)[2] > 1) {
        mZ_ <- Z[ret.gr$groups.cv[[kt]],]
      } else {
        mZ_ <- matrix(Z[ret.gr$groups.cv[[kt]],], nrow = length(ret.gr$groups.cv[[kt]]), ncol = 1)
      }
      ret <- .admm_glasso_cov_fast(W = mX_, Z = mZ_, y = vY_,
                                   groups = GRmat, group_weights = group_weights, var_weights = var_weights,
                                   lambda = lambda, rho_adaptation = adaptation, rho = rho, tau = tau.ada, mu.ada,
                                   reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
      
      # ret <- .Call("admm_glasso_cov_fast", 
      #              W = mX_, Z = mZ_, y = vY_,
      #              groups = GRmat, group_weights = group_weights, var_weights = var_weights, 
      #              lambda = lambda, rho_adaptation = adaptation, rho = rho, tau = tau.ada, mu.ada,
      #              reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
      # fit
      mSpRegP_     <- ret$sp.coef.path
      mRegP_       <- ret$coef.path
      fit          <- X.std[ret.gr$groups.pred[[kt]],] %*% t(mSpRegP_) + Z[ret.gr$groups.pred[[kt]],] %*% t(mRegP_)
    }
    
    # Evaluate MSE
    n_           <- length(y.std[ret.gr$groups.pred[[kt]]])
    mse          <- apply(fit, 2, function(x) {sum((y.std[ret.gr$groups.pred[[kt]]] - x)^2)/n_})
    vMse_cv[kt,] <- mse
  }
    
  # get mean squared error
  vMse <- apply(vMse_cv, 2, mean) 
  vSd  <- apply(vMse_cv, 2, sd) 

  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # system time
  endTime <- Sys.time()
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # compute run time
  eltime <- difftime(time1 = endTime, time2 = startTime, units = "secs")
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get lasso output
  min.mse      <- min(vMse)
  indi.min.mse <- which(vMse == min.mse) 
  lambda.min   <- lambda[indi.min.mse]
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # get estimate over the full sample
  rho_ <- ret$rho[[indi.min.mse]]
  if (is.null(Z)) {
    ret  <- .admm_glasso(A = X.std, b = y.std,
                         groups = GRmat, group_weights = group_weights, var_weights = var_weights,
                         u = t(ret$U[indi.min.mse,]), z = t(ret$Z[indi.min.mse,]), lambda = lambda.min,
                         rho_adaptation = adaptation, rho = rho_[length(rho_)], tau = tau.ada, mu = mu.ada,
                         reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
    
    # ret  <- .Call("admm_glasso", 
    #               A = X.std, b = y.std, 
    #               groups = GRmat, group_weights = group_weights, var_weights = var_weights, 
    #               u = t(ret$U[indi.min.mse,]), z = t(ret$Z[indi.min.mse,]), lambda = lambda.min, 
    #               rho_adaptation = adaptation, rho = rho_[length(rho_)], tau = tau.ada, mu = mu.ada,
    #               reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
    
    # get estimated parameters
    vSpRegP <- ret$x
  } else {
    ret  <- .admm_glasso_cov(W = X.std, Z = Z, y = y.std, u = t(ret$U[indi.min.mse,]), z = t(ret$Z[indi.min.mse,]),
                             groups = GRmat, group_weights = group_weights, var_weights = var_weights,
                             lambda = lambda.min, rho_adaptation = adaptation, rho = rho_[length(rho_)], tau = tau.ada, mu = mu.ada,
                             reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
    
    # ret  <- .Call("admm_glasso_cov", 
    #               W = X.std, Z = Z, y = y.std, u = t(ret$U[indi.min.mse,]), z = t(ret$Z[indi.min.mse,]),
    #               groups = GRmat, group_weights = group_weights, var_weights = var_weights,
    #               lambda = lambda.min, rho_adaptation = adaptation, rho = rho_[length(rho_)], tau = tau.ada, mu = mu.ada,
    #               reltol = reltol, abstol = abstol, maxiter = maxit, ping = 0)
    
    # get estimated parameters
    vSpRegP <- ret$x
    vRegP   <- ret$v
  }
  
  # Store output: full sample estimate
  niter <- ret$niter
  if (niter < maxiter) {
    converged <- TRUE
  } else {
    converged <- FALSE
  }
  dConv  <- ifelse(converged == TRUE, 1, 0) 
  nIterN <- ret$niter

  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Retrieve estimated parameters
  if (standardize.data == TRUE) {
    vSpRegP_ <- matrix(solve(mU) %*% vSpRegP %*% mV, nrow = p, ncol = 1)
    if (!is.null(Z)) {
      vRegP <- vRegP %*% mV
    }
  } else {
    vSpRegP_ <- matrix(vSpRegP, nrow = p, ncol = 1)
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Print to screen
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  if (print.out == TRUE) {
    cat("\n\n\n")
    cat("Linear regression model with group-LASSO penalty, Cross Validation\n")
    cat("Alternating direction method of multipliers\n")
    cat("running time (for ", reltol, " relative error):",
        endTime - startTime, "seconds \n\n\n")
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get output
  res.names <- c("sp.coefficients", 
                 "coefficients",
                 "mse",
                 "mse.sd",
                 "min.mse",
                 "lambda",
                 "lambda.min",
                 "convergence",
                 "elapsedTime",
                 "iternum",
                 "indi.min.mse",
                 "foldid") 
  res        <- vector(mode = "list", length = length(res.names))
  names(res) <- res.names
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Get output
  ret                 <- NULL
  ret$sp.coefficients <- vSpRegP_
  if (!is.null(Z)) {
    ret$coefficients <- vRegP
  }
  ret$mse          <- vMse
  ret$mse.sd       <- vSd
  ret$min.mse      <- min.mse
  ret$lambda       <- lambda
  ret$lambda.min   <- lambda.min
  ret$convergence  <- dConv
  ret$elapsedTime  <- eltime
  ret$iternum      <- nIterN
  ret$indi.min.mse <- indi.min.mse
  ret$foldid       <- ret.gr$foldid
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Return output
  return(ret)
}

#
## last update: 30 september 2025
#

#
## Description: compute dual norms for several Lasso-type penalties
#

# group-lasso norm
#' @keywords internal
glasso_norm <- function(x, groups, weights = NULL) {
  
  # get dimensions
  p <- length(x)
  G <- dim(groups)[1]

  # check for null weights
  if (is.null(weights)) {
    weights <- rep(1, G)
  }
  
  # compute the group lasso norm
  norm2 <- 0.0
  for (g in 1:G) {
    idx   <- which(groups[g,] == 1)
    norm2 <- norm2 + weights[g] * norm(x[idx], type = "2")
  }
  
  # return output
  return(norm2)
}
# overlap group-lasso norm
#' @keywords internal
ovglasso_norm <- function(x, groups, weights = NULL) {
  
  # return output
  return(glasso_norm(x = x, groups = groups, weights = weights))
}
# group-lasso dual norm
#' @keywords internal
glasso_dual_norm <- function(x, groups, weights = NULL) {
  
  # get dimensions
  p <- length(x)
  G <- dim(groups)[1]
  
  # check for null weights
  if (is.null(weights)) {
    weights <- rep(1, G)
  }
  
  # compute the glasso dual norm
  dual_norm2 <- rep(0, G)
  for (g in 1:G) {
    idx           <- which(groups[g,] == 1)
    dual_norm2[g] <- norm(x[idx], type = "2") / weights[g]
  }
  
  # return output
  return(max(dual_norm2))
}
# This function compute the non-overlap approximation of overlap group lasso norm
# provided by Qi and Li (2024, JoMLR) in Algorithm 2
#
# Input:  G  = matrice (g x p) binaria (righe = gruppi originari, colonne = variabili)
# Output: matrice (k x p) con gruppi non-overlapping
#' @keywords internal
ovg2g <- function(groups, weights = NULL, return_list = TRUE) {
  
  # get dimensions
  groups <- as.matrix(groups)
  G      <- nrow(groups)
  p      <- ncol(groups)
  
  # pattern per colonna (stringa di 0/1) — uso as.integer per gestire anche valori logici
  patterns <- apply(groups, 2, function(col) paste(as.integer(col), collapse = ""))
  
  # unici nell'ordine di primo apparire
  uniq <- unique(patterns)
  k    <- length(uniq)
  
  # costruisco G1
  G1           <- matrix(0, nrow = k, ncol = p)
  rownames(G1) <- paste0("group", seq_len(k))
  if (!is.null(colnames(groups))) {
    colnames(G1) <- colnames(groups) 
  } else {
    colnames(G1) <- paste0("var", seq_len(p))
  }
  parts       <- vector("list", length = k)
  orig_groups <- vector("list", length = k)
  
  for (i in seq_along(uniq)) {
    cols        <- which(patterns == uniq[i])
    G1[i, cols] <- 1
    parts[[i]]  <- cols
    # quali gruppi originari partecipano a questo pattern (indici righe)
    bits             <- strsplit(uniq[i], "")[[1]]
    orig_groups[[i]] <- which(bits == "1")
  }
  names(parts)       <- rownames(G1)
  names(orig_groups) <- rownames(G1)
  
  # get weights
  weights_ <- rep(0, length(orig_groups))
  if (is.null(weights)) {
    weights <- rep(1, G)
    for (i in 1:length(orig_groups)) {
      weights_[i] <- sum(weights[orig_groups[[i]]])
    }
  } else {
    for (i in 1:length(orig_groups)) {
      weights_[i] <- sum(weights[orig_groups[[i]]])
    }
  }
  
  # return output
  if (return_list) {
    output             <- NULL
    output$G1          <- G1
    output$orig_groups <- orig_groups
    output$weights     <- weights_
  } else {
    output <- G1
  }
  return(output)
}
# This function compute the upper bound of the overlap group lasso dual norm
# provided by Qi and Li (2024, JoMLR), Proposition 1 in Appendix C
#' @keywords internal
ovglasso_dual_norm_upper_bound <- function(x, groups, weights) {
  
  # get the diagonal matrix of overlap degrees
  H <- diag(1.0 / colSums(groups))
  
  # compute the approximate dual norm of the overlap group lasso
  res <- glasso_dual_norm(x = H %*% x, groups = groups, weights = weights)
  
  # return output
  return(res)
}
# This function compute the approximation of the overlap group lasso dual norm
# provided by Qi and Li (2024, JoMLR)
#' @keywords internal
ovglasso_dual_norm_approx <- function(x, groups, weights) {
  
  # get the non-overlap approximation of overlap group lasso norm
  res      <- ovg2g(groups = groups, weights = weights, return_list = TRUE)
  weights_ <- res$weights
  groups_  <- res$G1

  # compute the approximate dual norm of the overlap group lasso
  output <- glasso_dual_norm(x = x, groups = groups_, weights = weights_)
  
  # return output
  return(output)
}

#
## last update: 30 september 2025
#

#
## Description: compute the lambda max for linear regression 
##              model with several Lasso-type penalties
#

# Functions: 
#
#           1. lm_lambdamax_LASSO
#           2. lm_lambdamax_ADLASSO
#           3. lm_lambdamax_GLASSO
#           4. lm_lambdamax_spGLASSO
#           5. lm_lambdamax_OVGLASSO
#           6. lm_lambdamax

#' @keywords internal
lm_lambdamax_LASSO <- function(y, X, Z = NULL, 
                               lambda.min.ratio = NULL, 
                               maxl = NULL) {
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # check maxl
  if (is.null(maxl)) {
    maxl <- 30
  }
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # get the lambda max for LASSO
  n <- dim(X)[1]
  p <- dim(X)[2]
  if (is.null(Z)) {
    vXy        <- crossprod(X, y)
    lambda.max <- max(abs(vXy)) / n
  } else {
    regp       <- solve(crossprod(Z)) %*% crossprod(Z, y)
    y_         <- y - Z %*% regp
    vXy        <- crossprod(X, y_)
    lambda.max <- max(abs(vXy)) / n
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # get the lambda min for LASSO
  if (is.null(lambda.min.ratio)) {
    lambda.min.ratio <- ifelse(n < p, 0.01, 1e-04)
  }
  lambda.min <- lambda.min.ratio * lambda.max
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # get the sequence of lambdas
  lambda.seq <- logseq(from = lambda.max, to = lambda.min, length.out = maxl)
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # get output
  res            <- NULL
  res$lambda.min <- lambda.min
  res$lambda.max <- lambda.max
  res$lambda.seq <- lambda.seq
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # get output
  return(res)
}

#' @keywords internal
#' @export
lm_lambdamax_ADALASSO <- function(y, X, Z = NULL, 
                                  var_weights = NULL, 
                                  lambda.min.ratio = NULL, 
                                  maxl = NULL) {
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # check maxl
  if (is.null(maxl)) {
    maxl <- 30
  }
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # get the dimensions
  n <- dim(X)[1]
  p <- dim(X)[2]
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # check for adaptive-LASSO
  if (is.null(var_weights)) {
    var_weights <- rep(1L, p)
  } 
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # standardize data (to account adaptive weights)
  X_ <- X %*% diag(1.0 / var_weights)
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # get the lambda max for LASSO
  if (is.null(Z)) {
    vXy        <- crossprod(X_, y)
    lambda.max <- max(abs(vXy)) / n
  } else {
    regp       <- solve(crossprod(Z)) %*% crossprod(Z, y)
    y_         <- y - Z %*% regp
    vXy        <- crossprod(X_, y_)
    lambda.max <- max(abs(vXy)) / n
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # get the lambda min for LASSO
  if (is.null(lambda.min.ratio)) {
    lambda.min.ratio <- ifelse(n < p, 0.01, 1e-04)
  }
  lambda.min <- lambda.min.ratio * lambda.max
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # get the sequence of lambdas
  lambda.seq <- logseq(from = lambda.max, to = lambda.min, length.out = maxl)
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # get output
  res            <- NULL
  res$lambda.min <- lambda.min
  res$lambda.max <- lambda.max
  res$lambda.seq <- lambda.seq
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # get output
  return(res)
}

#' @keywords internal
#' @export
lm_lambdamax_GLASSO <- function(y, X, Z = NULL, 
                                groups = NULL, 
                                GRmat = NULL, 
                                group_weights = NULL, 
                                var_weights = NULL, 
                                lambda.min.ratio = NULL, 
                                maxl = NULL) {
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # check maxl
  if (is.null(maxl)) {
    maxl <- 30
  }
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # manage the non-penalized covariates (Z)
  if (is.null(Z)) {
    y_ <- y
  } else {
    regp <- solve(crossprod(Z)) %*% crossprod(Z, y)
    y_   <- y - Z %*% regp
  } 
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # get the relevant quantities
  n <- dim(X)[1]
  p <- dim(X)[2]
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Manage groups
  if (is.null(GRmat)) {
    if (is.list(groups) && !is.null(groups$Glen)) {
      nG      <- groups$Glen
      groups_ <- groups$groups
      GRmat   <- groups2mat_GLASSO(groups = groups_, Glen = nG)
      G       <- dim(GRmat)[1]
    } else {
      groups_ <- groups
      GRmat   <- groups2mat_GLASSO(groups = groups_, Glen = NULL)
      G       <- dim(GRmat)[1]
      nG      <- rowSums(GRmat)
    }
  } else {
    G  <- dim(GRmat)[1]
    nG <- rowSums(GRmat)
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Manage group weights
  if (is.null(group_weights)) {
    # Define the weights as in Yuan and Lin 2006, JRSSB
    group_weights <- sqrt(diag(tcrossprod(GRmat)))
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Manage variable weights
  if (is.null(var_weights)) {
    var_weights <- 1.0 / colSums(GRmat)
  }
  Tmat_INV <- diag(1.0 / var_weights)
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # standardize data (to account adaptive weights)
  X_ <- X %*% Tmat_INV
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # get the lambda max for Group LASSO
  lambda_ <- NULL
  for (g in 1:G) {
    idx     <- which(GRmat[g, ] == 1) 
    vXy_    <- as.numeric(crossprod(X_[,idx], y_))
    lam     <- norm(vXy_, type = "2") / group_weights[g]
    lambda_ <- c(lambda_, lam)
  }
  lambda.max <- max(lambda_)  / n
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # get the lambda min for LASSO
  if (is.null(lambda.min.ratio)) {
    lambda.min.ratio <- ifelse(n < p, 0.01, 1e-04)
  }
  lambda.min <- lambda.min.ratio * lambda.max
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # get the sequence of lambdas
  lambda.seq <- logseq(from = lambda.max, to = lambda.min, length.out = maxl)
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # get output
  res            <- NULL
  res$lambda.min <- lambda.min
  res$lambda.max <- lambda.max
  res$lambda.seq <- lambda.seq
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # get output
  return(res)
}

# lambdamax for sparse group lasso as in the original paper of
# Noah Simon, Jerome Friedman, Trevor Hastie & Robert Tibshirani, JoCGS 2013

#' @keywords internal
#' @export
spglasso_lambdamax_objfun <- function(x, X, y, 
                                      alpha, gwei) {
  
  # get dimensions
  n <- length(y)
  
  # get the relevant quantities
  XTy    <- as.numeric(crossprod(X, y)) / n
  lambda <- x
  
  # Evaluate the objective function
  Svec   <- softthreshold_VEC(xx = XTy, lambda = (lambda * alpha))
  objfun <- as.numeric(crossprod(Svec)) - gwei^2 * (1.0 - alpha)^2 * lambda^2
  
  # return
  return(objfun)
}
spglasso_lambdamax_objfun_VEC <- Vectorize(spglasso_lambdamax_objfun, vectorize.args = "x")
softthreshold_VEC <- Vectorize(FUN = softthreshold, vectorize.args = "xx")

lm_lambdamax_spGLASSO <- function(y, X, Z = NULL, groups, group_weights = NULL, var_weights = NULL, 
                                  var_weights_L1 = NULL, lambda.min.ratio = NULL, maxl = NULL, alpha = NULL, 
                                  tol = .Machine$double.eps^0.25) {
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # check maxl
  if (is.null(maxl)) {
    maxl <- 30
  }
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # manage the non-penalized covariates (Z)
  if (is.null(Z)) {
    y_ <- y
  } else {
    regp <- solve(crossprod(Z)) %*% crossprod(Z, y)
    y_   <- y - Z %*% regp
  } 
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # get the relevant quantities
  n <- dim(X)[1]
  p <- dim(X)[2]
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # get the lambda min for LASSO
  if (is.null(alpha)) {
    alpha <- 0.5
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Manage groups
  if (is.list(groups) && !is.null(groups$Glen)) {
    nG      <- groups$Glen
    groups_ <- groups$groups
    GRmat   <- groups2mat_GLASSO(groups = groups_, Glen = nG)
    G       <- dim(GRmat)[1]
  } else {
    groups_ <- groups
    GRmat   <- groups2mat_GLASSO(groups = groups_, Glen = NULL)
    G       <- dim(GRmat)[1]
    nG      <- rowSums(GRmat)
  }
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Manage group weights
  if (is.null(group_weights)) {
    # Define the weights as in Yuan and Lin 2006, JRSSB
    group_weights <- sqrt(diag(tcrossprod(GRmat)))
  }
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Manage variable weights: adaptive Lasso
  if (is.null(var_weights_L1)) {
    var_weights_L1 <- rep(1, p)
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Manage variable weights
  if (is.null(var_weights)) {
    var_weights <- 1.0 / colSums(GRmat)
  }
  Tmat_INV         <- diag(1.0 / var_weights)
  Tmat_L1_INV_SQRT <- diag(1.0 / sqrt(var_weights_L1))
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # get the lambda max for Group LASSO
  if (alpha == 0) {
    X_ <- X %*% Tmat_INV
    # Group LASSO
    lambda_ <- NULL
    for (g in 1:G) {
      idx     <- which(GRmat[g, ] == 1) 
      vXy_    <- as.numeric(crossprod(X_[,idx], y_))
      lam     <- norm(vXy_, type = "2") / group_weights[g]
      lambda_ <- c(lambda_, lam)
    }
    lambda.max <- max(lambda_)  / n
  } else if (alpha == 1) {
    # LASSO
    X_         <- X %*% Tmat_L1_INV_SQRT
    vXy        <- crossprod(X_, y_)
    lambda.max <- max(abs(vXy)) / n
  } else {
    X_ <- X %*% Tmat_INV
    # sparse GLASSO
    lambda_ <- NULL
    for (g in 1:G) {
      idx     <- which(GRmat[g, ] == 1) 
      lam     <- uniroot(f = spglasso_lambdamax_objfun, interval = c(0.0, 100.0),
                         X = X_[,idx], y = y_, alpha = alpha, gwei = group_weights[g], extendInt = "yes")$root
      lambda_ <- c(lambda_, lam)
    }
    lambda.max <- max(lambda_)  
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # get the lambda min for LASSO
  if (is.null(lambda.min.ratio)) {
    lambda.min.ratio <- ifelse(n < p, 0.01, 1e-04)
  }
  lambda.min <- lambda.min.ratio * lambda.max
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # get the sequence of lambdas
  lambda.seq <- logseq(from = lambda.max, to = lambda.min, length.out = maxl)
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # get output
  res            <- NULL
  res$lambda.min <- lambda.min
  res$lambda.max <- lambda.max
  res$lambda.seq <- lambda.seq
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # get output
  return(res)
}

#' @keywords internal
#' @export
lm_lambdamax_GLASSO_yuan_lin_2006_JRSSB <- function(y, X, groups, 
                                                    lambda.min.ratio = NULL, 
                                                    maxl = NULL) {
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # check maxl
  if (is.null(maxl)) {
    maxl <- 30
  }
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # get the relevant quantities
  n <- dim(X)[1]
  p <- dim(X)[2]
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Manage groups
  if (is.list(groups) && !is.null(groups$Glen)) {
    nG      <- groups$Glen
    groups_ <- groups$groups
    GRmat   <- groups2mat_GLASSO(groups = groups_, Glen = nG)
    G       <- dim(GRmat)[1]
  } else {
    groups_ <- groups
    GRmat   <- groups2mat_GLASSO(groups = groups_, Glen = NULL)
    G       <- dim(GRmat)[1]
    nG      <- rowSums(GRmat)
  }
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # get the lambda max for Group LASSO
  # (https://stats.stackexchange.com/questions/340486/how-is-the-minimum-lambda-computed-in-group-lasso)
  lambda_ <- NULL
  for (g in 1:G) {
    idx     <- which(GRmat[g, ] == 1) 
    X_      <- X[,idx] 
    vXy_    <- as.numeric(crossprod(X_, y))
    lam     <- norm(vXy_, type = "2") / sqrt(nG[g])
    lambda_ <- c(lambda_, lam)
  }
  lambda.max <- max(lambda_)  / n
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # get the lambda min for LASSO
  if (is.null(lambda.min.ratio)) {
    lambda.min.ratio <- ifelse(n < p, 0.01, 1e-04)
  }
  lambda.min <- lambda.min.ratio * lambda.max
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # get the sequence of lambdas
  lambda.seq <- logseq(from = lambda.max, to = lambda.min, length.out = maxl)
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # get output
  res            <- NULL
  res$lambda.min <- lambda.min
  res$lambda.max <- lambda.max
  res$lambda.seq <- lambda.seq
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # get output
  return(res)
}

#' @keywords internal
#' @export
lm_lambdamax_OVGLASSO <- function(y, X, Z = NULL, groups = NULL, GRmat = NULL, 
                                  group_weights = NULL, var_weights = NULL, 
                                  lambda.min.ratio = NULL, maxl = NULL) {
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # check maxl
  if (is.null(maxl)) {
    maxl <- 30
  }
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # get the relevant quantities
  n <- dim(X)[1]
  p <- dim(X)[2]
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # manage the non-penalized covariates (Z)
  if (is.null(Z)) {
    y_ <- y
  } else {
    regp <- solve(crossprod(Z)) %*% crossprod(Z, y)
    y_   <- y - Z %*% regp
  } 
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Manage groups
  if (is.null(GRmat)) {
    if (is.list(groups) && !is.null(groups$Glen)) {
      nG      <- groups$Glen
      groups_ <- groups$groups
      GRmat   <- groups2mat_OVGLASSO(groups = groups_, Glen = nG)
      G       <- dim(GRmat)[1]
    } else {
      groups_ <- groups
      GRmat   <- groups2mat_OVGLASSO(groups = groups_, Glen = NULL)
      G       <- dim(GRmat)[1]
      nG      <- rowSums(GRmat)
    }
  } else {
    G  <- dim(GRmat)[1]
    nG <- rowSums(GRmat)
  }
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Manage group weights
  if (is.null(group_weights)) {
    # Define the weights as in Yuan and Lin 2006, JRSSB
    #group_weights <- sqrt(diag(tcrossprod(GRmat)))
    # Modificata il 19 novembre 2024
    group_weights <- rep(1, G)
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Manage variable weights
  if (is.null(var_weights)) {
    var_weights <- 1.0 / sqrt(colSums(GRmat))
  }
  Tmat_INV <- diag(1.0 / var_weights)
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # standardize data (to account adaptive weights)
  X_ <- X %*% Tmat_INV
  #X_ <- X 
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # get the lambda max for Overlap Group LASSO
  # lambda_ <- NULL
  # for (g in 1:G) {
  #   idx     <- which(GRmat[g, ] == 1) 
  #   vXy_    <- as.numeric(crossprod(X_[,idx], y_))
  #   lam     <- norm(vXy_, type = "2") / group_weights[g]
  #   lambda_ <- c(lambda_, lam)
  # }
  # lambda.max <- max(lambda_)  / n
  vXy_    <- as.numeric(crossprod(X_, y_))
  lambda_ <- ovglasso_dual_norm_upper_bound(x = vXy_, 
                                            groups = GRmat, 
                                            weights = group_weights)
  lambda.max <- lambda_ / n
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # get the lambda min for LASSO
  if (is.null(lambda.min.ratio)) {
    lambda.min.ratio <- ifelse(n < p, 0.01, 1e-04)
  }
  lambda.min <- lambda.min.ratio * lambda.max
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # get the sequence of lambdas
  lambda.seq <- logseq(from = lambda.max, to = lambda.min, length.out = maxl)
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # get output
  res            <- NULL
  res$lambda.min <- lambda.min
  res$lambda.max <- lambda.max
  res$lambda.seq <- lambda.seq
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # get output
  return(res)
}

#' @keywords internal
#' @export
lm_lambdamax_spOVGLASSO <- function(y, X, Z = NULL, groups = NULL, GRmat = NULL, 
                                    group_weights = NULL, var_weights = NULL, 
                                    lambda.min.ratio = NULL, maxl = NULL) {
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # check maxl
  if (is.null(maxl)) {
    maxl <- 30
  }
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # get the relevant quantities
  n <- dim(X)[1]
  p <- dim(X)[2]
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # manage the non-penalized covariates (Z)
  if (is.null(Z)) {
    y_ <- y
  } else {
    regp <- solve(crossprod(Z)) %*% crossprod(Z, y)
    y_   <- y - Z %*% regp
  } 
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Manage groups
  if (is.null(GRmat)) {
    if (is.list(groups) && !is.null(groups$Glen)) {
      nG      <- groups$Glen
      groups_ <- groups$groups
      GRmat   <- groups2mat_OVGLASSO(groups = groups_, Glen = nG)
      GRmat   <- rbind(GRmat, diag(1, p))
      G       <- dim(GRmat)[1]
    } else {
      groups_ <- groups
      GRmat   <- groups2mat_OVGLASSO(groups = groups_, Glen = NULL)
      GRmat   <- rbind(GRmat, diag(1, p))
      G       <- dim(GRmat)[1]
      nG      <- rowSums(GRmat)
    }
  } else {
    G  <- dim(GRmat)[1]
    nG <- rowSums(GRmat)
  }
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Manage group weights
  if (is.null(group_weights)) {
    # Define the weights as in Yuan and Lin 2006, JRSSB
    #group_weights <- sqrt(diag(tcrossprod(GRmat)))
    # Modificata il 19 novembre 2024
    group_weights <- rep(1, G)
  }
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # Manage variable weights
  if (is.null(var_weights)) {
    var_weights <- 1.0 / sqrt(colSums(GRmat))
  }
  Tmat_INV <- diag(1.0 / var_weights)
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # standardize data (to account adaptive weights)
  X_ <- X %*% Tmat_INV
  #X_ <- X
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # get the lambda max for Overlap Group LASSO
  # lambda_ <- NULL
  # for (g in 1:G) {
  #   idx     <- which(GRmat[g, ] == 1) 
  #   vXy_    <- as.numeric(crossprod(X_[,idx], y_))
  #   lam     <- norm(vXy_, type = "2") / group_weights[g]
  #   lambda_ <- c(lambda_, lam)
  # }
  # lambda.max <- max(lambda_)  / n
  vXy_    <- as.numeric(crossprod(X_, y_))
  lambda_ <- ovglasso_dual_norm_upper_bound(x = vXy_, 
                                            groups = GRmat, 
                                            weights = group_weights)
  lambda.max <- max(lambda_)  / n
  
  # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # get the lambda min for LASSO
  if (is.null(lambda.min.ratio)) {
    lambda.min.ratio <- ifelse(n < p, 0.01, 1e-04)
  }
  lambda.min <- lambda.min.ratio * lambda.max
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # get the sequence of lambdas
  lambda.seq <- logseq(from = lambda.max, to = lambda.min, length.out = maxl)
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # get output
  res            <- NULL
  res$lambda.min <- lambda.min
  res$lambda.max <- lambda.max
  res$lambda.seq <- lambda.seq
  
  # :::::::::::::::::::::::::::::::::::::::::::
  # get output
  return(res)
}

#' @keywords internal
#' @export
lm_lambdamax <- function(method = c("lasso", "ovglasso"),
                         y, X, p = NULL, q = NULL, groups = NULL, 
                         lambda.min = NULL, maxl = NULL) {
    
  # :::::::::::::::::::::::::::::::::::::::::::
  # check maxl
  if (is.null(maxl)) {
    maxl <- 30
  }

  if (strcmpi(method, "lasso")) {
    res <- lm_lambdamax_LASSO(y, X, lambda.min, maxl)
  } else if (strcmpi(method, "ovglasso")) {
    res <- lm_lambdamax_OVGLASSO(y, X, groups, lambda.min, maxl)
  }
  return(res)
}

# for non consecutive grouping structures, nG specifies the number of variables
# belonging to each group
groups2mat_GLASSO <- function(groups, Glen = NULL) {
  if (is.null(Glen)) {
    grVec <- unique(groups)
    G     <- length(grVec)
    p     <- length(groups)
    GRmat <- matrix(data = 0.0, nrow = G, ncol = p)
    for (i in 1:G) {
      idx          <- groups == grVec[i]
      GRmat[i,idx] <- 1
    }
  } else {
    # non consecutive indeces
    G     <- length(Glen)
    p     <- length(groups)
    GRmat <- matrix(data = 0.0, nrow = G, ncol = p)
    idx1  <- 1
    for (i in 1:G) {
      idx2         <- idx1 + Glen[i] - 1
      idx          <- groups[idx1:idx2]
      GRmat[i,idx] <- 1
      idx1         <- idx2 + 1
    }
  }
  if (max(colSums(GRmat))>1) {
    stop("* groups2mat_GLASSO : groups overlaps.")  
  }
  return(GRmat)
}

# for non consecutive grouping structures, nG specifies the number of variables
# belonging to each group

#' @keywords internal
#' @export
groups2mat_OVGLASSO <- function(groups, Glen = NULL) {
  if (is.null(Glen)) {
    grVec <- unique(groups)
    G     <- length(grVec)
    p     <- length(groups)
    GRmat <- matrix(data = 0.0, nrow = G, ncol = p)
    for (i in 1:G) {
      idx          <- groups == grVec[i]
      GRmat[i,idx] <- 1
    }
  } else {
    # non consecutive indeces
    G     <- length(Glen)
    p     <- length(unique(groups))
    GRmat <- matrix(data = 0.0, nrow = G, ncol = p)
    idx1  <- 1
    for (i in 1:G) {
      idx2 <- idx1 + Glen[i] - 1
      idx          <- groups[idx1:idx2]
      GRmat[i,idx] <- 1
      idx1         <- idx2 + 1
    }
  }
  return(GRmat)
}
# Dual norm of overlap group-lasso via convex optimization
#' @keywords internal
matrix_to_groups <- function(GRmat) {
  apply(GRmat, 1, function(row) which(row == 1))
}
#' @keywords internal
#' @export
ovglasso_dual_norm <- function(x, groups, weights = NULL) {
  
  # require library
  #require(CVXR)
  
  # get dimensions
  p <- length(x)
  
  # convert groups matrix to list
  groups_list <- matrix_to_groups(groups)
  
  if (is.null(weights)) {
    weights <- rep(1, length(groups_list))
  }
  
  # primal variable
  z <- CVXR::Variable(p)
  
  # define primal OGL norm
  group_terms <- lapply(seq_along(groups_list), function(i) {
    g <- groups_list[[i]]
    w <- weights[i]
    w * CVXR::norm2(z[g])
  })
  ogl_norm <- Reduce("+", group_terms)
  
  # dual norm = sup { <x,z> : Omega(z) <= 1 }
  obj    <- CVXR::Maximize(t(x) %*% z)
  constr <- list(ogl_norm <= 1)
  prob   <- CVXR::Problem(obj, constr)
  result <- CVXR::psolve(prob)
  
  # return output
  return(result$value)
}

# Authors: Mauro Bernardi 
#          Department Statistical Sciences
#       	 University of Padova
#      	   Via Cesare Battisti, 241
#      	   35121 PADOVA, Italy
#          E-mail: mauro.bernardi@unipd.it  

# Last change: March 16, 2023

#' Sparse Adaptive Overlap Group Least Absolute Shrinkage and Selection Operator
#'
#' Sparse Adaptive overlap group-LASSO, or sparse adaptive group \eqn{L_2}-regularized regression, solves the following optimization problem
#' \deqn{\textrm{min}_{\beta,\gamma} ~ \frac{1}{2}\|y-X\beta-Z\gamma\|_2^2 + \lambda\Big[(1-\alpha) \sum_{g=1}^G \|S_g T\beta\|_2+\alpha\Vert T_1\beta\Vert_1\Big]}
#' to obtain a sparse coefficient vector \eqn{\beta\in\mathbb{R}^p} for the matrix of penalized predictors \eqn{X} and a coefficient vector \eqn{\gamma\in\mathbb{R}^q} 
#' for the matrix of unpenalized predictors \eqn{Z}. For each group \eqn{g}, each row of 
#' the matrix \eqn{S_g\in\mathbb{R}^{n_g\times p}} has non-zero entries only for those variables belonging 
#' to that group. These values are provided by the arguments \code{groups} and \code{group_weights} (see below). 
#' Each variable can belong to more than one group. The diagonal matrix \eqn{T\in\mathbb{R}^{p\times p}} contains the variable-specific weights. These values are
#' provided by the argument \code{var_weights} (see below). The diagonal matrix \eqn{T_1\in\mathbb{R}^{p\times p}} contains 
#' the variable-specific \eqn{L_1} weights. These values are provided by the argument \code{var_weights_L1} (see below).
#' The regularization path is computed for the sparse adaptive overlap group-LASSO penalty at a grid of values for the regularization 
#' parameter \eqn{\lambda} using the alternating direction method of multipliers (ADMM). See Boyd et al. (2011) and Lin et al. (2022) 
#' for details on the ADMM method. The regularization is a combination of \eqn{L_2} and
#' \eqn{L_1} simultaneous constraints. Different specifications of the \code{penalty} argument lead to different models choice:
#' \describe{
#' \item{LASSO}{The classical Lasso regularization (Tibshirani, 1996) can be obtained by specifying \eqn{\alpha = 1} 
#' and the matrix \eqn{T_1} as the \eqn{p \times p} identity matrix. An adaptive version of this model (Zou, 2006) can be obtained if \eqn{T_1} is 
#' a \eqn{p \times p}  diagonal matrix of adaptive weights. See also Hastie et al. (2015) for further details.}
#' \item{GLASSO}{The group-Lasso regularization (Yuan and Lin, 2006) can be obtained by specifying \eqn{\alpha = 0}, 
#' non-overlapping groups in \eqn{S_g} and by setting the matrix \eqn{T} equal to the \eqn{p \times p} identity matrix. 
#' An adaptive version of this model can be obtained if the matrix \eqn{T} is a \eqn{p \times p} diagonal matrix of adaptive weights. See also Hastie et al. (2015) for further details.}
#' \item{spGLASSO}{The sparse group-Lasso regularization (Simon et al., 2011) can be obtained by specifying \eqn{\alpha\in(0,1)},  
#' non-overlapping groups in \eqn{S_g} and by setting the matrices \eqn{T} and \eqn{T_1} equal to the \eqn{p \times p} identity matrix. 
#' An adaptive version of this model can be obtained if the matrices \eqn{T} and \eqn{T_1} are \eqn{p \times p}  
#' diagonal matrices of adaptive weights.}
#' \item{OVGLASSO}{The overlap group-Lasso regularization (Jenatton et al., 2011) can be obtained by specifying 
#' \eqn{\alpha = 0}, overlapping groups in \eqn{S_g} and by setting the matrix \eqn{T} equal to the \eqn{p \times p} identity matrix. An adaptive version of this model can be obtained if the matrix \eqn{T} is a \eqn{p \times p}  
#' diagonal matrix of adaptive weights.}
#' \item{spOVGLASSO}{The sparse overlap group-Lasso regularization (Jenatton et al., 2011) can be obtained by specifying 
#' \eqn{\alpha\in(0,1)}, overlapping groups in \eqn{S_g} and by setting the matrices \eqn{T} and \eqn{T_1} equal to the \eqn{p \times p} identity matrix. 
#' An adaptive version of this model can be obtained if the matrices \eqn{T} and \eqn{T_1} are \eqn{p \times p}  diagonal matrices of adaptive weights.}
#' }
#'  
#' @param X an \eqn{(n\times p)} matrix of penalized predictors.
#' @param Z an \eqn{(n\times q)} full column rank matrix of predictors that are not penalized. 
#' @param y a length-\eqn{n} response vector.
#' @param penalty choose one from the following options: 'LASSO', for the or adaptive-Lasso penalties, 'GLASSO', 
#' for the group-Lasso penalty, 'spGLASSO', for the sparse group-Lasso penalty, 'OVGLASSO', 
#' for the overlap group-Lasso penalty and 'spOVGLASSO', for the sparse overlap group-Lasso penalty.
#' @param groups either a vector of length \eqn{p} of consecutive integers describing the grouping of the coefficients, 
#' or a list with two elements: the first element 'groups' is a vector of length \eqn{\sum_{g=1}^G n_g} containing the variables belonging to each group, where \eqn{n_g} is the cardinality of the \eqn{g}-th group,
#' while the second element 'Glen' is a vector of length \eqn{G} containing the group lengths (see example below).
#' @param group_weights a vector of length \eqn{G} containing group-specific weights. The default is square root of the group cardinality, see Yuan and Lin (2006).
#' @param var_weights a vector of length \eqn{p} containing variable-specific weights. The default is a vector of ones.
#' @param var_weights_L1 a vector of length \eqn{p} containing variable-specific weights for the \eqn{L_1} penalty. The default is a vector of ones.
#' @param weights_adaptive logical. If \code{TRUE}, adaptive weights are computed from an initial 
#' least squares estimate. This option is ignored (and set to \code{FALSE}) when 
#' \code{group_weights} are provided by the user, since external weights override the adaptive scheme.
#' @param adaptive a flag for running adaptive Lasso. The default is FALSE.
#' @param standardize.data logical. Should data be standardized?
#' @param lambda either a regularization parameter or a vector of regularization parameters. In this latter case the routine computes the whole path. If it is NULL values for lambda are provided by the routine.
#' @param alpha the sparse overlap group-LASSO mixing parameter, with \eqn{0\leq\alpha\leq1}. This setting is only available for the sparse group-LASSO and the sparse overlap group-LASSO penalties, otherwise it is set to NULL. The LASSO and group-LASSO penalties are obtained
#' by specifying \eqn{\alpha = 1} and \eqn{\alpha = 0}, respectively.
#' @param lambda.min.ratio smallest value for lambda, as a fraction of the maximum lambda value. If \eqn{n>p}, the default is 0.0001, and if \eqn{n<p}, the default is 0.01. 
#' @param nlambda the number of lambda values - default is 30.
#' @param intercept logical. If it is TRUE, a column of ones is added to the design matrix. 
#' @param overall.group logical. This setting is only available for the overlap group-LASSO and the sparse overlap group-LASSO penalties, otherwise it is set to NULL. If it is TRUE, an overall group including all penalized covariates is added.
#' @param control	 a list of control parameters for the ADMM algorithm. See ‘Details’.
#'
#' @return A named list containing \describe{
#' \item{sp.coefficients}{a length-\eqn{p} solution vector for the parameters \eqn{\beta}. If \eqn{n_\lambda>1} then  the provided vector corresponds to the minimum in-sample MSE.}
#' \item{coefficients}{a length-\eqn{q} solution vector for the parameters \eqn{\gamma}. If \eqn{n_\lambda>1} then the provided vector corresponds to the minimum in-sample MSE.
#' It is provided only when either the matrix \eqn{Z} in input is not NULL or the intercept is set to TRUE.}
#' \item{sp.coef.path}{an \eqn{(n_\lambda\times p)} matrix of estimated \eqn{\beta} coefficients for each lambda of the provided sequence.}
#' \item{coef.path}{an \eqn{(n_\lambda\times q)} matrix of estimated \eqn{\gamma} coefficients for each lambda of the provided sequence.
#' It is provided only when either the matrix \eqn{Z} in input is not NULL or the intercept is set to TRUE.}
#' \item{lambda}{sequence of lambda.}
#' \item{lambda.min}{value of lambda that attains the minimum in sample MSE.}
#' \item{mse}{in-sample mean squared error.}
#' \item{min.mse}{minimum value of the in-sample MSE for the sequence of lambda.}
#' \item{convergence}{logical. 1 denotes achieved convergence.}
#' \item{elapsedTime}{elapsed time in seconds.}
#' \item{iternum}{number of iterations.}
#' }
#'
#' When you run the algorithm, output returns not only the solution, but also the iteration history recording
#' following fields over iterates:
#' \describe{
#' \item{objval}{objective function value}
#' \item{r_norm}{norm of primal residual}
#' \item{s_norm}{norm of dual residual}
#' \item{eps_pri}{feasibility tolerance for primal feasibility condition}
#' \item{eps_dual}{feasibility tolerance for dual feasibility condition.}
#' }
#' Iteration stops when both \code{r_norm} and \code{s_norm} values
#' become smaller than \code{eps_pri} and \code{eps_dual}, respectively.
#' @section Details: The control argument is a list that can supply any of the following components:\describe{
#' \item{adaptation}{logical. If it is TRUE, ADMM with adaptation is performed. The default value is TRUE. See Boyd et al. (2011) for details.}
#' \item{rho}{an augmented Lagrangian parameter. The default value is 1.}
#' \item{tau.ada}{an adaptation parameter greater than one. Only needed if adaptation = TRUE. The default value is 2. See Boyd et al. (2011) for details.}
#' \item{mu.ada}{an adaptation parameter greater than one. Only needed if adaptation = TRUE. The default value is 10. See Boyd et al. (2011) for details.}
#' \item{abstol}{absolute tolerance stopping criterion. The default value is sqrt(sqrt(.Machine$double.eps)).}
#' \item{reltol}{relative tolerance stopping criterion. The default value is sqrt(.Machine$double.eps).}
#' \item{maxit}{maximum number of iterations. The default value is 100.}
#' \item{dof.toler_c}{numeric tolerance for identifying effective zeros in the degrees-of-freedom calculation. The default is 5.}
#' \item{dof.toler_d}{numeric tolerance for identifying effective zeros in the degrees-of-freedom calculation. The default is 0.001.}
#' \item{print.out}{logical. If it is TRUE, a message about the procedure is printed. The default value is TRUE.}
#' }
#' 
#' @examples
#' 
#' ### generate sample data
#' set.seed(2023)
#' n    <- 50
#' p    <- 30 
#' X    <- matrix(rnorm(n*p), n, p)
#' 
#' ### Example 1, LASSO penalty
#' 
#' beta <- apply(matrix(rnorm(p, sd = 1), p, 1), 1, fdaSP::softhresh, 1.5)
#' y    <- X %*% beta + rnorm(n, sd = sqrt(crossprod(X %*% beta)) / 20)
#' 
#' ### set regularization parameter grid
#' lam   <- 10^seq(0, -2, length.out = 30)
#' 
#' ### set the hyper-parameters of the ADMM algorithm
#' maxit      <- 1000
#' adaptation <- TRUE
#' rho        <- 1
#' reltol     <- 1e-5
#' abstol     <- 1e-5
#' 
#' ### run example
#' mod <- lmSP(X = X, y = y, penalty = "LASSO", standardize.data = FALSE, intercept = FALSE, 
#'             lambda = lam, control = list("adaptation" = adaptation, "rho" = rho, 
#'                                          "maxit" = maxit, "reltol" = reltol, 
#'                                          "abstol" = abstol, "print.out" = FALSE)) 
#' 
#' ### graphical presentation
#' matplot(log(lam), mod$sp.coef.path, type = "l", main = "Lasso solution path",
#'         bty = "n", xlab = latex2exp::TeX("$\\log(\\lambda)$"), ylab = "")
#' 
#' ### Example 2, sparse group-LASSO penalty
#' 
#' beta <- c(rep(4, 12), rep(0, p - 13), -2)
#' y    <- X %*% beta + rnorm(n, sd = sqrt(crossprod(X %*% beta)) / 20)
#' 
#' ### define groups of dimension 3 each
#' group1 <- rep(1:10, each = 3)
#' 
#' ### set regularization parameter grid
#' lam   <- 10^seq(1, -2, length.out = 30)
#' 
#' ### set the alpha parameter 
#' alpha <- 0.5
#' 
#' ### set the hyper-parameters of the ADMM algorithm
#' maxit         <- 1000
#' adaptation    <- TRUE
#' rho           <- 1
#' reltol        <- 1e-5
#' abstol        <- 1e-5
#' 
#' ### run example
#' mod <- lmSP(X = X, y = y, penalty = "spGLASSO", groups = group1, standardize.data = FALSE,  
#'             intercept = FALSE, lambda = lam, alpha = 0.5, 
#'             control = list("adaptation" = adaptation, "rho" = rho, 
#'                            "maxit" = maxit, "reltol" = reltol, "abstol" = abstol, 
#'                            "print.out" = FALSE)) 
#' 
#' ### graphical presentation
#' matplot(log(lam), mod$sp.coef.path, type = "l", main = "Sparse Group Lasso solution path",
#'         bty = "n", xlab = latex2exp::TeX("$\\log(\\lambda)$"), ylab = "")
#' 
#' @references
#' \insertRef{bernardi_etal.2022}{fdaSP}
#' 
#' \insertRef{boyd_etal.2011}{fdaSP}
#'  
#' \insertRef{hastie_etal.2015}{fdaSP}
#' 
#' \insertRef{jenatton_etal.2011}{fdaSP}
#' 
#' \insertRef{lin_etal.2022}{fdaSP} 
#' 
#' \insertRef{simon_etal.2013}{fdaSP}
#' 
#' \insertRef{yuan_lin.2006}{fdaSP}
#' 
#' \insertRef{zou.2006}{fdaSP}
#' 
#' 
#' @export lmSP
lmSP <- function(X, Z = NULL, y, penalty = c("LASSO", "GLASSO", "spGLASSO", "OVGLASSO", "spOVGLASSO"), 
                 groups = NULL, group_weights = NULL, var_weights = NULL, var_weights_L1 = NULL, weights_adaptive = FALSE,
                 adaptive = FALSE, standardize.data = TRUE, intercept = FALSE, overall.group = FALSE,
                 lambda = NULL, alpha = NULL, lambda.min.ratio = NULL, nlambda = 30, 
                 control = list()) {
                   
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::
  # check the groups input (it should be a named list)
  if (!is.null(groups)) {
    if (is.list(groups)) {
      if (is.null(names(groups))) {
        names(groups) <- c("groups", "Glen")
      }
    }
  }
  
  # # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # # Standardise response and design matrix
  # if (standardize.data == TRUE) {
  #   res   <- standardizemat(X, y)
  #   X.std <- res$X.std
  #   y.std <- res$y.std
  # } else {
  #   X.std <- X
  #   y.std <- y
  # }
  # 
  # # ::::::::::::::::::::::::::::::::::::::::::::::::::::::::
  # # Manage group & overall group weights
  # # set weights_adaptive to FALSE if group_weights are provided by the user!
  # if (!is.null(group_weights)) {
  #   if (weights_adaptive == TRUE) {
  #     warning("* lmSP : weights_adaptive is set to FALSE if group_weights are provided by the user!\n")
  #     weights_adaptive <- FALSE
  #   }
  # }
  # # compute group_weights if they are not provided by the user
  # if (is.null(group_weights)) {
  #   # Define the weights as in Yuan and Lin 2006, JRSSB
  #   if (!is.null(weights_adaptive)) {
  #     if (weights_adaptive == TRUE) {
  #       if (is.null(Z)) {
  #         coeff_LS <- lm_ols(X = X.std, y = y.std)
  #         for (g in 1:G) {
  #           idx              <- which(GRmat[g,] == 1)
  #           group_weights[g] <- 1.0 / norm(coeff_LS[idx], type = "2")
  #         }
  #       } else {
  #         coeff_LS <- lm_ols_FWL(X = X.std, Z = Z, y = y.std)$coeff_X
  #         for (g in 1:G) {
  #           idx              <- which(GRmat[g,] == 1)
  #           group_weights[g] <- 1.0 / norm(coeff_LS[idx], type = "2")
  #         }
  #       }
  #     } else {
  #       group_weights <- rep(1, G)
  #     }
  #   } else {
  #     group_weights <- rep(1, G)
  #   }
  # } else {
  #   if (overall.group == TRUE) {
  #     check_group_weights(x = group_weights, n = G, algname = "lmSP", funname = "group_weights")
  #     if (length(group_weights) == (G-1)) {
  #       group_weights <- c(group_weights, sqrt(dim(GRmat)[2]))
  #     }
  #   } else {
  #     check_weights(x = group_weights, n = G, algname = "lmSP", funname = "group_weights")
  #   }
  # } 
  
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::
  # check the penalty input
  if (!strcmpi(penalty, "LASSO") && !strcmpi(penalty, "GLASSO") && !strcmpi(penalty, "spGLASSO") && !strcmpi(penalty, "OVGLASSO") && !strcmpi(penalty, "spOVGLASSO")) {
    stop("* lmSP : input 'penalty' must be correctly specified: 'LASSO', 'GLASSO', 'spGLASSO', 'OVGLASSO', 'spOVGLASSO'.")
  }
  
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::
  # run linear regression with LASSO-type penalty (means, LASSO or adaptive-LASSO)
  if (strcmpi(penalty, "LASSO")) {
    res <- linreg_ADMM_LASSO(X = X, Z = Z, y = y, var_weights = var_weights, 
                             adaptive = adaptive, 
                             standardize.data = standardize.data, 
                             lambda = lambda, lambda.min.ratio = lambda.min.ratio, 
                             nlambda = nlambda, 
                             intercept = intercept, control = control) 
  }
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::
  # run linear regression with group LASSO-type penalty 
  if (strcmpi(penalty, "GLASSO")) {
    res <- linreg_ADMM_GLASSO(X = X, Z = Z, y = y, groups = groups, group_weights = group_weights, var_weights = var_weights, 
                              standardize.data = standardize.data,
                              lambda = lambda, lambda.min.ratio = lambda.min.ratio, nlambda = nlambda, 
                              intercept = intercept, control = control)
  }
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::
  # run linear regression with sparse group LASSO-type penalty 
  if (strcmpi(penalty, "spGLASSO")) {
    res <- linreg_ADMM_spGLASSO(X = X, Z = Z, y = y, groups = groups, group_weights = group_weights, var_weights = var_weights, 
                                var_weights_L1 = var_weights_L1, standardize.data = standardize.data, 
                                lambda = lambda, alpha = alpha, lambda.min.ratio = lambda.min.ratio, nlambda = nlambda, 
                                intercept = intercept, control = control)
  }
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::
  # run linear regression with overlap group LASSO-type penalty 
  if (strcmpi(penalty, "OVGLASSO")) {
    res <- linreg_ADMM_OVGLASSO(X = X, Z = Z, y = y, groups = groups, group_weights = group_weights, var_weights = var_weights, 
                                standardize.data = standardize.data,
                                lambda = lambda, lambda.min.ratio = lambda.min.ratio, nlambda = nlambda,
                                intercept = intercept, overall.group = overall.group, control = control)
  }
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::
  # run linear regression with overlap group LASSO-type penalty 
  if (strcmpi(penalty, "spOVGLASSO")) {
    res <- linreg_ADMM_spOVGLASSO(X = X, Z = Z, y = y, groups = groups, group_weights = group_weights, var_weights = var_weights, 
                                  var_weights_L1 = var_weights_L1, standardize.data = standardize.data,
                                  lambda = lambda, alpha = alpha, lambda.min.ratio = lambda.min.ratio, nlambda = nlambda,
                                  intercept = intercept, overall.group = overall.group, control = control)
  }
  
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::
  # compute degrees of freedom
  # out.df <- lm_dof_OVGLASSO(y                = y.std,
  #                           X                = X.std, 
  #                           Z                = Z,
  #                           coeff            = res$coef.path,
  #                           lambda           = lambda,
  #                           adaptive_weights = weights_adaptive,
  #                           GRmat            = GRmat,
  #                           group_weights    = group_weights,
  #                           var_weights      = var_weights,
  #                           Umat             = res$U,
  #                           err_primal       = res$err_pri,
  #                           err_dual         = res$err_dual,
  #                           rho              = res$rho,
  #                           toler_c          = toler_c,
  #                           toler_d          = toler_d)

  # :::::::::::::::::::::::::::::::::::::::::::::::::::::
  # return
  return(res)
}

#' Cross-validation for Sparse Adaptive Overlap Group Least Absolute Shrinkage and Selection Operator
#'
#' Sparse Adaptive overlap group-LASSO, or sparse adaptive group \eqn{L_2}-regularized regression, solves the following optimization problem
#' \deqn{\textrm{min}_{\beta,\gamma} ~ \frac{1}{2}\|y-X\beta-Z\gamma\|_2^2 + \lambda\Big[(1-\alpha) \sum_{g=1}^G \|S_g T\beta\|_2+\alpha\Vert T_1\beta\Vert_1\Big]}
#' to obtain a sparse coefficient vector \eqn{\beta\in\mathbb{R}^p} for the matrix of penalized predictors \eqn{X} and a coefficient vector \eqn{\gamma\in\mathbb{R}^q} 
#' for the matrix of unpenalized predictors \eqn{Z}. For each group \eqn{g}, each row of 
#' the matrix \eqn{S_g\in\mathbb{R}^{n_g\times p}} has non-zero entries only for those variables belonging 
#' to that group. These values are provided by the arguments \code{groups} and \code{group_weights} (see below). 
#' Each variable can belong to more than one group. The diagonal matrix \eqn{T\in\mathbb{R}^{p\times p}} contains the variable-specific weights. These values are
#' provided by the argument \code{var_weights} (see below). The diagonal matrix \eqn{T_1\in\mathbb{R}^{p\times p}} contains 
#' the variable-specific \eqn{L_1} weights. These values are provided by the argument \code{var_weights_L1} (see below).
#' The regularization path is computed for the sparse adaptive overlap group-LASSO penalty at a grid of values for the regularization 
#' parameter \eqn{\lambda} using the alternating direction method of multipliers (ADMM). See Boyd et al. (2011) and Lin et al. (2022) 
#' for details on the ADMM method. The regularization is a combination of \eqn{L_2} and
#' \eqn{L_1} simultaneous constraints. Different specifications of the \code{penalty} argument lead to different models choice:
#' \describe{
#' \item{LASSO}{The classical Lasso regularization (Tibshirani, 1996) can be obtained by specifying \eqn{\alpha = 1} 
#' and the matrix \eqn{T_1} as the \eqn{p \times p} identity matrix. An adaptive version of this model (Zou, 2006) can be obtained if \eqn{T_1} is 
#' a \eqn{p \times p}  diagonal matrix of adaptive weights. See also Hastie et al. (2015) for further details.}
#' \item{GLASSO}{The group-Lasso regularization (Yuan and Lin, 2006) can be obtained by specifying \eqn{\alpha = 0}, 
#' non-overlapping groups in \eqn{S_g} and by setting the matrix \eqn{T} equal to the \eqn{p \times p} identity matrix. 
#' An adaptive version of this model can be obtained if the matrix \eqn{T} is a \eqn{p \times p} diagonal matrix of adaptive weights. See also Hastie et al. (2015) for further details.}
#' \item{spGLASSO}{The sparse group-Lasso regularization (Simon et al., 2011) can be obtained by specifying \eqn{\alpha\in(0,1)},  
#' non-overlapping groups in \eqn{S_g} and by setting the matrices \eqn{T} and \eqn{T_1} equal to the \eqn{p \times p} identity matrix. 
#' An adaptive version of this model can be obtained if the matrices \eqn{T} and \eqn{T_1} are \eqn{p \times p}  
#' diagonal matrices of adaptive weights.}
#' \item{OVGLASSO}{The overlap group-Lasso regularization (Jenatton et al., 2011) can be obtained by specifying 
#' \eqn{\alpha = 0}, overlapping groups in \eqn{S_g} and by setting the matrix \eqn{T} equal to the \eqn{p \times p} identity matrix. An adaptive version of this model can be obtained if the matrix \eqn{T} is a \eqn{p \times p}  
#' diagonal matrix of adaptive weights.}
#' \item{spOVGLASSO}{The sparse overlap group-Lasso regularization (Jenatton et al., 2011) can be obtained by specifying 
#' \eqn{\alpha\in(0,1)}, overlapping groups in \eqn{S_g} and by setting the matrices \eqn{T} and \eqn{T_1} equal to the \eqn{p \times p} identity matrix. 
#' An adaptive version of this model can be obtained if the matrices \eqn{T} and \eqn{T_1} are \eqn{p \times p}  diagonal matrices of adaptive weights.}
#' }
#'  
#' @param X an \eqn{(n\times p)} matrix of penalized predictors.
#' @param Z an \eqn{(n\times q)} full column rank matrix of predictors that are not penalized. 
#' @param y a length-\eqn{n} response vector.
#' @param penalty choose one from the following options: 'LASSO', for the or adaptive-Lasso penalties, 'GLASSO', 
#' for the group-Lasso penalty, 'spGLASSO', for the sparse group-Lasso penalty, 'OVGLASSO', 
#' for the overlap group-Lasso penalty and 'spOVGLASSO', for the sparse overlap group-Lasso penalty.
#' @param groups either a vector of length \eqn{p} of consecutive integers describing the grouping of the coefficients, 
#' or a list with two elements: the first element is a vector of length \eqn{\sum_{g=1}^G n_g} containing the variables belonging to each group, where \eqn{n_g} is the cardinality of the \eqn{g}-th group, 
#' while the second element is a vector of length \eqn{G} containing the group lengths (see example below).
#' @param group_weights a vector of length \eqn{G} containing group-specific weights. The default is square root of the group cardinality, see Yuan and Lin (2006).
#' @param var_weights a vector of length \eqn{p} containing variable-specific weights. The default is a vector of ones.
#' @param var_weights_L1 a vector of length \eqn{p} containing variable-specific weights for the \eqn{L_1} penalty. The default is a vector of ones.
#' @param cv.fold the number of folds - default is 5.
#' @param standardize.data logical. Should data be standardized?
#' @param lambda either a regularization parameter or a vector of regularization parameters. In this latter case the routine computes the whole path. If it is NULL values for lambda are provided by the routine.
#' @param alpha the sparse overlap group-LASSO mixing parameter, with \eqn{0\leq\alpha\leq1}. This setting is only available for the sparse group-LASSO and the sparse overlap group-LASSO penalties, otherwise it is set to NULL. The LASSO and group-LASSO penalties are obtained
#' by specifying \eqn{\alpha = 1} and \eqn{\alpha = 0}, respectively.
#' @param lambda.min.ratio smallest value for lambda, as a fraction of the maximum lambda value. If \eqn{n>p}, the default is 0.0001, and if \eqn{n<p}, the default is 0.01. 
#' @param nlambda the number of lambda values - default is 30.
#' @param intercept logical. If it is TRUE, a column of ones is added to the design matrix. 
#' @param overall.group logical. This setting is only available for the overlap group-LASSO and the sparse overlap group-LASSO penalties, otherwise it is set to NULL. If it is TRUE, an overall group including all penalized covariates is added.
#' @param control	 a list of control parameters for the ADMM algorithm. See ‘Details’.
#' 
#' @return A named list containing \describe{
#' \item{sp.coefficients}{a length-\eqn{p} solution vector for the parameters \eqn{\beta}. If \eqn{n_\lambda>1} then  the provided vector corresponds to the minimum cross-validated MSE.}
#' \item{coefficients}{a length-\eqn{q} solution vector for the parameters \eqn{\gamma}. If \eqn{n_\lambda>1} then  the provided vector corresponds to the minimum cross-validated MSE.
#' It is provided only when either the matrix \eqn{Z} in input is not NULL or the intercept is set to TRUE.}
#' \item{sp.coef.path}{an \eqn{(n_\lambda\times p)} matrix of estimated \eqn{\beta} coefficients for each lambda of the provided sequence.}
#' \item{coef.path}{an \eqn{(n_\lambda\times q)} matrix of estimated \eqn{\gamma} coefficients for each lambda of the provided sequence. 
#' It is provided only when either the matrix \eqn{Z} in input is not NULL or the intercept is set to TRUE.}
#' \item{lambda}{sequence of lambda.}
#' \item{lambda.min}{value of lambda that attains the minimum cross-validated MSE.}
#' \item{mse}{cross-validated mean squared error.}
#' \item{min.mse}{minimum value of the cross-validated MSE for the sequence of lambda.}
#' \item{convergence}{logical. 1 denotes achieved convergence.}
#' \item{elapsedTime}{elapsed time in seconds.}
#' \item{iternum}{number of iterations.}
#' \item{foldid}{a vector of values between 1 and cv.fold identifying what fold each observation is in.}
#' }
#'
#' When you run the algorithm, output returns not only the solution, but also the iteration history recording
#' following fields over iterates:
#' \describe{
#' \item{objval}{objective function value}
#' \item{r_norm}{norm of primal residual}
#' \item{s_norm}{norm of dual residual}
#' \item{eps_pri}{feasibility tolerance for primal feasibility condition}
#' \item{eps_dual}{feasibility tolerance for dual feasibility condition.}
#' }
#' Iteration stops when both \code{r_norm} and \code{s_norm} values
#' become smaller than \code{eps_pri} and \code{eps_dual}, respectively.
#' @section Details: The control argument is a list that can supply any of the following components:\describe{
#' \item{adaptation}{logical. If it is TRUE, ADMM with adaptation is performed. The default value is TRUE. See Boyd et al. (2011) for details.}
#' \item{rho}{an augmented Lagrangian parameter. The default value is 1.}
#' \item{tau.ada}{an adaptation parameter greater than one. Only needed if adaptation = TRUE. The default value is 2. See Boyd et al. (2011) for details.}
#' \item{mu.ada}{an adaptation parameter greater than one. Only needed if adaptation = TRUE. The default value is 10. See Boyd et al. (2011) for details.}
#' \item{abstol}{absolute tolerance stopping criterion. The default value is sqrt(sqrt(.Machine$double.eps)).}
#' \item{reltol}{relative tolerance stopping criterion. The default value is sqrt(.Machine$double.eps).}
#' \item{maxit}{maximum number of iterations. The default value is 100.}
#' \item{print.out}{logical. If it is TRUE, a message about the procedure is printed. The default value is TRUE.}
#' }
#' 
#' @examples 
#' 
#' ### generate sample data
#' set.seed(2023)
#' n    <- 50
#' p    <- 30 
#' X    <- matrix(rnorm(n * p), n, p)
#' 
#' ### Example 1, LASSO penalty
#' 
#' beta <- apply(matrix(rnorm(p, sd = 1), p, 1), 1, fdaSP::softhresh, 1.5)
#' y    <- X %*% beta + rnorm(n, sd = sqrt(crossprod(X %*% beta)) / 20)
#' 
#' ### set the hyper-parameters of the ADMM algorithm
#' maxit      <- 1000
#' adaptation <- TRUE
#' rho        <- 1
#' reltol     <- 1e-5
#' abstol     <- 1e-5
#' 
#' ### run cross-validation
#' mod_cv <- lmSP_cv(X = X, y = y, penalty = "LASSO", 
#'                   standardize.data = FALSE, intercept = FALSE,
#'                   cv.fold = 5, nlambda = 30, 
#'                   control = list("adaptation" = adaptation, 
#'                                  "rho" = rho, 
#'                                  "maxit" = maxit, "reltol" = reltol, 
#'                                  "abstol" = abstol, 
#'                                  "print.out" = FALSE)) 
#' 
#' ### graphical presentation
#' plot(log(mod_cv$lambda), mod_cv$mse, type = "l", col = "blue", lwd = 2, bty = "n", 
#'      xlab = latex2exp::TeX("$\\log(\\lambda)$"), ylab = "Prediction Error", 
#'      ylim = range(mod_cv$mse - mod_cv$mse.sd, mod_cv$mse + mod_cv$mse.sd),
#'      main = "Cross-validated Prediction Error")
#' fdaSP::confband(xV = log(mod_cv$lambda), yVmin = mod_cv$mse - mod_cv$mse.sd, 
#'                 yVmax = mod_cv$mse + mod_cv$mse.sd)       
#' abline(v = log(mod_cv$lambda[which(mod_cv$lambda == mod_cv$lambda.min)]), 
#'        col = "red", lwd = 1.0)
#' 
#' ### comparison with oracle error
#' mod <- lmSP(X = X, y = y, penalty = "LASSO", 
#'             standardize.data = FALSE, 
#'             intercept = FALSE,
#'             nlambda = 30, 
#'             control = list("adaptation" = adaptation, 
#'                            "rho" = rho, 
#'                            "maxit" = maxit, "reltol" = reltol, 
#'                            "abstol" = abstol, 
#'                            "print.out" = FALSE)) 
#'                                          
#' err_mod <- apply(mod$sp.coef.path, 1, function(x) sum((x - beta)^2))
#' plot(log(mod$lambda), err_mod, type = "l", col = "blue", lwd = 2, 
#'      xlab = latex2exp::TeX("$\\log(\\lambda)$"), 
#'      ylab = "Estimation Error", main = "True Estimation Error", bty = "n")
#' abline(v = log(mod$lambda[which(err_mod == min(err_mod))]), col = "red", lwd = 1.0)
#' abline(v = log(mod_cv$lambda[which(mod_cv$lambda == mod_cv$lambda.min)]), 
#'        col = "red", lwd = 1.0, lty = 2)
#' 
#' ### Example 2, sparse group-LASSO penalty
#' 
#' beta <- c(rep(4, 12), rep(0, p - 13), -2)
#' y    <- X %*% beta + rnorm(n, sd = sqrt(crossprod(X %*% beta)) / 20)
#' 
#' ### define groups of dimension 3 each
#' group1 <- rep(1:10, each = 3)
#' 
#' ### set regularization parameter grid
#' lam   <- 10^seq(1, -2, length.out = 30)
#' 
#' ### set the alpha parameter 
#' alpha <- 0.5
#' 
#' ### set the hyper-parameters of the ADMM algorithm
#' maxit         <- 1000
#' adaptation    <- TRUE
#' rho           <- 1
#' reltol        <- 1e-5
#' abstol        <- 1e-5
#' 
#' ### run cross-validation
#' mod_cv <- lmSP_cv(X = X, y = y, penalty = "spGLASSO", 
#'                   groups = group1, cv.fold = 5, 
#'                   standardize.data = FALSE,  intercept = FALSE, 
#'                   lambda = lam, alpha = 0.5, 
#'                   control = list("adaptation" = adaptation, 
#'                                  "rho" = rho,
#'                                  "maxit" = maxit, "reltol" = reltol, 
#'                                  "abstol" = abstol, 
#'                                  "print.out" = FALSE)) 
#' 
#' ### graphical presentation
#' plot(log(mod_cv$lambda), mod_cv$mse, type = "l", col = "blue", lwd = 2, bty = "n", 
#'      xlab = latex2exp::TeX("$\\log(\\lambda)$"), ylab = "Prediction Error", 
#'      ylim = range(mod_cv$mse - mod_cv$mse.sd, mod_cv$mse + mod_cv$mse.sd),
#'      main = "Cross-validated Prediction Error")
#' fdaSP::confband(xV = log(mod_cv$lambda), yVmin = mod_cv$mse - mod_cv$mse.sd, 
#'                 yVmax = mod_cv$mse + mod_cv$mse.sd)       
#' abline(v = log(mod_cv$lambda[which(mod_cv$lambda == mod_cv$lambda.min)]), 
#'        col = "red", lwd = 1.0)
#' 
#' ### comparison with oracle error
#' mod <- lmSP(X = X, y = y, 
#'             penalty = "spGLASSO", 
#'             groups = group1, 
#'             standardize.data = FALSE, 
#'             intercept = FALSE,
#'             lambda = lam, 
#'             alpha = 0.5, 
#'             control = list("adaptation" = adaptation, "rho" = rho, 
#'                            "maxit" = maxit, "reltol" = reltol, "abstol" = abstol, 
#'                            "print.out" = FALSE)) 
#'                                          
#' err_mod <- apply(mod$sp.coef.path, 1, function(x) sum((x - beta)^2))
#' plot(log(mod$lambda), err_mod, type = "l", col = "blue", lwd = 2, 
#'      xlab = latex2exp::TeX("$\\log(\\lambda)$"), 
#'      ylab = "Estimation Error", main = "True Estimation Error", bty = "n")
#' abline(v = log(mod$lambda[which(err_mod == min(err_mod))]), col = "red", lwd = 1.0)
#' abline(v = log(mod_cv$lambda[which(mod_cv$lambda == mod_cv$lambda.min)]), 
#'        col = "red", lwd = 1.0, lty = 2)
#' 
#' @references
#' \insertRef{bernardi_etal.2022}{fdaSP}
#' 
#' \insertRef{boyd_etal.2011}{fdaSP}
#'  
#' \insertRef{hastie_etal.2015}{fdaSP}
#' 
#' \insertRef{jenatton_etal.2011}{fdaSP}
#' 
#' \insertRef{lin_etal.2022}{fdaSP} 
#' 
#' \insertRef{simon_etal.2013}{fdaSP}
#' 
#' \insertRef{yuan_lin.2006}{fdaSP}
#' 
#' \insertRef{zou.2006}{fdaSP}
#' 
#' 
#' @export
lmSP_cv <- function(X, Z = NULL, y, penalty = c("LASSO", "GLASSO", "spGLASSO", "OVGLASSO", "spOVGLASSO"), 
                    groups = NULL, group_weights = NULL, var_weights = NULL, var_weights_L1 = NULL, cv.fold = 5,
                    standardize.data = TRUE, intercept = FALSE, overall.group = FALSE,
                    lambda = NULL, alpha = NULL, lambda.min.ratio = NULL, nlambda = 30, 
                    control = list()) {
                      
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::
  # check the groups input (it should be a named list)
  if (!is.null(groups)) {
    if (is.list(groups)) {
      if (is.null(names(groups))) {
        names(groups) <- c("groups", "Glen")
      }
    }
  }
    
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::
  # check the penalty input
  if (!strcmpi(penalty, "LASSO") && !strcmpi(penalty, "GLASSO") && !strcmpi(penalty, "spGLASSO") && !strcmpi(penalty, "OVGLASSO") && !strcmpi(penalty, "spOVGLASSO")) {
    stop("* lmSP : input 'penalty' must be correctly specified: 'LASSO', 'GLASSO', 'spGLASSO', 'OVGLASSO', 'spOVGLASSO'.") 
  }
  
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::
  # run linear regression with LASSO-type penalty (means, LASSO or adaptive-LASSO)
  if (strcmpi(penalty, "LASSO")) {
    res <- linreg_ADMM_LASSO_cv(X = X, Z = Z, y = y, var_weights = var_weights, standardize.data = standardize.data, 
                                lambda = lambda, lambda.min.ratio = lambda.min.ratio, nlambda = nlambda, cv.fold = cv.fold,
                                intercept = intercept, control = control) 
  }
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::
  # run linear regression with group LASSO-type penalty 
  if (strcmpi(penalty, "GLASSO")) {
    res <- linreg_ADMM_GLASSO_cv(X = X, Z = Z, y = y, groups = groups, group_weights = group_weights, var_weights = var_weights, 
                                 standardize.data = standardize.data,
                                 lambda = lambda, lambda.min.ratio = lambda.min.ratio, nlambda = nlambda, cv.fold = cv.fold,
                                 intercept = intercept, control = control)
  }
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::
  # run linear regression with sparse group LASSO-type penalty 
  if (strcmpi(penalty, "spGLASSO")) {
    res <- linreg_ADMM_spGLASSO_cv(X = X, Z = Z, y = y, groups = groups, group_weights = group_weights, var_weights = var_weights, 
                                   var_weights_L1 = var_weights_L1, standardize.data = standardize.data, 
                                   lambda = lambda, alpha = alpha, lambda.min.ratio = lambda.min.ratio, nlambda = nlambda, cv.fold = cv.fold,
                                   intercept = intercept, control = control)
  }
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::
  # run linear regression with overlap group LASSO-type penalty 
  if (strcmpi(penalty, "OVGLASSO")) {
    res <- linreg_ADMM_OVGLASSO_cv(X = X, Z = Z, y = y, groups = groups, group_weights = group_weights, var_weights = var_weights, 
                                   standardize.data = standardize.data,
                                   lambda = lambda, lambda.min.ratio = lambda.min.ratio, nlambda = nlambda, cv.fold = cv.fold,
                                   intercept = intercept, overall.group = overall.group, control = control)
  }
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::
  # run linear regression with overlap group LASSO-type penalty 
  if (strcmpi(penalty, "spOVGLASSO")) {
    res <- linreg_ADMM_spOVGLASSO_cv(X = X, Z = Z, y = y, groups = groups, group_weights = group_weights, var_weights = var_weights, 
                                     var_weights_L1 = var_weights_L1, standardize.data = standardize.data,
                                     lambda = lambda, alpha = alpha, lambda.min.ratio = lambda.min.ratio, nlambda = nlambda, cv.fold = cv.fold,
                                     intercept = intercept, overall.group = overall.group, control = control)
  }
  # :::::::::::::::::::::::::::::::::::::::::::::::::::::
  # return
  return(res)
}

\name{fdaSP-package}
\alias{fdaSP-package}
\alias{fdaSP}
\docType{package}
\title{\packageTitle{fdaSP}}
\description{\packageDescription{fdaSP}}
\section{Package Content}{\packageIndices{fdaSP}}
\author{\packageAuthor{fdaSP}}
\section{Maintainer}{\packageMaintainer{fdaSP}}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/f2freg.R
\name{f2fSP_cv}
\alias{f2fSP_cv}
\title{Cross-validation for Overlap Group Least Absolute Shrinkage and Selection Operator for function-on-function regression model}
\usage{
f2fSP_cv(
  mY,
  mX,
  L,
  M,
  group_weights = NULL,
  var_weights = NULL,
  standardize.data = FALSE,
  splOrd = 4,
  lambda = NULL,
  lambda.min.ratio = NULL,
  nlambda = NULL,
  cv.fold = 5,
  overall.group = FALSE,
  control = list()
)
}
\arguments{
\item{mY}{an \eqn{(n\times r_y)} matrix of observations of the functional response variable.}

\item{mX}{an \eqn{(n\times r_x)} matrix of observations of the functional covariate.}

\item{L}{number of elements of the B-spline basis vector \eqn{\theta(s)}.}

\item{M}{number of elements of the B-spline basis vector \eqn{\varphi(t)}.}

\item{group_weights}{a vector of length \eqn{G} containing group-specific weights. The default is square root of the group cardinality, see Bernardi et al. (2022).}

\item{var_weights}{a vector of length \eqn{ML} containing basis-specific weights. The default is a vector where 
each entry is the reciprocal of the number of groups including that basis. See Bernardi et al. (2022) for details.}

\item{standardize.data}{logical. Should data be standardized?}

\item{splOrd}{the order \eqn{d} of the spline basis.}

\item{lambda}{either a regularization parameter or a vector of regularization parameters. 
In this latter case the routine computes the whole path. If it is NULL values for lambda are provided by the routine.}

\item{lambda.min.ratio}{smallest value for lambda, as a fraction of the maximum lambda value. If \eqn{nr_y>LM}, the default is 0.0001, and if \eqn{nr_y<LM}, the default is 0.01.}

\item{nlambda}{the number of lambda values - default is 30.}

\item{cv.fold}{the number of folds - default is 5.}

\item{overall.group}{logical. If it is TRUE, an overall group including all penalized covariates is added.}

\item{control}{a list of control parameters for the ADMM algorithm. See ‘Details’.}
}
\value{
A named list containing \describe{
\item{sp.coefficients}{an \eqn{(M\times L)} solution matrix for the parameters \eqn{\Psi}, which corresponds to the minimum cross-validated MSE.}
\item{sp.fun}{an \eqn{(r_x\times r_y)} matrix providing the estimated functional coefficient for \eqn{\psi(t,s)} corresponding to the minimum cross-validated MSE.} 
\item{lambda}{sequence of lambda.}
\item{lambda.min}{value of lambda that attains the cross-validated minimum mean squared error.}
\item{indi.min.mse}{index of the lambda sequence corresponding to lambda.min.}
\item{mse}{cross-validated mean squared error.}
\item{min.mse}{minimum value of the cross-validated MSE for the sequence of lambda.}
\item{mse.sd}{standard deviation of the cross-validated mean squared error.}
\item{convergence}{logical. 1 denotes achieved convergence.}
\item{elapsedTime}{elapsed time in seconds.}
\item{iternum}{number of iterations.}
}

Iteration stops when both \code{r_norm} and \code{s_norm} values
become smaller than \code{eps_pri} and \code{eps_dual}, respectively.
}
\description{
Overlap Group-LASSO for function-on-function regression model solves the following optimization problem
\deqn{\textrm{min}_{\psi} ~ \frac{1}{2} \sum_{i=1}^n \int \left( y_i(s) - \int x_i(t) \psi(t,s) dt \right)^2 ds + \lambda \sum_{g=1}^{G} \Vert S_{g}T\psi \Vert_2}
to obtain a sparse coefficient vector \eqn{\psi=\mathsf{vec}(\Psi)\in\mathbb{R}^{ML}} for the functional penalized predictor \eqn{x(t)}, where the coefficient matrix \eqn{\Psi\in\mathbb{R}^{M\times L}}, 
the regression function \eqn{\psi(t,s)=\varphi(t)^\intercal\Psi\theta(s)}, 
\eqn{\varphi(t)} and \eqn{\theta(s)} are two B-splines bases of order \eqn{d} and dimension \eqn{M} and \eqn{L}, respectively. For each group \eqn{g}, each row of 
the matrix \eqn{S_g\in\mathbb{R}^{d\times ML}} has non-zero entries only for those bases belonging 
to that group. These values are provided by the arguments \code{groups} and \code{group_weights} (see below). 
Each basis function belongs to more than one group. The diagonal matrix \eqn{T\in\mathbb{R}^{ML\times ML}} contains 
the basis-specific weights. These values are provided by the argument \code{var_weights} (see below).
The regularization path is computed for the overlap group-LASSO penalty at a grid of values for the regularization 
parameter \eqn{\lambda} using the alternating direction method of multipliers (ADMM). See Boyd et al. (2011) and Lin et al. (2022) 
for details on the ADMM method.
}
\section{Details}{
 The control argument is a list that can supply any of the following components:\describe{
\item{adaptation}{logical. If it is TRUE, ADMM with adaptation is performed. The default value is TRUE. See Boyd et al. (2011) for details.}
\item{rho}{an augmented Lagrangian parameter. The default value is 1.}
\item{tau.ada}{an adaptation parameter greater than one. Only needed if adaptation = TRUE. The default value is 2. See Boyd et al. (2011) and Lin et al. (2022) for details.}
\item{mu.ada}{an adaptation parameter greater than one. Only needed if adaptation = TRUE. The default value is 10. See Boyd et al. (2011) and Lin et al. (2022) for details.}
\item{abstol}{absolute tolerance stopping criterion. The default value is sqrt(sqrt(.Machine$double.eps)).}
\item{reltol}{relative tolerance stopping criterion. The default value is sqrt(.Machine$double.eps).}
\item{maxit}{maximum number of iterations. The default value is 100.}
\item{print.out}{logical. If it is TRUE, a message about the procedure is printed. The default value is TRUE.}
}
}

\examples{

## generate sample data
set.seed(4321)
s  <- seq(0, 1, length.out = 100)
t  <- seq(0, 1, length.out = 100)
p1 <- 5
p2 <- 6
r  <- 10
n  <- 50

beta_basis1 <- splines::bs(s, df = p1, intercept = TRUE)    # first basis for beta
beta_basis2 <- splines::bs(s, df = p2, intercept = TRUE)    # second basis for beta

data_basis <- splines::bs(s, df = r, intercept = TRUE)    # basis for X

x_0   <- apply(matrix(rnorm(p1 * p2, sd = 1), p1, p2), 1, 
               fdaSP::softhresh, 1.5)  # regression coefficients 
x_fun <- beta_basis2 \%*\% x_0 \%*\%  t(beta_basis1)  

fun_data <- matrix(rnorm(n*r), n, r) \%*\% t(data_basis)
b        <- fun_data \%*\% x_fun + rnorm(n * 100, sd = sd(fun_data \%*\% x_fun )/3)

## set the hyper-parameters
maxit          <- 1000
rho_adaptation <- FALSE
rho            <- 0.01
reltol         <- 1e-5
abstol         <- 1e-5

## fit functional regression model
mod_cv <- f2fSP_cv(mY = b, mX = fun_data, L = p1, M = p2,
                   group_weights = NULL, var_weights = NULL, 
                   standardize.data = FALSE, splOrd = 4,
                   lambda = NULL, nlambda = 30, cv.fold = 5, 
                   lambda.min.ratio = NULL,
                   control = list("abstol" = abstol, 
                                  "reltol" = reltol, 
                                  "maxit" = maxit, 
                                  "adaptation" = rho_adaptation, 
                                  "rho" = rho, 
                                  "print.out" = FALSE))

### graphical presentation
plot(log(mod_cv$lambda), mod_cv$mse, type = "l", col = "blue", lwd = 2, bty = "n", 
     xlab = latex2exp::TeX("$\\\\log(\\\\lambda)$"), ylab = "Prediction Error", 
     ylim = range(mod_cv$mse - mod_cv$mse.sd, mod_cv$mse + mod_cv$mse.sd),
     main = "Cross-validated Prediction Error")
fdaSP::confband(xV = log(mod_cv$lambda), yVmin = mod_cv$mse - mod_cv$mse.sd, 
                yVmax = mod_cv$mse + mod_cv$mse.sd)       
abline(v = log(mod_cv$lambda[which(mod_cv$lambda == mod_cv$lambda.min)]), col = "red", lwd = 1.0)

### comparison with oracle error
mod <- f2fSP(mY = b, mX = fun_data, L = p1, M = p2,
             group_weights = NULL, var_weights = NULL, 
             standardize.data = FALSE, splOrd = 4,
             lambda = NULL, nlambda = 30, lambda.min.ratio = NULL, 
             control = list("abstol" = abstol, 
                            "reltol" = reltol, 
                            "maxit" = maxit,
                            "adaptation" = rho_adaptation, 
                            "rho" = rho, 
                            "print.out" = FALSE))
err_mod <- apply(mod$sp.coef.path, 1, function(x) sum((x - x_0)^2))
plot(log(mod$lambda), err_mod, type = "l", col = "blue", lwd = 2, 
     xlab = latex2exp::TeX("$\\\\log(\\\\lambda)$"), 
     ylab = "Estimation Error", main = "True Estimation Error", bty = "n")
abline(v = log(mod$lambda[which(err_mod == min(err_mod))]), col = "red", lwd = 1.0)
abline(v = log(mod_cv$lambda[which(mod_cv$lambda == mod_cv$lambda.min)]), 
       col = "red", lwd = 1.0, lty = 2)

}
\references{
\insertRef{bernardi_etal.2022}{fdaSP}

\insertRef{boyd_etal.2011}{fdaSP}

\insertRef{jenatton_etal.2011}{fdaSP}

\insertRef{lin_etal.2022}{fdaSP}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/f2sreg.R
\name{f2sSP}
\alias{f2sSP}
\title{Overlap Group Least Absolute Shrinkage and Selection Operator for scalar-on-function regression model}
\usage{
f2sSP(
  vY,
  mX,
  mZ = NULL,
  M,
  group_weights = NULL,
  var_weights = NULL,
  weights_adaptive = FALSE,
  standardize.data = TRUE,
  splOrd = 4,
  diff_order = 1,
  lambda1 = NULL,
  lambda2 = NULL,
  nlambda1 = 30,
  lambda1.min.ratio = NULL,
  intercept = FALSE,
  overall.group = FALSE,
  control = list()
)
}
\arguments{
\item{vY}{a length-\eqn{n} vector of observations of the scalar response variable.}

\item{mX}{a \eqn{(n\times r)} matrix of observations of the functional covariate.}

\item{mZ}{an \eqn{(n\times q)} full column rank matrix of scalar predictors that are not penalized.}

\item{M}{number of elements of the B-spline basis vector \eqn{\varphi(t)}.}

\item{group_weights}{a vector of length \eqn{G} containing group-specific weights. The default is square root of the group cardinality, see Bernardi et al. (2022).}

\item{var_weights}{a vector of length \eqn{M} containing basis-specific weights. The default is a vector where 
each entry is the reciprocal of the number of groups including that basis. See Bernardi et al. (2022) for details.}

\item{weights_adaptive}{logical. If \code{TRUE}, adaptive weights are computed from an initial 
least squares estimate. This option is ignored (and set to \code{FALSE}) when 
\code{group_weights} are provided by the user, since external weights override the adaptive scheme.}

\item{standardize.data}{logical. Should data be standardized?}

\item{splOrd}{the order \eqn{d} of the spline basis.}

\item{diff_order}{a positive integer specifying the order of the discrete
difference operator used in the smoothness penalty. The default is 1.}

\item{lambda1}{either a regularization parameter or a vector of regularization parameters.
In this latter case the routine computes the whole path. If it is NULL values for \code{lambda1} are provided by the routine.}

\item{lambda2}{either a non-negative smoothing regularization parameter or a vector of
smoothing regularization parameters. If \code{NULL}, no smoothness penalty is used.}

\item{nlambda1}{the number of \code{lambda1} values - default is 30.}

\item{lambda1.min.ratio}{smallest value for \code{lambda1}, as a fraction of the maximum \code{lambda1} value. If \eqn{n>M}, the default is 0.0001, and if \eqn{n<M}, the default is 0.01.}

\item{intercept}{logical. If it is TRUE, a column of ones is added to the design matrix.}

\item{overall.group}{logical. If it is TRUE, an overall group including all penalized covariates is added.}

\item{control}{a list of control parameters for the ADMM algorithm. See ‘Details’.}
}
\value{
A named list containing \describe{
\item{sp.coefficients}{a length-\eqn{M} solution vector for the parameters \eqn{\psi}, which corresponds to the minimum in-sample MSE.}
\item{sp.coef.path}{an \eqn{(n_{\lambda_1}\times M)} matrix of estimated \eqn{\psi} coefficients for each \code{lambda1}.}
\item{sp.fun}{a length-\eqn{r} vector providing the estimated functional coefficient for \eqn{\psi(t)}.}
\item{sp.fun.path}{an \eqn{(n_{\lambda_1}\times r)} matrix providing the estimated functional coefficients for \eqn{\psi(t)} for each \code{lambda1}.}
\item{coefficients}{a length-\eqn{q} solution vector for the parameters \eqn{\gamma}, which corresponds to the minimum in-sample MSE. 
It is provided only when either the matrix \eqn{Z} in input is not NULL or the intercept is set to TRUE.}
\item{coef.path}{an \eqn{(n_{\lambda_1}\times q)} matrix of estimated \eqn{\gamma} coefficients for each \code{lambda1}. 
It is provided only when either the matrix \eqn{Z} in input is not NULL or the intercept is set to TRUE.}
\item{dof}{estimated degrees of freedom for each value of the regularization parameter.}
\item{dof.groups_active}{active groups used in the degrees-of-freedom computation for each value of the regularization parameter.}
\item{dof.coeff_active}{active coefficient indices used in the degrees-of-freedom computation for each value of the regularization parameter.}
\item{dof.vars_active}{active basis coefficients used in the degrees-of-freedom computation for each value of the regularization parameter.}
\item{bic}{Bayesian information criterion computed along the regularization path.}
\item{lambda1.min.bic}{optimal value of the \code{lambda1} regularization parameter selected by minimizing the Bayesian Information Criterion (BIC).}
\item{lambda1.min.bic}{optimal value of the \code{lambda2} regularization parameter selected by minimizing the Bayesian Information Criterion (BIC).}
\item{lambda1}{sequence of regularization parameters \code{lambda1}.}
\item{lambda2}{sequence of smoothing regularization parameters \code{lambda2}. It is \code{NULL} when no smoothness penalty is used.}
\item{mse}{in-sample mean squared error.}
\item{lambda1.min.mse}{value of \code{lambda1} that attains the minimum in-sample MSE.}
\item{lambda2.min.mse}{value of \code{lambda2} that attains the minimum in-sample MSE.}
\item{min.mse}{minimum value of the in-sample MSE for the sequence of \code{lambda1}.}
\item{convergence}{logical. 1 denotes achieved convergence.}
\item{elapsedTime}{elapsed time in seconds.}
\item{iternum}{number of iterations.}
}

When you run the algorithm, output returns not only the solution, but also the iteration history recording
following fields over iterates,
\describe{
\item{objval}{objective function value.}
\item{r_norm}{norm of primal residual.}
\item{s_norm}{norm of dual residual.}
\item{eps_pri}{feasibility tolerance for primal feasibility condition.}
\item{eps_dual}{feasibility tolerance for dual feasibility condition.}
}
Iteration stops when both \code{r_norm} and \code{s_norm} values
become smaller than \code{eps_pri} and \code{eps_dual}, respectively.
}
\description{
Overlap Group-LASSO for scalar-on-function regression model solves the following optimization problem
\deqn{\textrm{min}_{\psi,\gamma} ~ \frac{1}{2} \sum_{i=1}^n \left( y_i - \int x_i(t) \psi(t) dt-z_i^\intercal\gamma \right)^2 + \lambda_1 \sum_{g=1}^{G} \Vert S_{g}T\psi \Vert_2+\lambda_2\Vert D\psi\Vert_2^2}
to obtain a sparse coefficient vector \eqn{\psi\in\mathbb{R}^{M}} for the functional penalized predictor \eqn{x(t)} and a coefficient vector \eqn{\gamma\in\mathbb{R}^q} for the unpenalized scalar predictors \eqn{z_1,\dots,z_q}. The regression function is \eqn{\psi(t)=\varphi(t)^\intercal\psi}
where \eqn{\varphi(t)} is a B-spline basis of order \eqn{d} and dimension \eqn{M}. For each group \eqn{g}, each row of 
the matrix \eqn{S_g\in\mathbb{R}^{d\times M}} has non-zero entries only for those bases belonging 
to that group. These values are provided by the arguments \code{groups} and \code{group_weights} (see below). 
Each basis function belongs to more than one group. The diagonal matrix \eqn{T\in\mathbb{R}^{M\times M}} contains 
the basis-specific weights. These values are provided by the argument \code{var_weights} (see below).
The matrix \eqn{D} is a discrete difference matrix whose order is specified
by the argument \code{diff_order}, which must be greater than or equal to one.
The penalty \eqn{\lambda_2 \lVert D\psi\rVert_2^2} controls the smoothness
of the estimated coefficient function.
The regularization path is computed for the overlap group-LASSO penalty at a grid of values for the regularization
parameter \eqn{\lambda_1} using the alternating direction method of multipliers (ADMM). See Boyd et al. (2011) and Lin et al. (2022) 
for details on the ADMM method.
}
\section{Details}{
 The control argument is a list that can supply any of the following components:\describe{
\item{adaptation}{logical. If it is TRUE, ADMM with adaptation is performed. The default value is TRUE. See Boyd et al. (2011) for details.}
\item{rho}{an augmented Lagrangian parameter. The default value is 1.}
\item{tau.ada}{an adaptation parameter greater than one. Only needed if adaptation = TRUE. The default value is 2. See Boyd et al. (2011) and Lin et al. (2022) for details.}
\item{mu.ada}{an adaptation parameter greater than one. Only needed if adaptation = TRUE. The default value is 10. See Boyd et al. (2011) and Lin et al. (2022) for details.}
\item{dof.toler_c}{tolerance parameter used in the degrees-of-freedom computation to determine active constraints.}
\item{dof.toler_d}{tolerance parameter used in the degrees-of-freedom computation to determine active coefficients or groups.}
\item{abstol}{absolute tolerance stopping criterion. The default value is sqrt(sqrt(.Machine$double.eps)).}
\item{reltol}{relative tolerance stopping criterion. The default value is sqrt(.Machine$double.eps).}
\item{maxit}{maximum number of iterations. The default value is 100.}
\item{print.out}{logical. If it is TRUE, a message about the procedure is printed. The default value is TRUE.}
}
}

\examples{

## generate sample data
set.seed(1)
n     <- 40
p     <- 18                                  # number of basis to GENERATE beta
r     <- 100
s     <- seq(0, 1, length.out = r)

beta_basis <- splines::bs(s, df = p, intercept = TRUE)    # basis
coef_data  <- matrix(rnorm(n*floor(p/2)), n, floor(p/2))        
fun_data   <- coef_data \%*\% t(splines::bs(s, df = floor(p/2), intercept = TRUE))     

x_0   <- apply(matrix(rnorm(p, sd=1),p,1), 1, fdaSP::softhresh, 1)  # regression coefficients 
x_fun <- beta_basis \%*\% x_0                

b     <- fun_data \%*\% x_fun + rnorm(n, sd = sqrt(crossprod(fun_data \%*\% x_fun ))/10)
l     <- 10^seq(2, -4, length.out = 30)

## set the hyper-parameters
maxit          <- 1000
rho_adaptation <- TRUE
rho            <- 1
reltol         <- 1e-5
abstol         <- 1e-5

# model fit
mod1 <- f2sSP(vY = b, mX = fun_data, 
              mZ = NULL, M = 5,
              group_weights = NULL, 
              var_weights = NULL, 
              standardize.data = FALSE, 
              intercept = FALSE, splOrd = 4,
              lambda1 = NULL, nlambda1 = 30, 
              lambda1.min.ratio = NULL, 
              weights_adaptive = TRUE, 
              overall.group = FALSE, 
              control = list("maxit"      = maxit,
                             "abstol"     = abstol, 
                             "reltol"     = reltol, 
                             "adaptation" = rho_adaptation, 
                             "rho"        = rho, 
                             "print.out"  = FALSE)) 

# plot coefficiente path
matplot(log(mod1$lambda1), mod1$sp.coef.path, type = "l", 
        xlab = latex2exp::TeX("$\\\\log(\\\\lambda_1)$"), ylab = "", bty = "n", lwd = 1.2)
        
 # model fit with smoothing      
 lam2 <- 10^seq(5, -3, length.out = 20)  # lambda2 sequence 
 mod2 <- f2sSP(vY = b, mX = fun_data, 
               mZ = NULL, M = 5,
               group_weights = NULL, 
               var_weights   = NULL, 
               standardize.data = FALSE, 
               intercept = FALSE, splOrd = 4,
               lambda1 = NULL, nlambda1 = 30, 
               lambda1.min.ratio = NULL,
               lambda2 = lam2, 
               diff_order = 2,
               weights_adaptive = TRUE, 
               overall.group = FALSE, 
               control = list("maxit"      = maxit,
                              "abstol"     = abstol, 
                              "reltol"     = reltol, 
                              "adaptation" = rho_adaptation, 
                              "rho"        = rho, 
                              "print.out"  = FALSE))       

idx <- which(mod2$lambda2 == mod2$lambda2.min.bic)
matplot(log(mod2$lambda1), mod2$sp.coef.path[, idx,, drop = TRUE], type = "l",
        xlab = latex2exp::TeX("$\\\\log(\\\\lambda_1)$"), ylab = "",
        bty = "n", lwd = 1.2)                              
                                         
}
\references{
\insertRef{bernardi_etal.2022}{fdaSP}

\insertRef{boyd_etal.2011}{fdaSP}

\insertRef{jenatton_etal.2011}{fdaSP}

\insertRef{lin_etal.2022}{fdaSP}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/f2sreg.R
\name{f2sSP_cv}
\alias{f2sSP_cv}
\title{Cross-validation for Overlap Group Least Absolute Shrinkage and Selection Operator on scalar-on-function regression model}
\usage{
f2sSP_cv(
  vY,
  mX,
  mZ = NULL,
  M,
  group_weights = NULL,
  var_weights = NULL,
  standardize.data = FALSE,
  splOrd = 4,
  diff_order = 1,
  lambda1 = NULL,
  lambda1.min.ratio = NULL,
  nlambda1 = NULL,
  lambda2 = NULL,
  cv.fold = 5,
  intercept = FALSE,
  overall.group = FALSE,
  control = list()
)
}
\arguments{
\item{vY}{a length-\eqn{n} vector of observations of the scalar response variable.}

\item{mX}{a \eqn{(n\times r)} matrix of observations of the functional covariate.}

\item{mZ}{an \eqn{(n\times q)} full column rank matrix of scalar predictors that are not penalized.}

\item{M}{number of elements of the B-spline basis vector \eqn{\varphi(t)}.}

\item{group_weights}{a vector of length \eqn{G} containing group-specific weights. The default is square root of the group cardinality, see Bernardi et al. (2022).}

\item{var_weights}{a vector of length \eqn{M} containing basis-specific weights. The default is a vector where 
each entry is the reciprocal of the number of groups including that basis. See Bernardi et al. (2022) for details.}

\item{standardize.data}{logical. Should data be standardized?}

\item{splOrd}{the order \eqn{d} of the spline basis.}

\item{diff_order}{a positive integer specifying the order of the discrete
difference operator used in the smoothness penalty. The default is 1.}

\item{lambda1}{either a regularization parameter or a vector of regularization parameters.}

\item{lambda1.min.ratio}{smallest value for \code{lambda1}, as a fraction of the maximum \code{lambda1} value. If \eqn{n>M}, the default is 0.0001, and if \eqn{n<M}, the default is 0.01.}

\item{nlambda1}{the number of \code{lambda1} values - default is 30.}

\item{lambda2}{either a non-negative smoothing regularization parameter or a vector of
smoothing regularization parameters. If \code{NULL}, no smoothness penalty is used.
In this latter case the routine computes the whole path. If it is NULL values for \code{lambda1} are provided by the routine.}

\item{cv.fold}{the number of folds - default is 5.}

\item{intercept}{logical. If it is TRUE, a column of ones is added to the design matrix.}

\item{overall.group}{logical. If it is TRUE, an overall group including all penalized covariates is added.}

\item{control}{a list of control parameters for the ADMM algorithm. See ‘Details’.}
}
\value{
A named list containing \describe{
\item{sp.coefficients}{a length-\eqn{M} solution vector for the parameters \eqn{\psi}, which corresponds to the minimum cross-validated MSE.}
\item{sp.fun}{a length-\eqn{r} vector providing the estimated functional coefficient for \eqn{\psi(t)} corresponding to the minimum cross-validated MSE.}
\item{coefficients}{a length-\eqn{q} solution vector for the parameters \eqn{\gamma}, which corresponds to the minimum cross-validated MSE.
It is provided only when either the matrix \eqn{Z} in input is not NULL or the intercept is set to TRUE.} 
\item{lambda1}{sequence of regularization parameters \code{lambda1}.}
\item{lambda2}{sequence of smoothing regularization parameters \code{lambda2}. It is \code{NULL} when no smoothness penalty is used.}
\item{mse}{cross-validated mean squared error.}
\item{min.mse}{minimum value of the cross-validated MSE for the sequence of lambda.}
\item{lambda1.min.mse}{value of \code{lambda1} that attains the minimum in-sample MSE.}
\item{lambda2.min.mse}{value of \code{lambda2} that attains the minimum in-sample MSE.}
\item{convergence}{logical. 1 denotes achieved convergence.}
\item{elapsedTime}{elapsed time in seconds.}
\item{iternum}{number of iterations.}
}

Iteration stops when both \code{r_norm} and \code{s_norm} values
become smaller than \code{eps_pri} and \code{eps_dual}, respectively.
}
\description{
Overlap Group-LASSO for scalar-on-function regression model solves the following optimization problem
\deqn{\textrm{min}_{\psi,\gamma} ~ \frac{1}{2} \sum_{i=1}^n \left( y_i - \int x_i(t) \psi(t) dt-z_i^\intercal\gamma \right)^2 + \lambda_1 \sum_{g=1}^{G} \Vert S_{g}T\psi \Vert_2+\lambda_2\Vert D\psi\Vert_2^2}
to obtain a sparse coefficient vector \eqn{\psi\in\mathbb{R}^{M}} for the functional penalized predictor \eqn{x(t)} and a coefficient vector \eqn{\gamma\in\mathbb{R}^q} for the unpenalized scalar predictors \eqn{z_1,\dots,z_q}. The regression function is \eqn{\psi(t)=\varphi(t)^\intercal\psi}
where \eqn{\varphi(t)} is a B-spline basis of order \eqn{d} and dimension \eqn{M}. 
For each group \eqn{g}, each row of the matrix \eqn{S_g\in\mathbb{R}^{d\times M}} has non-zero entries only for those bases belonging 
to that group. These values are provided by the arguments \code{groups} and \code{group_weights} (see below). 
Each basis function belongs to more than one group. The diagonal matrix \eqn{T\in\mathbb{R}^{M\times M}} contains 
the basis specific weights. These values are provided by the argument \code{var_weights} (see below).
The matrix \eqn{D} is a discrete difference matrix whose order is specified
by the argument \code{diff_order}, which must be greater than or equal to one.
The penalty \eqn{\lambda_2 \lVert D\psi\rVert_2^2} controls the smoothness
of the estimated coefficient function.
The regularization path is computed for the overlap group-LASSO penalty at a grid of values for the regularization
parameter \eqn{\lambda1} using the alternating direction method of multipliers (ADMM). See Boyd et al. (2011) and Lin et al. (2022) 
for details on the ADMM method.
}
\section{Details}{
 The control argument is a list that can supply any of the following components:\describe{
\item{adaptation}{logical. If it is TRUE, ADMM with adaptation is performed. The default value is TRUE. See Boyd et al. (2011) for details.}
\item{rho}{an augmented Lagrangian parameter. The default value is 1.}
\item{tau.ada}{an adaptation parameter greater than one. Only needed if adaptation = TRUE. The default value is 2. See Boyd et al. (2011) and Lin et al. (2022) for details.}
\item{mu.ada}{an adaptation parameter greater than one. Only needed if adaptation = TRUE. The default value is 10. See Boyd et al. (2011) and Lin et al. (2022) for details.}
\item{abstol}{absolute tolerance stopping criterion. The default value is sqrt(sqrt(.Machine$double.eps)).}
\item{reltol}{relative tolerance stopping criterion. The default value is sqrt(.Machine$double.eps).}
\item{maxit}{maximum number of iterations. The default value is 100.}
\item{print.out}{logical. If it is TRUE, a message about the procedure is printed. The default value is TRUE.}
}
}

\examples{

## generate sample data and functional coefficients
set.seed(1)
n     <- 40
p     <- 18                                 
r     <- 100
s     <- seq(0, 1, length.out = r)

beta_basis <- splines::bs(s, df = p, intercept = TRUE)    # basis
coef_data  <- matrix(rnorm(n*floor(p/2)), n, floor(p/2))        
fun_data   <- coef_data \%*\% t(splines::bs(s, 
                  df = floor(p/2), intercept = TRUE))     

x_0   <- apply(matrix(rnorm(p, sd=1),p,1), 1, fdaSP::softhresh, 1)  
x_fun <- beta_basis \%*\% x_0                

b     <- fun_data \%*\% x_fun + rnorm(n, sd = sqrt(crossprod(fun_data \%*\% x_fun ))/10)
l     <- 10^seq(2, -4, length.out = 30)

## set the hyper-parameters
maxit          <- 1000
rho_adaptation <- TRUE
rho            <- 1
reltol         <- 1e-5
abstol         <- 1e-5

## run cross-validation
mod_cv <- f2sSP_cv(vY = b, mX = fun_data, M = p,
                   group_weights = NULL, var_weights = NULL, 
                   standardize.data = FALSE, splOrd = 4,
                   lambda1 = NULL, lambda1.min.ratio = 1e-5, 
                   nlambda1 = 30, cv.fold = 5, intercept = FALSE, 
                   control = list("abstol" = abstol, 
                                  "reltol" = reltol, 
                                  "adaptation" = rho_adaptation,
                                  "rho" = rho, 
                                  "print.out" = FALSE))
                                          
### graphical presentation
plot(log(mod_cv$lambda1), mod_cv$mse, type = "l", 
     col = "blue", lwd = 2, bty = "n", 
     xlab = latex2exp::TeX("$\\\\log(\\\\lambda_1)$"), 
     ylab = "Prediction Error", 
     ylim = range(mod_cv$mse - mod_cv$mse.sd, 
     mod_cv$mse + mod_cv$mse.sd),
     main = "Cross-validated Prediction Error")
fdaSP::confband(xV = log(mod_cv$lambda1), 
                yVmin = mod_cv$mse - mod_cv$mse.sd, 
                yVmax = mod_cv$mse + mod_cv$mse.sd)       
idx <- which(mod_cv$lambda1 == mod_cv$lambda1.min.mse)
abline(v = log(mod_cv$lambda1[idx]), 
       col = "red", lwd = 1.0)

### comparison with oracle error
mod <- f2sSP(vY = b, mX = fun_data, M = p, 
             group_weights = NULL, var_weights = NULL, 
             standardize.data = FALSE, splOrd = 4,
             lambda1 = NULL, nlambda1 = 30, 
             lambda1.min.ratio = 1e-5, intercept = FALSE,
             control = list("abstol" = abstol, 
                            "reltol" = reltol, 
                            "adaptation" = rho_adaptation, 
                            "rho" = rho, 
                            "print.out" = FALSE))
                                    
err_mod <- apply(mod$sp.coef.path, 1, 
                 function(x) sum((x - x_0)^2))
plot(log(mod$lambda1), 
err_mod, type = "l", col = "blue", 
     lwd = 2, 
     xlab = latex2exp::TeX("$\\\\log(\\\\lambda_1)$"), 
     ylab = "Estimation Error", 
     main = "True Estimation Error", bty = "n")
abline(v = log(mod$lambda1[which(err_mod == min(err_mod))]), 
       col = "red", lwd = 1.0)
idx <- which(mod_cv$lambda1 == mod_cv$lambda1.min.mse)
abline(v = log(mod_cv$lambda1[idx]), 
       col = "red", lwd = 1.0, lty = 2)                                      

}
\references{
\insertRef{bernardi_etal.2022}{fdaSP}

\insertRef{boyd_etal.2011}{fdaSP}

\insertRef{jenatton_etal.2011}{fdaSP}

\insertRef{lin_etal.2022}{fdaSP}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{fdaSP.internal}
\alias{fdaSP.internal}
\alias{.lm_dof_LASSO_1lambda}
\alias{.lm_dof_OVGLASSO_1lambda}
\alias{.lm_cov_dof_OVGLASSO_1lambda}
\alias{.lm_adaptive_dof_OVGLASSO_1lambda}
\alias{.lm_cov_adaptive_dof_OVGLASSO_1lamba}
\alias{.f2s_dof_1lambda}
\alias{.f2s_cov_dof_1lambda}
\alias{.f2s_smo_dof_1lambda}
\alias{.f2s_cov_smo_dof_1lambda}
\alias{.f2s_adaptive_dof_1lambda}
\alias{.f2s_cov_adaptive_dof_1lambda}
\alias{.f2s_smo_adaptive_dof_1lambda}
\alias{.f2s_adaptive_cov_smo_dof_1lambda}
\alias{.f2s_bic}
\alias{.f2s_cov_bic}
\alias{.f2s_ebic}
\alias{.f2s_cov_ebic}
\alias{.admm_glasso}
\alias{.admm_glasso_cov}
\alias{.admm_glasso_fast}
\alias{.admm_glasso_cov_fast}
\alias{.admm_spglasso}
\alias{.admm_spglasso_cov}
\alias{.admm_spglasso_fast}
\alias{.admm_spglasso_cov_fast}
\alias{.admm_lasso}
\alias{.admm_lasso_cov}
\alias{.admm_adalasso}
\alias{.admm_adalasso_cov}
\alias{.admm_lasso_fast}
\alias{.admm_lasso_cov_fast}
\alias{.admm_adalasso_fast}
\alias{.admm_adalasso_cov_fast}
\alias{.admm_ovglasso}
\alias{.admm_ovglasso_smo}
\alias{.admm_ovglasso_cov}
\alias{.admm_ovglasso_cov_smo}
\alias{.admm_ovglasso_fast}
\alias{.admm_ovglasso_smo_fast}
\alias{.admm_ovglasso_cov_fast}
\alias{.admm_ovglasso_cov_smo_fast}
\alias{.admm_spovglasso}
\alias{.admm_spovglasso_cov}
\alias{.admm_spovglasso_fast}
\alias{.admm_spovglasso_cov_fast}
\alias{.forward_diff_penalty_matrix}
\alias{.forward_diff_difference_matrix}
\title{Compute the discrete difference penalty matrix Q = D^T D}
\usage{
.lm_dof_LASSO_1lambda(
  X,
  coeff,
  lambda,
  Uvec,
  err_primal,
  err_dual,
  rho,
  toler_c,
  toler_d
)

.lm_dof_OVGLASSO_1lambda(
  X,
  coeff,
  lambda,
  GRmat,
  group_weights,
  var_weights,
  Uvec,
  err_primal,
  err_dual,
  rho,
  toler_c,
  toler_d
)

.lm_cov_dof_OVGLASSO_1lambda(
  X,
  Z,
  coeff_X,
  lambda,
  GRmat,
  group_weights,
  var_weights,
  Uvec,
  err_primal,
  err_dual,
  rho,
  toler_c,
  toler_d
)

.lm_adaptive_dof_OVGLASSO_1lambda(
  X,
  coeff,
  coeff_LS,
  lambda,
  GRmat,
  group_weights,
  var_weights,
  Uvec,
  err_primal,
  err_dual,
  rho,
  toler_c,
  toler_d
)

.lm_cov_adaptive_dof_OVGLASSO_1lamba(
  X,
  Z,
  coeff_X,
  coeff_X_LS,
  lambda,
  GRmat,
  group_weights,
  var_weights,
  Uvec,
  err_primal,
  err_dual,
  rho,
  toler_c,
  toler_d
)

.f2s_dof_1lambda(
  W,
  coeff,
  lambda,
  GRmat,
  group_weights,
  var_weights,
  Uvec,
  err_primal,
  err_dual,
  rho,
  toler_c,
  toler_d
)

.f2s_cov_dof_1lambda(
  W,
  Z,
  coeff_W,
  lambda,
  GRmat,
  group_weights,
  var_weights,
  Uvec,
  err_primal,
  err_dual,
  rho,
  toler_c,
  toler_d
)

.f2s_smo_dof_1lambda(
  W,
  coeff,
  lambda,
  lambda2,
  diff_order,
  GRmat,
  group_weights,
  var_weights,
  Uvec,
  err_primal,
  err_dual,
  rho,
  toler_c,
  toler_d
)

.f2s_cov_smo_dof_1lambda(
  W,
  Z,
  coeff_W,
  lambda,
  lambda2,
  diff_order,
  GRmat,
  group_weights,
  var_weights,
  Uvec,
  err_primal,
  err_dual,
  rho,
  toler_c,
  toler_d
)

.f2s_adaptive_dof_1lambda(
  W,
  coeff,
  coeff_LS,
  lambda,
  GRmat,
  group_weights,
  var_weights,
  Uvec,
  err_primal,
  err_dual,
  rho,
  toler_c,
  toler_d
)

.f2s_cov_adaptive_dof_1lambda(
  W,
  Z,
  coeff_W,
  coeff_W_LS,
  lambda,
  GRmat,
  group_weights,
  var_weights,
  Uvec,
  err_primal,
  err_dual,
  rho,
  toler_c,
  toler_d
)

.f2s_smo_adaptive_dof_1lambda(
  W,
  coeff,
  coeff_LS,
  lambda,
  lambda2,
  diff_order,
  GRmat,
  group_weights,
  var_weights,
  Uvec,
  err_primal,
  err_dual,
  rho,
  toler_c,
  toler_d
)

.f2s_adaptive_cov_smo_dof_1lambda(
  W,
  Z,
  coeff_W,
  coeff_W_LS,
  lambda,
  lambda2,
  diff_order,
  GRmat,
  group_weights,
  var_weights,
  Uvec,
  err_primal,
  err_dual,
  rho,
  toler_c,
  toler_d
)

.f2s_bic(y, W, coeff_path, df)

.f2s_cov_bic(y, W, Z, W_coeff_path, Z_coeff_path, df)

.f2s_ebic(y, X, coeff_path, df, kappa)

.f2s_cov_ebic(y, W, Z, W_coeff_path, Z_coeff_path, df, kappa)

.admm_glasso(
  A,
  b,
  groups,
  group_weights,
  var_weights,
  u,
  z,
  lambda,
  rho_adaptation,
  rho,
  tau,
  mu,
  reltol,
  abstol,
  maxiter,
  ping
)

.admm_glasso_cov(
  W,
  Z,
  y,
  u,
  z,
  groups,
  group_weights,
  var_weights,
  lambda,
  rho_adaptation,
  rho,
  tau,
  mu,
  reltol,
  abstol,
  maxiter,
  ping
)

.admm_glasso_fast(
  A,
  b,
  groups,
  group_weights,
  var_weights,
  lambda,
  rho_adaptation,
  rho,
  tau,
  mu,
  reltol,
  abstol,
  maxiter,
  ping
)

.admm_glasso_cov_fast(
  W,
  Z,
  y,
  groups,
  group_weights,
  var_weights,
  lambda,
  rho_adaptation,
  rho,
  tau,
  mu,
  reltol,
  abstol,
  maxiter,
  ping
)

.admm_spglasso(
  A,
  b,
  groups,
  group_weights,
  var_weights,
  var_weights_L1,
  u,
  z,
  lambda1,
  lambda2,
  rho_adaptation,
  rho,
  tau,
  mu,
  reltol,
  abstol,
  maxiter,
  ping
)

.admm_spglasso_cov(
  W,
  Z,
  y,
  groups,
  group_weights,
  var_weights,
  var_weights_L1,
  u,
  z,
  lambda1,
  lambda2,
  rho_adaptation,
  rho,
  tau,
  mu,
  reltol,
  abstol,
  maxiter,
  ping
)

.admm_spglasso_fast(
  A,
  b,
  groups,
  group_weights,
  var_weights,
  var_weights_L1,
  lambda,
  alpha,
  rho_adaptation,
  rho,
  tau,
  mu,
  reltol,
  abstol,
  maxiter,
  ping
)

.admm_spglasso_cov_fast(
  W,
  Z,
  y,
  groups,
  group_weights,
  var_weights,
  var_weights_L1,
  lambda,
  alpha,
  rho_adaptation,
  rho,
  tau,
  mu,
  reltol,
  abstol,
  maxiter,
  ping
)

.admm_lasso(
  A,
  b,
  u,
  z,
  lambda,
  rho_adaptation,
  rho,
  tau,
  mu,
  reltol,
  abstol,
  maxiter,
  ping
)

.admm_lasso_cov(
  W,
  Z,
  y,
  u,
  z,
  lambda,
  rho_adaptation,
  rho,
  tau,
  mu,
  reltol,
  abstol,
  maxiter,
  ping
)

.admm_adalasso(
  A,
  b,
  var_weights,
  u,
  z,
  lambda,
  rho_adaptation,
  rho,
  tau,
  mu,
  reltol,
  abstol,
  maxiter,
  ping
)

.admm_adalasso_cov(
  W,
  Z,
  y,
  var_weights,
  u,
  z,
  lambda,
  rho_adaptation,
  rho,
  tau,
  mu,
  reltol,
  abstol,
  maxiter,
  ping
)

.admm_lasso_fast(
  A,
  b,
  lambda,
  rho_adaptation,
  rho,
  tau,
  mu,
  reltol,
  abstol,
  maxiter,
  ping
)

.admm_lasso_cov_fast(
  W,
  Z,
  y,
  lambda,
  rho_adaptation,
  rho,
  tau,
  mu,
  reltol,
  abstol,
  maxiter,
  ping
)

.admm_adalasso_fast(
  A,
  b,
  var_weights,
  lambda,
  rho_adaptation,
  rho,
  tau,
  mu,
  reltol,
  abstol,
  maxiter,
  ping
)

.admm_adalasso_cov_fast(
  W,
  Z,
  y,
  var_weights,
  lambda,
  rho_adaptation,
  rho,
  tau,
  mu,
  reltol,
  abstol,
  maxiter,
  ping
)

.admm_ovglasso(
  A,
  b,
  groups,
  group_weights,
  var_weights,
  u,
  z,
  lambda,
  rho_adaptation,
  rho,
  tau,
  mu,
  reltol,
  abstol,
  maxiter,
  ping
)

.admm_ovglasso_smo(
  A,
  b,
  groups,
  group_weights,
  var_weights,
  u,
  z,
  lambda,
  lambda2,
  diff_order,
  rho_adaptation,
  rho,
  tau,
  mu,
  reltol,
  abstol,
  maxiter,
  ping
)

.admm_ovglasso_cov(
  W,
  Z,
  y,
  u,
  z,
  groups,
  group_weights,
  var_weights,
  lambda,
  rho_adaptation,
  rho,
  tau,
  mu,
  reltol,
  abstol,
  maxiter,
  ping
)

.admm_ovglasso_cov_smo(
  W,
  Z,
  y,
  u,
  z,
  groups,
  group_weights,
  var_weights,
  lambda,
  lambda2,
  diff_order,
  rho_adaptation,
  rho,
  tau,
  mu,
  reltol,
  abstol,
  maxiter,
  ping
)

.admm_ovglasso_fast(
  A,
  b,
  groups,
  group_weights,
  var_weights,
  lambda,
  rho_adaptation,
  rho,
  tau,
  mu,
  reltol,
  abstol,
  maxiter,
  ping
)

.admm_ovglasso_smo_fast(
  A,
  b,
  groups,
  group_weights,
  var_weights,
  lambda,
  lambda2,
  diff_order,
  rho_adaptation,
  rho,
  tau,
  mu,
  reltol,
  abstol,
  maxiter,
  ping
)

.admm_ovglasso_cov_fast(
  W,
  Z,
  y,
  groups,
  group_weights,
  var_weights,
  lambda,
  rho_adaptation,
  rho,
  tau,
  mu,
  reltol,
  abstol,
  maxiter,
  ping
)

.admm_ovglasso_cov_smo_fast(
  W,
  Z,
  y,
  groups,
  group_weights,
  var_weights,
  lambda,
  lambda2,
  diff_order,
  rho_adaptation,
  rho,
  tau,
  mu,
  reltol,
  abstol,
  maxiter,
  ping
)

.admm_spovglasso(
  A,
  b,
  groups,
  group_weights,
  var_weights,
  var_weights_L1,
  u,
  z,
  lambda,
  alpha,
  rho_adaptation,
  rho,
  tau,
  mu,
  reltol,
  abstol,
  maxiter,
  ping
)

.admm_spovglasso_cov(
  W,
  Z,
  y,
  u,
  z,
  groups,
  group_weights,
  var_weights,
  var_weights_L1,
  lambda,
  alpha,
  rho_adaptation,
  rho,
  tau,
  mu,
  reltol,
  abstol,
  maxiter,
  ping
)

.admm_spovglasso_fast(
  A,
  b,
  groups,
  group_weights,
  var_weights,
  var_weights_L1,
  lambda,
  alpha,
  rho_adaptation,
  rho,
  tau,
  mu,
  reltol,
  abstol,
  maxiter,
  ping
)

.admm_spovglasso_cov_fast(
  W,
  Z,
  y,
  groups,
  group_weights,
  var_weights,
  var_weights_L1,
  lambda,
  alpha,
  rho_adaptation,
  rho,
  tau,
  mu,
  reltol,
  abstol,
  maxiter,
  ping
)

.forward_diff_penalty_matrix(n, h, d)

.forward_diff_difference_matrix(n, h, d)
}
\arguments{
\item{n}{Length of the vector the operator acts on.}

\item{h}{Step size (default = 1.0).}

\item{d}{Derivative order (integer >= 1).}
}
\value{
Symmetric n x n matrix Q = D^T D.
}
\description{
Constructs the d-th order forward-difference operator D (scaled by 1/h^d)
and returns Q = D^T D.
}
\examples{
#Q1 <- forward_diff_penalty_matrix(5, h = 1, d = 1)
#Q2 <- forward_diff_penalty_matrix(5, h = 1, d = 2)
#Q1
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lmreg.R
\name{lmSP_cv}
\alias{lmSP_cv}
\title{Cross-validation for Sparse Adaptive Overlap Group Least Absolute Shrinkage and Selection Operator}
\usage{
lmSP_cv(
  X,
  Z = NULL,
  y,
  penalty = c("LASSO", "GLASSO", "spGLASSO", "OVGLASSO", "spOVGLASSO"),
  groups = NULL,
  group_weights = NULL,
  var_weights = NULL,
  var_weights_L1 = NULL,
  cv.fold = 5,
  standardize.data = TRUE,
  intercept = FALSE,
  overall.group = FALSE,
  lambda = NULL,
  alpha = NULL,
  lambda.min.ratio = NULL,
  nlambda = 30,
  control = list()
)
}
\arguments{
\item{X}{an \eqn{(n\times p)} matrix of penalized predictors.}

\item{Z}{an \eqn{(n\times q)} full column rank matrix of predictors that are not penalized.}

\item{y}{a length-\eqn{n} response vector.}

\item{penalty}{choose one from the following options: 'LASSO', for the or adaptive-Lasso penalties, 'GLASSO', 
for the group-Lasso penalty, 'spGLASSO', for the sparse group-Lasso penalty, 'OVGLASSO', 
for the overlap group-Lasso penalty and 'spOVGLASSO', for the sparse overlap group-Lasso penalty.}

\item{groups}{either a vector of length \eqn{p} of consecutive integers describing the grouping of the coefficients, 
or a list with two elements: the first element is a vector of length \eqn{\sum_{g=1}^G n_g} containing the variables belonging to each group, where \eqn{n_g} is the cardinality of the \eqn{g}-th group, 
while the second element is a vector of length \eqn{G} containing the group lengths (see example below).}

\item{group_weights}{a vector of length \eqn{G} containing group-specific weights. The default is square root of the group cardinality, see Yuan and Lin (2006).}

\item{var_weights}{a vector of length \eqn{p} containing variable-specific weights. The default is a vector of ones.}

\item{var_weights_L1}{a vector of length \eqn{p} containing variable-specific weights for the \eqn{L_1} penalty. The default is a vector of ones.}

\item{cv.fold}{the number of folds - default is 5.}

\item{standardize.data}{logical. Should data be standardized?}

\item{intercept}{logical. If it is TRUE, a column of ones is added to the design matrix.}

\item{overall.group}{logical. This setting is only available for the overlap group-LASSO and the sparse overlap group-LASSO penalties, otherwise it is set to NULL. If it is TRUE, an overall group including all penalized covariates is added.}

\item{lambda}{either a regularization parameter or a vector of regularization parameters. In this latter case the routine computes the whole path. If it is NULL values for lambda are provided by the routine.}

\item{alpha}{the sparse overlap group-LASSO mixing parameter, with \eqn{0\leq\alpha\leq1}. This setting is only available for the sparse group-LASSO and the sparse overlap group-LASSO penalties, otherwise it is set to NULL. The LASSO and group-LASSO penalties are obtained
by specifying \eqn{\alpha = 1} and \eqn{\alpha = 0}, respectively.}

\item{lambda.min.ratio}{smallest value for lambda, as a fraction of the maximum lambda value. If \eqn{n>p}, the default is 0.0001, and if \eqn{n<p}, the default is 0.01.}

\item{nlambda}{the number of lambda values - default is 30.}

\item{control}{a list of control parameters for the ADMM algorithm. See ‘Details’.}
}
\value{
A named list containing \describe{
\item{sp.coefficients}{a length-\eqn{p} solution vector for the parameters \eqn{\beta}. If \eqn{n_\lambda>1} then  the provided vector corresponds to the minimum cross-validated MSE.}
\item{coefficients}{a length-\eqn{q} solution vector for the parameters \eqn{\gamma}. If \eqn{n_\lambda>1} then  the provided vector corresponds to the minimum cross-validated MSE.
It is provided only when either the matrix \eqn{Z} in input is not NULL or the intercept is set to TRUE.}
\item{sp.coef.path}{an \eqn{(n_\lambda\times p)} matrix of estimated \eqn{\beta} coefficients for each lambda of the provided sequence.}
\item{coef.path}{an \eqn{(n_\lambda\times q)} matrix of estimated \eqn{\gamma} coefficients for each lambda of the provided sequence. 
It is provided only when either the matrix \eqn{Z} in input is not NULL or the intercept is set to TRUE.}
\item{lambda}{sequence of lambda.}
\item{lambda.min}{value of lambda that attains the minimum cross-validated MSE.}
\item{mse}{cross-validated mean squared error.}
\item{min.mse}{minimum value of the cross-validated MSE for the sequence of lambda.}
\item{convergence}{logical. 1 denotes achieved convergence.}
\item{elapsedTime}{elapsed time in seconds.}
\item{iternum}{number of iterations.}
\item{foldid}{a vector of values between 1 and cv.fold identifying what fold each observation is in.}
}

When you run the algorithm, output returns not only the solution, but also the iteration history recording
following fields over iterates:
\describe{
\item{objval}{objective function value}
\item{r_norm}{norm of primal residual}
\item{s_norm}{norm of dual residual}
\item{eps_pri}{feasibility tolerance for primal feasibility condition}
\item{eps_dual}{feasibility tolerance for dual feasibility condition.}
}
Iteration stops when both \code{r_norm} and \code{s_norm} values
become smaller than \code{eps_pri} and \code{eps_dual}, respectively.
}
\description{
Sparse Adaptive overlap group-LASSO, or sparse adaptive group \eqn{L_2}-regularized regression, solves the following optimization problem
\deqn{\textrm{min}_{\beta,\gamma} ~ \frac{1}{2}\|y-X\beta-Z\gamma\|_2^2 + \lambda\Big[(1-\alpha) \sum_{g=1}^G \|S_g T\beta\|_2+\alpha\Vert T_1\beta\Vert_1\Big]}
to obtain a sparse coefficient vector \eqn{\beta\in\mathbb{R}^p} for the matrix of penalized predictors \eqn{X} and a coefficient vector \eqn{\gamma\in\mathbb{R}^q} 
for the matrix of unpenalized predictors \eqn{Z}. For each group \eqn{g}, each row of 
the matrix \eqn{S_g\in\mathbb{R}^{n_g\times p}} has non-zero entries only for those variables belonging 
to that group. These values are provided by the arguments \code{groups} and \code{group_weights} (see below). 
Each variable can belong to more than one group. The diagonal matrix \eqn{T\in\mathbb{R}^{p\times p}} contains the variable-specific weights. These values are
provided by the argument \code{var_weights} (see below). The diagonal matrix \eqn{T_1\in\mathbb{R}^{p\times p}} contains 
the variable-specific \eqn{L_1} weights. These values are provided by the argument \code{var_weights_L1} (see below).
The regularization path is computed for the sparse adaptive overlap group-LASSO penalty at a grid of values for the regularization 
parameter \eqn{\lambda} using the alternating direction method of multipliers (ADMM). See Boyd et al. (2011) and Lin et al. (2022) 
for details on the ADMM method. The regularization is a combination of \eqn{L_2} and
\eqn{L_1} simultaneous constraints. Different specifications of the \code{penalty} argument lead to different models choice:
\describe{
\item{LASSO}{The classical Lasso regularization (Tibshirani, 1996) can be obtained by specifying \eqn{\alpha = 1} 
and the matrix \eqn{T_1} as the \eqn{p \times p} identity matrix. An adaptive version of this model (Zou, 2006) can be obtained if \eqn{T_1} is 
a \eqn{p \times p}  diagonal matrix of adaptive weights. See also Hastie et al. (2015) for further details.}
\item{GLASSO}{The group-Lasso regularization (Yuan and Lin, 2006) can be obtained by specifying \eqn{\alpha = 0}, 
non-overlapping groups in \eqn{S_g} and by setting the matrix \eqn{T} equal to the \eqn{p \times p} identity matrix. 
An adaptive version of this model can be obtained if the matrix \eqn{T} is a \eqn{p \times p} diagonal matrix of adaptive weights. See also Hastie et al. (2015) for further details.}
\item{spGLASSO}{The sparse group-Lasso regularization (Simon et al., 2011) can be obtained by specifying \eqn{\alpha\in(0,1)},  
non-overlapping groups in \eqn{S_g} and by setting the matrices \eqn{T} and \eqn{T_1} equal to the \eqn{p \times p} identity matrix. 
An adaptive version of this model can be obtained if the matrices \eqn{T} and \eqn{T_1} are \eqn{p \times p}  
diagonal matrices of adaptive weights.}
\item{OVGLASSO}{The overlap group-Lasso regularization (Jenatton et al., 2011) can be obtained by specifying 
\eqn{\alpha = 0}, overlapping groups in \eqn{S_g} and by setting the matrix \eqn{T} equal to the \eqn{p \times p} identity matrix. An adaptive version of this model can be obtained if the matrix \eqn{T} is a \eqn{p \times p}  
diagonal matrix of adaptive weights.}
\item{spOVGLASSO}{The sparse overlap group-Lasso regularization (Jenatton et al., 2011) can be obtained by specifying 
\eqn{\alpha\in(0,1)}, overlapping groups in \eqn{S_g} and by setting the matrices \eqn{T} and \eqn{T_1} equal to the \eqn{p \times p} identity matrix. 
An adaptive version of this model can be obtained if the matrices \eqn{T} and \eqn{T_1} are \eqn{p \times p}  diagonal matrices of adaptive weights.}
}
}
\section{Details}{
 The control argument is a list that can supply any of the following components:\describe{
\item{adaptation}{logical. If it is TRUE, ADMM with adaptation is performed. The default value is TRUE. See Boyd et al. (2011) for details.}
\item{rho}{an augmented Lagrangian parameter. The default value is 1.}
\item{tau.ada}{an adaptation parameter greater than one. Only needed if adaptation = TRUE. The default value is 2. See Boyd et al. (2011) for details.}
\item{mu.ada}{an adaptation parameter greater than one. Only needed if adaptation = TRUE. The default value is 10. See Boyd et al. (2011) for details.}
\item{abstol}{absolute tolerance stopping criterion. The default value is sqrt(sqrt(.Machine$double.eps)).}
\item{reltol}{relative tolerance stopping criterion. The default value is sqrt(.Machine$double.eps).}
\item{maxit}{maximum number of iterations. The default value is 100.}
\item{print.out}{logical. If it is TRUE, a message about the procedure is printed. The default value is TRUE.}
}
}

\examples{

### generate sample data
set.seed(2023)
n    <- 50
p    <- 30 
X    <- matrix(rnorm(n * p), n, p)

### Example 1, LASSO penalty

beta <- apply(matrix(rnorm(p, sd = 1), p, 1), 1, fdaSP::softhresh, 1.5)
y    <- X \%*\% beta + rnorm(n, sd = sqrt(crossprod(X \%*\% beta)) / 20)

### set the hyper-parameters of the ADMM algorithm
maxit      <- 1000
adaptation <- TRUE
rho        <- 1
reltol     <- 1e-5
abstol     <- 1e-5

### run cross-validation
mod_cv <- lmSP_cv(X = X, y = y, penalty = "LASSO", 
                  standardize.data = FALSE, intercept = FALSE,
                  cv.fold = 5, nlambda = 30, 
                  control = list("adaptation" = adaptation, 
                                 "rho" = rho, 
                                 "maxit" = maxit, "reltol" = reltol, 
                                 "abstol" = abstol, 
                                 "print.out" = FALSE)) 

### graphical presentation
plot(log(mod_cv$lambda), mod_cv$mse, type = "l", col = "blue", lwd = 2, bty = "n", 
     xlab = latex2exp::TeX("$\\\\log(\\\\lambda)$"), ylab = "Prediction Error", 
     ylim = range(mod_cv$mse - mod_cv$mse.sd, mod_cv$mse + mod_cv$mse.sd),
     main = "Cross-validated Prediction Error")
fdaSP::confband(xV = log(mod_cv$lambda), yVmin = mod_cv$mse - mod_cv$mse.sd, 
                yVmax = mod_cv$mse + mod_cv$mse.sd)       
abline(v = log(mod_cv$lambda[which(mod_cv$lambda == mod_cv$lambda.min)]), 
       col = "red", lwd = 1.0)

### comparison with oracle error
mod <- lmSP(X = X, y = y, penalty = "LASSO", 
            standardize.data = FALSE, 
            intercept = FALSE,
            nlambda = 30, 
            control = list("adaptation" = adaptation, 
                           "rho" = rho, 
                           "maxit" = maxit, "reltol" = reltol, 
                           "abstol" = abstol, 
                           "print.out" = FALSE)) 
                                         
err_mod <- apply(mod$sp.coef.path, 1, function(x) sum((x - beta)^2))
plot(log(mod$lambda), err_mod, type = "l", col = "blue", lwd = 2, 
     xlab = latex2exp::TeX("$\\\\log(\\\\lambda)$"), 
     ylab = "Estimation Error", main = "True Estimation Error", bty = "n")
abline(v = log(mod$lambda[which(err_mod == min(err_mod))]), col = "red", lwd = 1.0)
abline(v = log(mod_cv$lambda[which(mod_cv$lambda == mod_cv$lambda.min)]), 
       col = "red", lwd = 1.0, lty = 2)

### Example 2, sparse group-LASSO penalty

beta <- c(rep(4, 12), rep(0, p - 13), -2)
y    <- X \%*\% beta + rnorm(n, sd = sqrt(crossprod(X \%*\% beta)) / 20)

### define groups of dimension 3 each
group1 <- rep(1:10, each = 3)

### set regularization parameter grid
lam   <- 10^seq(1, -2, length.out = 30)

### set the alpha parameter 
alpha <- 0.5

### set the hyper-parameters of the ADMM algorithm
maxit         <- 1000
adaptation    <- TRUE
rho           <- 1
reltol        <- 1e-5
abstol        <- 1e-5

### run cross-validation
mod_cv <- lmSP_cv(X = X, y = y, penalty = "spGLASSO", 
                  groups = group1, cv.fold = 5, 
                  standardize.data = FALSE,  intercept = FALSE, 
                  lambda = lam, alpha = 0.5, 
                  control = list("adaptation" = adaptation, 
                                 "rho" = rho,
                                 "maxit" = maxit, "reltol" = reltol, 
                                 "abstol" = abstol, 
                                 "print.out" = FALSE)) 

### graphical presentation
plot(log(mod_cv$lambda), mod_cv$mse, type = "l", col = "blue", lwd = 2, bty = "n", 
     xlab = latex2exp::TeX("$\\\\log(\\\\lambda)$"), ylab = "Prediction Error", 
     ylim = range(mod_cv$mse - mod_cv$mse.sd, mod_cv$mse + mod_cv$mse.sd),
     main = "Cross-validated Prediction Error")
fdaSP::confband(xV = log(mod_cv$lambda), yVmin = mod_cv$mse - mod_cv$mse.sd, 
                yVmax = mod_cv$mse + mod_cv$mse.sd)       
abline(v = log(mod_cv$lambda[which(mod_cv$lambda == mod_cv$lambda.min)]), 
       col = "red", lwd = 1.0)

### comparison with oracle error
mod <- lmSP(X = X, y = y, 
            penalty = "spGLASSO", 
            groups = group1, 
            standardize.data = FALSE, 
            intercept = FALSE,
            lambda = lam, 
            alpha = 0.5, 
            control = list("adaptation" = adaptation, "rho" = rho, 
                           "maxit" = maxit, "reltol" = reltol, "abstol" = abstol, 
                           "print.out" = FALSE)) 
                                         
err_mod <- apply(mod$sp.coef.path, 1, function(x) sum((x - beta)^2))
plot(log(mod$lambda), err_mod, type = "l", col = "blue", lwd = 2, 
     xlab = latex2exp::TeX("$\\\\log(\\\\lambda)$"), 
     ylab = "Estimation Error", main = "True Estimation Error", bty = "n")
abline(v = log(mod$lambda[which(err_mod == min(err_mod))]), col = "red", lwd = 1.0)
abline(v = log(mod_cv$lambda[which(mod_cv$lambda == mod_cv$lambda.min)]), 
       col = "red", lwd = 1.0, lty = 2)

}
\references{
\insertRef{bernardi_etal.2022}{fdaSP}

\insertRef{boyd_etal.2011}{fdaSP}
 
\insertRef{hastie_etal.2015}{fdaSP}

\insertRef{jenatton_etal.2011}{fdaSP}

\insertRef{lin_etal.2022}{fdaSP} 

\insertRef{simon_etal.2013}{fdaSP}

\insertRef{yuan_lin.2006}{fdaSP}

\insertRef{zou.2006}{fdaSP}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lmreg.R
\name{lmSP}
\alias{lmSP}
\title{Sparse Adaptive Overlap Group Least Absolute Shrinkage and Selection Operator}
\usage{
lmSP(
  X,
  Z = NULL,
  y,
  penalty = c("LASSO", "GLASSO", "spGLASSO", "OVGLASSO", "spOVGLASSO"),
  groups = NULL,
  group_weights = NULL,
  var_weights = NULL,
  var_weights_L1 = NULL,
  weights_adaptive = FALSE,
  adaptive = FALSE,
  standardize.data = TRUE,
  intercept = FALSE,
  overall.group = FALSE,
  lambda = NULL,
  alpha = NULL,
  lambda.min.ratio = NULL,
  nlambda = 30,
  control = list()
)
}
\arguments{
\item{X}{an \eqn{(n\times p)} matrix of penalized predictors.}

\item{Z}{an \eqn{(n\times q)} full column rank matrix of predictors that are not penalized.}

\item{y}{a length-\eqn{n} response vector.}

\item{penalty}{choose one from the following options: 'LASSO', for the or adaptive-Lasso penalties, 'GLASSO', 
for the group-Lasso penalty, 'spGLASSO', for the sparse group-Lasso penalty, 'OVGLASSO', 
for the overlap group-Lasso penalty and 'spOVGLASSO', for the sparse overlap group-Lasso penalty.}

\item{groups}{either a vector of length \eqn{p} of consecutive integers describing the grouping of the coefficients, 
or a list with two elements: the first element 'groups' is a vector of length \eqn{\sum_{g=1}^G n_g} containing the variables belonging to each group, where \eqn{n_g} is the cardinality of the \eqn{g}-th group,
while the second element 'Glen' is a vector of length \eqn{G} containing the group lengths (see example below).}

\item{group_weights}{a vector of length \eqn{G} containing group-specific weights. The default is square root of the group cardinality, see Yuan and Lin (2006).}

\item{var_weights}{a vector of length \eqn{p} containing variable-specific weights. The default is a vector of ones.}

\item{var_weights_L1}{a vector of length \eqn{p} containing variable-specific weights for the \eqn{L_1} penalty. The default is a vector of ones.}

\item{weights_adaptive}{logical. If \code{TRUE}, adaptive weights are computed from an initial 
least squares estimate. This option is ignored (and set to \code{FALSE}) when 
\code{group_weights} are provided by the user, since external weights override the adaptive scheme.}

\item{adaptive}{a flag for running adaptive Lasso. The default is FALSE.}

\item{standardize.data}{logical. Should data be standardized?}

\item{intercept}{logical. If it is TRUE, a column of ones is added to the design matrix.}

\item{overall.group}{logical. This setting is only available for the overlap group-LASSO and the sparse overlap group-LASSO penalties, otherwise it is set to NULL. If it is TRUE, an overall group including all penalized covariates is added.}

\item{lambda}{either a regularization parameter or a vector of regularization parameters. In this latter case the routine computes the whole path. If it is NULL values for lambda are provided by the routine.}

\item{alpha}{the sparse overlap group-LASSO mixing parameter, with \eqn{0\leq\alpha\leq1}. This setting is only available for the sparse group-LASSO and the sparse overlap group-LASSO penalties, otherwise it is set to NULL. The LASSO and group-LASSO penalties are obtained
by specifying \eqn{\alpha = 1} and \eqn{\alpha = 0}, respectively.}

\item{lambda.min.ratio}{smallest value for lambda, as a fraction of the maximum lambda value. If \eqn{n>p}, the default is 0.0001, and if \eqn{n<p}, the default is 0.01.}

\item{nlambda}{the number of lambda values - default is 30.}

\item{control}{a list of control parameters for the ADMM algorithm. See ‘Details’.}
}
\value{
A named list containing \describe{
\item{sp.coefficients}{a length-\eqn{p} solution vector for the parameters \eqn{\beta}. If \eqn{n_\lambda>1} then  the provided vector corresponds to the minimum in-sample MSE.}
\item{coefficients}{a length-\eqn{q} solution vector for the parameters \eqn{\gamma}. If \eqn{n_\lambda>1} then the provided vector corresponds to the minimum in-sample MSE.
It is provided only when either the matrix \eqn{Z} in input is not NULL or the intercept is set to TRUE.}
\item{sp.coef.path}{an \eqn{(n_\lambda\times p)} matrix of estimated \eqn{\beta} coefficients for each lambda of the provided sequence.}
\item{coef.path}{an \eqn{(n_\lambda\times q)} matrix of estimated \eqn{\gamma} coefficients for each lambda of the provided sequence.
It is provided only when either the matrix \eqn{Z} in input is not NULL or the intercept is set to TRUE.}
\item{lambda}{sequence of lambda.}
\item{lambda.min}{value of lambda that attains the minimum in sample MSE.}
\item{mse}{in-sample mean squared error.}
\item{min.mse}{minimum value of the in-sample MSE for the sequence of lambda.}
\item{convergence}{logical. 1 denotes achieved convergence.}
\item{elapsedTime}{elapsed time in seconds.}
\item{iternum}{number of iterations.}
}

When you run the algorithm, output returns not only the solution, but also the iteration history recording
following fields over iterates:
\describe{
\item{objval}{objective function value}
\item{r_norm}{norm of primal residual}
\item{s_norm}{norm of dual residual}
\item{eps_pri}{feasibility tolerance for primal feasibility condition}
\item{eps_dual}{feasibility tolerance for dual feasibility condition.}
}
Iteration stops when both \code{r_norm} and \code{s_norm} values
become smaller than \code{eps_pri} and \code{eps_dual}, respectively.
}
\description{
Sparse Adaptive overlap group-LASSO, or sparse adaptive group \eqn{L_2}-regularized regression, solves the following optimization problem
\deqn{\textrm{min}_{\beta,\gamma} ~ \frac{1}{2}\|y-X\beta-Z\gamma\|_2^2 + \lambda\Big[(1-\alpha) \sum_{g=1}^G \|S_g T\beta\|_2+\alpha\Vert T_1\beta\Vert_1\Big]}
to obtain a sparse coefficient vector \eqn{\beta\in\mathbb{R}^p} for the matrix of penalized predictors \eqn{X} and a coefficient vector \eqn{\gamma\in\mathbb{R}^q} 
for the matrix of unpenalized predictors \eqn{Z}. For each group \eqn{g}, each row of 
the matrix \eqn{S_g\in\mathbb{R}^{n_g\times p}} has non-zero entries only for those variables belonging 
to that group. These values are provided by the arguments \code{groups} and \code{group_weights} (see below). 
Each variable can belong to more than one group. The diagonal matrix \eqn{T\in\mathbb{R}^{p\times p}} contains the variable-specific weights. These values are
provided by the argument \code{var_weights} (see below). The diagonal matrix \eqn{T_1\in\mathbb{R}^{p\times p}} contains 
the variable-specific \eqn{L_1} weights. These values are provided by the argument \code{var_weights_L1} (see below).
The regularization path is computed for the sparse adaptive overlap group-LASSO penalty at a grid of values for the regularization 
parameter \eqn{\lambda} using the alternating direction method of multipliers (ADMM). See Boyd et al. (2011) and Lin et al. (2022) 
for details on the ADMM method. The regularization is a combination of \eqn{L_2} and
\eqn{L_1} simultaneous constraints. Different specifications of the \code{penalty} argument lead to different models choice:
\describe{
\item{LASSO}{The classical Lasso regularization (Tibshirani, 1996) can be obtained by specifying \eqn{\alpha = 1} 
and the matrix \eqn{T_1} as the \eqn{p \times p} identity matrix. An adaptive version of this model (Zou, 2006) can be obtained if \eqn{T_1} is 
a \eqn{p \times p}  diagonal matrix of adaptive weights. See also Hastie et al. (2015) for further details.}
\item{GLASSO}{The group-Lasso regularization (Yuan and Lin, 2006) can be obtained by specifying \eqn{\alpha = 0}, 
non-overlapping groups in \eqn{S_g} and by setting the matrix \eqn{T} equal to the \eqn{p \times p} identity matrix. 
An adaptive version of this model can be obtained if the matrix \eqn{T} is a \eqn{p \times p} diagonal matrix of adaptive weights. See also Hastie et al. (2015) for further details.}
\item{spGLASSO}{The sparse group-Lasso regularization (Simon et al., 2011) can be obtained by specifying \eqn{\alpha\in(0,1)},  
non-overlapping groups in \eqn{S_g} and by setting the matrices \eqn{T} and \eqn{T_1} equal to the \eqn{p \times p} identity matrix. 
An adaptive version of this model can be obtained if the matrices \eqn{T} and \eqn{T_1} are \eqn{p \times p}  
diagonal matrices of adaptive weights.}
\item{OVGLASSO}{The overlap group-Lasso regularization (Jenatton et al., 2011) can be obtained by specifying 
\eqn{\alpha = 0}, overlapping groups in \eqn{S_g} and by setting the matrix \eqn{T} equal to the \eqn{p \times p} identity matrix. An adaptive version of this model can be obtained if the matrix \eqn{T} is a \eqn{p \times p}  
diagonal matrix of adaptive weights.}
\item{spOVGLASSO}{The sparse overlap group-Lasso regularization (Jenatton et al., 2011) can be obtained by specifying 
\eqn{\alpha\in(0,1)}, overlapping groups in \eqn{S_g} and by setting the matrices \eqn{T} and \eqn{T_1} equal to the \eqn{p \times p} identity matrix. 
An adaptive version of this model can be obtained if the matrices \eqn{T} and \eqn{T_1} are \eqn{p \times p}  diagonal matrices of adaptive weights.}
}
}
\section{Details}{
 The control argument is a list that can supply any of the following components:\describe{
\item{adaptation}{logical. If it is TRUE, ADMM with adaptation is performed. The default value is TRUE. See Boyd et al. (2011) for details.}
\item{rho}{an augmented Lagrangian parameter. The default value is 1.}
\item{tau.ada}{an adaptation parameter greater than one. Only needed if adaptation = TRUE. The default value is 2. See Boyd et al. (2011) for details.}
\item{mu.ada}{an adaptation parameter greater than one. Only needed if adaptation = TRUE. The default value is 10. See Boyd et al. (2011) for details.}
\item{abstol}{absolute tolerance stopping criterion. The default value is sqrt(sqrt(.Machine$double.eps)).}
\item{reltol}{relative tolerance stopping criterion. The default value is sqrt(.Machine$double.eps).}
\item{maxit}{maximum number of iterations. The default value is 100.}
\item{dof.toler_c}{numeric tolerance for identifying effective zeros in the degrees-of-freedom calculation. The default is 5.}
\item{dof.toler_d}{numeric tolerance for identifying effective zeros in the degrees-of-freedom calculation. The default is 0.001.}
\item{print.out}{logical. If it is TRUE, a message about the procedure is printed. The default value is TRUE.}
}
}

\examples{

### generate sample data
set.seed(2023)
n    <- 50
p    <- 30 
X    <- matrix(rnorm(n*p), n, p)

### Example 1, LASSO penalty

beta <- apply(matrix(rnorm(p, sd = 1), p, 1), 1, fdaSP::softhresh, 1.5)
y    <- X \%*\% beta + rnorm(n, sd = sqrt(crossprod(X \%*\% beta)) / 20)

### set regularization parameter grid
lam   <- 10^seq(0, -2, length.out = 30)

### set the hyper-parameters of the ADMM algorithm
maxit      <- 1000
adaptation <- TRUE
rho        <- 1
reltol     <- 1e-5
abstol     <- 1e-5

### run example
mod <- lmSP(X = X, y = y, penalty = "LASSO", standardize.data = FALSE, intercept = FALSE, 
            lambda = lam, control = list("adaptation" = adaptation, "rho" = rho, 
                                         "maxit" = maxit, "reltol" = reltol, 
                                         "abstol" = abstol, "print.out" = FALSE)) 

### graphical presentation
matplot(log(lam), mod$sp.coef.path, type = "l", main = "Lasso solution path",
        bty = "n", xlab = latex2exp::TeX("$\\\\log(\\\\lambda)$"), ylab = "")

### Example 2, sparse group-LASSO penalty

beta <- c(rep(4, 12), rep(0, p - 13), -2)
y    <- X \%*\% beta + rnorm(n, sd = sqrt(crossprod(X \%*\% beta)) / 20)

### define groups of dimension 3 each
group1 <- rep(1:10, each = 3)

### set regularization parameter grid
lam   <- 10^seq(1, -2, length.out = 30)

### set the alpha parameter 
alpha <- 0.5

### set the hyper-parameters of the ADMM algorithm
maxit         <- 1000
adaptation    <- TRUE
rho           <- 1
reltol        <- 1e-5
abstol        <- 1e-5

### run example
mod <- lmSP(X = X, y = y, penalty = "spGLASSO", groups = group1, standardize.data = FALSE,  
            intercept = FALSE, lambda = lam, alpha = 0.5, 
            control = list("adaptation" = adaptation, "rho" = rho, 
                           "maxit" = maxit, "reltol" = reltol, "abstol" = abstol, 
                           "print.out" = FALSE)) 

### graphical presentation
matplot(log(lam), mod$sp.coef.path, type = "l", main = "Sparse Group Lasso solution path",
        bty = "n", xlab = latex2exp::TeX("$\\\\log(\\\\lambda)$"), ylab = "")

}
\references{
\insertRef{bernardi_etal.2022}{fdaSP}

\insertRef{boyd_etal.2011}{fdaSP}
 
\insertRef{hastie_etal.2015}{fdaSP}

\insertRef{jenatton_etal.2011}{fdaSP}

\insertRef{lin_etal.2022}{fdaSP} 

\insertRef{simon_etal.2013}{fdaSP}

\insertRef{yuan_lin.2006}{fdaSP}

\insertRef{zou.2006}{fdaSP}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/auxiliary_fun.R
\name{softhresh}
\alias{softhresh}
\title{Function to solve the soft thresholding problem}
\usage{
softhresh(x, lambda)
}
\arguments{
\item{x}{the data value.}

\item{lambda}{the lambda value.}
}
\value{
the solution to the soft thresholding operator.
}
\description{
Function to solve the soft thresholding problem
}
\references{
\insertRef{hastie_etal.2015}{fdaSP}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/confband.R
\name{confband}
\alias{confband}
\title{Function to plot the confidence bands}
\usage{
confband(xV, yVmin, yVmax)
}
\arguments{
\item{xV}{the values for the x-axis.}

\item{yVmin}{the minimum values for the y-axis.}

\item{yVmax}{the maximum values for the y-axis.}
}
\value{
a polygon.
}
\description{
Function to plot the confidence bands
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{forward_diff}
\alias{forward_diff}
\title{Forward finite difference approximation of order d}
\usage{
forward_diff(f, h, d)
}
\arguments{
\item{f}{Numeric vector containing function values.}

\item{h}{Step size (default = 1.0).}

\item{d}{Order of the derivative (integer >= 1).}
}
\value{
Numeric vector of forward differences of order d (length n, last d elements = NA).
#@examples
#f <- c(0, 1, 4, 9, 16)
#forward_diff(f = f, h = 1, d = 2)
}
\description{
Forward finite difference approximation of order d
}
